@@ -1,4 +1,6 @@
-/* @(#) akcs.c,v 1.2 1990/10/24 05:17:09 tron Exp */
+/*
+#ident "@(#)smail/contrib/akcs:RELEASE-3_2_0_115:akcs.c,v 1.3 2003/03/27 16:34:12 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -22,9 +24,13 @@
  *	    transport (name):  the name of the transport to use in delivering
  *		mail to local users.
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
 #include <stdio.h>
 #include <pwd.h>
-#include "defs.h"
+
 #include "../smail.h"
 #include "../smailconf.h"
 #include "../parse.h"
@@ -35,11 +41,10 @@
 #include "../exitcodes.h"
 #include "../dys.h"
 #include "akcs.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
+#include "../smailport.h"
 
 
 /*
