@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/contrib/smaillog:RELEASE-3_2_0_114:smaillog.c,v 1.8 1999/02/03 03:09:27 woods Exp"
+#ident	"@(#)smail/contrib/smaillog:RELEASE-3_2_0_115:smaillog.c,v 1.9 2003/03/25 20:33:09 woods Exp"
 */
 /*
  *
@@ -21,7 +21,7 @@
  *	Added options -l to show size of message, and
  *	      option -d to output the date stamp additionally
  *
- *	/cvs/master/smail/contrib/smaillog/smaillog.c,v 1.8 1999/02/03 03:09:27 woods Exp
+ *	/cvs/master/smail/contrib/smaillog/smaillog.c,v 1.9 2003/03/25 20:33:09 woods Exp
  */
 
 
@@ -184,8 +184,9 @@ char **argv;
  * This is the identifier that is used to index into the qlist struct.
  */
 		cp=buffer;
-		while(!isspace(*fg))
+		while (!isspace((int) *fg)) {
 			*cp++ = *fg++;
+		}
 		*cp = '\0';
 
 /*
@@ -248,7 +249,7 @@ char **argv;
 			if ( !strncmp(fg, "from: ", 6)) {
 				dest = from + strlen(from);
 				source = fg + 6;
-				while (*source && !isspace(*source) )
+				while (*source && !isspace((int) *source) )
 					*dest++ = *source++;
 				*dest++ = ' ';
 				*dest = '\0';
@@ -281,7 +282,7 @@ char **argv;
 			if ( !strncmp(fg, "via: ", 5)) {
 				dest = to + strlen(to);
 				source = fg + 5;
-				while (*source && !isspace(*source) )
+				while (*source && !isspace((int) *source) )
 					*dest++ = *source++;
 				if ((fg=skipfields(source, 1))
 							== (char *)NULL) {
@@ -296,7 +297,7 @@ char **argv;
 			if ( !strncmp(fg, "to: ", 4)) {
 				dest = to + strlen(to);
 				source = fg + 4;
-				while (*source && !isspace(*source) )
+				while (*source && !isspace((int) *source) )
 					*dest++ = *source++;
 				*dest++ = ' ';
 				*dest = '\0';
@@ -320,11 +321,11 @@ char **argv;
  * kill trailing ',' and whitespace in the 'to' and 'from' fields
  */
 			cp = to + strlen(to) - 1;
-			while ((*cp) && (isspace(*cp) || (*cp == ',')))
+			while ((*cp) && (isspace((int) *cp) || (*cp == ',')))
 				*cp-- = '\0';
 
 			cp = from + strlen(from) - 1;
-			while ((*cp) && (isspace(*cp) || (*cp == ',')))
+			while ((*cp) && (isspace((int) *cp) || (*cp == ',')))
 				*cp-- = '\0';
 /*
  * All of that for this to display it
@@ -454,17 +455,17 @@ int count;
 
 {
 	if ( count < 1 )
-		return( (char *)NULL);
+		return((char *) NULL);
 
 	while (count--) {
-		while((*str) && !(isspace(*str)))
+		while ((*str) && !(isspace((int) *str)))
 			str++;
 		if (!*str)
-			return( (char *)NULL);
-		while((*str) && (isspace(*str)))
+			return((char *) NULL);
+		while ((*str) && (isspace((int) *str)))
 			str++;
 		if (!*str)
-			return( (char *)NULL);
+			return((char *) NULL);
 	}
 	return(str);
 }
