@@ -1,4 +1,7 @@
-/* @(#) binmail.c,v 1.5 1991/09/17 11:38:26 tron Exp */
+/*
+
+#ident "@(#)smail/pd/binmail:RELEASE-3_2_0_115:binmail.c,v 1.6 2003/03/27 16:34:12 woods Exp"
+ */
 
 /* This program will be used in place of /bin/mail on System V sites.
  * It looks at the arguments and decides whether to call
@@ -11,9 +14,11 @@
  * This program works with SVR3 and SVR4 distributions from AT&T.
  */
 
-#include <stdio.h>
 #include "defs.h"
 
+#include <sys/types.h>
+#include <stdio.h>
+
 #ifndef LMAIL
 #define LMAIL "/bin/lmail"
 #endif
