@@ -1,5 +1,5 @@
 /*
- * @(#) get_opt.c,v 1.4 1991/12/09 01:57:22 tron Exp
+#ident "@(#)smail/pd/getopt:RELEASE-3_2_0_115:get_opt.c,v 1.5 2003/03/27 16:34:12 woods Exp"
  */
 
 /*
@@ -24,11 +24,13 @@
  * However, I am not about to post a copy of anything licensed by AT&T.
  */
 
-#include <stdio.h>
 #include "defs.h"
 
 #ifndef HAVE_GETOPT
 
+#include <sys/types.h>
+#include <stdio.h>
+
 /*LINTLIBRARY*/
 #define ERR(s, c)	if(opterr){\
 	extern int write();\
