@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/pd/getopt:RELEASE-3_2_0_114:getopt.c,v 1.6 2000/09/03 16:32:53 woods Exp"
+#ident	"@(#)smail/pd/getopt:RELEASE-3_2_0_115:getopt.c,v 1.7 2003/03/27 16:34:12 woods Exp"
  */
 /*
  * This file is in the public domain.
@@ -16,24 +16,21 @@
  * see the System V getopt(1) man page for usage instructions.
  */
 
-#include <stdio.h>
-#include <string.h>
 
 #include "defs.h"			/* from Smail build environment */
+
+#include <sys/types.h>
+#include <stdio.h>
+#include <string.h>
+#include <limits.h>
+
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
-#if defined(POSIX_OS)
-# include <limits.h>
+
+#if defined (HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_LIMITS_H)
-#  include <limits.h>
-# endif
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
+#endif
 
 #ifndef __P				/* in *BSD's <sys/cdefs.h>, included by everything!  */
 # if defined(__STDC__) || defined(__cplusplus)
