@@ -1,9 +1,10 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)addnode.c	9.8 95/05/04";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:addnode.c,v 1.7 1998/11/06 19:58:38 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:addnode.c,v 1.9 2003/03/27 18:57:05 woods Exp";
 #endif
 
+#include "defs.h"
 #include "def.h"
 
 #define EQ(n, s)	(*(n)->n_name == *(s) && strcmp((n)->n_name, (s)) == 0)
@@ -330,7 +331,7 @@ lowercase(s)
 	register char *s;
 {
 	do {
-		if (isupper(*s))
+		if (isupper((int) *s))
 			*s -= 'A' - 'a';	/* ASCII */
 	} while (*s++);
 }
