@@ -1,3 +1,7 @@
+/*
+#ident "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:config.h,v 1.20 2003/05/14 18:20:26 woods Exp"
+ */
+
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 
 /**************************************************************************
@@ -7,20 +11,47 @@
  **************************************************************************/
 
 /*
- * For smail3, get definitions from the defs.h file and convert them
- * for use with pathalias
+ * For smail3, get definitions from the generated "defs.h" file (#included at
+ * the top of every source file) and convert them for use with pathalias
  */
 
-#include "defs.h"
-#undef lowercase	/* to avoid conflict with SMAIL macro */
+#include <sys/types.h>
 
-#define SMAIL_3		/* changes for SMAIL-3 are #ifdef'd with this */
+#ifdef STDC_HEADERS
+# if !defined(UNIX_SUN_OS_4_1)	/* declares malloc() as "void *", but yacc doesn't */
+#  include <stdlib.h>
+# endif
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
 
-#undef STRCHR
-#define BZERO		/* always avail. to smail compile */
-#undef MEMSET
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
 
-#undef SMALL		/* run slower (definitely), use less vm (maybe). */
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
+#include "smail.h"
+
+#undef EQ		/* to avoid conflict with macro defined in smail.h */
+
+#define SMAIL_3	1	/* special changes for SMAIL-3 are #ifdef'd with this */
+
+#undef SMALL		/* smail-3 is big already, so pathalias can be too */
+
+#define MEMSET	1	/* smail-3 code uses memset() exclusively */
+#define STRCHR	1	/* smail-3 code use str[r]chr() exclusively */
 
 #undef UNAME
 #undef GETHOSTNAME
@@ -32,15 +63,11 @@
 # endif
 #endif
 
-#ifdef HAVE_BIND
-# define RESOLVER	/* have internet domain name resolver */
-#endif
-
 /* default place for dbm output of makedb (or use -o at run-time) */
 #ifdef PATH_PATHS_FILE
-#define ALIASDB PATH_PATHS_FILE
+# define ALIASDB	PATH_PATHS_FILE
 #else
-#define	ALIASDB	"/usr/local/lib/palias"
+# define ALIASDB	"/usr/local/lib/palias"
 #endif
 
 /*
@@ -61,7 +88,7 @@
 #ifdef MAIN
 #ifndef lint
 static char	*c_sccsid = "@(#)config.h	9.6 95/05/04";
-static char	*c_rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:config.h,v 1.10 1997/05/09 20:24:01 woods Exp";
+static char	*c_rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:config.h,v 1.20 2003/05/14 18:20:26 woods Exp";
 #endif /*lint*/
 #endif /*MAIN*/
 
@@ -84,47 +111,48 @@ static char	*c_rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:config.h,v 1.10
 #endif
 
 #ifdef MYMALLOC
-#define malloc mymalloc
-#define calloc(n, s) malloc ((n)*(s))
-#define free(s)
-#define cfree(s)
+
+# define malloc mymalloc
+# define calloc(n, s) malloc ((n)*(s))
+# define free(s)
+# define cfree(s)
 extern char *memget();
+
 #else /* !MYMALLOC */
-#if defined(SMAIL_3) && defined(ANSI_C) && !defined(NO_VOID_CALLOC)
-extern void *calloc();
-#else
+
+# ifndef MALLOC_DECLARED
+#  if defined(SMAIL_3) && !defined(NO_VOID_MALLOC)
+extern void *malloc __P((size_t));
+extern void *calloc __P((size_t, size_t));
+#  else
+extern char *malloc();
 extern char *calloc();
-#endif
+#  endif
+# endif
+
 #endif /* MYMALLOC */
 
-#ifndef SMAIL_3
 #ifdef STRCHR
-#define index strchr
-#define rindex strrchr
+# define index strchr
+# define rindex strrchr
 #else
-#define strchr index
-#define strrchr rindex
+# define strchr index
+# define strrchr rindex
 #endif
-#endif	/* SMAIL_3 */
 
 #ifdef BZERO
-#define strclear(s, n)	((void) bzero((s), (n)))
+# define strclear(s, n)	((void) bzero((s), (n)))
 #else /*!BZERO*/
-
-#ifdef MEMSET
+# ifdef MEMSET
+#  ifndef SMAIL_3
 extern char	*memset();
-#define strclear(s, n)	((void) memset((s), 0, (n)))
-#else /*!MEMSET*/
+#  endif
+#  define strclear(s, n) ((void) memset((s), 0, (n)))
+# else /*!MEMSET*/
 extern void	strclear();
-#endif /*MEMSET*/
-
+# endif /*MEMSET*/
 #endif /*BZERO*/
 
-#if defined(SMAIL_3) && defined(ANSI_C) && !defined(NO_VOID_MALLOC)
-extern void	*malloc();
-#else
-extern char	*malloc();
-#endif
 #ifndef SMAIL_3
 extern char	*strcpy(), *index(), *rindex();
 #endif
