@@ -1,10 +1,11 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)domain.c	9.5 92/08/25";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:domain.c,v 1.6 1998/11/06 19:58:38 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:domain.c,v 1.9 2003/03/27 18:59:56 woods Exp";
 #endif /* lint */
 
-#define NEED_SOCKETS			/* only for smail-3 */
+#include "defs.h"				/* Smail-3 */
+
 #include "def.h"
 
 /* imports */
@@ -97,14 +98,32 @@ movetofront(headp, d)
 	*headp = d;
 }
 
+/*
+ * WARNING: put no more code using "struct node" below here!!!
+ */
+
+#if defined(SMAIL_3) && defined(HAVE_BSD_NETWORKING) && defined(HAVE_BIND)
+# define RESOLVER	/* have internet domain name resolver */
+#endif
+
 #ifdef RESOLVER
+
+/* XXX at lease n_net buggers <netdb.h>, so clobber them all */
+#undef n_root					/* from def.h */
+#undef n_net					/* from def.h */
+#undef n_copy					/* from def.h */
+#undef n_private				/* from def.h */
+#undef n_parent					/* from def.h */
+
+#include "smailsock.h"				/* Smail-3 */
+
 int
 nslookup(domain)
 	char *domain;
 {	register HEADER *hp;
 	register int n;
-	char q[PACKETSZ], a[PACKETSZ];	/* query, answer */
-	char buf[PACKETSZ+1];
+	char q[NS_PACKETSZ], a[NS_PACKETSZ];	/* query, answer */
+	char buf[NS_PACKETSZ+1];
 
 	if ((n = strlen(domain)) >= PACKETSZ)
 		return 0;
@@ -117,13 +136,13 @@ nslookup(domain)
 	 * LIES! DAMN LIES!
 	 * 
 	 * The manual page resolver(3) says arg#7 is (struct rrec *) but the
-	 * prototypes say (const char *), others say (const u_char *).
+	 * prototypes and definitions all say (const char *) or [BNID-8 and
+	 * newer] (const u_char *).
 	 * 
-	 * Note though that res_mkquery() is not used by the rest of
-	 * smail-3 if OBSOLETE_ERSOLVER is defined [XXX use res_search()].
+	 * Note also that res_mkquery() is not used by the rest of smail-3.
 	 */
 	if ((n = res_mkquery(QUERY, buf, C_IN, T_ANY, (unsigned char *) NULL, 0,
-			     (struct rrec *) NULL, (unsigned char *) q,
+			     (const u_char *) NULL, (unsigned char *) q,
 			     sizeof(q))) < 0)
 		die("impossible res_mkquery error");
 	errno = 0;
