@@ -1,9 +1,11 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)local.c	9.3 91/06/11";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:local.c,v 1.4 1997/02/09 21:50:50 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:local.c,v 1.6 2003/03/27 18:57:05 woods Exp";
 #endif /* lint */
 
+#include "defs.h"				/* Smail-3 */
+
 #include "config.h"
 
 #include <stdio.h>
@@ -14,11 +16,11 @@ static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:local.c,v 1.4 199
 char	*
 local()
 {
-	static struct utsname utsname;
+	static struct utsname utsnm;
 	extern int uname();
 
-	(void) uname(&utsname);
-	return(utsname.nodename);
+	(void) uname(&utsnm);
+	return(utsnm.nodename);
 }
 
 #else /* !UNAME */
