@@ -1,10 +1,19 @@
-/* pathalias -- by steve bellovin, as told to peter honeyman */
+/*
+#ident "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:makedb.c,v 1.14 2003/03/27 18:57:05 woods Exp"
+ */
+
+/*
+ * pathalias -- by steve bellovin, as told to peter honeyman
+ */
 #ifndef lint
 static char	*sccsid = "@(#)makedb.c	9.1 87/10/04";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:makedb.c,v 1.9 1998/11/06 19:58:39 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:makedb.c,v 1.14 2003/03/27 18:57:05 woods Exp";
 #endif /* lint */
 
+#include "defs.h"				/* Smail-3 */
+
 #include <stdio.h>
+
 #include "config.h"
 
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
@@ -14,37 +23,35 @@ static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:makedb.c,v 1.9 19
 #  include <sys/file.h>
 # endif
 #endif
-#ifdef HAVE_STDLIB
-# include <stdlib.h>
+#ifdef __GLIBC__
+# include <sys/file.h>
 #endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
 
-#ifdef SMAIL_3
-# include "dbm_compat.h"
+#include "smailport.h"
+
+#include "dbm_compat.h"
+
+#if defined(HAVE_DBM) && !defined(HAVE_NDBM)
+static int dbfile __P((char *));
+static int dbcreat __P((char *));
 #else
-typedef struct {
-	char *dptr;
-	int dsize;
-} datum;
+# define dbfile(f)	(0)
+# define dbcreat(f,s)	(0)
 #endif
 
-int dbfile();
-int dbcreat();
-void makedb();
-void perror_();
+static void makedb __P((char *));
+static void perror_ __P((char *));
 
 char *Ofile = ALIASDB, *ProgName;
 
 #define USAGE "%s [-o dbmname] [-a] [file ...]\n"
 
+/* silly ANSI C warning avoidance... */
+int main __P((int, char **));
+
 int
 main(argc, argv)
+	int argc;
 	char *argv[];
 {	char *ofptr;
 	int c, append = 0;
@@ -73,10 +80,8 @@ main(argc, argv)
 		ofptr++;
 	else
 		ofptr = Ofile;
-#if defined(SMAIL_3) && defined(ANSI_C) && !defined(NO_SIZE_T_STRLEN)
+#if defined(ANSI_C) && !defined(NO_SIZE_T_STRLEN)
 	if (strlen(ofptr) > (size_t)10)
-#else
-	if (strlen(ofptr) > 10)
 #endif
 	{
 		ofptr[10] = 0;
@@ -103,14 +108,15 @@ main(argc, argv)
 	exit(EX_OK);
 }
 
-int
+#if defined(HAVE_DBM) && !defined(HAVE_NDBM)
+static int
 dbfile(dbf)
 	char *dbf;
 {
 	return (dbcreat(dbf, "dir") != 0 || dbcreat(dbf, "pag") != 0);
 }
 
-int
+static int
 dbcreat(dbf, suffix)
 	char *dbf, *suffix;
 {	char buf[BUFSIZ];
@@ -122,8 +128,9 @@ dbcreat(dbf, suffix)
 	(void) close(fd);
 	return(0);
 }
+#endif
 
-void
+static void
 makedb(ifile)
 	char *ifile;
 {	char line[BUFSIZ];
@@ -160,7 +167,7 @@ makedb(ifile)
 	}
 }
 
-void
+static void
 perror_(str)
 	char	*str;
 {
