@@ -1,9 +1,11 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)mapit.c	9.18 95/07/12";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:mapit.c,v 1.6 1998/11/06 19:58:40 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:mapit.c,v 1.8 2003/03/27 18:57:05 woods Exp";
 #endif
 
+#include "defs.h"				/* Smail-3 */
+
 #include "def.h"
 
 #define chkheap(i)	/* void */
@@ -134,12 +136,12 @@ heapchildren(n)
 		if (l->l_flag & LTERMINAL)
 			l->l_to = next = ncopy(n, l);
 
-		if ((n->n_flag & NTERMINAL) && (l->l_flag & LALIAS))
+		if ((n->n_flag & NTERMINAL) && (l->l_flag & LALIAS)) {
 			if (skipterminalalias(n, next))
 				continue;
 			else
 				l->l_to = next = ncopy(n, l);
-
+		}
 		if (next->n_flag & MAPPED) {
 			if (mtrace)
 				mtracereport(n, l, "-\talready mapped");
