@@ -1,9 +1,11 @@
 /* pathalias -- by steve bellovin, as told to peter honeyman */
 #ifndef lint
 static char	*sccsid = "@(#)printit.c	9.4 89/02/07";
-static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_114:printit.c,v 1.5 1997/02/09 21:51:08 woods Exp";
+static char	*rcsid = "@(#)smail/pd/pathalias:RELEASE-3_2_0_115:printit.c,v 1.7 2003/03/27 18:57:05 woods Exp";
 #endif
 
+#include "defs.h"				/* Smail-3 */
+
 #include "def.h"
 
 /*
@@ -195,12 +197,12 @@ setpath(l, ppath, npath)
 		return;
 	}
 		
-	if (netchar == '@')
+	if (netchar == '@') {
 		if (next->n_flag & ATSIGN)
 			netchar = '%';	/* shazam?  shaman? */
 		else
 			next->n_flag |= ATSIGN;
-
+	}
 	/* remainder should be a sprintf -- foo on '%' as an operator */
 	for ( ; (*npath = *ppath) != 0; ppath++) {
 		if (*ppath == '%') {
