@@ -1,32 +1,49 @@
-/* @(#) dba.c,v 1.1 1992/05/12 00:00:00 tron Exp%U&% */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:dba.c,v 1.2 2003/03/27 16:49:22 woods Exp"
+ */
 /*
  * dba	dbm analysis/recovery
  */
 
 #include "defs.h"
-#include "sdbm.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
-#ifdef UNIX_BSD
-#include <sys/file.h>
+
+#include "tune.h"
+#include "sdbm.h"
+
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
 #else
-#include <fcntl.h>
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
 #endif
 
 char *progname;
-extern void oops();
+extern int main __P((int, char **));
+extern void oops __P((char *, ...));		/* util.c */
+extern int okpage __P((char *));		/* util.c */
+
+static void sdump __P((int));
+static int pagestat __P((char *));
+
 
 int
 main(argc, argv)
-char **argv;
+	int argc;
+	char **argv;
 {
-	int n;
+	size_t n;
 	char *p;
 	char *name;
 	int pagf;
 
 	progname = argv[0];
 
-	if (p = argv[1]) {
+	if ((p = argv[1])) {
 		name = (char *) malloc((n = strlen(p)) + 5);
 		strcpy(name, p);
 		strcpy(name + n, ".pag");
@@ -39,17 +56,19 @@ char **argv;
 	else
 		oops("usage: %s dbname", progname);
 
-	return 0;
+	exit(0);
+	/* NOTREACHED */
 }
 
+static void
 sdump(pagf)
-int pagf;
+	int pagf;
 {
-	register b;
-	register n = 0;
-	register t = 0;
-	register o = 0;
-	register e;
+	register int b;
+	register int n = 0;
+	register int t = 0;
+	register int o = 0;
+	register int e;
 	char pag[PBLKSIZ];
 
 	while ((b = read(pagf, pag, PBLKSIZ)) > 0) {
@@ -72,11 +91,12 @@ int pagf;
 		oops("read failed: block %d", n);
 }
 
+static int
 pagestat(pag)
-char *pag;
+	char *pag;
 {
-	register n;
-	register free;
+	register int n;
+	register int free;
 	register short *ino = (short *) pag;
 
 	if (!(n = ino[0]))
