@@ -1,35 +1,51 @@
-/* @(#) dbd.c,v 1.1 1992/05/12 00:00:00 tron Exp%U&% */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:dbd.c,v 1.2 2003/03/27 16:49:22 woods Exp"
+ */
 /*
  * dbd - dump a dbm data file
  */
 
 #include "defs.h"
-#include "sdbm.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
-#ifdef UNIX_BSD
-#include <sys/file.h>
+
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
 #else
-#include <fcntl.h>
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
 #endif
 
+#include "tune.h"
+#include "sdbm.h"
+
 char *progname;
-extern void oops();
+extern int main __P((int, char **));
+extern void oops __P((char *, ...));		/* util.c */
+extern int okpage __P((char *));		/* util.c */
+
+static void sdump __P((int));
+static void dispage __P((char *));
 
 
 #define empty(page)	(((short *) page)[0] == 0)
 
 int
 main(argc, argv)
-char **argv;
+	int argc;
+	char **argv;
 {
-	int n;
+	size_t n;
 	char *p;
 	char *name;
 	int pagf;
 
 	progname = argv[0];
 
-	if (p = argv[1]) {
+	if ((p = argv[1])) {
 		name = (char *) malloc((n = strlen(p)) + 5);
 		strcpy(name, p);
 		strcpy(name + n, ".pag");
@@ -41,15 +57,18 @@ char **argv;
 	}
 	else
 		oops("usage: %s dbname", progname);
-	return 0;
+
+	exit(0);
+	/* NOTREACHED */
 }
 
+static void
 sdump(pagf)
-int pagf;
+	int pagf;
 {
-	register r;
-	register n = 0;
-	register o = 0;
+	register int r;
+	register int n = 0;
+	register int o = 0;
 	char pag[PBLKSIZ];
 
 	while ((r = read(pagf, pag, PBLKSIZ)) > 0) {
@@ -92,11 +111,13 @@ char *pag;
 	}
 }
 #else
+static void
 dispage(pag)
-char *pag;
+	char *pag;
 {
-	register i, n;
-	register off;
+	register int i;
+	register int n;
+	register int off;
 	register short *ino = (short *) pag;
 
 	off = PBLKSIZ;
