@@ -1,4 +1,6 @@
-/* @(#) dbm.h,v 1.1 1992/05/12 00:00:00 tron Exp%U&% */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:dbm.h,v 1.2 2001/10/12 20:09:09 woods Exp"
+ */
 /*
  * Copyright (c) 1983 The Regents of the University of California.
  * All rights reserved.
@@ -27,7 +29,7 @@
 #define    NULL    0
 #endif
 
-#include <ndbm.h>
+#include <sdbm.h>
 
 datum    fetch();
 datum    firstkey();
