@@ -1,19 +1,29 @@
-/* @(#) dbu.c,v 1.1 1992/05/12 00:00:00 tron Exp%U&% */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:dbu.c,v 1.3 2003/03/27 19:01:24 woods Exp"
+ */
+
 #include "defs.h"
-#ifdef SDBM
-#include "sdbm.h"
-#else
-#include <ndbm.h>
-#endif
+
+#include <sys/types.h>
 #include <stdio.h>
-#ifdef UNIX_BSD
-#include <sys/file.h>
+
+#ifdef SDBM
+# include "tune.h"
+# include "sdbm.h"
 #else
-#include <fcntl.h>
+# include <ndbm.h>
 #endif
 
-extern int	getopt();
-extern void	oops();
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
+#else
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
+#endif
+
+extern int	main __P((int, char **));
+extern void	oops __P((char *, char *));
 
 char *progname;
 
@@ -39,34 +49,36 @@ typedef struct {
 
 static cmd cmds[] = {
 
-	"fetch", DLOOK, 	O_RDONLY,
-	"get", DLOOK,		O_RDONLY,
-	"look", DLOOK,		O_RDONLY,
-	"add", DINSERT,		O_RDWR,
-	"insert", DINSERT,	O_RDWR,
-	"store", DINSERT,	O_RDWR,
-	"delete", DDELETE,	O_RDWR,
-	"remove", DDELETE,	O_RDWR,
-	"dump", DCAT,		O_RDONLY,
-	"list", DCAT, 		O_RDONLY,
-	"cat", DCAT,		O_RDONLY,
-	"creat", DCREAT,	O_RDWR | O_CREAT | O_TRUNC,
-	"new", DCREAT,		O_RDWR | O_CREAT | O_TRUNC,
-	"build", DBUILD,	O_RDWR | O_CREAT,
-	"squash", DPRESS,	O_RDWR,
-	"compact", DPRESS,	O_RDWR,
-	"compress", DPRESS,	O_RDWR
+	{ "fetch", DLOOK, 	O_RDONLY },
+	{ "get", DLOOK,		O_RDONLY },
+	{ "look", DLOOK,	O_RDONLY },
+	{ "add", DINSERT,	O_RDWR },
+	{ "insert", DINSERT,	O_RDWR },
+	{ "store", DINSERT,	O_RDWR },
+	{ "delete", DDELETE,	O_RDWR },
+	{ "remove", DDELETE,	O_RDWR },
+	{ "dump", DCAT,		O_RDONLY },
+	{ "list", DCAT, 	O_RDONLY },
+	{ "cat", DCAT,		O_RDONLY },
+	{ "creat", DCREAT,	O_RDWR | O_CREAT | O_TRUNC },
+	{ "new", DCREAT,	O_RDWR | O_CREAT | O_TRUNC },
+	{ "build", DBUILD,	O_RDWR | O_CREAT },
+	{ "squash", DPRESS,	O_RDWR },
+	{ "compact", DPRESS,	O_RDWR },
+	{ "compress", DPRESS,	O_RDWR }
 };
 
 #define CTABSIZ (sizeof (cmds)/sizeof (cmd))
 
-static cmd *parse();
-static void badk(), doit(), prdatum();
+static cmd *parse __P((char *));
+static void badk __P((char *));
+static void doit __P((cmd *, char *));
+static void prdatum __P((FILE *, datum));
 
 int
 main(argc, argv)
-int	argc;
-char *argv[];
+	int	argc;
+	char *argv[];
 {
 	int c;
 	register cmd *act;
@@ -93,13 +105,15 @@ char *argv[];
 		badk(argv[optind]);
 	optind++;
 	doit(act, argv[optind]);
-	return 0;
+
+	exit(0);
+	/* NOTREACHED */
 }
 
 static void
 doit(act, file)
-register cmd *act;
-char *file;
+	register cmd *act;
+	char *file;
 {
 	datum key;
 	datum val;
@@ -197,7 +211,7 @@ char *file;
 
 static void
 badk(word)
-char *word;
+	char *word;
 {
 	register int i;
 
@@ -214,7 +228,7 @@ char *word;
 
 static cmd *
 parse(str)
-register char *str;
+	register char *str;
 {
 	register int i = CTABSIZ;
 	register cmd *p;
@@ -227,8 +241,8 @@ register char *str;
 
 static void
 prdatum(stream, d)
-FILE *stream;
-datum d;
+	FILE *stream;
+	datum d;
 {
 	register int c;
 	register char *p = d.dptr;
