@@ -1,4 +1,6 @@
-/* @(#) $Id: hash.c,v 1.1 1992/05/12 11:03:49 tron Exp $ */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:hash.c,v 1.2 2003/03/27 16:49:22 woods Exp"
+ */
 /*
  * sdbm - ndbm work-alike hashed database library
  * based on Per-Aake Larson's Dynamic Hashing algorithms. BIT 18 (1978).
@@ -9,6 +11,8 @@
  */
 
 #include "defs.h"
+
+#include <sys/types.h>
 #include "sdbm.h"
 
 /*
