@@ -1,4 +1,6 @@
-/* @(#) $Id: pair.c,v 1.3 1992/08/01 20:02:20 tron Exp $ */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:pair.c,v 1.4 2003/03/27 16:49:22 woods Exp"
+ */
 /*
  * sdbm - ndbm work-alike hashed database library
  * based on Per-Aake Larson's Dynamic Hashing algorithms. BIT 18 (1978).
@@ -9,26 +11,23 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: pair.c,v 1.3 1992/08/01 20:02:20 tron Exp $";
+static char rcsid[] = "pair.c,v 1.4 2003/03/27 16:49:22 woods Exp";
 #endif
 
-#include "defs.h"
-#include "sdbm.h"
+#include "defs.h"				/* Smail-3 */
+
+#include <sys/types.h>
+
 #include "tune.h"
+#include "sdbm.h"
 #include "pair.h"
 
-#ifdef __STDC__
-#define P_(p) p
-#else
-#define P_(p) ()
-#endif
-
 #define exhash(item)	dbm_hash((item).dptr, (item).dsize)
 
 /* 
  * forward 
  */
-static int seepair P_((char *, int, char *, int));
+static int seepair __P((char *, int, char *, int));
 
 /*
  * page format:
@@ -84,13 +83,13 @@ datum val;
  * enter the key first
  */
 	off -= key.dsize;
-	(void) memcpy(pag + off, key.dptr, key.dsize);
+	(void) memcpy(pag + off, key.dptr, (size_t) key.dsize);
 	ino[n + 1] = off;
 /*
  * now the data
  */
 	off -= val.dsize;
-	(void) memcpy(pag + off, val.dptr, val.dsize);
+	(void) memcpy(pag + off, val.dptr, (size_t) val.dsize);
 	ino[n + 2] = off;
 /*
  * adjust item count
@@ -200,7 +199,7 @@ datum key;
 		}
 #else
 #if (defined (HAVE_MEMMOVE) || defined(ANSI_C)) && !defined(NO_HAVE_MEMMOVE)
-		memmove(dst, src, m);
+		memmove(dst, src, (size_t) m);
 #else
 		while (m--)
 			*--dst = *--src;
@@ -236,7 +235,7 @@ register int siz;
 
 	for (i = 1; i < n; i += 2) {
 		if (siz == off - ino[i] &&
-		    memcmp(key, pag + ino[i], siz) == 0)
+		    memcmp(key, pag + ino[i], (size_t) siz) == 0)
 			return i;
 		off = ino[i + 1];
 	}
