@@ -1,4 +1,6 @@
-/* @(#) sdbm.c,v 1.2 1992/08/14 23:37:55 tron Exp */
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:sdbm.c,v 1.3 2003/03/27 16:49:22 woods Exp"
+ */
 /*
  * sdbm - ndbm work-alike hashed database library
  * based on Per-Aake Larson's Dynamic Hashing algorithms. BIT 18 (1978).
@@ -9,44 +11,36 @@
  */
 
 #ifndef lint
-static char rcsid[] = "sdbm.c,v 1.2 1992/08/14 23:37:55 tron Exp";
+static char rcsid[] = "sdbm.c,v 1.3 2003/03/27 16:49:22 woods Exp";
 #endif
 
-#include "defs.h"
-#include "sdbm.h"
-#include "tune.h"
-#include "pair.h"
+#include "defs.h"				/* Smail-3 */
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifdef UNIX_BSD
-#include <sys/file.h>
-#else
-#include <fcntl.h>
-#include <memory.h>
-#endif
 #include <errno.h>
-#include <string.h>
 
-#ifndef SEEK_SET
-#define SEEK_SET  0
-#endif	/* SEEK_SET */
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
+#else
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
+#endif
+
+#include "tune.h"
+#include "sdbm.h"
+#include "pair.h"
 
 /*
  * forward
  */
 
-#ifdef ANSI_C
-#define P_(p) p
-#else
-#define P_(p) ()
-#endif
-
-static int getdbit P_((DBM *, long));
-static int setdbit P_((DBM *, long));
-static int getpage P_((DBM *, long));
-static datum getnext P_((DBM *));
-static int makroom P_((DBM *, long, int));
+static int getdbit __P((DBM *, long));
+static int setdbit __P((DBM *, long));
+static int getpage __P((DBM *, long));
+static datum getnext __P((DBM *));
+static int makroom __P((DBM *, long, int));
 
 /*
  * useful macros
@@ -55,8 +49,8 @@ static int makroom P_((DBM *, long, int));
 #define exhash(item)	dbm_hash((item).dptr, (item).dsize)
 #define ioerr(db)	((db)->flags |= DBM_IOERR)
 
-#define OFF_PAG(off)	(long) (off) * PBLKSIZ
-#define OFF_DIR(off)	(long) (off) * DBLKSIZ
+#define OFF_PAG(off)	(off_t) ((off) * PBLKSIZ)
+#define OFF_DIR(off)	(off_t) ((off) * DBLKSIZ)
 
 static long masks[] = {
 	000000000000, 000000000001, 000000000003, 000000000007,
