@@ -1,34 +1,58 @@
-/* @(#) util.c,v 1.1 1992/05/12 11:03:56 tron Exp */
-#include "defs.h"
-#ifdef SDBM
-#include "sdbm.h"
-#else
-#include <ndbm.h>
-#endif
-#include <stdio.h>
+/*
+#ident "@(#)smail/pd/sdbm:RELEASE-3_2_0_115:util.c,v 1.2 2003/03/27 16:49:22 woods Exp"
+ */
 
-#ifdef ANSI_C
-#include <string.h>
+#include "defs.h"				/* smail-3 */
+
+#include <sys/types.h>
+#include <stdio.h>
 #include <errno.h>
+
+#ifdef SDBM
+# include "tune.h"
+# include "sdbm.h"
 #else
+# include <ndbm.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+# define VA_START(args, lastarg)       va_start(args, lastarg)
+#else
+# include <varargs.h>
+# define VA_START(args, lastarg)       va_start(args)
+#endif
+
+#ifndef __STDC__
 extern int errno, sys_nerr;
 extern char *sys_errlist[];
 #endif
 
+extern void oops __P((register char *fmt, ...));
+extern int okpage __P((char *));
+
 extern char *progname;
 
+/* VARARGS1 */
 void
-oops(s1, s2)
-register char *s1;
-register char *s2;
+#ifdef __STDC__
+oops(register char *fmt, ...)
+#else
+oops(fmt, va_alist)
+	register char *fmt;
+	va_dcl
+#endif
 {
 	int e = errno;
 	char *emsg = NULL;
+	va_list ap;
 
 	if (progname)
 		fprintf(stderr, "%s: ", progname);
-	fprintf(stderr, s1, s2);
-#ifdef ANSI_C
+	VA_START(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+#ifdef HAVE_STRERROR
 	emsg = strerror(e);
 #else
 	if (e > 0 && e < sys_nerr)
@@ -45,7 +69,7 @@ okpage(pag)
 char *pag;
 {
 	register unsigned n;
-	register off;
+	register int off;
 	register short *ino = (short *) pag;
 
 	if ((n = ino[0]) > PBLKSIZ / sizeof(short))
