@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/pd/uuwho:RELEASE-3_2_0_114:uuwho.c,v 1.22 2000/02/06 16:31:51 woods Exp"
+#ident	"@(#)smail/pd/uuwho:RELEASE-3_2_0_115:uuwho.c,v 1.26 2003/04/02 22:15:07 woods Exp"
  */
 /*
  * uuwho:
@@ -22,14 +22,32 @@
  * Modified to search parent domains by Chip Salzenberg.
  */
 
-#include <stdio.h>
-#include <ctype.h>
-
 #include "defs.h"
 
-#ifdef HAVE_STDLIB_H
+#include <sys/types.h>
+#include <stdio.h>
+#include <ctype.h>
+#include <limits.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
 # include <fcntl.h>
 #else
@@ -37,47 +55,42 @@
 #  include <sys/file.h>
 # endif
 #endif
-#if defined(POSIX_OS)
-# include <limits.h>
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_LIMITS_H)
-#  include <limits.h>
-# endif
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
+#endif
 
 #ifdef UUWHO_USE_DBM
-#include "dbm_compat.h"
+# include "dbm_compat.h"
 #endif
 
 #ifdef HAVE_UNAME
-#include <sys/utsname.h>
+# include <sys/utsname.h>
 #endif /* HAVE_UNAME */
 
+#include "smail.h"
+#include "smailport.h"
+
 #ifndef UUWHO_FILE
-#define UUWHO_FILE "uuwho"	/* database basename in SMAIL_LIB_DIR */
+# define UUWHO_FILE	 "uuwho"	/* database basename in SMAIL_LIB_DIR */
 #endif
 
-#define NLEN	512
+#define NLEN		512
 
-void getfile();
-void usage();
-
-static char *cmdname;		/* same as argv[0] */
+static char *argv0 = "uuwho";
 
 /* variable length record stored in who database */
 typedef struct {
 	long	pos;
 	char	*name;
-} position;
+} position_t;
 
-static int find();		/* find host's map entry */
-static void printmap();		/* print a map form using the location form dbm */
-static char *skip_map_dir();	/* skip map directory in pathname */
-static char *get_hostname();	/* get our histname */
+static void getfile __P((char *));		/* get a file into the who db */
+static void usage __P((void));			/* print usage message */
+static int find __P((position_t *, char *));	/* find host's map entry */
+static void printmap __P((position_t *));		/* print a map form using the location form dbm */
+static char *skip_map_dir __P((char *, unsigned int));	/* skip map directory in pathname */
+static char *get_hostname __P((void));		/* get our histname */
 
 static char line[BUFSIZ + 1];
 static FILE *fp;
@@ -87,16 +100,22 @@ static FILE *db;		/* pipe to sort process, or open database */
 
 static int rawformat = 0;
 
-extern int errno;
+#if !STDC_HEADERS && !defined(AHVE_STDLIB_H)
 extern long atol();
-#if defined(ANSI_C) && !defined(NO_VOID_MALLOC)
+#endif
+
+#ifndef MALLOC_DECLARED
+# if defined(ANSI_C) && !defined(NO_VOID_MALLOC)
 extern void	*malloc();
 extern void	*realloc();
-#else
+# else
 extern char	*malloc();
 extern char	*realloc();
+# endif
 #endif
 
+extern int main __P((int, char **));
+
 int
 main(argc, argv)
     int argc;
@@ -108,15 +127,16 @@ main(argc, argv)
     char *hname;		/* host name to lookup */
     int opt;			/* flag char */
     int newdbm = 0;		/* 1 ==> new database creation is requested */
-    position pos;		/* the fetched record */
+    position_t pos;		/* the fetched record */
     extern char *optarg;	/* the arg of a flag */
     extern int optind;		/* the first non flag */
 
     /*
      * parse args
      */
-    cmdname = argv[0];
-    while ((opt = getopt(argc,argv,"ird:")) != EOF) {
+    argv0 = (argv0 = strrchr(argv[0], '/')) ? argv0 + 1 : argv[0];
+
+    while ((opt = getopt(argc, argv, "ird:")) != -1) {
 	switch (opt) {
 	case 'i':	/* new database being formed */
 	    newdbm = 1;
@@ -147,7 +167,7 @@ main(argc, argv)
     } else {
 	fname = malloc(strlen(SMAIL_LIB_DIR)+1+strlen(fbasename)+1);
 	if (fname == NULL) {
-	    perror(cmdname);
+	    perror(argv0);
 	    exit(1);
 	}
 	sprintf(fname, "%s/%s", SMAIL_LIB_DIR, fbasename);
@@ -156,7 +176,7 @@ main(argc, argv)
 #ifdef UUWHO_USE_DBM
     touchdbm(fname,newdbm);
     if (dbminit(fname) < 0) {
-	fprintf(stderr, "%s: dbinit(%s) failed\n", cmdname, fname);
+	fprintf(stderr, "%s: dbinit(%s) failed\n", argv0, fname);
 	exit(2);
     }
 #else
@@ -166,7 +186,7 @@ main(argc, argv)
 
 	sortcmd = malloc(strlen(fname) * 2 + sizeof("sort +0 -1 > \"\""));
 	if (sortcmd == NULL) {
-	    perror(cmdname);
+	    perror(argv0);
 	    exit(1);
 	}
 	strcpy(sortcmd, "sort +0 -1 > \"");
@@ -188,7 +208,7 @@ main(argc, argv)
 
 	db = popen(sortcmd, "w");
 	if (db == NULL) {
-	    perror(cmdname);
+	    perror(argv0);
 	    exit(1);
 	}
     } else {
@@ -238,14 +258,14 @@ main(argc, argv)
 
 static int
 find(posp, hname)
-    position *posp;
+    position_t *posp;
     char *hname;
 {
     char *h, *t;
     int ret = 0;
 
     if ((t = malloc(strlen(hname) + 2)) == NULL) {
-	perror(cmdname);
+	perror(argv0);
 	exit(1);
     }
     *t = '.';
@@ -271,7 +291,7 @@ find(posp, hname)
 
 static int
 find_exact(posp, hname)
-    position *posp;
+    position_t *posp;
     char *hname;
 {
     datum fileinfo;
@@ -284,12 +304,12 @@ find_exact(posp, hname)
     if (fileinfo.dptr == NULL)
 	return 0;
     p = fileinfo.dptr;
-    while (isspace(*p))
+    while (isspace((int) *p))
 	p++;
     posp->pos = atol(p);
-    while (*p && !isspace(*p))
+    while (*p && !isspace((int) *p))
 	p++;
-    while (isspace(*p))
+    while (isspace((int) *p))
 	p++;
     posp->name = p;
     return 1;
@@ -299,7 +319,7 @@ find_exact(posp, hname)
 
 static int
 find(posp, hname)
-    position *posp;
+    position_t *posp;
     char *hname;
 {
     long middle, hi, lo;
@@ -310,7 +330,7 @@ find(posp, hname)
     char *p;
 
 #ifndef SEEK_END
-#define SEEK_END 2
+# define SEEK_END 2
 #endif
     lo = 0;
     fseek(db, 0L, SEEK_END);
@@ -327,16 +347,16 @@ find(posp, hname)
 	if (middle != 0) {		/* to beginning of next line */
 	    while((c = getc(db)) != EOF && c != '\n') ;
 	    if (c == EOF && ferror(db)) {
-		perror(cmdname);
+		perror(argv0);
 		exit(1);
 	    }
 	}
 	p = ibuf;
-	while ((c = getc(db)) != EOF && !isspace(c) && c != ':') {
+	while ((c = getc(db)) != EOF && !isspace((int) c) && c != ':') {
 	    *p++ = c;
 	}
 	if (c == EOF && ferror(db)) {
-	    perror(cmdname);
+	    perror(argv0);
 	    exit(1);
 	}
 	*p = '\0';
@@ -360,7 +380,7 @@ find(posp, hname)
     flag = 0;
     gotnum = 0;
     while ((c = getc(db)) != EOF && c != '\n') {
-	if (isspace(c)) {
+	if (isspace((int) c)) {
 	    if (! flag)
 		continue;
 	    if (! gotnum) {
@@ -376,11 +396,11 @@ find(posp, hname)
 	*p++ = c;
     }
     if (c == EOF && ferror(db)) {
-	perror(cmdname);
+	perror(argv0);
 	exit(1);
     }
     if (! gotnum) {
-	fprintf(stderr, "%s: warning: bad format in uuwho database\n", cmdname);
+	fprintf(stderr, "%s: warning: bad format in uuwho database\n", argv0);
 	return 0;
     }
     *p = '\0';
@@ -396,7 +416,7 @@ find(posp, hname)
  */
 static void
 printmap(p)
-    position *p;		/* the fetched record */
+    position_t *p;		/* the fetched record */
 {
     int comment;		/* 1 ==> within a #comment set */
     int pathdata;		/* 1 ==> have seen path data */
@@ -414,7 +434,7 @@ printmap(p)
     if (p->name[0] != '/') {
 	cp = malloc(sizeof(UNSHAR_MAP_DIR) + strlen(p->name) + 1);
 	if (cp == NULL) {
-	    perror(cmdname);
+	    perror(argv0);
 	    exit(1);
 	}
 	sprintf(cp, "%s/%s", UNSHAR_MAP_DIR, p->name);
@@ -425,7 +445,7 @@ printmap(p)
 	cp = p->name;
     }
     if ((fp = fopen(cp, "r")) == NULL) {
-	fprintf(stderr, "%s: Can't open data file %s\n", cmdname, cp);
+	fprintf(stderr, "%s: Can't open data file %s\n", argv0, cp);
 	return;
     }
 
@@ -559,7 +579,7 @@ printmap(p)
 /*
  * get a file and install it in the who database
  */
-void
+static void
 getfile(filename)
     char *filename;			/* the file to add */
 {
@@ -582,10 +602,10 @@ getfile(filename)
 	filename = "[stdin]";
     } else if (input == 1 && strcmp(filename, "-") == 0) {
 	fprintf(stderr,
-		"%s: already processed standard input\n", cmdname);
+		"%s: already processed standard input\n", argv0);
 	return;
     } else if ((fp = fopen(filename, "r")) == (FILE *)NULL) {
-	fprintf(stderr, "%s: Can't open %s\n", cmdname, filename);
+	fprintf(stderr, "%s: Can't open %s\n", argv0, filename);
 	return;
     }
 
@@ -630,14 +650,14 @@ getfile(filename)
 		continue;	/* not a useful line */
 	    }
 	    /* omit default map directory */
-	    p = skip_map_dir(p, q - p);
+	    p = skip_map_dir(p, (unsigned int) (q - p));
 	    /* form new fileinfo module */
-	    filename = malloc(q - p + 1);
+	    filename = malloc((size_t) (q - p + 1));
 	    if (filename == NULL) {
-		perror(cmdname);
+		perror(argv0);
 		exit(1);
 	    }
-	    strncpy(filename, p, q-p);
+	    strncpy(filename, p, (size_t) (q - p));
 	    filename[q-p] = '\0';
 	    pos = 0;	/* assume beginning of the filename */
 	    continue;
@@ -692,7 +712,7 @@ getfile(filename)
 	if (linelen >= BUFSIZ-2) {	/* avoid excessively bogus lines */
 	    fprintf(stderr,
 		    "%s: %s has a line >= %d chars\n",
-		    cmdname, filename, BUFSIZ-2);
+		    argv0, filename, BUFSIZ-2);
 	    exit(1);
 	}
 	pos += (long)linelen;
@@ -705,7 +725,7 @@ getfile(filename)
 static char *
 skip_map_dir(p, len)
     char *p;
-    unsigned len;
+    unsigned int len;
 {
 #ifdef UNSHAR_MAP_DIR
     if (len >= sizeof(UNSHAR_MAP_DIR)
@@ -724,11 +744,11 @@ skip_map_dir(p, len)
 /*
  * show command usage, and die
  */
-void
+static void
 usage()
 {
-    fprintf(stderr, "usage: %s [-d dbmfile] -i mapfile ...\n", cmdname);
-    fprintf(stderr, "usage: %s [-d dbmfile] [sitename ...]\n", cmdname);
+    fprintf(stderr, "usage: %s [-d dbmfile] -i mapfile ...\n", argv0);
+    fprintf(stderr, "usage: %s [-d dbmfile] [sitename ...]\n", argv0);
     exit(-1);
 }
 
@@ -749,7 +769,7 @@ touchdbm(name,wr)
      */
     filename = malloc(strlen(name) + 4 + 1);
     if (filename == NULL) {
-	perror(cmdname);
+	perror(argv0);
 	exit(1);
     }
     
@@ -759,10 +779,10 @@ touchdbm(name,wr)
     sprintf(filename, "%s.pag", name);
     fd = open(filename, wr);
     if (fd < 0 && ! wr) {
-	fprintf(stderr, "%s: can not open %s\n", cmdname, filename);
+	fprintf(stderr, "%s: can not open %s\n", argv0, filename);
 	exit(1);
     } else if (fd < 0 && (fd = creat(filename, 0666)) < 0) {
-	fprintf(stderr, "%s: can not create or write %s\n", cmdname, filename);
+	fprintf(stderr, "%s: can not create or write %s\n", argv0, filename);
 	exit(1);
     }
     close(fd);
@@ -773,10 +793,10 @@ touchdbm(name,wr)
     sprintf(filename, "%s.dir", name);
     fd = open(filename, wr);
     if (fd < 0 && ! wr) {
-	fprintf(stderr, "%s: can not open %s\n", cmdname, filename);
+	fprintf(stderr, "%s: can not open %s\n", argv0, filename);
 	exit(1);
     } else if (fd < 0 && (fd = creat(filename, 0664)) < 0) {
-	fprintf(stderr, "%s: can not create or write %s\n", cmdname, filename);
+	fprintf(stderr, "%s: can not create or write %s\n", argv0, filename);
 	exit(1);
     }
     close(fd);
@@ -799,7 +819,7 @@ get_hostname()
      */
     hostname = malloc(NLEN+1+1);
     if (hostname == NULL) {
-	fprintf(stderr, "%s: bad mallof of hostname\n", cmdname);
+	fprintf(stderr, "%s: bad mallof of hostname\n", argv0);
 	exit(3);
     }
     if (gethostname(hostname, NLEN+1) < 0) {
@@ -812,11 +832,12 @@ get_hostname()
 static char *
 get_hostname()
 {
-    static struct utsname utsname;
+    static struct utsname utsnm;
 
-    (void) uname(&utsname);
+    (void) uname(&utsnm);
     /* is the sysname tag used for something interesting? */
-    return(utsname.nodename);
+
+    return (utsnm.nodename);
 }
 #else	/* not HAVE_UNAME */
 static char *
