@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:addr.c,v 1.38 2001/07/28 18:05:02 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:addr.c,v 1.50 2003/04/17 22:16:26 woods Exp"
  */
 
 /*
@@ -19,19 +19,42 @@
  *			     address_token, back_address_token, alloc_addr,
  *			     insert_addr_list, addr_sort, split_addr_list
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
 
-#define NEED_SOCKETS			/* Required for IP address validation */
-
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
+#include "smailsock.h"
 #include "main.h"
 #include "config.h"
 #include "parse.h"
@@ -39,21 +62,18 @@
 #include "field.h"
 #include "route.h"
 #include "direct.h"
-#include "transport.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
+#include "transport.h"
 #include "smailconf.h"
 #include "bindlib.h"
+#include "transports/smtplib.h"			/* rfc821_is_*() decls. */
 #include "debug.h"
 #include "extern.h"
-
-#ifndef INADDR_NONE
-# define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
-#endif
-#ifndef INADDR_LOOPBACK
-# define INADDR_LOOPBACK	((unsigned long) 0x7f000001) /* XXX not 64-bit clean */
-#endif
+#include "smailport.h"
 
 /* functions local to this file */
 static int check_target_and_remainder __P((char **, char **));
@@ -89,8 +109,9 @@ preparse_address(address, error)
     char *address;			/* address to be preparsed */
     char **error;			/* return error message here */
 {
-    char * ignore;
-    return preparse_address_1(address, error, & ignore);
+    char *ignore;
+
+    return preparse_address_1(address, error, &ignore);
 }
 
 char *
@@ -158,7 +179,7 @@ preparse_address_1(address, error, rest)
      */
     if (*ap == '@') {
 	/* valid route-addr, not a mutant one */
-	ap = xmalloc((unsigned)(strlen(address) + 1));
+	ap = xmalloc((size_t) (strlen(address) + 1));
 	strcpy(ap, address);
 	if (mark_end) {
 	    *mark_end++ = '>';		/* widen the original address */
@@ -178,7 +199,7 @@ preparse_address_1(address, error, rest)
 	    return NULL;
 	}
 	if (*ap != '!') {
-	    ap = xmalloc((unsigned)(strlen(address) + 1));
+	    ap = xmalloc((size_t) (strlen(address) + 1));
 	    strcpy(ap, address);
 	    if (mark_end) {
 		*mark_end++ = '>';	/* widden the original address */
@@ -190,7 +211,7 @@ preparse_address_1(address, error, rest)
 	ap++;
 	if (*ap == '@') {
 	    /* matched host!(host!)*@route -- build the !-route */
-	    register char *p = xmalloc((unsigned)strlen(address));
+	    register char *p = xmalloc((size_t) strlen(address));
 	    DEBUG(DBG_ADDR_MID, "found host!(host!)*@route form--ugh!\n");
 	    /* first part already !-route */
 	    strncpy(p, address, (size_t) (ap - address));
@@ -199,7 +220,9 @@ preparse_address_1(address, error, rest)
 	    }
 	    ap = build_uucp_route(ap, error, 0); /* build !-route */
 	    if (ap == NULL) {
-		DEBUG(DBG_ADDR_MID, "preparse_address returns: (null)\n");
+		DEBUG1(DBG_ADDR_LO,
+		       "preparse_address(): build_uucp_route() failed: %s: returns: (null)\n",
+		       *error);
 		return NULL;
 	    }
 	    strcat(p, ap);		/* concatenate together */
@@ -209,7 +232,7 @@ preparse_address_1(address, error, rest)
 	    return p;			/* transformed */
 	}
     }
-    ap = xmalloc((unsigned)(strlen(address) + 1));
+    ap = xmalloc((size_t) (strlen(address) + 1));
     strcpy(ap, address);
     if (mark_end) {
 	*mark_end++ = '>';	/* widden the original address */
@@ -522,78 +545,165 @@ parse_address(address, target, remainder, flagp)
     return LOCAL;
 }
 
-/* check_target_and_remainder - check for glaring problems */
+/*
+ * check_target_and_remainder - check for glaring problems
+ *
+ * Returns SUCCEED if all is well, FAIL otherwise.
+ *
+ * A pointer to the error message related to any problems found is returned in
+ * the (*remainderp) pointer.
+ */
 static int
-check_target_and_remainder(target, remainder)
-    char **target;
-    char **remainder;
+check_target_and_remainder(targetp, remainderp)
+    char **targetp;				/* ptr to hostname str */
+    char **remainderp;				/* ptr to mailbox str */
 {
     int c;
     char *p;
 
+    DEBUG2(DBG_ADDR_HI, "check_target_and_remainder(): hostname='%s', mailbox='%s'\n", *targetp, *remainderp);
     /*
      * first check the remainder...
+     *
+     * We should be checking for full RFC [2]822 conformance (i.e. the
+     * following syntax) at a minimum, but we would need
+     * rfc822_is_quoted_string() to do that.  Also note that much of the
+     * parsing and removal of extraneous parts such as CFWS and FWS has already
+     * been done by our callers.
+     *
+     *	local-part      =       dot-atom / quoted-string / obs-local-part
+     *
+     *	dot-atom        =       [CFWS] dot-atom-text [CFWS]
+     *
+     *	dot-atom-text   =       1*atext *("." 1*atext)
+     *
+     *	atext           =       ALPHA / DIGIT / ; Any character except controls,
+     *	                        "!" / "#" /     ;  SP, and specials.
+     *	                        "$" / "%" /     ;  Used for atoms
+     *	                        "&" / "'" /
+     *	                        "*" / "+" /
+     *	                        "-" / "/" /
+     *	                        "=" / "?" /
+     *	                        "^" / "_" /
+     *	                        "`" / "{" /
+     *	                        "|" / "}" /
+     *	                        "~"
+     *
+     *	quoted-string   =       [CFWS]
+     *	                        DQUOTE *([FWS] qcontent) [FWS] DQUOTE
+     *	                        [CFWS]
+     *
+     *	qcontent        =       qtext / quoted-pair
+     *
+     *	qtext           =       NO-WS-CTL /     ; Non white space controls
+     *	                        %d33 /          ; The rest of the US-ASCII
+     *	                        %d35-91 /       ;  characters not including "\"
+     *	                        %d93-126        ;  or the quote character
+     *
+     *	quoted-pair     =       ("\" text) / obs-qp
+     *
+     *	text            =       %d1-9 /         ; Characters excluding CR and LF
+     *	                        %d11 /
+     *	                        %d12 /
+     *	                        %d14-127 /
+     *	                        obs-text
+     *
+     *	NO-WS-CTL       =       %d1-8 /         ; US-ASCII control characters
+     *	                        %d11 /          ;  that do not include the
+     *	                        %d12 /          ;  carriage return, line feed,
+     *	                        %d14-31 /       ;  and white space characters
+     *	                        %d127
+     *
+     *	ctext           =       NO-WS-CTL /     ; Non white space controls
+     *	                        %d33-39 /       ; The rest of the US-ASCII
+     *	                        %d42-91 /       ;  characters not including "(",
+     *	                        %d93-126        ;  ")", or "\"
+     *
+     *	ccontent        =       ctext / quoted-pair / comment
+     *
+     *	comment         =       "(" *([FWS] ccontent) [FWS] ")"
+     *
+     *	CFWS            =       *([FWS] comment) (([FWS] comment) / FWS)
+     *
+     *	FWS             =       ([*WSP CRLF] 1*WSP) /   ; Folding white space
+     *	                        obs-FWS
+     *
+     * Instead we will simply check for strict RFC 821 compatability as will be
+     * needed for any addresses used in SMTP.  Other transports may not be
+     * quite so restrictive as this (or may even be more restrictive!).
      */
-    p = *remainder;
+    p = *remainderp;
     if (*p == '\0') {
-	*remainder = "no remainder address";
-	DEBUG1(DBG_ADDR_MID, "parse_address: %s\n", *remainder);
+	*remainderp = "no remainder address";
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s, ''): %s\n", *targetp, *remainderp);
 	return FAIL;
     }
+    if (!rfc821_is_dot_string(*remainderp) && !rfc821_is_quoted_string(*remainderp)) {
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s, %s): mailbox is not a valid RFC-821 local-part.", *targetp, *remainderp);
+	*remainderp = "mailbox is not a valid RFC-821 local-part";
+	return FAIL;
+    }	    
     /*
      * now check the target...
      */
-    p = *target;
-    if (*p == '[') {
+    p = *targetp;
 #ifdef HAVE_BSD_NETWORKING
-# if defined(INET_ADDR_USE_STRUCT)
-	struct in_addr s_inet;		/* internet address */
-# endif
+    if (*p == '[') {
 	unsigned long inet;		/* IP address */
 	char *p2;			/* pointer to closing bracket (]) */
 
 	p2 = strchr(p, ']');
 	*p2 = '\0';
-# ifdef INET_ADDR_USE_STRUCT
-	s_inet = inet_addr(&(p[1]));
-	inet = s_inet.s_addr;
-# else
-	inet = inet_addr(&(p[1]));
-# endif
+	inet = get_inet_addr(&(p[1]));
 	*p2 = ']';
-	DEBUG3(DBG_ADDR_HI, "parse_address(%s): inet given: [0x%lx] aka %s\n", p,
-	       ntohl(inet), COPY_STRING(inet_ntoa(*((struct in_addr *) &inet))));
+	DEBUG3(DBG_ADDR_HI, "parse_address(%s): inet addr given: [0x%lx] aka [%s]\n", p,
+	       ntohl(inet), inet_ntoa(*((struct in_addr *) &inet)));
 	if (inet == INADDR_NONE) {
-	    *remainder = "Invalid host address form";
-	    DEBUG2(DBG_ADDR_LO, "parse_address(%s): inet_addr() failed: %s.\n", p, *remainder);
+	    DEBUG2(DBG_ADDR_LO, "parse_address(%s, %s): get_inet_addr() failed: Invalid host address literal form\n", *targetp, *remainderp);
+	    *remainderp = "Invalid host address literal form";
 	    return FAIL;
 	}
-#else /* !HAVE_BSD_NETWORKING */
-	/* XXX shouldn't be lazy -- should verify the hard way! */
-#endif
 	return SUCCEED;
     }
+#endif
     if (*p == '-') {
-	*remainder = "target cannot begin with `-'";
-	DEBUG2(DBG_ADDR_MID, "parse_address(%s): %s\n", p, *remainder);
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s, %s): hostname cannot begin with `-'\n", *targetp, *remainderp);
+	*remainderp = "hostname cannot begin with `-'";
 	return FAIL;
     }
+    if (*p == '.') {
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s, %s): hostname cannot begin with `.'\n", *targetp, *remainderp);
+	*remainderp = "hostname cannot begin with `.'";
+	return FAIL;
+    }
+#ifdef HAVE_BSD_NETWORKING		/* XXX having to ifdef this is bogus... */
+    if (get_inet_addr(p) != INADDR_NONE) {
+	DEBUG2(DBG_ADDR_MID, "parse_address(%s, %s): hostname cannot be a bare IP address\n", *targetp, *remainderp);
+	*remainderp = "hostname cannot be a bare IP address";
+	return FAIL;
+    }
+#endif
     /*
      * this test is more in line with RFC-821's restrictions, which are based
      * on RFC-1035's hostname restrictions.  Other transports may not be quite
      * so restrictive as this (or may even be more restrictive!).
      *
-     * WARNING: we should probably use something like CERT's IsValidHostname()
-     * to validate total length, length of each label, start char of each
-     * label, etc.
+     * WARNING: we should probably also ensure the total length, the length of
+     * each label, the start char of each label, etc., are all valid.
+     *
+     * There's somewhat more careful code in smtprecv.c:verify_host(), but at
+     * this point we don't really care if the name is fully qualified or not,
+     * nor if it's a bare IP address (i.e. missing the square brackets), as
+     * eventually we'll find out if it's routable or not one way or another.
      */
     while ((c = *p++)) {
-	if (!(isalnum(c) || c == '.' || c == '-')) {
-	    *remainder = "illegal character in hostname";
-	    DEBUG2(DBG_ADDR_MID, "parse_address(%s): %s\n", *target, *remainder);
+	if (!isascii(c) || !(isalnum(c) || c == '.' || c == '-')) {
+	    DEBUG2(DBG_ADDR_MID, "parse_address(%s, %s): illegal character in hostname\n", *targetp, *remainderp);
+	    *remainderp = "illegal character in hostname";
 	    return FAIL;
 	}
     }
+
     return SUCCEED;
 }
 
@@ -712,7 +822,7 @@ internal_build_uucp_route(address, error, partial, flag)
      * allocate a new copy of the address so it can be examined destructively.
      * XXX this seems to be bogus....
      */
-    storage = remainder = xmalloc((unsigned)(strlen(address) + 1));
+    storage = remainder = xmalloc((size_t) (strlen(address) + 1));
     strcpy(storage, address);
 
     /* initialize for copy into string region */
@@ -767,8 +877,8 @@ internal_build_uucp_route(address, error, partial, flag)
 		xfree(storage);		/* free garbage */
 		STR_DONE(sp);
 		DEBUG1(DBG_ADDR_HI, "internal_build_uucp_route returns: %s\n",
-		       sp->p);
-		return sp->p;		/* return completed !-route */
+		       STR(sp));
+		return STR(sp);		/* return completed !-route */
 	    }
 	    /*NOTREACHED*/
 
@@ -873,10 +983,11 @@ split_addr_list(arg, app)
 	new->gid = nobody_gid;
 	new->succ = *app;
 	*app = new;
-	if (q)
+	if (q) {
 	    p = q;
-	else
+	} else {
 	    break;
+	}
     }
 
     return;
@@ -962,7 +1073,7 @@ strip_rfc822_whitespace(s)
 
     p = q = s;
     while ((c = *p++)) {
-	if (isspace(c)) {
+	if (isspace((int) c)) {
 	    space = 1;
 	    continue;
 	}
@@ -1383,7 +1494,7 @@ escaped(ba, ap)
     register char *ba;			/* beginning of string */
     register char *ap;			/* character position in string */
 {
-    register unsigned i = 0;		/* count of \ characters */
+    register size_t i = 0;		/* count of \ characters */
 
     /*
      * count the number of preceding \ characters, but don't go past
@@ -1415,7 +1526,7 @@ alloc_addr()
     addr = (struct addr *)xmalloc(sizeof(*addr));
 
     /* preset the proper values */
-    bzero((char *)addr, sizeof(*addr));	/* XXX hope NULL pointers are all zeros! */
+    (void) memset((char *) addr, '\0', sizeof(*addr)); /* XXX hope NULL pointers are all zeros! */
     addr->match_count = -1;
     addr->uid = (unsigned int) BOGUS_USER;	/* the identity is not known yet */
     addr->gid = (unsigned int) BOGUS_GROUP;	/* the identity is not known yet */
@@ -1476,10 +1587,10 @@ static int sort_offset;
 struct addr *
 addr_sort(in, offset)
     struct addr *in;
-    int offset;
+    int offset;				/* XXX should be unsigned? */
 {
     struct addr **addrv;		/* array of addresses */
-    register unsigned int addrc;	/* count of addresses */
+    register size_t addrc;		/* count of addresses */
     register struct addr **addrp;	/* temp addr pointer */
     register struct addr *a;		/* address list or current address */
 
@@ -1493,7 +1604,7 @@ addr_sort(in, offset)
     }
 
     /* allocate space for an array for that many pointers */
-    addrv = (struct addr **) xmalloc((unsigned) (addrc * sizeof(*addrv)));
+    addrv = (struct addr **) xmalloc(addrc * sizeof(*addrv));
 
     /* build the array from the input list */
     for (addrp = addrv, a = in; a; a = a->succ) {
@@ -1524,10 +1635,10 @@ addrcmp(x, y)
     const void *x;
     const void *y;
 {
-    char **a = (char **) x;
-    char **b = (char **) y;
+    const char *a = *((const char * const *) x);
+    const char *b = *((const char * const *) y);
 
-    return strcmp(*(char **)(*a + sort_offset), *(char **)(*b + sort_offset));
+    return strcmp((a + sort_offset), (b + sort_offset));
 }
 
 /*
@@ -1540,7 +1651,7 @@ note_error(info, message)
 {
     struct error *ret = (struct error *)xmalloc(sizeof(*ret));
 
-    DEBUG2(DBG_ADDR_MID, "note_error(%ld, %s)\n", info, message);
+    DEBUG2(DBG_ADDR_MID, "note_error(ERR_%ld, %s)\n", (info & ERR_MASK), message);
 
     ret->info = info;
     ret->message = message;
@@ -1599,7 +1710,7 @@ main(argc, argv)
     /*
      * if first argument is a number, change the debug level
      */
-    if (argc > 1 && isdigit(argv[1][0])) {
+    if (argc > 1 && isdigit((int) argv[1][0])) {
 	debug = atoi(*++argv);
 	argc--;
     }
