@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:addr.h,v 1.13 2000/09/03 15:43:54 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:addr.h,v 1.14 2003/02/26 19:36:28 woods Exp"
  */
 
 /*
@@ -26,9 +26,11 @@ struct addr {
     int parseflags;		/* parse flags for parse_address() */
     struct addr *parent;	/* addr from which this one is derived */
     struct addr *true_addr;	/* point to addr which had an error */
-    char *in_addr;		/* address from header or args */
-    char *target;		/* next explicit destination */
-    char *remainder;		/* address beyond explicit site */
+    char *in_addr;		/* original address from header or args */
+    char *target;		/* generally the target domain name */
+    char *remainder;		/* generally the local mailbox name */
+    char *rem_prefix;		/* prefix stripped from remainder */
+    char *rem_suffix;		/* suffix stripped from remainder */
     char *work_addr;		/* working area */
     int match_count;		/* chars of target matched by router */
     char *local_name;           /* domain matched as local */
