@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:alloc.c,v 1.20 2000/09/02 23:32:14 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:alloc.c,v 1.24 2003/04/02 22:07:46 woods Exp"
  */
 
 /*
@@ -31,22 +31,42 @@
  *			    balloc, brealloc, bfree,
  *			    malloc_block, realloc_block, free_block
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
 #endif
-#include "smail.h"
-#include "exitcodes.h"
-#include "log.h"
-#include "alloc.h"
 
 #ifdef USE_VMALLOC
 # include <vmalloc.h>
 #endif
 
+#include "smail.h"
+#include "exitcodes.h"
+#include "log.h"
+#include "alloc.h"
+#include "smailport.h"
+
 /* the master block of blocks */
 static struct block master = {(struct bseg *)0, 0};
 
@@ -86,7 +106,7 @@ extern void free();				/* int on SunOS */
  */
 char *
 xmalloc(size)
-    unsigned size;			/* size of region to allocate */
+    size_t size;			/* size of region to allocate */
 {
     register char *ret;			/* region returned by malloc */
 
@@ -102,7 +122,7 @@ xmalloc(size)
 char *
 xrealloc(region, size)
     char *region;			/* region to be realloc'd */
-    unsigned size;			/* realloc size */
+    size_t size;			/* realloc size */
 {
     register char *ret;			/* region returned by realloc */
 
@@ -154,7 +174,7 @@ xfree(region)
  */
 char *
 bmalloc(size, block)
-    unsigned size;		/* number of bytes to allocate */
+    size_t size;		/* number of bytes to allocate */
     struct block *block;	/* block that data belongs to */
 {
     struct bseg *newseg;	/* the new segment for this alloc */
@@ -200,7 +220,7 @@ bmalloc(size, block)
 char *
 brealloc(data, size, block)
     char *data;			/* data being re-allocated */
-    unsigned size;		/* number of bytes to re-allocate */
+    size_t size;		/* number of bytes to re-allocate */
     struct block *block;	/* block that data belongs to */
 {
     struct bseg *oldseg;	/* old segment associated with data */
@@ -309,7 +329,7 @@ malloc_block()
     struct block *block;	/* ptr to the new block */
 
     /* allocate the block */
-    block = (struct block *) bmalloc((unsigned) sizeof(struct block), &master);
+    block = (struct block *) bmalloc(sizeof(struct block), &master);
     block->cnt = 0;
     block->next = NULL;
 
