@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:alloc.h,v 1.11 2000/09/03 16:29:01 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:alloc.h,v 1.13 2002/10/01 21:44:45 woods Exp"
  */
 
 /*
@@ -71,12 +71,16 @@ struct bseg {
 #define X_NO_PANIC() (x_dont_panic = TRUE)
 #define X_PANIC_OKAY() (x_dont_panic = FALSE)
 
+/* external variables defined in alloc.c */
+extern struct block *perm;		/* permanent allocation block */
+extern int x_dont_panic;		/* dont panic in X_CHECK() */
+
 /* external functions defined in alloc.c */
-extern char *xmalloc __P((unsigned int));
-extern char *xrealloc __P((char *, unsigned int));
+extern char *xmalloc __P((size_t));
+extern char *xrealloc __P((char *, size_t));
 extern void xfree __P((char *));
-extern char *bmalloc __P((unsigned int, struct block *));
-extern char *brealloc __P((char *, unsigned int, struct block *));
+extern char *bmalloc __P((size_t, struct block *));
+extern char *brealloc __P((char *, size_t, struct block *));
 extern void bfree __P((char *, struct block *));
 extern struct block *malloc_block __P((void));
 extern void realloc_block __P((char *, struct block *, struct block *));
