@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)src/bindlib.h:RELEASE-3_2_0_114:bindlib.h,v 1.10 2000/09/03 15:53:01 woods Exp"
+#ident	"@(#)src/bindlib.h:RELEASE-3_2_0_115:bindlib.h,v 1.11 2003/02/19 22:25:15 woods Exp"
 */
 
 /*
@@ -63,52 +63,52 @@ struct bindlib_private {
 
 /* configuration description */
 #define BIND_ATTRIBUTES(STRUCT,BL_PRIV_MEM) \
-    { "local_mx_okay", t_boolean, NULL, NULL, BIND_LOCAL_MX_OKAY },     \
-    { "defer_no_connect", t_boolean, NULL, NULL, BIND_DEFER_NO_CONN },  \
-    { "defnames", t_boolean, NULL, NULL, BIND_DEFNAMES },               \
-    { "dns_search", t_boolean, NULL, NULL, BIND_DNS_SEARCH },           \
-    { "domain_required", t_boolean, NULL, NULL, BIND_DOMAIN_REQUIRED }, \
-    { "mx_only", t_boolean, NULL, NULL, BIND_MX_ONLY },                 \
-    { "uk_try_inverting", t_boolean, NULL, NULL, BIND_UK_TRY_INVERT },  \
-    { "uk_greybook_worldorder", t_boolean, NULL, NULL, BIND_UK_GREY_WORLD }, \
-    { "match_domains", t_string, NULL, NULL,				\
-	  OFFSET(STRUCT, BL_PRIV_MEM.match_domains) },			\
-    { "ignore_domains", t_string, NULL, NULL,                           \
-          OFFSET(STRUCT, BL_PRIV_MEM.ignore_domains) },                 \
-    { "required", t_string, NULL, NULL,                                 \
-          OFFSET(STRUCT, BL_PRIV_MEM.required) },                       \
-    { "widen_domains", t_string, NULL, NULL,                            \
-          OFFSET(STRUCT, BL_PRIV_MEM.widen_domains) },                  \
-    { "mx_domains", t_string, NULL, NULL,                               \
-          OFFSET(STRUCT, BL_PRIV_MEM.mx_domains) },                     \
-    { "gateways", t_string, NULL, NULL,                                 \
-          OFFSET(STRUCT, BL_PRIV_MEM.gateways) },                       \
-    { "uk_ignore_gateways", t_string, NULL, NULL,                       \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_ignore_gateways) },             \
-    { "uk_greybook_transport", t_string, NULL, NULL,                    \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_transport) },          \
-    { "uk_max_precedence", t_int, NULL, NULL,                           \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_max_precedence) },              \
-    { "uk_greybook_relay", t_string, NULL, NULL,                        \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_relay) },              \
-    { "uk_greybook_hosts_list", t_string, NULL, NULL,                   \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_list) },         \
-    { "uk_greybook_hosts_file", t_string, NULL, NULL,                   \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_file) },         \
-    { "uk_greybook_hosts_proto", t_string, NULL, NULL,                  \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_proto) },        \
-    { "uk_greybook_host_retries", t_int, NULL, NULL,			\
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_retries) },       \
-    { "uk_greybook_host_interval", t_int, NULL, NULL,			\
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_interval) },      \
-    { "uk_suffix", t_string, NULL, NULL,                                \
-          OFFSET(STRUCT, BL_PRIV_MEM.uk_suffix) },                      \
-    { "rewrite_headers", t_int, NULL, NULL,                             \
-          OFFSET(STRUCT, BL_PRIV_MEM.rewrite_headers) },                \
-    { "cname_limit", t_int, NULL, NULL,                                 \
-          OFFSET(STRUCT, BL_PRIV_MEM.cname_limit) },                    \
-    { "fallback_gateway", t_string, NULL, NULL,                         \
-          OFFSET(STRUCT, BL_PRIV_MEM.fallback_gateway) }                \
+    { "local_mx_okay", t_boolean, 0, NULL, NULL, BIND_LOCAL_MX_OKAY },	\
+    { "defer_no_connect", t_boolean, 0, NULL, NULL, BIND_DEFER_NO_CONN }, \
+    { "defnames", t_boolean, 0, NULL, NULL, BIND_DEFNAMES },		\
+    { "dns_search", t_boolean, 0, NULL, NULL, BIND_DNS_SEARCH },	\
+    { "domain_required", t_boolean, 0, NULL, NULL, BIND_DOMAIN_REQUIRED }, \
+    { "mx_only", t_boolean, 0, NULL, NULL, BIND_MX_ONLY },		\
+    { "uk_try_inverting", t_boolean, 0, NULL, NULL, BIND_UK_TRY_INVERT }, \
+    { "uk_greybook_worldorder", t_boolean, 0, NULL, NULL, BIND_UK_GREY_WORLD },	\
+    { "match_domains", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.match_domains) },			\
+    { "ignore_domains", t_string, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.ignore_domains) },			\
+    { "required", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.required) },				\
+    { "widen_domains", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.widen_domains) },			\
+    { "mx_domains", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.mx_domains) },			\
+    { "gateways", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.gateways) },				\
+    { "uk_ignore_gateways", t_string, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_ignore_gateways) },		\
+    { "uk_greybook_transport", t_string, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_transport) },		\
+    { "uk_max_precedence", t_int, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_max_precedence) },		\
+    { "uk_greybook_relay", t_string, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_relay) },		\
+    { "uk_greybook_hosts_list", t_string, 0, NULL, NULL,		\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_list) },		\
+    { "uk_greybook_hosts_file", t_string, 0, NULL, NULL,		\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_file) },		\
+    { "uk_greybook_hosts_proto", t_string, 0, NULL, NULL,		\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_hosts_proto) },		\
+    { "uk_greybook_host_retries", t_int, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_retries) },		\
+    { "uk_greybook_host_interval", t_int, 0, NULL, NULL,		\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_greybook_host_interval) },	\
+    { "uk_suffix", t_string, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.uk_suffix) },			\
+    { "rewrite_headers", t_int, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.rewrite_headers) },			\
+    { "cname_limit", t_int, 0, NULL, NULL,				\
+	OFFSET(STRUCT, BL_PRIV_MEM.cname_limit) },			\
+    { "fallback_gateway", t_string, 0, NULL, NULL,			\
+	OFFSET(STRUCT, BL_PRIV_MEM.fallback_gateway) }
 
 #define BIND_TEMPLATE_FLAGS 0
 
