@@ -1,4 +1,6 @@
-/* @(#) child.h,v 1.4 1998/04/12 01:59:48 woods Exp */
+/*
+#ident "@(#)smail/src:RELEASE-3_2_0_115:child.h,v 1.5 2001/11/02 17:39:34 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1987, 1988 Ronald S. Karr and Landon Curt Noll
@@ -10,7 +12,7 @@
 
 /*
  * child.h:
- *	interface file for the child process control functions in child.c.
+ *	interface file for the child_*() functions in sysdep.c.
  */
 
 /* bit values for the flag parameter to open_child */
