@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:config.c,v 1.44 2001/06/25 22:09:55 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:config.c,v 1.58 2003/03/27 16:34:10 woods Exp"
  */
 
 /*
@@ -18,19 +18,30 @@
  *
  *	no external functions.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
-#include "defs.h"
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "config.h"
 #include "extern.h"
 
+int allow_one_mx_target_cname_hack = ALLOW_ONE_MX_TARGET_CNAME_HACK; /* relax DNS MX checks a bit */
 char *listen_name = NULL; 		/* DNS name of interface to listen on */
- char *sending_name = NULL; 		/* DNS name of interface to send mail from */
+char *sending_name = NULL; 		/* DNS name of interface to send mail from */
 char *visible_name = VISIBLE_NAME;	/* hostname used in outgoing addrs */
 char *visible_domains = VISIBLE_DOMAINS; /* domain used in outgoing headers */
 char *uucp_name = UUCP_NAME;		/* hostname used in !-routes */
@@ -47,8 +58,8 @@ char *cons_fn = CONSOLE;		/* name of console device file */
 char *spool_dirs = SPOOL_DIRS;		/* names of spooling directories */
 int spool_mode = SPOOL_MODE;		/* mode for spool files */
 int lock_mode = LOCK_MODE;		/* mode for lock files */
-int log_mode = LOG_MODE;		/* mode for system log files */
-int message_log_mode = MESSAGE_LOG_MODE; /* mode for per-message log files */
+unsigned int log_mode = LOG_MODE;	/* mode for system log files */
+unsigned int message_log_mode = MESSAGE_LOG_MODE; /* mode for per-message log files */
 int spool_grade = SPOOL_GRADE;		/* default spool grade character */
 int open_retries = OPEN_RETRIES;	/* max open retries on config files */
 int open_interval = OPEN_INTERVAL;	/* sleep between open retries */
@@ -71,11 +82,12 @@ char *date_field = DATE_FIELD;		/* Date: field string */
 char *from_field = FROM_FIELD;		/* From: field string */
 char *return_path_field = RETURN_PATH_FIELD; /* Return-Path: field string */
 char *smail = SMAIL;			/* location of the smail program */
+char *daemon_pidfile = DAEMON_PIDFILE;	/* location of the PID file */
 double max_load_ave = MAX_LOAD_AVE;	/* spool mail > this load agerage */
 char *trusted = TRUSTED;		/* : list of trusted users */
 char *trusted_groups = TRUSTED_GROUPS;	/* : list of trusted groups */
-unsigned message_bufsiz = MESSAGE_BUF_SIZE; /* size of message buffers */
-int hit_table_len = HIT_TABLE_LEN;	/* #entries in address hit table */
+size_t message_bufsiz = MESSAGE_BUF_SIZE; /* size of message buffers */
+unsigned int hit_table_len = HIT_TABLE_LEN; /* #entries in address hit table */
 int flock_mailbox = FLOCK_MAILBOX;	/* TRUE to use lock_fd_wait() macro */
 int fnlock_retries = FNLOCK_RETRIES;	/* retries for lock_file() creat */
 unsigned int fnlock_interval = FNLOCK_INTERVAL;	/* retry intervals for lock_file() */
@@ -97,28 +109,43 @@ int require_configs = REQUIRE_CONFIGS;	/* TRUE to require config files */
 char *postmaster_address = POSTMASTER_ADDRESS; /* default addr of postmaster */
 int smtp_accept_max = SMTP_ACCEPT_MAX;	/* max simultaneous SMTPs to accept */
 int smtp_accept_queue = SMTP_ACCEPT_QUEUE; /* simultaneous SMTPs to queueonly */
-char *smtp_banner = SMTP_BANNER;	/* smtp startup banner message */
 int smtp_allow_debug = SMTP_ALLOW_DEBUG; /* allow DEBUG command in SMTP */
 int smtp_allow_expn = SMTP_ALLOW_EXPN;	/* allow EXPN commanda in SMTP */
-char *smtp_recipient_no_verify = SMTP_RECIPIENT_NO_VERIFY; /* ip list for not verifying RCPT TO: */
-char *smtp_remote_allow = SMTP_REMOTE_ALLOW; /* ip list that can send remote smtp */
+char *smtp_bad_mx_targets = SMTP_BAD_MX_TARGETS; /* ip list of undesirable MX targets */
+char *smtp_banner = SMTP_BANNER;	/* smtp startup banner message */
+unsigned int smtp_debug_pause = 0;	/* seconds to pause after child fork for debugger connect */
 int smtp_hello_verify = SMTP_HELLO_VERIFY; /* verify HELO/EHLO host has A RR matching sockaddr */
 char *smtp_hello_broken_allow = SMTP_HELLO_BROKEN_ALLOW; /* ip list that can avoid hello verification */
+char *smtp_hello_dnsbl_domains = SMTP_HELLO_DNSBL_DOMAINS; /* list of DNSBL domains to check sender_host */
+char *smtp_hello_dnsbl_except = SMTP_HELLO_DNSBL_EXCEPT; /* list of exceptions for smtp_hello_dnsbl_domains */
 int smtp_hello_reject_dns_paranoid = SMTP_HELLO_REJECT_DNS_PARANOID; /* like TCP Wrappers PARANOID check */
-char *smtp_hello_reject_hosts = SMTP_HELLO_REJECT_HOSTS; /* like TCP Wrappers */
+char *smtp_hello_reject_hostnames = SMTP_HELLO_REJECT_HOSTNAMES; /* list of hostnames to match sender_host */
 int smtp_hello_verify_literal = SMTP_HELLO_VERIFY_LITERAL; /* verify HELO/EHLO domain literal has PTR RR */
 int smtp_hello_verify_ptr = SMTP_HELLO_VERIFY_PTR; /* verify HELO/EHLO matches PTR RR for sockaddr */
+char *smtp_host_dnsbl_domains = SMTP_HOST_DNSBL_DOMAINS; /* list of DNSBL domains to check sender_host_really */
+char *smtp_host_dnsbl_except = SMTP_HOST_DNSBL_EXCEPT; /* list of exceptions for smtp_host_dnsbl_domains */
+char *smtp_host_reject_hostnames = SMTP_HOST_REJECT_HOSTNAMES; /* list of hostname REs to match sender_host_really */
+char *smtp_local_sender_allow = SMTP_LOCAL_SENDER_ALLOW;/* locally deliverable mailboxes which might come back to us */
+int smtp_local_sender_restrict = SMTP_LOCAL_SENDER_RESTRICT; /* verify locally deliverable sender clients match smtp_remote_allow */
+unsigned int smtp_max_recipients = SMTP_MAX_RECIPIENTS; /* maximum recipients per SMTP connect */
+unsigned int smtp_max_bounce_recipients = SMTP_MAX_BOUNCE_RECIPIENTS; /* maximum recipients per bounce message */
+char *smtp_permit_mx_backup = SMTP_PERMIT_MX_BACKUP; /* list of domains we will allow for secondary MX */
 char *smtp_rbl_domains = SMTP_RBL_DOMAINS; /* list of RBL domains */
 char *smtp_rbl_except = SMTP_RBL_EXCEPT; /* list of exceptions to RBL lookups */
+char *smtp_recipient_no_verify = SMTP_RECIPIENT_NO_VERIFY; /* ip list for not verifying RCPT TO: */
+char *smtp_reject_hosts = SMTP_REJECT_HOSTS; /* list of IP/Nets to match client address */
+char *smtp_remote_allow = SMTP_REMOTE_ALLOW; /* ip list that can send remote smtp */
 char *smtp_sender_no_verify = SMTP_SENDER_NO_VERIFY; /* ip list for not verifying MAIL FROM: */
+char *smtp_sender_reject = SMTP_SENDER_REJECT; /* list of REs to match whole sender addresses */
+char *smtp_sender_reject_db = SMTP_SENDER_REJECT_DB; /* database to match whole sender addresses */
+char *smtp_sender_reject_db_proto = SMTP_SENDER_REJECT_DB_PROTO; /* smtp_sender_reject_db access protocol */
+char *smtp_sender_reject_hostnames = SMTP_SENDER_REJECT_HOSTNAMES; /* list of hostname REs to match sender addresses */
 char *smtp_sender_rhsbl_domains = SMTP_SENDER_RHSBL_DOMAINS; /* list of RHSBL domains for sender addresses */
 char *smtp_sender_rhsbl_except = SMTP_SENDER_RHSBL_EXCEPT; /* list of exceptions to sender RHSBL lookups */
 int smtp_sender_verify_mx_only = SMTP_SENDER_VERIFY_MX_ONLY; /* require MX RRs for MAIL FROM: hosts */
-int smtp_max_recipients = SMTP_MAX_RECIPIENTS; /* maximum recipients per SMTP connect */
 unsigned int smtp_expn_delay = SMTP_EXPN_DELAY;	/* seconds to delay after an EXPN command */
 unsigned int smtp_vrfy_delay = SMTP_VRFY_DELAY;	/* seconds to delay after a VRFY command */
 unsigned int smtp_error_delay = SMTP_ERROR_DELAY; /* seconds to delay after any SMTP error */
-unsigned int debug_pause_for_smtp_connections = 0; /* seconds to pause after child fork for debugger connect */
 char *sender_env_variable = SENDER_ENV_VARIABLE; /* env variable naming user */
 int switch_percent_and_bang = SWITCH_PERCENT_AND_BANG;
 					/* switch precedence of % and ! */
@@ -127,10 +154,8 @@ int error_copy_postmaster = ERROR_COPY_POSTMASTER;
 long retry_interval = RETRY_INTERVAL;	/* default delivery retry interval */
 long retry_duration = RETRY_DURATION;	/* default delivery retry duration */
 long host_lock_timeout = HOST_LOCK_TIMEOUT; /* timeout for host lock */
-unsigned int smtp_receive_command_timeout = SMTP_RECEIVE_COMMAND_TIMEOUT;
-					/* timeout for smtp command reads */
-unsigned int smtp_receive_message_timeout = SMTP_RECEIVE_MESSAGE_TIMEOUT;
-					/* timeout for smtp message */
+long smtp_receive_command_timeout = SMTP_RECEIVE_COMMAND_TIMEOUT; /* timeout for smtp command reads */
+long smtp_receive_message_timeout = SMTP_RECEIVE_MESSAGE_TIMEOUT; /* timeout for smtp message */
 char *auth_domains = AUTH_DOMAINS;	/* authoritative domain list */
 long rfc1413_query_timeout = -1;	/* Timeout on RFC1413 queries - initially disabled */
 long resolve_timeout = RESOLVE_TIMEOUT;	/* timeout on directors/routers */
