@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:debug.h,v 1.16 2000/09/29 21:18:39 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:debug.h,v 1.18 2002/08/21 19:11:51 woods Exp"
  */
 
 /*
@@ -300,5 +300,16 @@
 
 /* debug levels for startup file generation code in smailconf.c */
 #define DBG_CONF_HI	50
-#define DBG_CONF_MID	8
+#define DBG_CONF_MID	11
 #define DBG_CONF_LO	1
+
+/*
+ * variables exported by main.c but used primarily only by debug.h
+ */
+extern int debug;			/* debugging level, 0 is off */
+extern FILE *errfile;			/* file to write debug messages to */
+
+/*
+ * external functions defined in string.c but used primarily only by debug.h
+ */
+extern int dprintf __P((FILE *, char *, ...));
