@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:direct.c,v 1.31 2000/09/03 17:00:12 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:direct.c,v 1.40 2003/04/14 18:39:06 woods Exp"
  */
 
 /*
@@ -20,28 +20,60 @@
  *			    find_direct_driver, read_director_file.
  *                          dump_director_config
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "config.h"
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "direct.h"
-#include "transport.h"
+#include "directors/user.h"
 #include "log.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
+#include "transport.h"
 #include "smailconf.h"
 #include "debug.h"
 #include "extern.h"
 #include "error.h"
+#include "smailport.h"
 
 /* enums local to this file */
 enum local_form {
@@ -72,19 +104,19 @@ static char *director_driv_function __P((char *, struct attribute *));
 
 /* Configuration template information */
 static struct attr_table director_generic[] = {
-    { "driver", t_string, NULL, NULL, OFFSET(director, driver) },
-    { "caution", t_boolean, NULL, NULL, CAUTION_DIRECTOR },
-    { "nobody", t_boolean, NULL, NULL, NOBODY_DIRECTOR },
-    { "sender_okay", t_boolean, NULL, NULL, SENDER_OKAY },
-    { "ignore_alias_match", t_boolean, NULL, NULL, IGNORE_ALIAS_MATCH },
-    { "owner", t_string, NULL, NULL, OFFSET(director, owner) },
-    { "default_user", t_string, NULL, NULL, OFFSET(director, default_user) },
-    { "default_group", t_string, NULL, NULL, OFFSET(director, default_group) },
-    { "default_home", t_string, NULL, NULL, OFFSET(director, default_home) },
-    { "domains", t_string, NULL, NULL, OFFSET(director, domains) },
-    { "set_user", t_string, NULL, NULL, OFFSET(director, set_user) },
-    { "set_group", t_string, NULL, NULL, OFFSET(director, set_group) },
-    { "set_home", t_string, NULL, NULL, OFFSET(director, set_home) },
+    { "driver", t_string, 0, NULL, NULL, OFFSET(director, driver) },
+    { "caution", t_boolean, 0, NULL, NULL, CAUTION_DIRECTOR },
+    { "nobody", t_boolean, 0, NULL, NULL, NOBODY_DIRECTOR },
+    { "sender_okay", t_boolean, 0, NULL, NULL, SENDER_OKAY },
+    { "ignore_alias_match", t_boolean, 0, NULL, NULL, IGNORE_ALIAS_MATCH },
+    { "owner", t_string, 0, NULL, NULL, OFFSET(director, owner) },
+    { "default_user", t_string, 0, NULL, NULL, OFFSET(director, default_user) },
+    { "default_group", t_string, 0, NULL, NULL, OFFSET(director, default_group) },
+    { "default_home", t_string, 0, NULL, NULL, OFFSET(director, default_home) },
+    { "domains", t_string, 0, NULL, NULL, OFFSET(director, domains) },
+    { "set_user", t_string, 0, NULL, NULL, OFFSET(director, set_user) },
+    { "set_group", t_string, 0, NULL, NULL, OFFSET(director, set_group) },
+    { "set_home", t_string, 0, NULL, NULL, OFFSET(director, set_home) },
 };
 
 
@@ -469,7 +501,6 @@ finish_direct_other_form(dp, addr)
 
     switch (form) {
     case FAIL:
-    case PARSE_ERROR:
 	xfree(temp_work_addr);
 	break;
 
@@ -553,7 +584,7 @@ finish_direct_special_form(addr, dp, form, new, out, defer, fail, includes)
     struct addr *addr;			/* addr structure to finish */
     struct director *dp;		/* matching director */
     enum local_form form;		/* local address form */
-    struct addr **new;			/* put new addrs to retry here (not currently used) */
+    struct addr **new;			/* UNUSED put new addrs to retry here */
     struct addr **out;			/* resolved addrs */
     struct addr **defer;		/* put config errors here */
     struct addr **fail;			/* put failed addrs here */
@@ -914,6 +945,7 @@ addr_type(s)
 /*
  * director_user_info - fill an addr structure with user information
  *
+ * XXX FIXME! This function should probably be in directors/user.c
  *
  * if the remainder field if the addr structure matches a username,
  * fill in fields which pertain to the user's passwd file entry.
@@ -934,8 +966,9 @@ director_user_info(addr)
     pw = getpwbyname(addr->flags & USER_IGNORE_CASE ?
 		     str2lower((dupe = COPY_STRING(addr->remainder))) :
 		     addr->remainder);
-    if (dupe)
+    if (dupe) {
 	    xfree(dupe);
+    }
     if (pw) {
 	/* passwd entry found */
 	addr->flags |= ADDR_ISUSER;
@@ -1006,6 +1039,7 @@ read_director_file()
     FILE *f;				/* open director file */
     char *error;			/* error from read_standard_file() */
     struct stat statbuf;
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_director_generic = ENDTABLE(director_generic);
     static struct director director_template = {
 	NULL,				/* name */
@@ -1098,6 +1132,7 @@ dump_director_config(f)
      FILE * f;
 {
     struct director *dp;		/* temp for stepping thru directors */
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_director_generic = ENDTABLE(director_generic);
     char * name;			/* name of director */
     long flags;				/* flags for director */
