@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:aliasfile.c,v 1.26 2001/08/03 13:35:41 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:aliasfile.c,v 1.34 2003/04/02 22:19:37 woods Exp"
  */
 
 /*
@@ -65,49 +65,76 @@
  *	    tryagain:  if set, then if the open fails, try again on a
  *		later spool directory queue run.
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <pwd.h>
 #include <grp.h>
 #include <errno.h>
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
-#include "../direct.h"
-#include "../transport.h"
 #include "../exitcodes.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../direct.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "aliasfile.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../error.h"
-# include "../debug.h"
-#endif
+#include "../extern.h"
+#include "../error.h"
+#include "../debug.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static int alias_secure __P((struct aliasfile_private *, struct stat *));
 
 static struct attr_table aliasfile_attributes[] = {
-    { "file", t_string, NULL, NULL, OFFSET(aliasfile_private, file) },
-    { "proto", t_string, NULL, NULL, OFFSET(aliasfile_private, proto) },
-    { "modemask", t_mode, NULL, NULL, OFFSET(aliasfile_private, modemask) },
-    { "owners", t_string, NULL, NULL, OFFSET(aliasfile_private, owners) },
-    { "owngroups", t_string, NULL, NULL,
+    { "file", t_string, 0, NULL, NULL, OFFSET(aliasfile_private, file) },
+    { "proto", t_string, 0, NULL, NULL, OFFSET(aliasfile_private, proto) },
+    { "modemask", t_mode, 0, NULL, NULL, OFFSET(aliasfile_private, modemask) },
+    { "owners", t_string, 0, NULL, NULL, OFFSET(aliasfile_private, owners) },
+    { "owngroups", t_string, 0, NULL, NULL,
 	  OFFSET(aliasfile_private, owngroups) },
-    { "retries", t_int, NULL, NULL, OFFSET(aliasfile_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(aliasfile_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, ALIAS_REOPEN },
-    { "optional", t_boolean, NULL, NULL, ALIAS_OPTIONAL },
-    { "tryagain", t_boolean, NULL, NULL, ALIAS_TRYAGAIN },
+    { "retries", t_int, 0, NULL, NULL, OFFSET(aliasfile_private, retries) },
+    { "interval", t_int, 0, NULL, NULL, OFFSET(aliasfile_private, interval) },
+    { "reopen", t_boolean, 0, NULL, NULL, ALIAS_REOPEN },
+    { "optional", t_boolean, 0, NULL, NULL, ALIAS_OPTIONAL },
+    { "tryagain", t_boolean, 0, NULL, NULL, ALIAS_TRYAGAIN },
 };
 static struct attr_table *end_aliasfile_attributes =
 	ENDTABLE(aliasfile_attributes);
@@ -125,6 +152,7 @@ static struct aliasfile_private aliasfile_template = {
     2,					/* interval */
     0,					/* flags_set -- for internal use */
     NULL,				/* database -- for internal use */
+    NULL,				/* error_text -- for internal use */
 };
 
 
@@ -253,9 +281,9 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 	    error = xprintf("director %s: alias database %s: %s",
 			    dp->name, priv->file, priv->error_text);
 #endif
-	    err = note_error((dp->flags & ALIAS_TRYAGAIN) || ret == DB_AGAIN?
-				ERR_CONFERR|ERR_112:
-				ERR_112,
+	    err = note_error(((dp->flags & ALIAS_TRYAGAIN) || ret == DB_AGAIN) ?
+			         (ERR_CONFERR | ERR_112) :
+			         ERR_112,
 			     error);
 	    insert_addr_list(in, defer, err);
 #ifdef USE_TARGET_DOMAIN
@@ -349,7 +377,7 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 					cur->remainder)) ||
 		!(preparsed_owner = preparse_address(owner, &error)))
 	    {
-		DEBUG1(DBG_DRIVER_HI, "dtd_aliasfile: no owner set for %s.\n", cur->remainder);
+		DEBUG1(DBG_DRIVER_LO, "dtd_aliasfile: no owner address set for %s!\n", cur->remainder);
 		owner = NULL;
 	    } else {
 		/* 
@@ -385,7 +413,9 @@ dtd_aliasfile(dp, in, out, new, defer, fail)
 		    exitvalue = oexitval;
 		    if (!okay) {
 			xfree(owner);
-			owner = NULL;
+			/* XXX some might claim this should be a paniclog entry, but it could be too frequent... */
+			DEBUG1(DBG_DRIVER_LO, "dtd_aliasfile: couldn't verify owner %s, using 'postmaster'\n", preparsed_owner);
+			owner = COPY_STRING("postmaster");
 		    }
 		    xfree((char *) new2);
 		    if (was_reopen) {
