@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:altuser.c,v 1.10 2000/09/03 17:00:21 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:altuser.c,v 1.17 2003/04/02 22:19:29 woods Exp"
  */
 
 /*
@@ -65,23 +65,42 @@
  *	    tryagain:  if set, then if the open fails, try again on a
  *		later spool directory queue run.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <pwd.h>
 #include <grp.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
 
 #include "../smail.h"
 #include "../main.h"
@@ -89,10 +108,12 @@
 #include "../addr.h"
 #include "../field.h"
 #include "../log.h"
+#include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
+#include "../dys.h"
 #include "../direct.h"
 #include "../transport.h"
-#include "../alloc.h"
-#include "../dys.h"
 #include "../smailconf.h"
 #include "../exitcodes.h"
 #include "../lookup.h"
@@ -100,30 +121,24 @@
 #include "../extern.h"
 #include "../error.h"
 #include "../debug.h"
-
-#undef P_
-#ifdef ANSI_C
-# define P_(x) x
-#else
-# define P_(x) ()
-#endif
+#include "../smailport.h"
 
 /* functions local to this file */
-static int altuserfile_secure P_((struct altuser_private *, struct stat *));
-static int parse_pwline P_((char *, char *, int *, int *, char * *));
+static int altuserfile_secure __P((struct altuser_private *, struct stat *));
+static int parse_pwline __P((char *, char *, int *, int *, char * *));
 
 static struct attr_table altuser_attributes[] = {
-    { "pwfile", t_string, NULL, NULL, OFFSET(altuser_private, pwfile) },
-    { "proto", t_string, NULL, NULL, OFFSET(altuser_private, proto) },
-    { "modemask", t_mode, NULL, NULL, OFFSET(altuser_private, modemask) },
-    { "owners", t_string, NULL, NULL, OFFSET(altuser_private, owners) },
-    { "owngroups", t_string, NULL, NULL,
+    { "pwfile", t_string, 0, NULL, NULL, OFFSET(altuser_private, pwfile) },
+    { "proto", t_string, 0, NULL, NULL, OFFSET(altuser_private, proto) },
+    { "modemask", t_mode, 0, NULL, NULL, OFFSET(altuser_private, modemask) },
+    { "owners", t_string, 0, NULL, NULL, OFFSET(altuser_private, owners) },
+    { "owngroups", t_string, 0, NULL, NULL,
 	  OFFSET(altuser_private, owngroups) },
-    { "retries", t_int, NULL, NULL, OFFSET(altuser_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(altuser_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, ALTUSER_REOPEN },
-    { "tryagain", t_boolean, NULL, NULL, ALTUSER_TRYAGAIN },
-    { "transport", t_string, NULL, NULL, OFFSET(altuser_private, transport) },
+    { "retries", t_int, 0, NULL, NULL, OFFSET(altuser_private, retries) },
+    { "interval", t_int, 0, NULL, NULL, OFFSET(altuser_private, interval) },
+    { "reopen", t_boolean, 0, NULL, NULL, ALTUSER_REOPEN },
+    { "tryagain", t_boolean, 0, NULL, NULL, ALTUSER_TRYAGAIN },
+    { "transport", t_string, 0, NULL, NULL, OFFSET(altuser_private, transport) },
 };
 static struct attr_table *end_altuser_attributes =
 	ENDTABLE(altuser_attributes);
@@ -141,6 +156,8 @@ static struct altuser_private altuser_template = {
     2,					/* interval */
     0,					/* flags_set -- for internal use */
     NULL,				/* database -- for internal use */
+    NULL,				/* transport -- for internal use */
+    NULL,				/* error_text -- for internal use */
 };
 
 
@@ -153,9 +170,9 @@ dtd_altuser(dp, in, out, new, defer, fail)
     struct director *dp;		/* director entry */
     struct addr *in;			/* input local-form addrs */
     struct addr **out;			/* output resolved addrs */
-    struct addr **new;			/* output new addrs to resolve */
+    struct addr **new;			/* UNUSED output new addrs to resolve */
     struct addr **defer;		/* addrs to defer to a later time */
-    struct addr **fail;			/* unresolvable addrs */
+    struct addr **fail;			/* UNUSED unresolvable addrs */
 {
     register struct addr *cur;		/* temp for processing input */
     struct addr *pass = NULL;		/* addrs to pass to next director */
@@ -259,9 +276,9 @@ dtd_altuser(dp, in, out, new, defer, fail)
 	    error = xprintf("director %s: altuser database %s: %s",
 			    dp->name, priv->pwfile, priv->error_text);
 #endif
-	    err = note_error((dp->flags & ALTUSER_TRYAGAIN) || ret == DB_AGAIN?
-				ERR_CONFERR|ERR_112:
-				ERR_112,
+	    err = note_error(((dp->flags & ALTUSER_TRYAGAIN) || ret == DB_AGAIN) ?
+			         (ERR_CONFERR | ERR_112) :
+			         ERR_112,
 			     error);
 	    insert_addr_list(in, defer, err);
 #ifdef USE_TARGET_DOMAIN
