@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:fwdfile.c,v 1.37 2001/08/03 13:35:41 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:fwdfile.c,v 1.49 2003/04/14 18:30:45 woods Exp"
  */
 
 /*
@@ -75,6 +75,9 @@
  *	       `checkowner' is set, no checks are made for ownership
  *	       restrictions.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -82,13 +85,51 @@
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
-#include "defs.h"
-#ifdef UNIX_BSD
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
+#else
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
+#endif
+#ifdef __GLIBC__
 # include <sys/file.h>
 #endif
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
+#ifdef HAVE_SYS_LOCKING_H
+# include <sys/locking.h>
 #endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
@@ -97,6 +138,7 @@
 #include "../log.h"
 #include "../direct.h"
 #include "../alloc.h"
+#include "../smailstring.h"
 #include "../dys.h"
 #include "../exitcodes.h"
 #include "../smailconf.h"
@@ -104,6 +146,7 @@
 #include "../extern.h"
 #include "../error.h"
 #include "../debug.h"
+#include "../smailport.h"
 
 #ifndef HAVE_STDLIB_H
 long atol __P((char *));
@@ -119,24 +162,24 @@ static int forward_secure __P((struct director *, struct addr *, struct stat *,
 static int secure_match __P((char *, char *, char *));
 
 static struct attr_table forwardfile_attributes[] = {
-    { "file", t_string, NULL, NULL, OFFSET(forwardfile_private, file) },
-    { "modemask", t_mode, NULL, NULL,
+    { "file", t_string, 0, NULL, NULL, OFFSET(forwardfile_private, file) },
+    { "modemask", t_mode, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, modemask) },
-    { "caution", t_string, NULL, NULL,
+    { "caution", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, caution) },
-    { "unsecure", t_string, NULL, NULL,
+    { "unsecure", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, unsecure) },
-    { "owners", t_string, NULL, NULL,
+    { "owners", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, owners) },
-    { "owngroups", t_string, NULL, NULL,
+    { "owngroups", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, owngroups) },
-    { "prefix", t_string, NULL, NULL,
+    { "prefix", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, prefix) },
-    { "suffix", t_string, NULL, NULL,
+    { "suffix", t_string, 0, NULL, NULL,
 	  OFFSET(forwardfile_private, suffix) },
-    { "checkowner", t_boolean, NULL, NULL, FWD_CHECKOWNER },
-    { "forwardto", t_boolean, NULL, NULL, FWD_FORWARDTO },
-    { "lock", t_boolean, NULL, NULL, FWD_LOCKFWDFILE },
+    { "checkowner", t_boolean, 0, NULL, NULL, FWD_CHECKOWNER },
+    { "forwardto", t_boolean, 0, NULL, NULL, FWD_FORWARDTO },
+    { "lock", t_boolean, 0, NULL, NULL, FWD_LOCKFWDFILE },
 };
 static struct attr_table *end_forwardfile_attributes =
     ENDTABLE(forwardfile_attributes);
@@ -147,6 +190,8 @@ static struct forwardfile_private forwardfile_template = {
     NULL,				/* unsecure */
     NULL,				/* owners */
     NULL,				/* owngroups */
+    NULL,				/* prefix */
+    NULL,				/* suffix */
 };
 
 
@@ -172,7 +217,7 @@ dtd_forwardfile(dp, in, out, new, defer, fail)
     /* loop over all of the input addrs */
     for (cur = in; cur; cur = next) {
 	next = cur->succ;
-	if (cur->remainder[0] == '.' || index(cur->remainder, '/')) {
+	if (cur->remainder[0] == '.' || strchr(cur->remainder, '/')) {
 	    /* do not match hidden files or files in subdirectories */
 	    cur->succ = pass;
 	    pass = cur;
@@ -269,7 +314,7 @@ finish_forward(dp, addr, out, new, defer, fail)
     char *error = NULL;			/* error message */
     struct str fwdtobuf;
     char *remainder = addr->remainder;
-    int c_save = 0;				/* saved start of suffix */
+    int c_save = 0;			/* saved start of suffix */
     char *suffixp = NULL;		/* start of suffix in remainder */
     char *owner = NULL;                 /* owner address */
     char *preparsed_owner = NULL;       /* preparsed version */
@@ -280,7 +325,7 @@ finish_forward(dp, addr, out, new, defer, fail)
 
     /* remove prefix and suffix from the user name */
     if (priv->prefix) {
-	unsigned int len = strlen(priv->prefix);
+	size_t len = strlen(priv->prefix);
 
 	if (strncmpic(priv->prefix, remainder, len) == 0) {
 	    remainder += len;
@@ -304,8 +349,8 @@ finish_forward(dp, addr, out, new, defer, fail)
     }
 
     /* compute the file name */
-    fn = expand_string(priv->file, (struct addr *)NULL,
-		       addr->flags&ADDR_ISUSER? addr->home: (char *)NULL,
+    fn = expand_string(priv->file, (struct addr *) NULL,
+		       (addr->flags & ADDR_ISUSER) ? addr->home : (char *) NULL,
 		       remainder);
     if (suffixp) {
 	*suffixp = c_save;
@@ -420,12 +465,12 @@ finish_forward(dp, addr, out, new, defer, fail)
 
 	buf = STR(&fwdtobuf) + sizeof("forward to ") - 1;
     } else {
-	unsigned int fsize;
+	size_t fsize;
 
 	fsize = statbuf.st_size;
 	buf = xmalloc(fsize + 1); /* leave room for trailing nul */
 
-	if (fread(buf, (size_t) statbuf.st_size, 1, fp) < 1) {
+	if (fread(buf, (size_t) statbuf.st_size, (size_t) 1, fp) < 1) {
 	    write_log(WRITE_LOG_SYS, "read failed on %s, director=%s", fn, dp->name);
 	    (void) fclose(fp);
 	    xfree(buf);
@@ -502,7 +547,7 @@ finish_forward(dp, addr, out, new, defer, fail)
 				remainder)) ||
         !(preparsed_owner = preparse_address(owner, &error)))
     {
-	DEBUG1(DBG_DRIVER_HI, "dtd_forwardfile: no owner set for %s.\n", addr->remainder);
+	DEBUG1(DBG_DRIVER_LO, "dtd_forwardfile: no owner set for %s!\n", addr->remainder);
         owner = NULL;
     } else {
 	/* Must verify that this owner is sensible (to avoid things like
@@ -533,7 +578,9 @@ finish_forward(dp, addr, out, new, defer, fail)
 	    exitvalue = oexitval;
 	    if (!okay) {
 		xfree(owner);
-		owner = NULL;
+		/* XXX some might claim this should be a paniclog entry, but it could be too frequent... */
+		DEBUG1(DBG_DRIVER_LO, "dtd_forwardfile: couldn't verify owner '%s', using 'postmaster'\n", preparsed_owner);
+		owner = COPY_STRING("postmaster");
 	    }
 
 	    xfree((char *) new2);
@@ -587,9 +634,9 @@ forward_secure(dp, addr, statp, fn)
     if (priv->unsecure) {
 	char *temp = priv->unsecure;
 	int found = FALSE;
-	char *save_dir = rindex(fn, '/'); /* temp put a nul byte here */
+	char *save_dir = strrchr(fn, '/'); /* temp put a nul byte here */
 
-	*save_dir = '\0';
+	*save_dir = '\0';		/* XXX FIXME writable storage? */
 	for (temp = strcolon(temp); temp; temp = strcolon((char *)NULL)) {
 	    if (secure_match(temp, addr->remainder, fn)) {
 		found = TRUE;
@@ -612,10 +659,10 @@ forward_secure(dp, addr, statp, fn)
     if (priv->caution) {
 	char *temp = priv->caution;
 	int found = FALSE;
-	char *save_dir = rindex(fn, '/'); /* temp put a nul byte here */
+	char *save_dir = strrchr(fn, '/'); /* temp put a nul byte here */
 
 	/* pass the directory name to secure_match */
-	*save_dir = '\0';
+	*save_dir = '\0';		/* XXX FIXME writable storage? */
 	for (temp = strcolon(temp); temp; temp = strcolon((char *)NULL)) {
 	    if (secure_match(temp, addr->remainder, fn)) {
 		found = TRUE;
@@ -720,7 +767,7 @@ secure_match(string, user, dirn)
 	return EQ(dir, dirn);
     }
 
-    if (isdigit(string[0])) {
+    if (isdigit((int) string[0])) {
 	for (p = string; *p; p++) {
 	    if (*p == '-') {
 		if (delim)
@@ -728,7 +775,7 @@ secure_match(string, user, dirn)
 		delim = p;
 		continue;
 	    }
-	    if (! isdigit(*p))
+	    if (! isdigit((int) *p))
 		goto not_range;
 	}
 	pw = getpwbyname(user);
