@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/directors:RELEASE-3_2_0_114:smartuser.c,v 1.14 2000/09/03 17:00:21 woods Exp"
+#ident	"@(#)smail/directors:RELEASE-3_2_0_115:smartuser.c,v 1.23 2003/04/14 18:29:09 woods Exp"
  */
 
 /*
@@ -35,49 +35,85 @@
  *	  match anything, while making it possible to enable the smartuser
  *	  driver using only a config file.
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../field.h"
+#include "../lookup.h"
 #include "../log.h"
-#include "../direct.h"
-#include "../transport.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
 #include "../exitcodes.h"
+#include "../direct.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "smartuser.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../extern.h"
+#include "../debug.h"
+#include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static char *make_well_formed __P((char *));
 static char *make_passable __P((char *));
 
 static struct attr_table smartuser_attributes[] = {
-    { "new_user", t_string, NULL, NULL,
+    { "new_user", t_string, 0, NULL, NULL,
 	  OFFSET(smartuser_private, new_user) },
-    { "transport", t_string, NULL, NULL,
+    { "transport", t_string, 0, NULL, NULL,
 	  OFFSET(smartuser_private, transport) },
-    { "well_formed_only", t_boolean, NULL, NULL, SMARTUSER_WELLFORMED },
+    { "prefix", t_string, 0, NULL, NULL,
+	  OFFSET(smartuser_private, prefix) },
+    { "suffix", t_string, 0, NULL, NULL,
+	  OFFSET(smartuser_private, suffix) },
+    { "well_formed_only", t_boolean, 0, NULL, NULL, SMARTUSER_WELLFORMED },
 };
 static struct attr_table *end_smartuser_attributes =
 	ENDTABLE(smartuser_attributes);
 static struct smartuser_private smartuser_template = {
     NULL,				/* new_user */
     NULL,				/* transport */
+    NULL,				/* prefix */
+    NULL,				/* suffix */
     NULL,				/* transport_ptr */
 };
 
@@ -93,9 +129,10 @@ dtd_smartuser(dp, in, out, new, defer, fail)
     struct addr **out;			/* output resolved addrs */
     struct addr **new;			/* output new addrs to resolve */
     struct addr **defer;		/* addrs to defer to a later time */
-    struct addr **fail;			/* unresolvable addrs */
+    struct addr **fail;			/* UNUSED unresolvable addrs */
 {
     register struct addr *cur;		/* temp for processing input */
+    struct addr *new2 = NULL;		/* addr created if prefix or suffix was stripped */
     struct addr *next;			/* next value for cur */
     struct addr *pass = NULL;		/* addrs to pass to next director */
     char *error = NULL;
@@ -140,6 +177,77 @@ dtd_smartuser(dp, in, out, new, defer, fail)
 	    remainder = make_passable(cur->remainder);
 	}
 
+	if (priv->prefix) {
+	    char *anchored_re;
+	    char *start = NULL;
+	    char *beg = NULL;
+	    char *end = NULL;
+	    char *last = NULL;
+
+	    /*
+	     * if we are testing against a prefix, strip the prefix,
+	     * remembering it in rem_prefix
+	     */
+	    anchored_re = xprintf("^%s", priv->prefix);
+	    if (match_re(cur->remainder, anchored_re, TRUE, &start, &beg, &end, &last) <= 0) {
+		cur->succ = pass;
+		pass = cur;
+		xfree(anchored_re);
+		continue;		/* did not start with prefix */
+	    }
+	    xfree(anchored_re);
+	    new2 = alloc_addr();
+	    new2->remainder = COPY_STRING(last);
+	    new2->rem_prefix = rcopy(beg, end);
+	    DEBUG2(DBG_DRIVER_HI, "dtd_smartuser(): stripped address: %s, prefix: %s.\n", new2->remainder, new2->rem_prefix);
+	}
+	if (priv->suffix) {
+	    char *newrem;
+	    char *anchored_re;
+	    char *start = NULL;
+	    char *beg = NULL;
+	    char *end = NULL;
+	    char *last = NULL;
+
+	    /*
+	     * if we are testing against a suffix, strip the suffix,
+	     * remembering it in rem_suffix
+	     */
+	    if (!new2) {
+		newrem = cur->remainder;
+	    } else {
+		newrem = new2->remainder; /* use new value without prefix */
+	    }
+	    anchored_re = xprintf("%s$", priv->suffix);
+	    if (match_re(newrem, anchored_re, TRUE, &start, &beg, &end, &last) <= 0) {
+		cur->succ = pass;
+		pass = cur;
+		xfree(anchored_re);
+		continue;		/* did not end with suffix */
+	    }
+	    xfree(anchored_re);
+	    if (!new2) {
+		new2 = alloc_addr();
+	    } else {
+		xfree(new2->remainder);	/* don't leak copy just made above! */
+	    }
+	    new2->remainder = rcopy(newrem, start);
+	    new2->rem_suffix = COPY_STRING(beg);
+	    DEBUG2(DBG_DRIVER_HI, "dtd_smartuser(): stripped address: %s, suffix: %s.\n", new2->remainder, new2->rem_suffix);
+	}
+	/*
+	 * If we stripped a prefix and/or suffix string then substitute the new
+	 * fully resolved addr structure in place of the original, leaving the
+	 * original now as its parent.
+	 */
+	if (new2 && !(cur->flags & ADDR_VRFY_ONLY)) {
+	    DEBUG2(DBG_DRIVER_MID, "dtd_smartuser: re-writing addr %s to %s.\n", cur->in_addr, new2->remainder);
+	    new2->in_addr = COPY_STRING(new2->remainder);
+	    new2->director = dp;
+	    new2->parent = cur;
+	    cur = new2;
+	}
+
 	cur->director = dp;		/* match anything, at this point */
 
 	/*
@@ -313,7 +421,7 @@ make_well_formed(name)
     register char *name;		/* input string */
 {
     static char *result = NULL;		/* region for forming result */
-    static unsigned int result_size;		/* allocated region length */
+    static size_t result_size;		/* allocated region length */
     register char *p;			/* point to result */
     register int c;			/* char from name string */
 
@@ -321,7 +429,7 @@ make_well_formed(name)
     if (result == NULL) {
 	result = xmalloc(result_size = strlen(name) + 1);
     } else {
-	register unsigned int new_size = strlen(name) + 1;
+	register size_t new_size = strlen(name) + 1;
 
 	if (new_size > result_size) {
 	    result = xrealloc(result, new_size);
@@ -342,9 +450,9 @@ make_well_formed(name)
     while ((c = *name++)) {
 	if (isalnum(c) || c == '-' || c == '_' || c == '+') {
 	    *p++ = c;
-	} else if (isspace(c) || c == '.') {
+	} else if (isspace((int) c) || c == '.') {
 	    *p++ = '.';
-	    while (isspace(c = *name) || c == '.') {
+	    while (isspace((int) (c = *name)) || c == '.') {
 		name++;
 	    }
 	} else {
@@ -374,8 +482,8 @@ make_passable(name)
     int need_quotes = FALSE;		/* true if we should put in quotes */
     int esc_ct = 0;			/* count of \'s required */
     static char *result = NULL;		/* region for building result */
-    static unsigned int result_size;	/* size of result region */
-    unsigned int new_size;		/* minimum size required */
+    static size_t result_size;		/* size of result region */
+    size_t new_size;			/* minimum size required */
     register char *p;			/* temp for building result */
     register char *q;			/* temp for scanning name */
     register int c;			/* char from name */
@@ -391,7 +499,7 @@ make_passable(name)
 	    }
 	} else {
 	    need_quotes = TRUE;
-	    if (c == '"' || c == '\\' || !isprint(c)) {
+	    if (c == '"' || c == '\\' || !isprint((int) c)) {
 		esc_ct++;
 	    }
 	}
@@ -409,7 +517,7 @@ make_passable(name)
 	p = result;
 	*p++ = '"';
 	for (q = name; (c = *q); q++) {
-	    if (c == '"' || c == '\\' || !isprint(c)) {
+	    if (c == '"' || c == '\\' || !isprint((int) c)) {
 		*p++ = '\\';
 		*p++ = c;
 	    } else {
