@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:smartuser.h,v 1.5 2000/09/03 16:26:33 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:smartuser.h,v 1.6 2003/04/14 18:36:13 woods Exp"
  */
 
 /*
@@ -22,6 +22,8 @@
 struct smartuser_private {
     char *new_user;			/* template for new address */
     char *transport;			/* transport if routing to "smart" transport */
+    char *prefix;			/* prefix, e.g., "owner-" */
+    char *suffix;			/* suffix */
     struct transport *transport_ptr;	/* transport structure */
 };
 
