@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:user.c,v 1.18 2000/09/03 17:00:22 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:user.c,v 1.26 2003/04/02 22:15:05 woods Exp"
  */
 
 /*
@@ -22,37 +22,68 @@
  *	    transport (name):  the name of the transport to use in delivering
  *		mail to local users.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <pwd.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../log.h"
-#include "../direct.h"
-#include "../transport.h"
+#include "../lookup.h"
 #include "../exitcodes.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../direct.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "user.h"
-#ifndef DEPEND
-# include "../extern.h"
-# include "../debug.h"
-# include "../error.h"
-#endif
+#include "../debug.h"
+#include "../error.h"
+#include "../extern.h"
+#include "../smailport.h"
 
 
 static struct attr_table user_attributes[] = {
-    { "transport", t_string, NULL, NULL, OFFSET(user_private, transport) },
-    { "prefix", t_string, NULL, NULL, OFFSET(user_private, prefix) },
-    { "ignore-case", t_boolean, NULL, NULL, USER_IGNORE_CASE },
+    { "transport", t_string, 0, NULL, NULL, OFFSET(user_private, transport) },
+    { "prefix", t_string, 0, NULL, NULL, OFFSET(user_private, prefix) },
+    { "suffix", t_string, 0, NULL, NULL, OFFSET(user_private, suffix) },
+    { "ignore-case", t_boolean, 0, NULL, NULL, USER_IGNORE_CASE },
 #ifdef HAVE_FGETPWENT
-    { "pwfile", t_string, NULL, NULL, OFFSET(user_private, pwfile) },
+    { "pwfile", t_string, 0, NULL, NULL, OFFSET(user_private, pwfile) },
 #endif
 };
 static struct attr_table *end_user_attributes =
@@ -60,20 +91,14 @@ static struct attr_table *end_user_attributes =
 static struct user_private user_template = {
     "local",			/* transport */
     NULL,			/* prefix */
+    NULL,			/* suffix */
 #ifdef HAVE_FGETPWENT
     NULL,			/* pwfile */
 #endif
 };
 
-#undef P_
-#ifdef ANSI_C
-# define P_(x) x
-#else
-# define P_(x) ()
-#endif
-
 #ifdef HAVE_FGETPWENT
-void direct_userinfo_pwfile P_((struct addr *, char *));
+void direct_userinfo_pwfile __P((struct addr *, char *));
 #endif
 
 
@@ -86,39 +111,91 @@ dtd_user(dp, in, out, new, defer, fail)
     struct director *dp;		/* director entry */
     struct addr *in;			/* input local-form addrs */
     struct addr **out;			/* output resolved addrs */
-    struct addr **new;			/* output new addrs to resolve */
+    struct addr **new;			/* UNUSED output new addrs to resolve */
     struct addr **defer;		/* addrs to defer to a later time */
-    struct addr **fail;			/* unresolvable addrs */
+    struct addr **fail;			/* UNUSED unresolvable addrs */
 {
     register struct addr *cur;		/* temp for processing input */
+    struct addr *new2 = NULL;		/* addr created if prefix or suffix was stripped */
     struct addr *pass = NULL;		/* addrs to pass to next director */
     struct addr *next;			/* next value for cur */
     struct user_private *priv;
 
-    if (!(priv = (struct user_private *)dp->private)) {
+    if (!(priv = (struct user_private *) dp->private)) {
 	priv = &user_template;
     }
 
-    DEBUG(DBG_DRIVER_HI, "dtd_user called\n");
+    DEBUG1(DBG_DRIVER_HI, "dtd_user called for director %s\n", dp->name);
+
     for (cur = in; cur; cur = next) {
 	next = cur->succ;
 
 	if (priv->prefix) {
-	    struct addr *new2;
-	    unsigned int len = strlen(priv->prefix);
+	    char *anchored_re;
+	    char *start = NULL;
+	    char *beg = NULL;
+	    char *end = NULL;
+	    char *last = NULL;
 
 	    /*
-	     * if we are testing against a prefix, strip the prefix and
-	     * lookup the name in the passwd file.  If found, return a
-	     * new fully resolved addr.
+	     * if we are testing against a prefix, strip the prefix,
+	     * remembering it in rem_prefix
 	     */
-	    if (strncmpic(priv->prefix, cur->remainder, len) != 0) {
+	    anchored_re = xprintf("^%s", priv->prefix);
+	    if (match_re(cur->remainder, anchored_re, (unsigned) (cur->flags & USER_IGNORE_CASE), &start, &beg, &end, &last) <= 0) {
 		cur->succ = pass;
 		pass = cur;
+		xfree(anchored_re);
 		continue;		/* did not start with prefix */
 	    }
+	    xfree(anchored_re);
 	    new2 = alloc_addr();
-	    new2->remainder = COPY_STRING(cur->remainder + len);
+	    new2->remainder = COPY_STRING(last);
+	    new2->rem_prefix = rcopy(beg, end);
+	    DEBUG2(DBG_DRIVER_HI, "dtd_user(): stripped address: %s, prefix: %s.\n", new2->remainder, new2->rem_prefix);
+	}
+	if (priv->suffix) {
+	    char *newrem;
+	    char *anchored_re;
+	    char *start = NULL;
+	    char *beg = NULL;
+	    char *end = NULL;
+	    char *last = NULL;
+
+	    /*
+	     * if we are testing against a suffix, strip the suffix,
+	     * remembering it in rem_suffix
+	     */
+	    if (!new2) {
+		newrem = cur->remainder;
+	    } else {
+		newrem = new2->remainder; /* use new value without prefix */
+	    }
+	    anchored_re = xprintf("%s$", priv->suffix);
+	    if (match_re(newrem, anchored_re, (unsigned) (cur->flags & USER_IGNORE_CASE), &start, &beg, &end, &last) <= 0) {
+		cur->succ = pass;
+		pass = cur;
+		xfree(anchored_re);
+		continue;		/* did not end with suffix */
+	    }
+	    xfree(anchored_re);
+	    if (!new2) {
+		new2 = alloc_addr();
+	    } else {
+		xfree(new2->remainder);	/* don't leak copy just made above! */
+	    }
+	    new2->remainder = rcopy(newrem, start);
+	    new2->rem_suffix = COPY_STRING(beg);
+	    DEBUG2(DBG_DRIVER_HI, "dtd_user(): stripped address: %s, suffix: %s.\n", new2->remainder, new2->rem_suffix);
+	}
+	/*
+	 * If we stripped a prefix and/or suffix string then lookup the
+	 * resulting name (now in the remainder field) in the passwd file.  If
+	 * a matching username is found, then substitute the new fully resolved
+	 * addr structure in place of the original, leaving the original now as
+	 * its parent.
+	 */
+	if (new2) {
 #ifdef HAVE_FGETPWENT
 	    if (priv->pwfile) {
 		direct_userinfo_pwfile(new2, priv->pwfile);
@@ -129,10 +206,30 @@ dtd_user(dp, in, out, new, defer, fail)
 	    }
 
 	    if ((new2->flags & ADDR_NOTUSER) || (new2->error)) {
-		/* we did not match a user on the local host */
-	        cur->error = new2->error;
+		/*
+		 * the stripped name apparently did not match a username
+		 */
+	        cur->error = new2->error;	/* keep any error info */
+
+		/* clean up temporary new addr */
 		xfree(new2->remainder);
-		xfree((char *)new2);
+		if (new2->rem_prefix) {
+		    xfree(new2->rem_prefix);
+		}
+		if (new2->rem_suffix) {
+		    xfree(new2->rem_suffix);
+		}
+		xfree((char *) new2);
+
+		/*
+		 * NOTE: we can't easily factor out the error handling of these
+		 * two blocks because we can't corrupt cur->flags with
+		 * ADDR_NOTUSER.  If the prefix and/or suffix didn't match then
+		 * some other director must still be given a chance, including
+		 * another which calls director_user_info() and which might try
+		 * to match a username with some different prefix and/or
+		 * suffix.
+		 */
 		if (cur->error) {
 		    cur->succ = *defer;
 		    *defer = cur;
@@ -140,25 +237,19 @@ dtd_user(dp, in, out, new, defer, fail)
 		    cur->succ = pass;
  		    pass = cur;
 		}
-		continue;
+		continue;			/* no such user with pre/suf */
 	    }
 	    if (!(cur->flags & ADDR_VRFY_ONLY)) {
-		DEBUG2(DBG_DRIVER_HI, "dtd_user: re-writing addr %s to %s.\n", cur->in_addr, new2->remainder);
+		DEBUG2(DBG_DRIVER_MID, "dtd_user: re-writing addr %s to %s.\n", cur->in_addr, new2->remainder);
 		new2->in_addr = COPY_STRING(new2->remainder);
 		new2->director = dp;
 		new2->parent = cur;
 		cur = new2;
 	    }
 	} else {
-	    /* fill in any user information */
 #ifdef HAVE_FGETPWENT
 	    if (priv->pwfile) {
 		direct_userinfo_pwfile(cur, priv->pwfile);
-		if (cur->error) {
-		    cur->succ = *defer;
-		    *defer = cur;
-		    continue;
-		}		    
 	    } else
 #endif
 	    {
@@ -166,18 +257,25 @@ dtd_user(dp, in, out, new, defer, fail)
 	    }
 
 	    /* don't match it if it is not a local user */
-	    if (cur->flags & ADDR_NOTUSER) {
-		cur->succ = pass;
-		pass = cur;
-		continue;
+	    if ((cur->flags & ADDR_NOTUSER) || (cur->error)) {
+		if (cur->error) {
+		    cur->succ = *defer;
+		    *defer = cur;
+		} else {
+		    cur->succ = pass;
+		    pass = cur;
+		}
+		continue;			/* no such user */
 	    }
 	}
 
 	DEBUG2(DBG_DRIVER_LO, "director %s matched user %s\n",
 	       dp->name, cur->remainder);
-	cur->director = dp;		/* matched address */
 
-	/* attach the transport */
+	/* attach this director */
+	cur->director = dp;
+
+	/* attach the specified transport */
 	cur->transport = find_transport(priv->transport);
 	if (cur->transport == NULL) {
 	    /*
@@ -195,18 +293,22 @@ dtd_user(dp, in, out, new, defer, fail)
 	     *      correct transport for local delivery.
 	     */
 	    cur->error = note_error(ERR_CONFERR|ERR_122,
-				    xprintf("director %s: transport not defined",
+				    xprintf("director %s: no transport defined!",
 					    dp->name));
 	    cur->succ = *defer;
 	    *defer = cur;
 	    continue;
 	}
+
+	/* assign the final delivery address from the remainder */
 	cur->next_addr = COPY_STRING(cur->remainder); /* XXX avoid if ADDR_VRFY_ONLY? */
+
+	/* hook this address into the resolved address list */
 	cur->succ = *out;
 	*out = cur;
     }
 
-    return pass;			/* return addrs for next director */
+    return pass;	/* return remaining addrs for the next director */
 }
 
 /*
@@ -294,7 +396,7 @@ direct_userinfo_pwfile(addr, pwfile)
 				 xprintf("director user: specified passwd file missing"));
 	return;
     } 
-    while (pw = (struct passwd *) fgetpwent(pwfd)) {
+    while ((pw = (struct passwd *) fgetpwent(pwfd))) {
 	if (strcmp(pw->pw_name, addr->remainder) == 0) {
 	    /* passwd entry found */
 	    addr->flags |= ADDR_ISUSER;
