@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/directors:RELEASE-3_2_0_114:user.h,v 1.7 2000/09/03 16:29:03 woods Exp"
+#ident	"@(#)smail/src/directors:RELEASE-3_2_0_115:user.h,v 1.10 2003/04/14 18:37:27 woods Exp"
  */
 
 /*
@@ -15,11 +15,17 @@
  *	interface file for driver code in user.c
  */
 
+/* boolean attributes for director.flags field */
+#define USER_IGNORE_CASE 0x0001000	/* lower user b4 matching w/mailbox */
+
 /* private information from director file entry */
 struct user_private {
     char *transport;			/* name of the transport */
     char *prefix;			/* prefix, e.g., "real-" */
+    char *suffix;			/* suffix, e.g., "-.*" */
+#ifdef HAVE_FGETPWENT
     char *pwfile;			/* name of password file */
+#endif
 };
 
 extern struct addr *dtd_user __P((struct director *,
