@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:dys.h,v 1.12 2000/09/03 15:57:14 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:dys.h,v 1.16 2002/10/03 06:28:01 woods Exp"
  */
 
 /*
@@ -26,13 +26,6 @@
  *	xrealloc() is required.
  */
 
-/* type for a dynamic string region */
-struct str {
-    char *p;				/* xmalloc'd text region */
-    unsigned int i;			/* index into text region */
-    unsigned int a;			/* current allocation size */
-};
-
 /* STR - access the string in question directly */
 #define STR(sp)		((sp)->p)
 
@@ -42,6 +35,9 @@ struct str {
 /* STR_CHECK - call X_CHECK (from alloc.h) for a string */
 #define STR_CHECK(sp)	X_CHECK((sp)->p)
 
+/* STR_ALLOCSZ - access the allocated size of the string buffer */
+#define STR_ALLOCSZ(sp)	((sp)->a)
+
 /* STR_BUMP - the basic quantum of allocation space */
 #define STR_BUMP	64
 
@@ -89,12 +85,16 @@ struct str {
 #define STR_TRIM(sp, len)	((sp)->i = (len))
 
 /*
- * STR_CAT - concatenate a string onto the end of a dynamic string
- * region, growing as necessary.
+ * STR_CAT - concatenate a C string onto the end of a dynamic string region
+ * (thus NUL-terminating it), growing the storage region as necessary.
  *
- * This is now implemented as a function in string.c.
+ * STR_NCAT - concatenate a C string onto the end of a dynamic string region
+ * (thus NUL-terminating it), growing the storage region as necessary.
+ *
+ * These are now implemented as functions in string.c.
  */
 #define STR_CAT(sp, cs)		str_cat((sp), (cs))
+#define STR_NCAT(sp, cs, len)	str_ncat((sp), (cs), (len))
 
 /*
  * STR_DONE - finish building a dynamic string region.  This is not
@@ -103,7 +103,7 @@ struct str {
  * stock malloc and realloc functions.
  */
 #define STR_DONE(sp)	((sp)->p = xrealloc((char *) ((sp)->p),		\
-					    (unsigned) ((sp)->i + 1)),	\
+					    ((sp)->i + 1)),		\
 			 (sp)->a = (sp)->i + 1)
 
 /*
@@ -119,4 +119,4 @@ struct str {
 /*
  * COPY_STRING - copy a C-style string to a new xmalloc'd region
  */
-#define COPY_STRING(sp)	(strcpy(xmalloc((unsigned int) (strlen((sp)) + 1)), (sp)))
+#define COPY_STRING(sp)	(strcpy(xmalloc((size_t) (strlen((sp)) + 1)), (sp)))
