@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:extern.h,v 1.82 2001/07/29 16:10:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:extern.h,v 1.100 2003/03/21 01:31:05 woods Exp"
  */
 
 /*
@@ -21,11 +21,9 @@ extern int errno;			/* previous system error value */
 /*
  * External variables used in the smail program
  */
-/* external variables defined in alloc.c */
-extern struct block *perm;		/* permanent allocation block */
-extern int x_dont_panic;		/* dont panic in X_CHECK() */
 
 /* external variables defined in config.c */
+extern int allow_one_mx_target_cname_hack;	/* relax DNS MX checks a bit */
 extern char *listen_name;		/* DNS name of interface to listen on */
 extern char *sending_name;		/* DNS name of interface to send mail from */
 extern char *visible_name;		/* hostname used in outgoing addrs */
@@ -44,8 +42,8 @@ extern char *cons_fn;			/* name of console device file */
 extern char *spool_dirs;		/* names of spooling directories */
 extern int spool_mode;			/* mode for spool files */
 extern int lock_mode;			/* mode for lock files */
-extern int log_mode;			/* mode for system log files */
-extern int message_log_mode;		/* mode for per-message log files */
+extern unsigned int log_mode;		/* mode for system log files */
+extern unsigned int message_log_mode;	/* mode for per-message log files */
 extern int spool_grade;			/* default spool grade character */
 extern int open_retries;		/* max open retries on startup file */
 extern int open_interval;		/* sleep between open retries */
@@ -68,11 +66,12 @@ extern char *date_field;		/* Date: field string */
 extern char *from_field;		/* From: field string */
 extern char *return_path_field;		/* Return-Path: field string */
 extern char *smail;			/* location of the smail program */
+extern char *daemon_pidfile;		/* location of the PID file */
 extern double max_load_ave;		/* spool mail > this load agerage */
 extern char *trusted;			/* : list of trusted users */
 extern char *trusted_groups;		/* : list of trusted groups */
-extern unsigned message_bufsiz;		/* size of message buffers */
-extern int hit_table_len;		/* #entries in address hit table */
+extern size_t message_bufsiz;		/* size of message buffers */
+extern unsigned int hit_table_len;	/* #entries in address hit table */
 extern int flock_mailbox;		/* TRUE to use lock_fd_wait() macro */
 extern int fnlock_retries;		/* retries for lock_file() creat */
 extern unsigned int fnlock_interval;	/* retry intervals for lock_file() */
@@ -92,39 +91,54 @@ extern int auto_mkdir;			/* TRUE to auto create directories */
 extern unsigned int auto_mkdir_mode;	/* the mode for auto directories */
 extern int require_configs;		/* TRUE to require config files */
 extern char *postmaster_address;	/* default addr of postmaster */
-extern char *smtp_banner;		/* smtp startup banner message */
 extern int smtp_accept_max;		/* max simultaneous SMTPs to accept */
 extern int smtp_accept_queue;		/* simultaneous SMTPs until queueonly */
 extern int smtp_allow_debug;		/* allow DEBUG command in SMTP */
 extern int smtp_allow_expn;		/* allow EXPN command in SMTP */
-extern char *smtp_recipient_no_verify;	/* ip list for not verifying RCPT TO: */
-extern char *smtp_remote_allow;		/* ip list that can send remote smtp */
-extern char *smtp_local_net;		/* the local network calculated from src. addr */
+extern char *smtp_bad_mx_targets;	/* ip list of undesirable MX targets */
+extern char *smtp_banner;		/* smtp startup banner message */
+extern unsigned int smtp_debug_pause;	/* seconds to pause after child fork for debugger connect */
 extern int smtp_hello_verify;		/* verify HELO/EHLO host has A RR matching sockaddr */
 extern char *smtp_hello_broken_allow;	/* ip list that can avoid hello verification */
+extern char *smtp_hello_dnsbl_domains;	/* list of DNSBL domains to check sender_host */
+extern char *smtp_hello_dnsbl_except;	/* list of exceptions for smtp_hello_dnsbl_domains */
 extern int smtp_hello_reject_dns_paranoid; /* like TCP Wrappers PARANOID check */
-extern char *smtp_hello_reject_hosts;	/* like TCP Wrappers */
+extern char *smtp_hello_reject_hostnames; /* list of hostnames to match sender_host */
 extern int smtp_hello_verify_literal;	/* verify HELO/EHLO domain literal has PTR RR */
 extern int smtp_hello_verify_ptr;	/* verify HELO/EHLO matches PTR RR for sockaddr */
+extern char *smtp_host_dnsbl_domains;	/* list of DNSBL domains to check sender_host_really */
+extern char *smtp_host_dnsbl_except;	/* list of exceptions for smtp_host_dnsbl_domains */
+extern char *smtp_host_reject_hostnames; /* list of hostname REs to match sender_host_really */
+extern char *smtp_local_net;		/* the local network calculated from src. addr */
+extern char *smtp_local_sender_allow;	/* locally deliverable mailboxes which might come back to us */
+extern int smtp_local_sender_restrict;	/* verify locally deliverable sender clients match smtp_remote_allow */
+extern unsigned int smtp_max_recipients; /* maximum recipients per SMTP connect */
+extern unsigned int smtp_max_bounce_recipients; /* maximum recipients per bounce message */
+extern char *smtp_permit_mx_backup;	/* list of domains we will allow for secondary MX */
 extern char *smtp_rbl_domains;		/* list of RBL domains */
 extern char *smtp_rbl_except;		/* list of exceptions to RBL lookups */
+extern char *smtp_recipient_no_verify;	/* ip list for not verifying RCPT TO: */
+extern char *smtp_reject_hosts;		/* list of IP/Nets to match client address */
+extern char *smtp_remote_allow;		/* ip list that can send remote smtp */
 extern char *smtp_sender_no_verify;	/* ip list for not verifying MAIL FROM: */
+extern char *smtp_sender_reject;	/* list of REs to match whole sender addresses */
+extern char *smtp_sender_reject_db;	/* database to match whole sender addresses */
+extern char *smtp_sender_reject_db_proto; /* smtp_sender_reject_db access protocol */
+extern char *smtp_sender_reject_hostnames; /* list of hostname REs to match sender addresses */
 extern char *smtp_sender_rhsbl_domains;	/* list of RHSBL domains for sender addresses */
 extern char *smtp_sender_rhsbl_except;	/* list of exceptions to sender RHSBL lookups */
 extern int smtp_sender_verify_mx_only;	/* require MX RRs for MAIL FROM: hosts */
-extern int smtp_max_recipients;		/* maximum recipients per SMTP connect */
 extern unsigned int smtp_expn_delay;	/* seconds to delay after an EXPN command */
 extern unsigned int smtp_vrfy_delay;	/* seconds to delay after a VRFY command */
 extern unsigned int smtp_error_delay;	/* seconds to delay after any SMTP error */
-extern unsigned int debug_pause_for_smtp_connections; /* seconds to pause after child fork for debugger connect */
 extern char *sender_env_variable;	/* env variable naming user */
 extern int switch_percent_and_bang;	/* switch precedence of % and ! */
 extern int error_copy_postmaster;	/* copy postmaster on errors */
 extern long retry_interval;		/* default delivery retry interval */
 extern long retry_duration;		/* default delivery retry duration */
 extern long host_lock_timeout;		/* timeout for host exclusivity lock */
-extern unsigned int smtp_receive_command_timeout; /* timeout for receiving SMTP cmds */
-extern unsigned int smtp_receive_message_timeout; /* timeout for receiving SMTP msg */
+extern long smtp_receive_command_timeout; /* timeout for receiving SMTP cmds */
+extern long smtp_receive_message_timeout; /* timeout for receiving SMTP msg */
 extern char *auth_domains;		/* authoritative domain list */
 extern long rfc1413_query_timeout;	/* Timeout on RFC1413 queries - initially disabled */
 extern long resolve_timeout;		/* timeout on directors/routers */
@@ -153,62 +167,18 @@ extern struct route_driver route_drivers[]; /* compiled router drivers */
 extern struct transport_driver transport_drivers[]; /* transport drivers */
 #endif
 
-/* external variables defined in header.c */
-extern struct list *header;		/* list of header fields */
-
 /* external variables defined in ldinfo.c */
 extern int compile_num;
 extern char *compile_date;
 
 /* external variables defined in log.c */
+extern FILE *panicfile;			/* open stream to panic log file */
+extern FILE *logfile;			/* open stream to system log file */
 extern FILE *msg_logfile;		/* open stream to per-message log */
 
-/* external variables defined in main.c */
-extern int islocal;			/* TRUE if mail originated locally */
-extern int exitvalue;			/* call exit with this value */
-extern char *program;			/* argv[0] from main */
-extern char *sender;			/* sender of message */
-extern char *local_sender;		/* local sender of message */
-extern int error_sender;		/* TRUE if special sender <> given */
-extern char *sender_name;		/* full name of sender */
-extern int debug;			/* debugging level, 0 is off */
-extern int dont_deliver;		/* if TRUE, don't actually deliver */
-extern int process_queue;		/* process spooled files */
-extern unsigned queue_interval;		/* process queues at this interval */
-extern int hop_count;			/* hop count so far for message */
-extern int do_aliasing;			/* do aliasing for local addresses */
-extern int extract_addresses;		/* get recipients from header */
-#ifdef MAIN_H
-extern enum op_mode operation_mode;	/* mode of operation */
-extern enum dot_usage dot_usage;	/* how do we treat . on input */
-extern enum deliver_mode deliver_mode;	/* foreground, background or queued */
-extern enum er_proc error_processing;	/* method of displaying errors */
-extern enum prog_type prog_type;	/* type of program we are running as */
-#endif	/* MAIN_H */
-extern int me_too;			/* sender allowed in aliases */
-extern struct addr *recipients;		/* list of recipient addresses */
-extern int num_recipients;		/* number of entries in 'recipients' */
-extern char *primary_name;		/* primary local name from hostnames */
-extern FILE *errfile;			/* file to write debug messages to */
-extern unsigned int real_uid;		/* saved real uid before ruid setup */
-extern unsigned int real_gid;		/* saved real gid before rgid setup */
-extern char **save_argv;		/* saved pointer to arguments */
-extern int some_deferred_addrs;		/* don't unlink spool file */
-					/* as some addrs were deferred */
-extern unsigned int prog_euid;		/* effective uid of program */
-extern unsigned int prog_egid;		/* effective gid of program */
-extern int force_zero_exitvalue;	/* if TRUE always exit with status 0 */
-extern int call_freeze_message;		/* if TRUE must call freeze_message() */
-extern int sender_is_trusted;		/* TRUE if sender is a trusted user */
-extern char *sender_host;		/* name of sender's host */
-extern char *sender_host_addr;		/* inet address of sender's host */
-extern char *sender_proto;		/* name of sender's sending protocol */
-extern char *sender_program;		/* name of program that spooled msg */
-extern char * ident_sender;		/* The calculated identity of the sender */
-extern char * ident_method;		/* Method used to determine ident_sender */
-
 /* external variables defined in modes.c */
 extern int daemon_pid;			/* pid of daemon process */
+extern int mailq_summary_only;		/* does mailq print only the summary tally? */
 
 /* external variables defined in notify.c */
 extern int send_to_postmaster;		/* set TRUE to mail to postmaster */
@@ -220,7 +190,7 @@ extern char *on;			/* boolean on attribute value */
 
 /* external variables defined in queue.c */
 extern int msg_grade;			/* grade level for this message */
-extern long msg_body_size;		/* size of message body */
+extern unsigned long msg_body_size;	/* size of message body */
 
 /* external variables defined in resolve.c */
 extern struct hash_table *hit_table;	/* table to recognize address hits */
@@ -241,7 +211,7 @@ extern char *msg_ptr;			/* read placeholder in msg_buf */
 extern char *msg_max;			/* last valid char in msg_buf */
 extern char *end_msg_buf;		/* end of msg_buf */
 extern off_t msg_foffset;		/* file offset for msg_buf contents */
-extern off_t msg_size;			/* size of spool file */
+extern unsigned long msg_size;		/* size of spool file */
 
 /* external variables defined in version.c */
 extern char *version_number;		/* string defining version number */
@@ -292,7 +262,7 @@ extern void print_version __P((void));
 extern void print_copying_file __P((void));
 extern void print_variables __P((void));
 extern void print_queue __P((void));
-extern void smtp_mode __P((FILE *, FILE *));
+extern void smtp_mode __P((FILE *, FILE *, void *));
 extern int process_spool_file __P((char *));
 extern int fork_wait __P((void));
 extern void run_queue __P((int));
@@ -345,34 +315,7 @@ extern int retry_host_lock __P((struct transport *, char *, int *, struct error
 extern void retry_host_unlock __P((time_t, struct error *));
 
 /* external functions defined in smtprecv.c */
-extern char **receive_smtp __P((FILE *, FILE *));
-
-/* external functions defined in string.c */
-extern char *str2lower __P((char *));
-extern int strcmpic __P((char *, char *));
-extern int strncmpic __P((char *, char *, unsigned int));
-extern int strip __P((char *));
-extern char *strcolon __P((char *));
-extern int is_string_in_list __P((char *, char *));
-#ifndef HAVE_STRERROR
-extern char *strerror __P((int));
-#endif
-extern char *strsysexit __P((int));
-extern void str_printf __P((struct str *, char *, ...));
-extern char *xprintf __P((char *, ...));
-extern int dprintf __P((FILE *, char *, ...));
-extern long c_atol __P((char *, char **));
-extern char *base62 __P((unsigned long));
-extern char *read_line __P((FILE *));
-extern void str_cat __P((struct str *, char *));
-extern void str_ncat __P((struct str *, char *, unsigned int));
-#ifndef HAVE_VFPRINTF
-extern int vfprintf __P((FILE *, char *, va_list));
-#endif
-extern long ivaltol __P((char *));
-extern char *ltoival __P((long));
-extern char *copy __P((char *));
-extern char *rcopy __P((char *, char *));
+extern char **receive_smtp __P((FILE *, FILE *, void *));
 
 /* external functions defined in sysdep.c */
 extern char *time_stamp __P((void));
@@ -381,11 +324,15 @@ extern int get_local_year __P((void));
 extern char *unix_date __P((void));
 extern void compute_local_sender __P((void));
 extern void getfullname __P((void));
+#ifdef HAVE_BSD_NETWORKING
+extern unsigned long get_inet_addr __P((char *));
+#endif
 extern FILE *fopen_as_user __P((char *, char *, unsigned int, unsigned int, unsigned int));
 extern int lock_file __P((char *, FILE *));
 extern void unlock_file __P((char *, FILE *));
 #ifdef	USE_FCNTL_RD_LOCK
 extern int fcntl_rd_lock __P((int));
+extern int fcntl_rd_lock_wait __P((int));
 #endif
 extern char *compute_hostname __P((void));
 extern char *compute_domain __P((char *));
@@ -395,9 +342,20 @@ extern void close_all __P((void));
 extern char *scan_dir __P((char *));
 extern int touch __P((char *, time_t, time_t));
 extern int fsetsize __P((int, off_t, off_t));
-#ifdef NEED_HSTRERROR
-extern const char *hstrerror __P((int));
+#ifndef HAVE_VFPRINTF
+extern int vfprintf __P((FILE *, char *, va_list));
 #endif
+#ifndef HAVE_STRERROR
+extern char *strerror __P((int));
+#endif
+extern char *strsysexit __P((int));
+#ifndef HAVE_STRSIGNAL
+extern char *strsignal __P((int));
+#endif
+#ifndef HAVE_SIG2STR
+extern int sig2str __P((int, char *));
+#endif
+extern int set_sigchld_handler __P((void (*)(int)));
 
 /* external functions defined in libcompat XXX should have compat.h */
 int dummy_reference __P((void));
