@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:field.c,v 1.16 2000/09/29 22:15:05 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:field.c,v 1.24 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -28,28 +28,58 @@
  *
  *	external functions:  process_field, tokenize, detokenize, dump_tokens
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "field.h"
 #include "log.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
 #include "debug.h"
 #include "extern.h"
+#include "smailport.h"
 
 /* functions local to this file */
-static char *finish_mod_clean __P((char *, unsigned int, struct token *, unsigned len));
+static char *finish_mod_clean __P((char *, size_t, struct token *, size_t));
 static void insert_comma __P((char *));
-static int match_route_or_group __P((struct token *, struct token **, struct addr **, int, unsigned int *, char *, char *, int, char **));
+static int match_route_or_group __P((struct token *, struct token **, struct addr **, int, size_t *, char *, char *, int, char **));
 static int match_group_term __P((struct token *, struct token **, struct addr **, int, char **));
-static int match_general __P((struct token *, struct token **, unsigned int *, struct addr **, char *, char *, int, char **));
+static int match_general __P((struct token *, struct token **, size_t *, struct addr **, char *, char *, int, char **));
 static char *queue_qualify_domain __P((struct token *, struct token *));
 static int enqueue_address __P((struct addr **, struct token *, struct token *, char **));
 
@@ -126,7 +156,7 @@ tokenize(field, ret_q, alias, space)
     STR_INIT(sp);
 
     /* allocate space for initial token */
-    *ret_q = tq = (struct token *)xmalloc(sizeof(*tq));
+    *ret_q = tq = (struct token *) xmalloc(sizeof(*tq));
 
     /* begin by reading through white space */
     state = s_space;
@@ -136,7 +166,7 @@ tokenize(field, ret_q, alias, space)
      * going through the state machine to build up tokens.
      */
     for (fp = field; *fp != '\0'; fp++) {
-	switch(state) {
+	switch (state) {
 
 	/*
 	 * initial state
@@ -165,7 +195,7 @@ tokenize(field, ret_q, alias, space)
 			STR_NEXT(sp, *fp);
 		    }
 		    break;
-		} else if (isspace(*fp)) {
+		} else if (isspace((int) *fp)) {
 		    if (space) {
 			STR_NEXT(sp, *fp);
 		    }
@@ -202,20 +232,20 @@ tokenize(field, ret_q, alias, space)
 		    break;
 		default:
 		    if (alias && *fp == ':' &&
-			strncmpic(fp, ":include:", sizeof(":include:")-1) == 0)
-		    {
+			strncmpic(fp, ":include:", sizeof(":include:")-1) == 0) {
 			p = fp + sizeof(":include:") - 1;
-			while (isspace(*p) && *p != '\n')
+			while (isspace((int) *p) && *p != '\n') {
 			    p++;
-			if (*p && index(non_text_tokens, *p) == NULL) {
-			    str_ncat(sp, fp, (unsigned) (p - fp));
+			}
+			if (*p && strchr(non_text_tokens, *p) == NULL) {
+			    str_ncat(sp, fp, (size_t) (p - fp));
 			    fp = p;
 			    state = s_text;
 			    tq->form = T_TEXT;
 			    break;
 			}
 		    }
-		    if (index(non_text_tokens, *fp)) {
+		    if (strchr(non_text_tokens, *fp)) {
 			state = s_newtok;
 			tq->form = T_OPER;
 		    } else {
@@ -278,7 +308,7 @@ tokenize(field, ret_q, alias, space)
 	    	STR_NEXT(sp, *fp);	/* copy char into token */
 		save_state = s_text;
 		state = s_cquote;
-	    } else if (index(non_text_tokens, *fp)) {
+	    } else if (strchr(non_text_tokens, *fp)) {
 		/* space or an operator follows a text literal */
 		fp--;		/* re-scan character */
 		state = s_newtok;
@@ -362,8 +392,8 @@ tokenize(field, ret_q, alias, space)
 	    STR_DONE(sp);
 	    /* create new token which is the current token's successor */
 	    tq->succ = (struct token *)xmalloc(sizeof(*tq));
-	    tq->space = sp->p;		/* mark pointer to white space */
-	    tq->text = sp->p + text_offs; /* mark pointer to token text */
+	    tq->space = STR(sp);	/* mark pointer to white space */
+	    tq->text = STR(sp) + text_offs; /* mark pointer to token text */
 	    tq = tq->succ;
 	    /* scan through white space next */
 	    state = s_space;
@@ -396,10 +426,10 @@ tokenize(field, ret_q, alias, space)
 	/* last token does contain some text */
 	STR_NEXT(sp, '\0');
 	STR_DONE(sp);
-	tq->text = sp->p + text_offs;
+	tq->text = STR(sp) + text_offs;
     }
 
-    tq->space = sp->p;	/* first part of p is the white space and comments */
+    tq->space = STR(sp);	/* first part of p is the white space and comments */
 
     /*
      * if the current token is white space, then make it the ending
@@ -412,7 +442,7 @@ tokenize(field, ret_q, alias, space)
 	tq->succ = NULL;
     } else {
 	struct token *end_q;
-	end_q = tq->succ = (struct token *)xmalloc(sizeof(*end_q));
+	end_q = tq->succ = (struct token *) xmalloc(sizeof(*end_q));
 	end_q->text = end_q->space = "";
 	end_q->form = T_END;
 	end_q->succ = NULL;
@@ -485,22 +515,24 @@ detokenize(space, buf, tq_head, tq_end)
     register struct token *tq;		/* temp for scanning through tokens */
 
     if (buf) {
-	register char *bp= buf;		/* point to buf */
+	register char *bp = buf;	/* point to buf */
 
 	bp[0] = '\0';
 	/* loop through contatenating space and text from tokens */
 	for (tq = tq_head; !ENDTOK(tq->form); tq = tq->succ) {
 	    if (space) {
-		(void)strcat(bp, tq->space);
+		(void) strcat(bp, tq->space);
 	    }
-	    (void)strcat(bp, tq->text);
+	    (void) strcat(bp, tq->text);
 	    if (tq == tq_end) {
 		return bp;
 	    }
 	}
 	/* get the white space from the ending token */
-	(void)strcat(bp, tq->space);
+	(void) strcat(bp, tq->space);
+
 	return bp;			/* return the buffer */
+
     } else {
 	struct str str;
 	register struct str *sp = &str;	/* dynamic string region */
@@ -516,7 +548,7 @@ detokenize(space, buf, tq_head, tq_end)
 		STR_NEXT(sp, '\0');	/* null terminate */
 		STR_DONE(sp);		/* finish dynamic string */
 
-		return sp->p;		/* return string */
+		return STR(sp);		/* return string */
 	    }
 	}
 
@@ -524,8 +556,10 @@ detokenize(space, buf, tq_head, tq_end)
 	STR_NEXT(sp, '\0');		/* null terminate */
 	STR_DONE(sp);			/* finish dynamic string */
 
-	return sp->p;			/* return it */
+	return STR(sp);			/* return it */
     }
+
+    /* NOTREACHED */
 }
 
 
@@ -600,7 +634,7 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
     struct token *tq_new;		/* new anchor found by pattern scan */
     int new_group = FALSE;		/* set if group: pattern newly found */
     int group = FALSE;			/* set when inside of a group */
-    unsigned len = 0;			/* length of cleaned header */
+    size_t len = 0;			/* length of cleaned header */
     int need_comma = FALSE;		/* TRUE if we may need a , at anchor */
     int check_route = TRUE;		/* TRUE if we must scan for routes */
     int i;				/* temp */
@@ -639,7 +673,7 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
     tq_anchor = tq_head;
     while (!ENDTOK(tq_anchor->form)) {
 	tq_new = NULL;			/* set when address pattern found */
-	if (check_route) {
+	if (check_route) {		/* XXX currently always true! */
 	    /* scan for:  phrase <route-addr> or phrase : */
 	    i = match_route_or_group(tq_anchor, &tq_new, extract_q, group,
 				     &len, domain, uucp_host, flags, error);
@@ -769,7 +803,7 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
 
     if (modified && field != NULL) {
 	/* copy finished results into buffer for returning to caller */
-	field = finish_mod_clean(field, (unsigned)(fp-field), tq_head, len);
+	field = finish_mod_clean(field, (size_t) (fp-field), tq_head, len);
     }
 
     DEBUG1(DBG_FIELD_MID, "process_field: return %s.\n", field);
@@ -799,23 +833,24 @@ process_field(field, fp, domain, uucp_host, extract_q, flags, error)
 static char *
 finish_mod_clean(field_name, name_len, tq_head, len)
     char *field_name;			/* field name string */
-    unsigned name_len;			/* length of field name */
+    size_t name_len;			/* length of field name */
     struct token *tq_head;		/* head of list to convert to string */
-    unsigned len;			/* computed length of result */
+    size_t len;				/* computed length of result */
 {
     register char *p = xmalloc(len);	/* where to store result */
 
-    DEBUG(DBG_FIELD_HI, "field was modified--build string for return\n");
-    DEBUG1(DBG_FIELD_HI, "field = %s\n", field_name);
+    DEBUG(DBG_FIELD_HI, "field was modified during parsing--build new string for return\n");
+    DEBUG1(DBG_FIELD_HI, "finish_mod_clean(): old field = %s\n", field_name);
+
     /* copy field name up to colon */
-    (void)memcpy(p, field_name, (size_t) name_len);
+    (void) memcpy(p, field_name, name_len);
 
     /*
      * copy space and text from queued tokens
      */
-    (void)detokenize(TRUE, p+name_len, tq_head, (struct token *)NULL);
+    (void) detokenize(TRUE, (p + name_len), tq_head, (struct token *) NULL);
 
-    DEBUG1(DBG_FIELD_HI, "completed string: %s\n", p);
+    DEBUG1(DBG_FIELD_HI, "completed new field string: %s\n", p);
 
     return p;
 }
@@ -844,7 +879,7 @@ insert_comma(s)
     register char *q;			/* temp pointer */
 
     /* put comma at beginning of white space or after last comment */
-    p = rindex(s, ')');
+    p = strrchr(s, ')');
     if (!p) {
 	p = s;
     } else {
@@ -917,7 +952,7 @@ match_route_or_group(tq_anchor, tq_new, extract_q, group,
     struct token **tq_new;		/* return last matched token here */
     struct addr **extract_q;		/* queue in which to put addresses */
     int group;				/* TRUE if group would be recursive */
-    unsigned *len;			/* len variable from process_field */
+    size_t *len;			/* len variable from process_field */
     char *domain;			/* domain to add to local addresses */
     char *uucp_host;			/* uucp host to prepend to ! routes */
     int flags;				/* miscellaneous flags */
@@ -1072,7 +1107,7 @@ static int
 match_group_term(tq_anchor, tq_new, extract_q, group, error)
     struct token *tq_anchor;		/* anchor point from process_field */
     struct token **tq_new;		/* return last matched token here */
-    struct addr **extract_q;		/* queue in which to add addresses */
+    struct addr **extract_q;		/* UNUSED queue in which to add addresses */
     int group;				/* TRUE if we are processing a group */
     char **error;			/* store error message here */
 {
@@ -1173,7 +1208,7 @@ match_general(tq_anchor, tq_new, len, extract_q, domain,
 	      uucp_host, flags, error)
     struct token *tq_anchor;		/* anchor point from process_field */
     struct token **tq_new;		/* return last matched token here */
-    unsigned *len;			/* len variable from process_field */
+    size_t *len;			/* len variable from process_field */
     struct addr **extract_q;		/* queue in which to add addresses */
     char *domain;			/* domain to add to local addresses */
     char *uucp_host;			/* uucp host to prepend to ! routes */
@@ -1213,11 +1248,11 @@ match_general(tq_anchor, tq_new, len, extract_q, domain,
 	break;
     }
 
-    while (!ENDTOK(tq->succ->form) && index("!%@", tq->succ->text[0]) &&
+    while (!ENDTOK(tq->succ->form) && strchr("!%@", tq->succ->text[0]) &&
 	   (WORDTOK(tq->succ->succ->form) ||
 	    tq->succ->succ->text[0] == '.'))
     {
-	switch(tq->succ->text[0]) {
+	switch (tq->succ->text[0]) {
 
 	case '!':			/* take first host in ! route */
 	    bang_route = TRUE;
@@ -1273,10 +1308,12 @@ match_general(tq_anchor, tq_new, len, extract_q, domain,
      * the domain WORD1, if necessary by appending a qualifier
      * to the domain.  len is updated to reflect length change.
      */
-    if (domain_address && (flags&F_LOCAL)) {
+    if (domain_address && (flags & F_LOCAL)) {
 	char *s = queue_qualify_domain(tq_mark, tq);
 
 	if (s) {
+	    DEBUG2(DBG_FIELD_HI, "match_general(): appending '.%s' to '%s'\n", s, (*tq_new)->text);
+	    
 	    /* append .s */
 	    (*tq_new)->text = xprintf("%s.%s", (*tq_new)->text, s);
 
@@ -1286,37 +1323,30 @@ match_general(tq_anchor, tq_new, len, extract_q, domain,
 	}
     }
 
-    if (!tq_mark && uucp_host && (bang_route || (flags&F_STRICT))) {
+    if (!tq_mark && uucp_host && bang_route) {
 	/*
 	 * we have a bang route, prepend uucp_host! to the route.
-	 * Also prepend uucp_host! to the route if we are doing
-	 * strict RFC822.  In this case an address will be
-	 * prepended with the route back to the sender and
-	 * appended with the current domain.
 	 */
+	DEBUG2(DBG_FIELD_HI, "match_general(): prepending '%s!' to '%s'\n", uucp_host, tq_anchor->text);
 	tq_anchor->text = xprintf("%s!%s", uucp_host, tq_anchor->text);
 	/* field length increased */
 	*len += 1 + strlen(uucp_host);
 	ret_val = T_MODIFIED;
-    }
-
-    /*
-     * if we want strict addresses and we don't have one, or if
-     * we have a local address but we don't want one (for a local
-     * message only), then append @domain
-     */
-
-    if (domain) {
+    } else if (domain) {
+	/*
+	 * if we want strict addresses and we don't have one, or if we have a
+	 * local address but we don't want one (for a local message only), then
+	 * append @domain
+	 */
 	if ((flags & F_STRICT && ! at_found) ||
 	    ((flags & F_LOCAL) &&
-	     ! (domain_address || bang_route || at_found)))
-	    {
-		(*tq_new)->text = xprintf("%s@%s", (*tq_new)->text, domain);
-
-		/* field length increased */
-		*len += 1 + strlen(domain);
-		ret_val = T_MODIFIED;
-	    }
+	     ! (domain_address || bang_route || at_found))) {
+	    DEBUG2(DBG_FIELD_HI, "match_general(): appending '@%s' to '%s'\n", domain, (*tq_new)->text);
+	    (*tq_new)->text = xprintf("%s@%s", (*tq_new)->text, domain);
+	    /* field length increased */
+	    *len += 1 + strlen(domain);
+	    ret_val = T_MODIFIED;
+	}
     }
 
     if (extract_q) {
@@ -1374,8 +1404,8 @@ queue_qualify_domain(tq_start, tq_end)
     STR_NEXT(sp, '\0');
 
     /* send out for the actual qualification */
-    ret = qualify_domain(sp->p);
-    DEBUG2(200, "qualify_domain(%s) returns %s\n", sp->p, ret? ret: "(null)");
+    ret = qualify_domain(STR(sp));
+    DEBUG2(DBG_FIELD_HI+1, "qualify_domain(%s) returns %s\n", STR(sp), ret ? ret: "(null)");
 
     STR_FREE(sp);		/* free region */
 
@@ -1413,7 +1443,7 @@ enqueue_address(q, tq_start, tq_end, error)
     char *parse_error;			/* error from parse_address() */
 
     /* grab the string corresponding to the tokens */
-    s = detokenize(FALSE, (char *)NULL, tq_start, tq_end);
+    s = detokenize(FALSE, (char *) NULL, tq_start, tq_end);
 
     DEBUG1(DBG_FIELD_LO, "enqueue_address(%s)\n", s);
     /* insert it into the queue */
@@ -1422,7 +1452,9 @@ enqueue_address(q, tq_start, tq_end, error)
     temp_q->in_addr = s;
     /* work_addr gets a mungeable copy */
     if ((temp_q->work_addr = preparse_address(s, &parse_error)) == NULL) {
-	*error = xprintf("%s: %s", s, error);
+	/* preparse_address() DEBUG()s will have explained error */
+	*error = xprintf("%s: %s", s, parse_error);
+	xfree((char *) temp_q);
 	return FAIL;
     }
     *q = temp_q;			/* insert at beginning of list */
@@ -1449,12 +1481,12 @@ void
 dump_tokens(tq)
     register struct token *tq;		/* dump these tokens on errfile */
 {
-    (void)fprintf(errfile, "token list:\n");
+    (void) fprintf(errfile, "token list:\n");
     while (tq) {
 	register char *s;
 	char buf[100+1];
 
-	switch(tq->form) {
+	switch (tq->form) {
 
 	case T_QUOTE:
 	    s = "T_QUOTE";
@@ -1475,14 +1507,15 @@ dump_tokens(tq)
 	    s = "T_ERROR";
 	    break;
 	default:
-	    (void)sprintf(s = buf, "form=%d", tq->form);
+	    (void) sprintf(buf, "form=%d", tq->form);
+	    s = buf;
 	    break;
 	}
-	(void)fprintf(errfile, "\t|%s|%s|%s|\n",
-		      tq->space, tq->text, s);
+	(void) fprintf(errfile, "\t|%s|%s|%s|\n",
+		       tq->space, tq->text, s);
 	tq = tq->succ;
     }
-    (void)fprintf(errfile, "end of list\n");
+    (void) fprintf(errfile, "end of list\n");
 }
 
 
@@ -1570,7 +1603,7 @@ main(argc, argv)
     if (argc > 0) {
 	while (*argv) {
 	    (void)fprintf(stderr, "input:  %s\n", *argv);
-	    s = index(*argv, ':');
+	    s = strchr(*argv, ':');
 	    if (s) {
 		s++;
 	    } else {
@@ -1602,7 +1635,7 @@ main(argc, argv)
 		line[len - 1] = '\0';
 	    }
 	    (void)fprintf(stderr, "input:  <%s>\n", line);
-	    s = index(line, ':');
+	    s = strchr(line, ':');
 	    if (s) {
 		s++;
 	    } else {
