@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:hash.c,v 1.17 2000/09/03 17:00:13 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:hash.c,v 1.25 2003/04/02 22:22:29 woods Exp"
  */
 
 /*
@@ -30,31 +30,55 @@
  *		write_hash_table, walk_hash, free_hash_element,
  *		free_hash_table
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
+#include "smailarch.h"
 #include "main.h"
 #include "addr.h"
 #include "exitcodes.h"
 #include "hash.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "log.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-#endif
+#include "extern.h"
+#include "debug.h"
+#include "smailport.h"
 
 /* static functions used in this file */
-static int hash_str __P((char *, int));
-static int hash_stric __P((char *, int));
-static struct hash *new_hash_element __P((char *, char *, int, struct hash_table *));
+static int hash_str __P((char *, unsigned int));
+static int hash_stric __P((char *, unsigned int));
+static struct hash *new_hash_element __P((char *, char *, size_t, struct hash_table *));
 
 #ifdef STANDALONE
 int debug = 0;	/* default debug level is NONE */
@@ -101,7 +125,7 @@ extern void free();				/* int on SunOS */
 static int
 hash_str(str, mod)
     register char *str;			/* the string to hash */
-    int mod;				/* prime modulus, size of hash table */
+    unsigned int mod;			/* prime modulus, size of hash table */
 {
     register unsigned long val;		/* current hash value */
     register unsigned long excess;	/* upper/excess bits in val */
@@ -122,14 +146,14 @@ hash_str(str, mod)
 	/* hash each character using the L_ values */
 	for (val = 0; (c = (unsigned long) *str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & (unsigned long) L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val & L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
 	for (val = 0; (c = (unsigned long) *str); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & (unsigned long) H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val & H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
@@ -165,8 +189,8 @@ hash_str(str, mod)
  */
 static int
 hash_stric(str, mod)
-    register char *str;			/* the string to hash disregarding case */
-    int mod;				/* prime modulus, size of hash table */
+    register char *str;		/* the string to hash disregarding case */
+    unsigned int mod;		/* prime modulus, size of hash table */
 {
     register unsigned long val;		/* current hash value */
     register unsigned long excess;	/* upper/excess bits in val */
@@ -185,16 +209,16 @@ hash_stric(str, mod)
      */
     if (mod < HASH_LEVEL) {
 	/* hash each character using the L_ values */
-	for (val = 0; (c = (unsigned long) lowercase((unsigned char) *str)); ++str) {
+	for (val = 0; (c = (unsigned long) tolower((int) *str)); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & (unsigned long) L_HASH_MASK)) >> L_DOWN_SHIFT);
+	    val ^= ((excess = (val & L_HASH_MASK)) >> L_DOWN_SHIFT);
 	    val ^= excess;
 	}
     } else {
 	/* hash each character using the H_ values */
-	for (val = 0; (c = (unsigned long) lowercase((unsigned char) *str)); ++str) {
+	for (val = 0; (c = (unsigned long) tolower((int) *str)); ++str) {
 	    val = (val << HASH_UP_SHIFT) + c;
-	    val ^= ((excess = (val & (unsigned long) H_HASH_MASK)) >> H_DOWN_SHIFT);
+	    val ^= ((excess = (val & H_HASH_MASK)) >> H_DOWN_SHIFT);
 	    val ^= excess;
 	}
     }
@@ -226,8 +250,8 @@ walk_hash(cur, table)
     struct hash_table *table;	/* hash table being walked */
 {
     register struct hash *prev;	/* our previous walk location */
-    register int indx;		/* our previous hash slot */
-    register int len;		/* the table length */
+    register unsigned int indx;	/* our previous hash slot */
+    register unsigned int len;	/* the table length */
 
     /*
      * firewall
@@ -326,11 +350,11 @@ static struct hash *
 new_hash_element(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to be added */
     char *data;				/* the associated data if any */
-    int datalen;			/* length of data,  0 ==> no data */
+    size_t datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* hash table being added to */
 {
     struct hash *new;		/* the new slot chain location */
-    int keylen;			/* the length of the string, padded */
+    size_t keylen;		/* the length of the string, padded */
     unsigned long lk;		/* temp var for key length */
 
     /*
@@ -343,19 +367,19 @@ new_hash_element(keystr, data, datalen, table)
     if (lk >= (unsigned long) (1L << BITS_PER_SHORT)) {
 	panic(EX_SOFTWARE, "new_hash_element: key too long");
     }
-    keylen = (int)lk;		/* now we know it will fit in an int */
+    keylen = (size_t) lk;		/* now we know it will fit in an int */
     /* firewall - check against bad data being passed to us */
-    if (datalen < 0 || (datalen > 0 && data == NULL) || 
-      (unsigned long) datalen >= (unsigned long) (1L << BITS_PER_SHORT))  {
+    if ((datalen > 0 && data == NULL) || 
+	datalen >= (unsigned long) (1L << BITS_PER_SHORT))  {
 	panic(EX_SOFTWARE,
-	    "new_hash_element: bad data passed with: <%s>  datalen: <%d>",
+	    "new_hash_element: bad data passed with: <%s>  datalen: <%lu>",
 	    keystr, datalen);
     }
 
     /*
      * malloc the storage
      */
-    new = (struct hash *) bmalloc((unsigned int) hashslot_size(keylen, datalen), table->life);
+    new = (struct hash *) bmalloc((size_t) hashslot_size(keylen, datalen), table->life);
     /* firewall */
     if (is_odd(new)) {
 	panic(EX_SOFTWARE,
@@ -402,7 +426,7 @@ new_hash_element(keystr, data, datalen, table)
 void
 free_hash_element(cur, search, table)
     struct hash *cur;		/* what we will delete */
-    int search;			/* non-zero => delete `cur' first */
+    unsigned int search;	/* non-zero => delete `cur' first */
     struct hash_table *table;	/* table `cur' does/would_have belonged to */
 {
     /*
@@ -443,26 +467,23 @@ free_hash_element(cur, search, table)
  */
 struct hash_table *
 new_hash_table(tablelen, life, flags)
-    int tablelen;		/* number of slots in the hash table */
+    unsigned int tablelen;	/* number of slots in the hash table */
     struct block *life;		/* is the hash table permanent or temporary */
     int flags;			/* hash table flag as per struct hash_table */
 {
-    register int i;			/* index */
+    register unsigned int i;		/* index */
     struct hash_table *table;		/* the malloced hash table */
 
     /*
      * firewalls
      */
-    if (tablelen <= 0) {
-	panic(EX_SOFTWARE, "new_hash_table: tablelen: %d", tablelen);
-    }
     DEBUG3(DBG_HASH_LO,
-	   "new_hash_table: tablelen: %d life: 0x%lx flags: 0x%x\n", tablelen, (long)life, flags);
+	   "new_hash_table: tablelen: %u life: 0x%lx flags: 0x%x\n", tablelen, (long)life, flags);
 
     /*
      * malloc the hash table
      */
-    table = (struct hash_table *)bmalloc(table_size(tablelen), life);
+    table = (struct hash_table *) bmalloc(table_size(tablelen), life);
 
     /* firewall */
     if (is_odd(table)) {
@@ -536,7 +557,7 @@ int
 add_to_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to be added */
     char *data;				/* the associated data if any */
-    int datalen;			/* length of data,  0 ==> no data */
+    size_t datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -633,7 +654,7 @@ struct hash *
 replace_in_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to replace */
     char *data;				/* the associated data if any */
-    int datalen;			/* length of data,  0 ==> no data */
+    size_t datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -717,7 +738,7 @@ struct hash *
 store_in_hash(keystr, data, datalen, table)
     char *keystr;    			/* the keystring to replace */
     char *data;				/* the associated data if any */
-    int datalen;			/* length of data,  0 ==> no data */
+    size_t datalen;			/* length of data,  0 ==> no data */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;		/* the current slot chain location */
@@ -822,12 +843,12 @@ int
 lookup_in_hash(keystr, data, datalen, table)
     char *keystr;			/* the key to lookup */
     char **data;			/* where to point at data, or NULL */
-    int *datalen;			/* where to place data len or NULL */
+    size_t *datalen;			/* where to place data len or NULL */
     struct hash_table *table;		/* the hash table to add it to */
 {
     register struct hash *cur;	/* the slot chain location */
-    int loc;			/* the hash slot to add onto */
-    int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
+    unsigned int loc;		/* the hash slot to add onto */
+    unsigned int caseflag;	/* 0 ==> use strcmp, 1 ==> strcmpic */
     int cmp;			/* compare function result */
 
     /*
@@ -899,9 +920,9 @@ delete_from_hash(keystr, table)
 {
     register struct hash *cur;		/* the slot chain location */
     register struct hash *prev;		/* previous element */
-    int loc;			/* the hash slot to add onto */
-    int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
-    int cmp;			/* compare function result */
+    unsigned int loc;			/* the hash slot to add onto */
+    unsigned int caseflag;		/* 0 ==> use strcmp, 1 ==> strcmpic */
+    int cmp;				/* compare function result */
 
     /*
      * firewall - watch for NULLs
@@ -970,11 +991,11 @@ write_hash_table(table, stream)
     struct hash_table *table;	/* the table to write */
     FILE *stream;		/* the stream on which to write table */
 {
-    register int i;		/* index */
-    long tab_loc;		/* file location of hash_table start */
+    register size_t i;		/* index */
+    off_t tab_loc;		/* file location of hash_table start */
     long loc;			/* current location */
     int size;			/* size of the current element */
-    long offset;                /* current offset within the file */
+    off_t offset;		/* current offset within the file */
     long disc_succ;             /* cur->succ as written to disc */
     struct hash *cur;		/* the current hash element */
     struct hash_table *tab;	/* disk copy of table */
@@ -1005,7 +1026,7 @@ write_hash_table(table, stream)
      * skip the hash table block
      */
     tab_loc = ftell(stream);
-    if (fseek(stream, (long)table_size(table->len), 1) < 0) {
+    if (fseek(stream, (long int) table_size(table->len), 1) < 0) {
 	panic(EX_IOERR, "write_hash_table: bad skip of %d over table",
 	      table_size(table->len));
     }
@@ -1018,7 +1039,7 @@ write_hash_table(table, stream)
 	/* don't write out chains that don't exist */
 	if (empty_slot(table->slot[i])) {
 		/* slot is empty, deal with it quickly */
-		set_ptr(tab->slot[i], (int)NULL);
+		set_ptr(tab->slot[i], (int) NULL);
 		continue;
 	}
 
@@ -1042,14 +1063,14 @@ write_hash_table(table, stream)
 	    offset += size;		/* note file movement */
 	    /* write the hash element disk successor element */
 	    disc_succ = (cur->succ == NULL) ? 0 : to_odd(offset);
-	    if (fwrite((char *)&disc_succ, sizeof(disc_succ), 1, stream) != 1) {
+	    if (fwrite((char *)&disc_succ, sizeof(disc_succ), (size_t) 1, stream) != 1) {
 		panic(EX_IOERR,
 		      "write_hash_table: bad succ write <%s> on slot %d",
 		      cur->keystr, i);
 	    }
 	    /* write the rest of the hash element data */
 	    if (fwrite((char *)cur+sizeof(cur->succ), size-sizeof(cur->succ),
-		       1, stream) != 1) {
+		       (size_t) 1, stream) != 1) {
 		panic(EX_IOERR,
 		      "write_hash_table: bad write <%s> on slot %d",
 		      cur->keystr, i);
@@ -1062,13 +1083,13 @@ write_hash_table(table, stream)
      * current position
      */
     loc = ftell(stream);	/* remember our current location */
-    if (fseek(stream, (long)tab_loc, 0) < 0) {
+    if (fseek(stream, (long int) tab_loc, 0) < 0) {
 	panic(EX_IOERR, "write_hash_table: bad skip back to table at 0x%lx", (long)tab_loc);
     }
-    if (fwrite((char *)tab, table_size(tab->len), 1, stream) != 1) {
+    if (fwrite((char *)tab, table_size(tab->len), (size_t) 1, stream) != 1) {
 	panic(EX_IOERR, "write_hash_table: bad table write");
     }
-    if (fseek(stream, (long)loc, 0) < 0) {
+    if (fseek(stream, (long int) loc, 0) < 0) {
 	panic(EX_IOERR, "write_hash_table: bad end seek to 0x%lx", (long)loc);
     }
 
@@ -1086,6 +1107,7 @@ write_hash_table(table, stream)
 #include <sys/stat.h>
 #include <limits.h>
 #include "dys.h"
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -1252,13 +1274,13 @@ test(buf, table)
 {
     register struct hash *cur;	/* the current hash entry */
     char *data;			/* the data we stored */
-    int datalen;		/* length of data */
-    int buflen;			/* length of the buffer string + NULL */
+    size_t datalen;		/* length of data */
+    size_t buflen;		/* length of the buffer string + NULL */
     int i;			/* index */
     register int caseflag;	/* 0 ==> use strcmp, 1 ==> strcmpic */
 
     /* test adding */
-    buflen = strlen(buf)+1;
+    buflen = strlen(buf) + 1;
     i = add_to_hash(buf, buf, buflen, table);
     DEBUG2(DBG_HASH_LO, "test: <%s> add: %d\n", buf, i);
 
