@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:hash.h,v 1.9 2000/09/11 19:11:36 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:hash.h,v 1.10 2002/10/01 21:44:44 woods Exp"
  */
 
 /*
@@ -34,14 +34,14 @@
  * for hash sizes where the hash mod < HASH_LEVEL
  */
 #define L_MASK_SHIFT	(HASH_LEVEL_SHIFT)
-#define L_HASH_MASK	(~((1 << L_MASK_SHIFT) - 1)) /* mask of excess bits */
+#define L_HASH_MASK	(~(((unsigned long) 1 << L_MASK_SHIFT) - 1)) /* mask of excess bits */
 #define L_DOWN_SHIFT	(L_MASK_SHIFT - 1)	/* shift down excess */
 
 /*
  * for hash sizes where the hash mod >= HASH_LEVEL
  */
 #define H_MASK_SHIFT	(BITS_PER_LONG - HASH_UP_SHIFT - 2)
-#define H_HASH_MASK	(~((1 << H_MASK_SHIFT) - 1)) /* mask of excess bits */
+#define H_HASH_MASK	(~(((unsigned long) 1 << H_MASK_SHIFT) - 1)) /* mask of excess bits */
 #define H_DOWN_SHIFT	(H_MASK_SHIFT - 2)	/* shift down excess */
 
 /*
@@ -63,7 +63,7 @@
  * slot pointers.  A slot that does not have a chain has the value NULL.
  */
 struct hash_table {
-    int len;		/* the number of hash slots in this table */
+    unsigned int len;	/* the number of hash slots in this table */
     int flags;		/* see flags section, default == 0 */
     int indx;		/* the walk_hash() slot location */
     struct hash *prev;	/* the walk_hash() current element location */
@@ -303,13 +303,13 @@ struct hash {
 #define next_hash(cur, table) hash_addr(cur->succ, table)
 
 /* external functions defined in hash.c */
-extern int add_to_hash __P((char *, char *, int, struct hash_table *));
-extern int lookup_in_hash __P((char *, char **, int *, struct hash_table *));
-extern struct hash_table *new_hash_table __P((int, struct block *, int));
+extern int add_to_hash __P((char *, char *, size_t, struct hash_table *));
+extern int lookup_in_hash __P((char *, char **, size_t *, struct hash_table *));
+extern struct hash_table *new_hash_table __P((unsigned int, struct block *, int));
 extern void write_hash_table __P((struct hash_table *, FILE *));
 extern struct hash *delete_from_hash __P((char *, struct hash_table *));
-extern struct hash *replace_in_hash __P((char *, char *, int, struct hash_table *));
-extern struct hash *store_in_hash __P((char *, char *, int, struct hash_table *));
+extern struct hash *replace_in_hash __P((char *, char *, size_t, struct hash_table *));
+extern struct hash *store_in_hash __P((char *, char *, size_t, struct hash_table *));
 extern struct hash *walk_hash __P((struct hash *, struct hash_table *));
-extern void free_hash_element __P((struct hash *, int, struct hash_table *));
+extern void free_hash_element __P((struct hash *, unsigned int, struct hash_table *));
 extern void free_hash_table __P((struct hash_table *));
