@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:header.c,v 1.34 2000/09/03 17:00:13 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:header.c,v 1.43 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -17,29 +17,60 @@
  *	external functions: process_header, read_header, write_header
  */
 
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <pwd.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "field.h"
-#include "transport.h"
 #include "main.h"
 #include "spool.h"
 #include "log.h"
 #include "exitcodes.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
+#include "transport.h"
+#include "header.h"
 #include "extern.h"
 #include "debug.h"
+#include "smailport.h"
 
 /* variables defined in this file */
-struct list *header;			/* list of header fields */
+charplist_t *header;			/* list of header fields */
 
 /* variables local to this file */
 static int use_resent;			/* TRUE if Resent- fields exist */
@@ -61,8 +92,8 @@ static int found_date;			/* TRUE if Date: field was found */
 static int found_sender;		/* TRUE if Sender: field was found */
 static int count_received;		/* count of Received: fields found */
 static struct addr *header_from;	/* list of From: addrs */
-static struct list *remote_header;	/* simple remote-form header */
-static struct list *strict_header;	/* strict RFC822 header */
+static charplist_t *remote_header;	/* simple remote-form header */
+static charplist_t *strict_header;	/* strict RFC822 header */
 
 /* functions local to this file */
 static void check_for_resent __P((void));
@@ -167,7 +198,6 @@ process_header(rq)
 	     */
 	    switch (form) {
 	    case FAIL:
-	    case PARSE_ERROR:
 		dobuild = TRUE;
 		break;
 
@@ -188,17 +218,19 @@ process_header(rq)
 		/*FALLTHROUGH*/
 
 	    case LOCAL:
-		pw = getpwbyname(remain);	/* WARNING: may need
-						 * lowercase(), but
-						 * getpwbyname() does lowercase
-						 * before calling getpwnam().
+		pw = getpwbyname(remain);	/* WARNING: may need tolower(),
+						 * but getpwbyname() does
+						 * lowercaseing before calling
+						 * getpwnam().
 						 */
-		if (pw == NULL || pw->pw_uid != real_uid)
+		if (pw == NULL || pw->pw_uid != real_uid) {
 		    dobuild = TRUE;
+		}
 		break;
 	    }
-	    if (dobuild)
+	    if (dobuild) {
 		new_field(build_from_field(sender_fn));
+	    }
 	    xfree(work_addr);
 	}
     }
@@ -234,7 +266,7 @@ process_header(rq)
 static void
 check_for_resent()
 {
-    register struct list *q;		/* index to headers */
+    register charplist_t *q;		/* index to headers */
 
     /* on the first pass check for Resent- headers */
     for (q = header; q; q = q->succ) {
@@ -292,7 +324,7 @@ scan_header(rq, error)
     struct addr **rq;			/* put recipients here, if non-NULL */
     char **error;			/* store error message here */
 {
-    register struct list *q;		/* temp for scanning header */
+    register charplist_t *q;		/* temp for scanning header */
 
     for (q = header; q; q = q->succ) {
 	if (hop_count < 0 && HDREQ("Received", q->text)) {
@@ -300,105 +332,86 @@ scan_header(rq, error)
 	} else if (HDREQ(to_fn, q->text)) {
 	    found_to = TRUE;
 	    if (islocal) {
-		q->text = process_field(q->text,
-					index(q->text,':')+1,
-					(char *)NULL,
-					(char *)NULL,
-					rq,
-					islocal?F_LOCAL:0,
-					error);
+		q->text = process_field(q->text, strchr(q->text, ':') + 1,
+					(char *) NULL, (char *) NULL,
+					rq, (islocal ? F_LOCAL : 0), error);
 	    } else if (rq) {
-		(void) process_field((char *)NULL, index(q->text, ':') + 1,
-				     (char *)NULL, (char *)NULL, rq,
-				     0, error);
+		(void) process_field((char *) NULL, strchr(q->text, ':') + 1,
+				     (char *) NULL, (char *) NULL,
+				     rq, 0, error);
 	    }
 	} else if (HDREQ("apparently-to", q->text) ||
 		   HDREQ("to", q->text) ||
-		   HDREQ("resent-to", q->text))
-	{
+		   HDREQ("resent-to", q->text))	{
 	    found_to = TRUE;
 	} else if (HDREQ(message_id_fn, q->text)) {
 	    found_message_id = TRUE;
 	} else if (HDREQ(from_fn, q->text)) {
 	    found_from = TRUE;
 	    if (islocal) {
-		q->text = process_field(q->text,
-					index(q->text, ':')+1,
-					(char *)NULL,
-					(char *)NULL,
-					&header_from,
-					islocal?F_LOCAL:0,
-					error);
+		q->text = process_field(q->text, strchr(q->text, ':') + 1,
+					(char *) NULL, (char *) NULL,
+					&header_from, (islocal ? F_LOCAL : 0), error);
 	    } else {
-		(void) process_field((char *)NULL, index(q->text, ':') + 1,
-				     (char *)NULL, (char *)NULL,
+		(void) process_field((char *) NULL, strchr(q->text, ':') + 1,
+				     (char *) NULL, (char *) NULL,
 				     &header_from, 0, error);
 	    }
 	} else if (HDREQ(date_fn, q->text)) {
 	    found_date = TRUE;
 	} else if (HDREQ(sender_fn, q->text)) {
 	    if (!sender_is_trusted) {
-		struct list **pq;
+		charplist_t **pq;
 		/*
 		 * Non-trusted users can't supply their own Sender: fields,
 		 * so delete this field.
 		 */
-		for (pq = &header; *pq && *pq != q; pq = &(*pq)->succ) ;
+		for (pq = &header; *pq && *pq != q; pq = &(*pq)->succ) {
+		    ;
+		}
 		if (*pq) {
 		    *pq = q->succ;
 		}
 	    } else {
 		found_sender = TRUE;
 		if (islocal) {
-		    q->text = process_field(q->text,
-					    index(q->text, ':')+1,
-					    (char *)NULL,
-					    (char *)NULL,
-					    &header_from,
-					    islocal?F_LOCAL:0,
-					    error);
+		    q->text = process_field(q->text, strchr(q->text, ':') + 1,
+					    (char *) NULL, (char *) NULL,
+					    &header_from, (islocal ? F_LOCAL : 0), error);
 		} else {
-		    (void) process_field((char *)NULL, index(q->text, ':') + 1,
-					 (char *)NULL, (char *)NULL,
+		    (void) process_field((char *) NULL, strchr(q->text, ':') + 1,
+					 (char *) NULL, (char *) NULL,
 					 &header_from, 0, error);
 		}
 	    }
 	} else if (HDREQ(cc_fn, q->text)) {
 	    found_cc = TRUE;
 	    if (islocal) {
-		q->text = process_field(q->text,
-					index(q->text, ':') + 1,
-					(char *)NULL,
-					(char *)NULL,
-					rq,
-					islocal?F_LOCAL:0,
-					error);
+		q->text = process_field(q->text, strchr(q->text, ':') + 1,
+					(char *) NULL, (char *) NULL,
+					rq, (islocal ? F_LOCAL : 0), error);
 	    } else if (rq) {
-		(void) process_field((char *)NULL, index(q->text, ':') + 1,
-				     (char *)NULL, (char *)NULL, rq,
-				     0, error);
+		(void) process_field((char *) NULL, strchr(q->text, ':') + 1,
+				     (char *) NULL, (char *) NULL,
+				     rq, 0, error);
 	    }
 	} else if (HDREQ(bcc_fn, q->text)) {
 	    found_bcc = TRUE;
 	    if (islocal) {
-		q->text = process_field(q->text,
-					index(q->text, ':') + 1,
-					(char *)NULL,
-					(char *)NULL,
-					rq,
-					islocal?F_LOCAL:0,
-					error);
+		q->text = process_field(q->text, strchr(q->text, ':') + 1,
+					(char *) NULL, (char *) NULL,
+					rq, (islocal ? F_LOCAL : 0), error);
 	    } else if (rq) {
-		(void) process_field((char *)NULL, index(q->text, ':') + 1,
-				     (char *)NULL, (char *)NULL, rq,
-				     0, error);
+		(void) process_field((char *) NULL, strchr(q->text, ':') + 1,
+				     (char *) NULL, (char *) NULL,
+				     rq, 0, error);
 	    }
 	} else if (HDREQ("precedence", q->text)) {
 	    /*
 	     * a precedence field is specified, use it to set the message
 	     * priority
 	     */
-	    int newgrd = parse_precedence(index(q->text, ':'));
+	    int newgrd = parse_precedence(strchr(q->text, ':'));
 
 	    if (newgrd) {
 		msg_grade = newgrd;
@@ -503,9 +516,8 @@ build_date_field()
     register char *s;			/* returned header field */
 
     if (date_field &&
-	(s = expand_string(date_field, (struct addr *)NULL,
-			   (char *)NULL, (char *)NULL)))
-    {
+	(s = expand_string(date_field, (struct addr *) NULL,
+			   (char *) NULL, (char *) NULL))) {
 	if (use_resent) {
 	    return xprintf("Resent-%s", s);
 	} else {
@@ -524,9 +536,8 @@ build_message_id_field()
     register char *s;			/* returned header field */
 
     if (message_id_field &&
-	(s = expand_string(message_id_field, (struct addr *)NULL,
-			   (char *)NULL, (char *)NULL)))
-    {
+	(s = expand_string(message_id_field, (struct addr *) NULL,
+			   (char *) NULL, (char *) NULL))) {
 	if (use_resent) {
 	    return xprintf("Resent-%s", s);
 	} else {
@@ -545,12 +556,15 @@ build_from_field(fn)
 {
     char *s = NULL;
 
-    if (from_field)
-	s = expand_string(from_field, (struct addr *)0, (char *)0, sender);
-    if (s == NULL || *s == '\0')
+    if (from_field) {
+	s = expand_string(from_field, (struct addr *) NULL, (char *) NULL, sender);
+    }
+    if (s == NULL || *s == '\0') {
 	return "";
-    if (! HDREQ("from", s))
+    }
+    if (! HDREQ("from", s)) {
 	return COPY_STRING(s);
+    }
     return xprintf("%s%s", fn, s + 4);
 }
 
@@ -561,9 +575,9 @@ static void
 new_field(field)
     char *field;			/* new field to add */
 {
-    register struct list *f;		/* header field entry */
+    register charplist_t *f;		/* header field entry */
 
-    f = (struct list *)xmalloc(sizeof(struct list)); /* create it */
+    f = (charplist_t *) xmalloc(sizeof(charplist_t)); /* create it */
     f->text = field;
     f->succ = header;			/* link it in */
     header = f;
@@ -580,7 +594,7 @@ int
 parse_precedence(field)
     char *field;
 {
-    unsigned int len;			/* length of token */
+    size_t len;				/* length of token */
     struct token *tok;			/* list of tokens from tokenize */
     register char *p = grades;		/* temp for scanning grades */
     char *error;			/* error message from tokenize */
@@ -603,10 +617,10 @@ parse_precedence(field)
 	    return p[len+1];
 	}
 	/* skip to end of precedence string */
-	p = index(p, ':');
+	p = strchr(p, ':');
 	if (p) {
 	    /* skip past grade character */
-	    p = index(p, ':');
+	    p = strchr(p, ':');
 	    if (p) {
 		p++;			/* now at next precedence string */
 	    }
@@ -632,7 +646,7 @@ read_header()
     struct str field;			/* input field */
     register int c;			/* current input char */
     int last_c = EOF;			/* last input char */
-    struct list **link_addr;		/* forward link for next field */
+    charplist_t **link_addr;		/* forward link for next field */
     off_t line_mark = 0;		/* marks beginning of a line */
 
     link_addr = &header;
@@ -661,11 +675,11 @@ read_header()
 		 */
 		line_mark = tell_spool();
 	    }
-	    while ((c = GETSPOOL()) != EOF && c != READ_FAIL && !isspace(c) && c != ':') {
+	    while ((c = GETSPOOL()) != EOF && c != READ_FAIL && !isspace((int) c) && c != ':') {
 		STR_NEXT(&field, c);
 		last_c = c;
 	    }
-	    while (c != EOF && c != READ_FAIL && isspace(c)) {
+	    while (c != EOF && c != READ_FAIL && isspace((int) c)) {
 		    STR_NEXT(&field, c);
 		    c = GETSPOOL();
 	    }
@@ -704,7 +718,7 @@ read_header()
 
 	STR_NEXT(&field, '\0');
 	STR_DONE(&field);
-	*link_addr = (struct list *) xmalloc(sizeof(struct list));
+	*link_addr = (charplist_t *) xmalloc(sizeof(charplist_t));
 	(*link_addr)->text = STR(&field);
 	link_addr = &(*link_addr)->succ;
 	/*
@@ -752,15 +766,16 @@ write_header(f, addr)
     struct addr *addr;			/* transport */
 {
     long flags = addr->transport->flags; /* transport flags */
-    struct list *q, *q2;		/* temp for processing header */
+    charplist_t *q, *q2;		/* temp for processing header */
     int crlf = (flags & PUT_CRLF)?1:0;	/* put \r\n at end of each line? */
     char *s;
 
-    /* output transport-specific insertion headers */
+    /* output transport-specific "insert_header" headers */
     for (q2 = addr->transport->hdrinsert; q2; q2 = q2->succ) {
 	if (write_field(expand_string(q2->text, addr,
-			(char *)NULL, (char *)NULL), f, crlf) == FAIL)
+				      (char *) NULL, (char *) NULL), f, crlf) == FAIL) {
 	    return FAIL;
+	}
     }
 
     /* output a Return-Path: header if required */
@@ -827,22 +842,24 @@ write_header(f, addr)
 	if ((flags & PUT_RETURNPATH) && HDREQ("Return-Path", q->text)) {
 	    continue;
 	}
-	/* output transport-specific insertion headers */
+	/* skip over any transport-specific "remove_header" headers */
 	for (q2 = addr->transport->hdrremove; q2; q2 = q2->succ) {
-	    s = expand_string(q2->text, addr, (char *)NULL, (char *)NULL);
-	    if (s && HDREQ(s, q->text))
+	    s = expand_string(q2->text, addr, (char *) NULL, (char *) NULL);
+	    if (s && HDREQ(s, q->text)) {
 		break;
+	    }
 	}
 	if (q2 == NULL && write_field(q->text, f, crlf) == FAIL) {
 	    return FAIL;
 	}
     }
 
-    /* output transport-specific insertion headers */
+    /* output transport-specific "append_header" headers */
     for (q2 = addr->transport->hdrappend; q2; q2 = q2->succ) {
 	if (write_field(expand_string(q2->text, addr,
-			(char *)NULL, (char *)NULL), f, crlf) == FAIL)
+				      (char *) NULL, (char *) NULL), f, crlf) == FAIL) {
 	    return FAIL;
+	}
     }
 
     DEBUG(DBG_HEADER_LO, "\n");
@@ -899,7 +916,7 @@ build_received_field(addr)
     register char *s;			/* returned header field */
 
     if (received_field &&
-	(s = expand_string(received_field, addr, (char *)NULL, (char *)NULL)))
+	(s = expand_string(received_field, addr, (char *) NULL, (char *) NULL)))
     {
 	return s;
     }
@@ -917,8 +934,7 @@ build_return_path(addr)
 
     if (return_path_field &&
 	(s = expand_string(return_path_field, addr,
-			   (char *)NULL, (char *)NULL)))
-    {
+			   (char *) NULL, (char *) NULL))) {
 	return s;
     }
     return "";
@@ -935,8 +951,8 @@ build_return_path(addr)
 static void
 build_remote_header()
 {
-    struct list **rhq = &remote_header; /* where to put next field */
-    struct list *hq;			/* temp for scanning header */
+    charplist_t **rhq = &remote_header; /* where to put next field */
+    charplist_t *hq;			/* temp for scanning header */
     char *error;			/* (ignored) error message */
 
     /*
@@ -944,7 +960,7 @@ build_remote_header()
      * sender and recipient fields as appropriate.
      */
     for (hq = header; hq; hq = hq->succ) {
-	*rhq = (struct list *)xmalloc(sizeof(**rhq));
+	*rhq = (charplist_t *) xmalloc(sizeof(**rhq));
 	(*rhq)->succ = NULL;
 	if (HDREQ(from_fn, hq->text) || HDREQ(sender_fn, hq->text) || HDREQ(reply_to_fn, hq->text)) {
 	    /*
@@ -954,32 +970,30 @@ build_remote_header()
 	     *    qualify known domain abbreviations, if local
 	     */
 	    (*rhq)->text = process_field(hq->text,
-					 index(hq->text, ':') + 1,
+					 strchr(hq->text, ':') + 1,
 					 visible_name,
 					 uucp_name,
-					 (struct addr **)NULL,
-					 (islocal? F_LOCAL: 0),
+					 (struct addr **) NULL,
+					 (islocal ? F_LOCAL : 0),
 					 &error);
 	} else if (islocal && (HDREQ(to_fn, hq->text) ||
 			       HDREQ(cc_fn, hq->text) ||
-			       HDREQ(bcc_fn, hq->text)))
-	{
+			       HDREQ(bcc_fn, hq->text))) {
 	    /*
 	     * locally generated field containing recipient addresses:
 	     *    append @visible_name after local-form addresses
 	     *    qualify known domain abbreviations
 	     */
 	    (*rhq)->text = process_field(hq->text,
-					 index(hq->text, ':') + 1,
+					 strchr(hq->text, ':') + 1,
 					 visible_name,
-					 (char *)NULL,
-					 (struct addr **)NULL,
+					 (char *) NULL,
+					 (struct addr **) NULL,
 					 F_LOCAL,
 					 &error);
 	} else {
 	    (*rhq)->text = hq->text;
 	}
-
 	rhq = &(*rhq)->succ;
     }
 }
@@ -997,8 +1011,8 @@ static void
 build_strict_header()
 {
     char *error;			/* error messages */
-    struct list **shq = &strict_header; /* where to put next field */
-    struct list *hq;			/* temp for scanning header */
+    charplist_t **shq = &strict_header; /* where to put next field */
+    charplist_t *hq;			/* temp for scanning header */
     char *sender_path;			/* path to sender's machine */
 
     /*
@@ -1024,7 +1038,7 @@ build_strict_header()
 	p = back_address_token(sender_path, p);
 	if (p == NULL) {
 	    write_log(WRITE_LOG_MLOG,
-		 "error building path to sender <%s>: unexpected end of token",
+		      "error building path to sender <%s>: unexpected end of token",
 		      sender);
 	    sender_path = NULL;
 	} else if (p == sender_path) {
@@ -1039,24 +1053,26 @@ build_strict_header()
      * the necessary transformations on sender and recipient fields
      */
     for (hq = header; hq; hq = hq->succ) {
-	*shq = (struct list *)xmalloc(sizeof(**shq));
+	*shq = (charplist_t *) xmalloc(sizeof(**shq));
 	(*shq)->succ = NULL;
-	if (HDREQ(from_fn, hq->text) || HDREQ(sender_fn, hq->text) || HDREQ(reply_to_fn, hq->text)) {
+	if (HDREQ(from_fn, hq->text) ||
+	    HDREQ(sender_fn, hq->text) ||
+	    HDREQ(reply_to_fn, hq->text)) {
 	    /*
 	     * field containing a sender address:
 	     *    append @visiblename after local-form addresses
 	     *    qualify known domain abbreviations, if local
 	     */
 	    (*shq)->text = process_field(hq->text,
-					 index(hq->text, ':') + 1,
+					 strchr(hq->text, ':') + 1,
 					 visible_name,
-					 (char *)NULL,
-					 (struct addr **)NULL,
-					 (islocal? F_LOCAL: 0)|F_STRICT,
+					 (char *) NULL,
+					 (struct addr **) NULL,
+					 (islocal ? F_LOCAL : 0) | F_STRICT,
 					 &error);
-	} else if (HDREQ(to_fn, hq->text) || HDREQ(cc_fn, hq->text) ||
-		   HDREQ(bcc_fn, hq->text))
-	{
+	} else if (HDREQ(to_fn, hq->text) ||
+		   HDREQ(cc_fn, hq->text) ||
+		   HDREQ(bcc_fn, hq->text)) {
 	    /*
 	     * field containing a recipient address:
 	     *    append @visiblename after !-route, local-form or u%d form
@@ -1064,11 +1080,11 @@ build_strict_header()
 	     *    qualify known domain abbreviations, if local
 	     */
 	    (*shq)->text = process_field(hq->text,
-					 index(hq->text, ':') + 1,
+					 strchr(hq->text, ':') + 1,
 					 visible_name,
-					 EQIC(sender_path,visible_name) ? NULL : sender_path,
-					 (struct addr **)NULL,
-					 (islocal? F_LOCAL: 0)|F_STRICT,
+					 EQIC(sender_path,visible_name) ? (char *) NULL : sender_path,
+					 (struct addr **) NULL,
+					 (islocal ? F_LOCAL : 0) | F_STRICT,
 					 &error);
 	} else {
 	    (*shq)->text = hq->text;
