@@ -1,4 +1,6 @@
-/* @(#) jump.h,v 1.2 1992/09/20 18:45:12 tron Exp -  */
+/*
+#ident "@(#)smail/src:RELEASE-3_2_0_115:jump.h,v 1.4 2002/10/01 21:44:43 woods Exp"
+ */
 
 /*
  *    Copyright (C) 1992  Chip Salzenberg
@@ -9,9 +11,6 @@
 #ifndef JUMP_H
 #define JUMP_H
 
-#include <setjmp.h>
-#include <signal.h>
-
 #if defined(POSIX_OS) && !defined(NO_SIGSETJMP)
 
 typedef struct {
@@ -42,7 +41,7 @@ typedef struct {
 #else /* !POSIX_OS || NO_SIGSETJMP */
 
 typedef struct {
-    void (*func)();
+    void (*func) __P((void));
 } JUMPSIG;
 
 #define JUMP_SETSIG(SIG,FUNC,JS)	{ (JS)->func = signal(SIG,FUNC); }
