@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:log.c,v 1.42 2001/01/18 07:34:14 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:log.c,v 1.57 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -22,39 +22,42 @@
  *			     open_msg_log, close_msg_log, unlink_msg_log,
  *			     panic, write_log, send_log, scan_msg_log
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include "defs.h"
-#ifdef ANSI_C
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
 # include <stdarg.h>
 #else
 # include <varargs.h>
 #endif
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
 
-#include "smail.h"
-#include "main.h"
-#include "addr.h"
-#include "alloc.h"
-#include "dys.h"
-#include "log.h"
-#include "spool.h"
-#include "exitcodes.h"
-#ifndef DEPEND
-# include "debug.h"
-# include "extern.h"
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
 #endif
 
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
@@ -64,6 +67,22 @@
 #  include <sys/file.h>
 # endif
 #endif
+#ifdef __GLIBC__
+# include <sys/file.h>
+#endif
+
+#include "smail.h"
+#include "main.h"
+#include "addr.h"
+#include "alloc.h"
+#include "smailstring.h"
+#include "dys.h"
+#include "log.h"
+#include "spool.h"
+#include "exitcodes.h"
+#include "debug.h"
+#include "extern.h"
+#include "smailport.h"
 
 #ifdef STANDALONE
 # define xmalloc malloc
@@ -71,27 +90,17 @@
 # define xfree free
 #endif	/* STANDALONE */
 
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
-
 /* exported variables */
-FILE *msg_logfile = NULL;		/* open stream to per-message log */
+FILE *msg_logfile = NULL;	/* open stream to per-message log */
+FILE *panicfile = NULL;		/* open stream to panic log file */
+FILE *logfile = NULL;		/* open stream to system log file */
 
 /* variables local to this file */
-static FILE *panicfile = NULL;		/* open stream to panic log file */
-static FILE *logfile = NULL;		/* open stream to system log file */
-static char *msg_log_fn;		/* name of per-message log file */
+static char *msg_log_fn;	/* name of per-message log file */
 
 /* functions local to this file */
 static void build_msg_log_fn __P((void));
+static FILE *open_a_log __P((char *, char *, char *, unsigned int));
 static void try_mkdir __P((char *));
 static void write_log_va __P((int, char *, va_list));
 
@@ -112,78 +121,106 @@ static void write_log_va __P((int, char *, va_list));
 void
 open_system_logs()
 {
+    static char panicbuf[BUFSIZ];	/* stdio buffer to avoid unchecked mallocs */
+    static char logbuf[BUFSIZ];		/* stdio buffer to avoid unchecked mallocs */
+
+    if (!panicfile) {
+	panicfile = open_a_log(panic_fn, "panic log", panicbuf, log_mode);
+	if (!panicfile) {
+	    if (only_testing) {
+		fprintf(stderr, "%s: [%lu] cannot open panic log %s: %s\n", program, (long unsigned int) getpid(), panic_fn, strerror(errno));
+	    } else {
+		fprintf(stderr, "%s: cannot open panic log %s: %s\n", program, panic_fn, strerror(errno));
+	    }
+	    panicfile = stderr;
+	}
+    }
+    if (!logfile) {
+	logfile = open_a_log(log_fn, "system log", logbuf, log_mode);
+	if (!logfile) {
+	    if (only_testing) {
+		fprintf(stderr, "%s: [%lu] cannot open system log %s: %s\n", program, (long unsigned int) getpid(), log_fn, strerror(errno));
+	    } else {
+		fprintf(stderr, "%s: cannot open system log %s: %s\n", program, log_fn, strerror(errno));
+	    }
+	    logfile = stderr;
+	}
+    }
+    return;
+}
+
+/*
+ * open_a_log() - open a log file for append, creating it if necessary, and if
+ *                auto_mkdir is set then perhaps also creating its directory.
+ *
+ * If 'buf' is set then call setbuf() with it.
+ *
+ * Returns a pointer to a STDIO FILE structure.
+ */
+/*
+ * XXX this should probably be rewritten to use fopen(fqfn, "a")
+ * [with a fix to use umask((~mode) & 0777)) too of course]
+ */
+static FILE *
+open_a_log(fqfn, fn, buf, mode)
+    char *fqfn;				/* fully qualified filename */
+    char *fn;				/* internal filename (for debug/error msgs) */
+    char *buf;				/* stdio buffer for fp */
+    unsigned int mode;			/* access modes to create with */
+{
+    FILE *fp = NULL;
     int fd;
-    static char panicbuf[BUFSIZ];	/* stdio buffer to avoid mallocs */
-    static char logbuf[BUFSIZ];		/* stdio buffer to avoid mallocs */
+    unsigned int oumask = umask(0);	/* we supply an explicit mode */
 
-    /*
-     * first open panic log so we can panic if we can't open
-     * the system log
-     */
-#ifdef	O_APPEND
-    fd = open(panic_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
-#else
-    /* limited to V7 open semantics */
-    fd = open(panic_fn, 1);
-    if (fd < 0) {
-	fd = creat(panic_fn, log_mode);
-    } else {
-	(void) lseek(fd, (off_t) 0L, SEEK_END);
-    }
-#endif
-    if (fd < 0) {
-	/* perhaps the directory just needs to be created */
-	if (auto_mkdir && errno == ENOENT) {
-	    try_mkdir(panic_fn);
-	    /* try opening the file again */
 #ifdef O_APPEND
-	    fd = open(panic_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
+    fd = open(fqfn, O_CREAT | O_APPEND | O_WRONLY, mode);
 #else
-	    fd = creat(panic_fn, log_mode);
-#endif
-	}
-    }
+    /* limited to V7 open semantics  XXX this old junk should go away.... */
+    fd = open(fqfn, 1);
     if (fd < 0) {
-	fprintf(stderr, "%s: [%lu] cannot open %s: %s\n", program, (long unsigned int) getpid(), panic_fn, strerror(errno));
-	panicfile = stderr;
+	DEBUG3(DBG_LOG_HI, "open_a_log(%s): %s: open() failed, will try creat(): %s\n", fqfn, fn, strerror(errno));
+	fd = creat(fqfn, mode);
     } else {
-	panicfile = fdopen(fd, "a");
-	(void) setbuf(panicfile, panicbuf);   /* associate stream and buffer */
+	(void) lseek(fd, (off_t) 0L, SEEK_END);	/* XXX could this ever fail? */
     }
+#endif
+    if (fd < 0) {
+#ifdef O_APPEND
+	DEBUG3(DBG_LOG_MID, "open_a_log(%s): %s: initial open() failed: %s\n", fqfn, fn, strerror(errno));
+#else
+	DEBUG3(DBG_LOG_MID, "open_a_log(%s): %s: initial creat() failed: %s\n", fqfn, fn, strerror(errno));
+#endif
 
-    /*
-     * next open the system log file after which we can start issuing
-     * log messages.
-     */
-#ifdef	O_APPEND
-    fd = open(log_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
-#else
-    /* limited to V7 open semantics */
-    fd = open(log_fn, 1);
-    if (fd < 0) {
-	fd = creat(log_fn, log_mode);
-    } else {
-	(void) lseek(fd, (off_t) 0L, SEEK_END);
-    }
-#endif
-    if (fd < 0) {
 	/* perhaps the directory just needs to be created */
 	if (auto_mkdir && errno == ENOENT) {
-	    try_mkdir(log_fn);
-	    /* try opening the file again */
-#ifdef	O_APPEND
-	    fd = open(log_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
+	    try_mkdir(fqfn);
+#ifdef O_APPEND
+	    fd = open(fqfn, O_CREAT | O_APPEND | O_WRONLY, mode);
 #else
-	    fd = creat(log_fn, log_mode);
+	    /* limited to V7 open semantics  XXX this old junk should go away.... */
+	    /* this time we know it's not there so just create it */
+	    fd = creat(fqfn, mode);
 #endif
-	}
-	if (fd < 0) {
-	    panic(EX_OSFILE, "cannot open %s: %s", log_fn, strerror(errno));
-	    /*NOTREACHED*/
+	    if (fd < 0) {
+#ifdef O_APPEND
+		DEBUG3(DBG_LOG_LO, "open_a_log(%s): %s: second try open() failed: %s\n", fqfn, fn, strerror(errno));
+#else
+		DEBUG3(DBG_LOG_LO, "open_a_log(%s): %s: second try creat() failed: %s\n", fqfn, fn, strerror(errno));
+#endif
+	    }
 	}
     }
-    logfile = fdopen(fd, "a");
-    (void) setbuf(logfile, logbuf);	/* associate stream and buffer */
+    if (fd >= 0) {
+	if (!(fp = fdopen(fd, "a"))) {
+	    DEBUG3(DBG_LOG_LO, "open_a_log(%s): %s: fdopen() failed: %s\n", fqfn, fn, strerror(errno));
+	}
+	if (fp && buf) {
+	    setbuf(fp, buf);
+	}
+    }
+    (void) umask(oumask);
+
+    return fp;
 }
 
 /*
@@ -193,7 +230,7 @@ static void
 try_mkdir(fn)
     char *fn;
 {
-    char *slash = rindex(fn, '/');
+    char *slash = strrchr(fn, '/');
     char *dr;
     struct stat st;
 
@@ -202,29 +239,39 @@ try_mkdir(fn)
     }
 
     /* figure directory name */
-    while (slash > fn && *(slash - 1) == '/')
+    while (slash > fn && *(slash - 1) == '/') {
 	--slash;
-    if (slash == fn)
+    }
+    if (slash == fn) {
 	return;			/* root directory */
-    dr = xmalloc((unsigned) (slash - fn + 1));
+    }
+    dr = xmalloc((size_t) (slash - fn + 1));
     (void) memcpy(dr, fn, (size_t) (slash - fn));
     dr[slash - fn] = '\0';
 
-    DEBUG1(DBG_LOG_LO, "make directory %s\n", dr);
+    DEBUG1(DBG_LOG_LO, "make directory %s: ", dr);
+    errno = 0;
     (void) mkdir(dr, auto_mkdir_mode);
+    DEBUG1(DBG_LOG_LO, "%s\n", strerror(errno));
 
     if (stat(dr, &st) == -1 && errno == ENOENT) {
-	char *slash2 = rindex(dr, '/');
+	char *slash2 = strrchr(dr, '/');
 
 	if (slash2) {
-	    while (slash2 > dr && *(slash2 - 1) == '/')
+	    while (slash2 > dr && *(slash2 - 1) == '/') {
 		--slash2;
+	    }
 	    if (slash2 != dr) {
 		*slash2 = '\0';
-		DEBUG1(DBG_LOG_LO, "    make parent directory %s\n", dr);
+		DEBUG1(DBG_LOG_LO, "    make parent directory %s: ", dr);
+		errno = 0;
 		(void) mkdir(dr, auto_mkdir_mode);
+		DEBUG1(DBG_LOG_LO, "%s\n", strerror(errno));
 		*slash2 = '/';
+		DEBUG1(DBG_LOG_LO, "    re-make directory %s: ", dr);
+		errno = 0;
 		(void) mkdir(dr, auto_mkdir_mode);
+		DEBUG1(DBG_LOG_LO, "%s\n", strerror(errno));
 	    }
 	}
     }
@@ -261,60 +308,26 @@ close_system_logs()
 void
 open_msg_log()
 {
-    int fd;
-    static char msgbuf[BUFSIZ];		/* stdio buffer to avoid mallocs */
+    static char msgbuf[BUFSIZ];		/* stdio buffer to avoid unchecked mallocs */
 
     /*
      * if msg_fn not yet set up create a suitably unique value
      */
     if (msg_logfile) {
 	(void) fclose(msg_logfile);
+	msg_logfile = NULL;
     }
     build_msg_log_fn();
-#ifdef	O_APPEND
-    fd = open(msg_log_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
-#else
-    /* limited to V7 open semantics */
-    fd = open(msg_log_fn, 1);
-    if (fd < 0) {
-	fd = creat(msg_log_fn, log_mode);
-    } else {
-	(void) lseek(fd, (off_t) 0L, SEEK_END);
+    msg_logfile = open_a_log(msg_log_fn, "message log", msgbuf, message_log_mode);
+    if (!msg_logfile) {
+	/*
+	 * otherwise, panic.  We are assuming that the mail queue entry can be
+	 * scanned at a later date (note: errno may not be useful from open_a_log)
+	 */
+	panic(EX_OSFILE, "cannot open %s/%s: %s", spool_dir, msg_log_fn, strerror(errno));
+	/*NOTREACHED*/
     }
-#endif
-    if (fd < 0) {
-	if (errno == ENOENT) {
-	    /* the directory did not exist, try to create it */
-	    DEBUG1(DBG_LOG_LO, "make directory %s/msglog\n",
-		   spool_dir);
-	    (void) mkdir("msglog", auto_mkdir_mode);
-	} else {
-	    /* alternate idea, permissions wrong so try to remove it */
-	    (void) unlink(msg_log_fn);
-	}
-#ifdef	O_APPEND
-	fd = open(msg_log_fn, O_CREAT|O_APPEND|O_WRONLY, log_mode);
-#else
-	/* limited to V7 open semantics */
-	fd = open(msg_log_fn, 1);
-	if (fd < 0) {
-	    fd = creat(msg_log_fn, log_mode);
-	} else {
-	    (void) lseek(fd, (off_t) 0L, SEEK_END);
-	}
-#endif
-	if (fd < 0) {
-	    /*
-	     * otherwise, panic.  We are assuming that the mail
-	     * queue entry can be scanned at a later date
-	     */
-	    panic(EX_OSFILE, "cannot open %s/%s: %s",
-		  spool_dir, msg_log_fn, strerror(errno));
-	    /*NOTREACHED*/
-	}
-    }
-    msg_logfile = fdopen(fd, "a");
-    (void) setbuf(msg_logfile, msgbuf);	/* associate stream and buffer */
+    return;
 }
 
 /*
@@ -372,7 +385,7 @@ unlink_msg_log()
  *
  */
 /*VARARGS2*/
-#ifdef ANSI_C
+#ifdef __STDC__
 void
 panic(int exitcode, char *fmt, ...)
 #else
@@ -386,45 +399,41 @@ panic(exitcode, fmt, va_alist)
     va_list ap;
 
     if (errfile) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_TTY, fmt, ap);
 	va_end(ap);
+	if (only_testing) {
+	    exit(force_zero_exitvalue ? 0 : exitcode);
+	}
     }
-#ifdef ANSI_C
-    va_start(ap, fmt);
-#else
-    va_start(ap);
-#endif
+    VA_START(ap, fmt);
     write_log_va(WRITE_LOG_PANIC, fmt, ap);
     va_end(ap);
 
-#ifdef ANSI_C
-    va_start(ap, fmt);
-#else
-    va_start(ap);
-#endif
+    VA_START(ap, fmt);
     write_log_va(WRITE_LOG_SYS, fmt, ap);
     va_end(ap);
 
     if (daemon_pid != 0 && daemon_pid == getpid()) {
 	write_log(WRITE_LOG_SYS, "smail daemon exiting on panic");
+	if (daemon_pidfile && *daemon_pidfile == '/') {
+	    (void) unlink(daemon_pidfile);
+	}
     }
     if (spool_fn) {
 	freeze_message();		/* put message in error/ directory */
     }
-    if (force_zero_exitvalue) {
-	exit(0);
-    }
-    exit(exitcode);
+
+    /* XXX we _REALLY_ should have an option to (try to) trigger a core dump here.... */
+
+    exit(force_zero_exitvalue ? 0 : exitcode);
+    /* NOTREACHED */
 }
 
 #ifndef NODEBUG
 # define WRITE_LOG_DEBUG	(WRITE_LOG_LAST << 2) /* magic for write_log_va() */
 #endif
+
 /*
  * write_log - write to the per-message log, and perhaps the system log
  *
@@ -434,7 +443,7 @@ panic(exitcode, fmt, va_alist)
  * Note that messages with a leading 'X' are for per-message logs.
  */
 /*VARARGS2*/
-#ifdef ANSI_C
+#ifdef __STDC__
 void
 write_log(int who, char *fmt, ...)
 #else
@@ -446,58 +455,65 @@ write_log(who, fmt, va_alist)
 #endif
 {
     va_list ap;
-
 #ifndef NODEBUG
+    int wct = 0;
+
     if (debug && errfile && (debug > 1 || fmt[0] != 'X')) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	/* if we are debugging at all, print all logged messages */
+	(void) fputs("write_log(", errfile);
+	if (who & WRITE_LOG_SYS) {
+	    (void) fputs("SYS", errfile);
+	    wct++;
+	}
+	if (who & WRITE_LOG_PANIC) {
+	    (void) fprintf(errfile, "%sPANIC", wct++ ? "|" : "");
+	}
+	if (who & WRITE_LOG_MLOG) {
+	    (void) fprintf(errfile, "%sMLOG", wct++ ? "|" : "");
+	}
+	if (who & WRITE_LOG_TTY) {
+	    (void) fprintf(errfile, "%sTTY", wct++ ? "|" : "");
+	}
+	if (who & WRITE_LOG_DEBUG) {
+	    (void) fprintf(errfile, "%sDEBUG", wct++ ? "|" : "");
+	}
+	(void) fputs("): ", errfile);
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_DEBUG, fmt, ap);
 	va_end(ap);
+	if (only_testing || who == WRITE_LOG_TTY) {
+	    return;
+	}
     }
-#endif
+#endif /* NODEBUG */
     if (errfile && ((who & WRITE_LOG_TTY) ||
 		   ((who & (WRITE_LOG_MLOG|WRITE_LOG_PANIC)) &&
 		    (error_processing == TERMINAL ||
 		     error_processing == ERROR_DEFAULT) && /* XXX ??? */
 		    fmt[0] != 'X'))) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_TTY, fmt, ap);
 	va_end(ap);
     }
+    if (only_testing) {
+	return;
+    }
     if (who & WRITE_LOG_SYS) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_SYS, fmt, ap);
 	va_end(ap);
     }
     if (who & WRITE_LOG_PANIC) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_PANIC, fmt, ap);
 	va_end(ap);
     }
     if (who & WRITE_LOG_MLOG) {
-#ifdef ANSI_C
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
+	VA_START(ap, fmt);
 	write_log_va(WRITE_LOG_MLOG, fmt, ap);
 	va_end(ap);
     }
+    return;
 }
 
 static void
@@ -506,31 +522,9 @@ write_log_va(who, fmt, ap)
     char *fmt;				/* printf(3) format */
     va_list ap;                         /* arguments for printf */
 {
-    int wct = 0;
-
     switch (who) {
 #ifndef NODEBUG
     case WRITE_LOG_DEBUG:
-	/* if we are debugging at all, print logging messages */
-	(void) fprintf(errfile, "write_log:");
-
-	/* when debugging, tell which log files are to be used */
-	if (debug > 1 && (who & WRITE_LOG_SYS)) {
-	    (void) fprintf(errfile, " SYS");
-	    wct++;
-	}
-	if (debug > 1 && (who & WRITE_LOG_PANIC)) {
-	    (void) fprintf(errfile, "%cPANIC", wct++ ? '|' : ' ');
-	}
-	if (debug > 1 && (who & WRITE_LOG_MLOG)) {
-	    (void) fprintf(errfile, "%cMLOG", wct++ ? '|' : ' ');
-	}
-	if (debug > 1 && (who & WRITE_LOG_TTY)) {
-	    (void) fprintf(errfile, "%cTTY", wct++ ? '|' : ' ');
-	}
-	if (debug > 1) {
-	    (void) fprintf(errfile, " ");
-	}
 	(void) vfprintf(errfile, fmt, ap);
 	/* log messages don't come with their own \n */
 	(void) fprintf(errfile, "\n");
@@ -540,7 +534,11 @@ write_log_va(who, fmt, ap)
 #endif	/* NODEBUG */
 
     case WRITE_LOG_TTY:
-	(void) fprintf(errfile, "%s: [%lu] ", program, (long unsigned int) getpid());
+	if (only_testing) {
+	    (void) fprintf(errfile, "%s: ", program);
+	} else {
+	    (void) fprintf(errfile, "%s: %s: [%lu] ", program, time_stamp(), (long unsigned int) getpid());
+	}
 	if (message_id) {
 	    (void) fprintf(errfile, "[%s] ", message_id);
 	}
@@ -548,6 +546,15 @@ write_log_va(who, fmt, ap)
 	/* log messages don't come with their own \n */
 	(void) fprintf(errfile, "\n");
 	(void) fflush(errfile);
+	if (ferror(errfile)) {
+	    int oerrno = errno;
+	    int fd;
+
+	    fd = fileno(errfile);
+	    errfile = NULL;
+	    panic(EX_IOERR, "error writing %s[%d]: %s", "errfile", fd, strerror(oerrno));
+	    /*NOTREACHED*/
+	}
 	break;
 
     case WRITE_LOG_SYS:
@@ -556,10 +563,13 @@ write_log_va(who, fmt, ap)
 	    open_system_logs();		/* if system log not open, open it */
 	}
 #ifdef LOG_EXTRA_NEWLINE
-	(void) fprintf(logfile, "\n%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
-#else
-	(void) fprintf(logfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
+	(void) fputc('\n', logfile);
 #endif
+	if (only_testing) {
+	    (void) fprintf(logfile, "%s: %s: [%lu] log: ", program, time_stamp(), (long unsigned int) getpid());
+	} else {
+	    (void) fprintf(logfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
+	}
 	if (message_id) {
 	    (void) fprintf(logfile, "[%s] ", message_id);
 	}
@@ -568,7 +578,16 @@ write_log_va(who, fmt, ap)
 	(void) fprintf(logfile, "\n");
 	(void) fflush(logfile);
 	if (ferror(logfile)) {
-	    panic(EX_IOERR, "error writing %s: %s", log_fn, strerror(errno));
+	    int oerrno = errno;
+	    int fd;
+	    struct str str;
+		
+	    STR_INIT(&str);
+	    str_printf_va(&str, fmt, ap);
+	    fd = fileno(logfile);
+	    logfile = NULL;
+	    panic(EX_IOERR, "error writing the following message to %s[%d]: %s\n%s",
+		  log_fn, fd, strerror(oerrno), STR(&str));
 	    /*NOTREACHED*/
 	}
 	break;
@@ -578,10 +597,13 @@ write_log_va(who, fmt, ap)
 	    open_system_logs();		/* if system log not open, open it */
 	}
 #ifdef LOG_EXTRA_NEWLINE
-	(void) fprintf(panicfile, "\n%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
-#else
-	(void) fprintf(panicfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
+	(void) fputc('\n', logfile);
 #endif
+	if (only_testing) {
+	    (void) fprintf(panicfile, "%s: %s: [%lu] panic: ", program, time_stamp(), (long unsigned int) getpid());
+	} else {
+	    (void) fprintf(panicfile, "%s: [%lu] ", time_stamp(), (long unsigned int) getpid());
+	}
 	if (message_id) {
 	    (void) fprintf(panicfile, "[%s] ", message_id);
 	}
@@ -590,9 +612,33 @@ write_log_va(who, fmt, ap)
 	(void) fprintf(panicfile, "\n");
 	(void) fflush(panicfile);
 	if (ferror(panicfile)) {
+	    int oerrno = errno;
+	    int fd;
+
+	    /*
+	     * Note: no sense potentially going in a loop here by calling
+	     * panic() again, so instead just clear the errant panicfile so
+	     * that nothing tries again, and so in case panic() itself calls
+	     * itself then mabye the open_system_logs() above will work.  In
+	     * the mean time just try writing the message to errfile and
+	     * logfile if possible.
+	     */
+	    fd = fileno(panicfile);
 	    panicfile = NULL;
-	    panic(EX_IOERR, "error writing %s: %s", log_fn, strerror(errno));
-	    /*NOTREACHED*/
+	    if (errfile) {
+		(void) fprintf(errfile, "error writing following message to %s[%d]: %s\n", panic_fn, fd, strerror(oerrno));
+		(void) vfprintf(errfile, fmt, ap);
+		/* log messages don't come with their own \n */
+		(void) fprintf(errfile, "\n");
+		(void) fflush(errfile);
+	    }
+	    if (logfile) {
+		(void) fprintf(logfile, "error writing following message to %s[%d]: %s", panic_fn, fd, strerror(oerrno));
+		(void) vfprintf(logfile, fmt, ap);
+		/* log messages don't come with their own \n */
+		(void) fprintf(logfile, "\n");
+		(void) fflush(logfile);
+	    }
 	}
 	break;
 
@@ -613,8 +659,16 @@ write_log_va(who, fmt, ap)
 	    (void) fprintf(msg_logfile, "\n");
 	    (void) fflush(msg_logfile);
 	    if (ferror(msg_logfile)) {
-		panic(EX_IOERR, "error writing %s/%s: %s",
-		      spool_dir, msg_log_fn, strerror(errno));
+		int oerrno = errno;
+		int fd;
+		struct str str;
+		
+		STR_INIT(&str);
+		str_printf_va(&str, fmt, ap);
+		fd = fileno(msg_logfile);
+		msg_logfile = NULL;
+		panic(EX_IOERR, "error writing the following message to %s/%s[%d]: %s\n%s",
+		      spool_dir, msg_log_fn, fileno(msg_logfile), strerror(oerrno), STR(&str));
 		/*NOTREACHED*/
 	    }
 	}
@@ -647,7 +701,7 @@ send_log(f, all, banner)
     build_msg_log_fn();
     /* reopen log file to avoid changing the write position */
     if (! (mlog = fopen(msg_log_fn, "r"))) {
-	DEBUG2(DBG_LOG_MID, "send_log(): no message log file: %s: %s.\n", msg_log_fn, strerror(errno));
+	DEBUG3(DBG_LOG_MID, "send_log(): no message log file: %s/%s: %s.\n", spool_dir, msg_log_fn, strerror(errno));
 	return;
     }
     /* copy from message-log to the specified file */
@@ -665,7 +719,7 @@ send_log(f, all, banner)
 	}
 	if (lastx) {
 	    /* ignore any SMTP replies following the 'X' line too */
-	    if (strlen(mln) >= 4 && isdigit(mln[0]) && isdigit(mln[1]) && isdigit(mln[2])) {
+	    if (strlen(mln) >= 4 && isdigit((int) mln[0]) && isdigit((int) mln[1]) && isdigit((int) mln[2])) {
 		if (mln[3] == ' ') {
 		    lastx = FALSE;
 		}
