@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:lookup.c,v 1.57 2001/07/30 02:41:24 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:lookup.c,v 1.84 2003/06/08 22:54:36 woods Exp"
  */
 
 /*
@@ -16,50 +16,72 @@
  *	access method.
  *
  *	external functions:  open_database, close_database, lookup_database,
- *			     match_ip
+ *			     match_re, match_ip, match_ip_net, flip_inet_addr,
+ *			     match_hostname
  *
  * NeXT NetInfo aliases support added by Dan Danz <dan@az.stratus.com>
  */
 
+#include "defs.h"
+#ifdef STANDALONE
+# undef HAVE_LIBWHOSON
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <setjmp.h>
 #include <signal.h>
 #include <errno.h>
-#define NEED_SOCKETS				/* for match_ip(), etc. */
-#include "defs.h"
-#ifdef	UNIX_BSD
-# include <sys/file.h>
-#endif
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
 #ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
 # include <string.h>
 #endif
-#ifdef	HAVE_YP
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
+# include <fcntl.h>
+#else
+# if defined(UNIX_BSD)
+#  include <sys/file.h>
+# endif
+#endif
+#ifdef __GLIBC__
+# include <sys/file.h>
+#endif
+#ifdef HAVE_SYS_LOCKING_H
+# include <sys/locking.h>
+#endif
+
+#ifdef HAVE_YP
 # include "jump.h"
 # include <rpcsvc/ypclnt.h>
 #endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
 
-#include "smail.h"
-#include "main.h"
-#include "parse.h"
-#include "addr.h"
-#include "alloc.h"
-#include "dys.h"
-#include "log.h"
-#include "lookup.h"			/* XXX only really needed with LIBWHOSON for now */
-#include "exitcodes.h"
-#include "smailconf.h"
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
 
 #ifdef HAVE_NIALIAS
 # include <aliasdb.h>
@@ -71,35 +93,52 @@
 #ifdef  HAVE_NISPLUS
 # include <rpcsvc/nis.h>
 #endif
-#ifdef USE_LSEARCH_REGEXCMP
-# ifdef UNIX_SYS5_4
-#  include <regexpr.h>			/* simple form of the following */
-# else
-#  if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
-    /* most use "INIT" before other declaritions, Linux uses "INIT;" as the
-     * last, luckily */
-#   define INIT		register char *sp = instring;
-#   define ESIZE	BUFSIZ
-#   define GETC()	(*sp++)
-#   define PEEKC()	(*sp)
-#   define UNGETC(c)	(--sp)
-#   define RETURN(c)	return (c);
-#   define ERROR(c)	{ regerr = c; return ((char *) NULL); }
+
+#ifdef UNIX_SYS5_4
+# include <regexpr.h>			/* simple form of the following */
+extern int regerrno;
+#else
+# if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
+/* XXX this should be deprecated and replaced with pd/regex XXX */
+   /* most use "INIT" before other declaritions, Linux uses "INIT;" as the
+    * last, luckily */
+#  define INIT		register char *sp = instring;
+#  define ESIZE		BUFSIZ
+#  define GETC()	(*sp++)
+#  define PEEKC()	(*sp)
+#  define UNGETC(c)	(--sp)
+#  define RETURN(c)	return (c);
+#  define ERROR(c)	{ regerr = c; return ((char *) NULL); }
 static int	regerr = 0;		/* error code from RE compiler */
-#   include <regexp.h>			/* from UNIX SysV (incl. SunOS) or GNU */
-#  else
-#   include <regex.h>			/* P1003.2 C, Henry Spencer, 4.4BSD, XPG3 */
-#   ifndef REG_BASIC
-#    define REG_BASIC	0		/* only in Henry Spencer's 4.4BSD version? */
-#   endif
+#  include <regexp.h>			/* from UNIX SysV (incl. SunOS) or GNU */
+# else
+#  include <regex.h>			/* P1003.2 C, Henry Spencer, 4.4BSD, XPG3 */
+#  ifndef REG_BASIC
+#   define REG_BASIC	0		/* only in Henry Spencer's 4.4BSD version? */
 #  endif
 # endif
-#endif /* USE_LSEARCH_REGEXCMP */
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
 #endif
 
+#ifdef HAVE_LIBWHOSON
+# include <whoson.h>
+#endif
+
+#include "smail.h"
+#include "smailsock.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
+#include "alloc.h"
+#include "smailstring.h"
+#include "dys.h"
+#include "log.h"
+#include "lookup.h"
+#include "exitcodes.h"
+#include "smailconf.h"
+#include "extern.h"
+#include "debug.h"
+#include "smailport.h"
+
 #if defined(HAVE_DBM) && !defined(HAVE_NDBM)
 # undef NULL
 # include <dbm.h>
@@ -108,16 +147,15 @@ static int	regerr = 0;		/* error code from RE compiler */
 #else
 # ifdef HAVE_NDBM
 #  include <ndbm.h>
+#  ifdef __bsdi__
+extern int dbm_error __P((DBM *));
+#  endif
 # else
 #  include "sdbm.h"
 #  define HAVE_NDBM
 # endif
 #endif
 
-#ifdef HAVE_LIBWHOSON
-# include <whoson.h>
-#endif
-
 #ifdef STANDALONE
 # define xmalloc malloc
 # define xrealloc realloc
@@ -149,7 +187,7 @@ static int lsearch_open __P((char *, struct proto *, int, unsigned int, struct s
 static void lsearch_close __P((struct generic_db *));
 static int lsearch_lookup __P((struct generic_db *, char *, char **, char **));
 #ifdef USE_LSEARCH_REGEXCMP
-static int lsearch_regexcmp __P((char *, char *, unsigned int *));
+static int lsearch_regexcmp __P((char *, char *, size_t *));
 #endif
 
 static int dbmbase_open __P((char *, struct proto *, int, unsigned int, struct stat *, char **, char **));
@@ -223,7 +261,8 @@ struct proto *end_protos = protos + sizeof(protos)/sizeof(protos[0]);
  *   FILE_FAIL	if an unrecoverable failure occured.  An error will be
  *		stored in *error.
  *   FILE_NOMATCH
- *		if the database does not appear to exist.
+ *		if the database does not appear to exist.  An error will
+ *		be stored in *error.
  */
 int
 open_database(name, proto, retries, interval, statp, db, error)
@@ -239,9 +278,12 @@ open_database(name, proto, retries, interval, statp, db, error)
 
     DEBUG2(DBG_DRIVER_HI, "open_database: db name = %s, db proto = %s.\n", name, proto);
 
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     for (pp = protos; proto && pp < end_protos; pp++) {
+	DEBUG1(DBG_DRIVER_HI, "open_database: checking against db proto = %s.\n", pp->proto);
 	if (EQ(proto, pp->proto)) {
 	    /* found the requested access method */
+	    DEBUG1(DBG_DRIVER_HI, "open_database: found matching db proto = %s.\n", pp->proto);
 	    return (*pp->open)(name, pp, retries, interval, statp, db, error);
 	}
     }
@@ -365,8 +407,8 @@ bsearch_open(name, proto, retries, interval, statp, db, error)
 	}
     }
 
-#ifdef lock_fd_rd_wait
-    if (lock_fd_rd_wait(fileno(f)) < 0) {
+#ifdef lock_fd_rd
+    if (lock_fd_rd(fileno(f)) < 0) {
 	*error = "database is locked, try again later";
 	return FILE_AGAIN;
     }
@@ -459,7 +501,7 @@ bsearch_lookup(gdb, key, value, error)
 	    }
 	}
 	STR_CLEAR(&str);
-	while ((c = getc(db->f)) != EOF && !isspace(c) && c != ':') {
+	while ((c = getc(db->f)) != EOF && !isspace((int) c) && c != ':') {
 	    STR_NEXT(&str, c);
 	}
 	if (c == EOF && ferror(db->f)) {
@@ -487,7 +529,7 @@ bsearch_lookup(gdb, key, value, error)
     i = -1;				/* index to last non-space */
     STR_CLEAR(&str);			/* clear out the region */
     while (((c = getc(db->f)) != EOF) && (c != '\n')) {
-	if (!isspace(c)) {
+	if (!isspace((int) c)) {
 	    if (c == '#') {
 		/* comment puts an end to the entry */
 		break;
@@ -568,8 +610,8 @@ lsearch_open(name, proto, retries, interval, statp, db, error)
 	}
     }
 
-#ifdef lock_fd_rd_wait
-    if (lock_fd_rd_wait(fileno(f)) < 0) {
+#ifdef lock_fd_rd
+    if (lock_fd_rd(fileno(f)) < 0) {
 	*error = "database is locked, try again later";
 	return FILE_AGAIN;
     }
@@ -605,36 +647,29 @@ lsearch_close(gdb)
 #ifdef USE_LSEARCH_REGEXCMP
 /*
  * lsearch_regexcmp - do a regular expression matching
+ *
+ * returns 0 on match, non-zero on no match [just like strcmp() does]
  */
 static int
 lsearch_regexcmp(entry, key, len)
     char *entry;
     char *key;
-    unsigned int *len;
+    size_t *len;
 {
     static int inited = FALSE;
     static struct str exp;
-# if defined(UNIX_SYS5_4)
-    char *rcstr;
-# else
-#  if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
-    char expbuf[ESIZE];	
-    char *rcstr;
-#  else /* P1003.2 / 4.4bsd / Spencer regex */
-    regex_t preg;
-#  endif
-# endif
     char *s;
-    int  rc;
 	
+# ifndef NODEBUG
     if (*entry != '\"') {
-	panic(EX_SOFTWARE, "lsearch_lookup: called for non-RE <%s>", entry);
+	panic(EX_SOFTWARE, "lsearch_regexcmp: called for non-RE <%s>", entry);
 	/*NOTREACHED*/
     }
+# endif
     s = &entry[1];			/* note this style may allow for multibyte */
     if (!strchr(s, '\"')) {
-	DEBUG1(DBG_DRIVER_MID, "lsearch_lookup: un-terminated RE <%s>\n", entry);
-	return (-1);
+	write_log(WRITE_LOG_PANIC, "lsearch_regexcmp: un-terminated RE <%s>", entry);
+	return (-1);			/* equivalent to "no-match" */
     }
     if (!inited) {
 	STR_INIT(&exp);
@@ -642,58 +677,16 @@ lsearch_regexcmp(entry, key, len)
     } else {
 	STR_LEN(&exp) = 0;
     }
-    STR_NEXT(&exp, '^');
-    while (*s && *s != '\"')
+    STR_NEXT(&exp, '^');		/* create a fully anchored RE */
+    while (*s && *s != '\"') {
 	STR_NEXT(&exp, *s++);
-    *len = s + 1 - entry;		/* mark char following closing quote.... */
-    STR_NEXT(&exp, '$');
-    STR_NEXT(&exp, '\0');
-    DEBUG2(DBG_DRIVER_HI, "lsearch_regexcmp: matching <%s> against expression <%s>\n", key, STR(&exp));
-
-# if defined(UNIX_SYS5_4)
-    if (!(rcstr = compile(STR(&exp), (char *) NULL, (char *) NULL))) {
-	DEBUG1(DBG_DRIVER_MID, "lsearch_regexcmp: bad regular expression <%s>\n", STR(&exp));
-	if (rcstr)
-	    free(rcstr);
-	return (-1);
     }
-# else
-#  if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
-    if (!(rcstr = compile(STR(&exp), expbuf, &expbuf[ESIZE], '\0'))) {
-	DEBUG2(DBG_DRIVER_MID, "lsearch_regexcmp: bad regular expression <%s>, regerr = %d\n",
-	       STR(&exp), regerr);
-	return (-1);
-    }
-#  else
-    if ((rc = regcomp(&preg, STR(&exp), (REG_BASIC | REG_ICASE | REG_NOSUB)))) {
-	char re_errmsg[BUFSIZ];
+    STR_CAT(&exp, "$");
 
-	(void) regerror(rc, &preg, re_errmsg, sizeof(re_errmsg));
-	DEBUG2(DBG_DRIVER_MID, "lsearch_regexcmp: bad regular expression <%s>: %s\n",
-	       STR(&exp), re_errmsg);
-	regfree(&preg);
-	return (-1);
-    }
-#  endif
-# endif
+    *len = s + 1 - entry;		/* mark the char following closing quote.... */
 
-# if defined(UNIX_SYS5_4)
-    /* non-zero indicates match */
-    rc = (!(advance(key, rcstr)));	/* advance() forces leading '^' */
-    if (rcstr)
-	free(rcstr);			/* no reusing this one....  :-( */
-# else
-#  if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
-    /* non-zero indicates match */
-    rc = (!(advance(key, expbuf)));	/* advance() forces leading '^' */
-#  else /* P1003.2 / 4.4bsd / Spencer regex */
-    /* zero indicates match */
-    rc = regexec(&preg, key, (size_t) 0, (regmatch_t *) NULL, 0);
-    regfree(&preg);
-#  endif
-# endif
-
-    return (rc);
+    /* ignore errors, act like strcmp() */
+    return (match_re(key, STR(&exp), 1, (char **) NULL, (char **) NULL, (char **) NULL, (char **) NULL) == 1 ? 0 : 1);
 }
 #endif /* USE_LSEARCH_REGEXCMP */
 
@@ -708,7 +701,7 @@ lsearch_lookup(gdb, key, value, error)
     char **value;
     char **error;
 {
-    unsigned int len;			/* length of comparison */
+    size_t len;				/* length of comparison */
     register char *entry;		/* entry from read_entry() */
     register char *p;			/* temp */
     register struct lsearch_db *db = (struct lsearch_db *) gdb;
@@ -733,7 +726,7 @@ lsearch_lookup(gdb, key, value, error)
 #endif
 	    rc = strncmpic(entry, key, len);
 
-	if ((rc == 0) && (entry[len] == ':' || isspace(entry[len]))) {
+	if ((rc == 0) && (entry[len] == ':' || isspace((int) entry[len]))) {
 	    char *ret;			/* value to be returned */
 
 	    /* found a matching entry in the file, find the data */
@@ -741,18 +734,26 @@ lsearch_lookup(gdb, key, value, error)
 	    /*
 	     * skip <whitespace>:<whitespace>
 	     */
-	    while (isspace(*entry)) entry++;
+	    while (isspace((int) *entry)) {
+		entry++;
+	    }
 	    if (*entry == ':') {
 		entry++;
-		while (isspace(*entry)) entry++;
+		while (isspace((int) *entry)) {
+		    entry++;
+		}
 	    }
 
 	    /*
 	     * skip comments
 	     */
 	    while (*entry == '#') {
-		while (*entry && *entry != '\n') entry++;
-		while (isspace(*entry)) entry++;
+		while (*entry && *entry != '\n') {
+		    entry++;
+		}
+		while (isspace((int) *entry)) {
+		    entry++;
+		}
 	    }
 	    ret = entry;		/* return from this point on */
 
@@ -760,7 +761,7 @@ lsearch_lookup(gdb, key, value, error)
 	    p = entry - 1;
 	    /* find the last character which is not white-space or comment */
 	    while (*entry) {
-		if (!isspace(*entry)) {
+		if (!isspace((int) *entry)) {
 		    if (*entry == '#') {
 			while (*entry && *entry != '\n') entry++;
 			continue;
@@ -942,9 +943,9 @@ dbmbase_lookup(gdb, key, value, error)
 {
     datum the_key;
     datum the_value;
-    static unsigned int temp_size;	/* size of temp_key area */
+    static size_t temp_size;		/* size of temp_key area */
     static char *temp_data = NULL;	/* growable temp area */
-    int len;				/* length of key */
+    size_t len;				/* length of key */
     register struct dbmbase_db *db = (struct dbmbase_db *) gdb;
 
 #ifndef HAVE_NDBM
@@ -993,7 +994,7 @@ dbmbase_lookup(gdb, key, value, error)
 #endif
 
     if (the_value.dptr) {
-	if (temp_size < the_value.dsize + 1) {
+	if (temp_size < (size_t) the_value.dsize + 1) {
 	    temp_size = the_value.dsize + 1;
 	    temp_data = xrealloc(temp_data, temp_size);
 	}
@@ -1042,8 +1043,14 @@ static int common_yp_lookup __P((register struct yp_db *, char *, char **, char
 
 #ifdef notyet
 
+#if ((__STDC__ - 0) <= 0)
+/* most traditional compilers won't optimize global variables */
+int save_time;				/* saved value from alarm() */
+JUMPSIG save_sigalrm;			/* previous SIGALRM handler */
+#else
 static volatile int save_time;		/* saved value from alarm() */
 static volatile JUMPSIG save_sigalrm;	/* previous SIGALRM handler */
+#endif
 
 static JUMP_ENVBUF alarm_jmp;		/* jump here on SIGALRM */
 
@@ -1097,7 +1104,7 @@ yp_open(name, proto, retries, interval, statp, db, error)
     pname = COPY_STRING(name);
 
     /* is a YP domain specified? */
-    p = index(pname, ':');
+    p = strchr(pname, ':');
     if (p == NULL) {
 	/* no, use the default YP domain */
 	priv->domain = NULL;
@@ -1110,7 +1117,7 @@ yp_open(name, proto, retries, interval, statp, db, error)
 
     /* if stat required, just zero it out, there is nothing to put there */
     if (statp) {
-	(void) bzero((char *)statp, sizeof(*statp));
+	(void) memset((char *) statp, '\0', sizeof(*statp));
     }
 
     if (priv->domain) {
@@ -1254,7 +1261,7 @@ common_yp_lookup(db, key, value, error, aliasflag)
 {
     int keylen;				/* length of key */
     static char *temp_key = NULL;	/* area for lower-case conversion */
-    static unsigned int temp_size = 0;	/* size allocated for temp_key */
+    static size_t temp_size = 0;	/* size allocated for temp_key */
     register char *p;
     char *matchval;			/* matched data */
     int matchlen;			/* length of matched data */
@@ -1275,7 +1282,7 @@ common_yp_lookup(db, key, value, error, aliasflag)
 	temp_key = xrealloc(temp_key, temp_size);
     }
     for (p = temp_key; *key; p++, key++) {
-	*p = lowercase((int) *key);
+	*p = tolower((int) *key);
     }
     *p = '\0';
 
@@ -1401,7 +1408,7 @@ aliasni_open(name, proto, retries, interval, statp, db, error)
 
     /* if stat required, just zero it out, there is nothing to put there */
     if (statp) {
-	(void) bzero((char *)statp, sizeof(*statp));
+	(void) memset((char *) statp, '\0', sizeof(*statp));
     }
 
     *db = (char *)priv;
@@ -1545,7 +1552,7 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 	char	*offset;
 
         priv->fieldnum = atoi(priv->name + 1);
-	offset = index(priv->name,',');
+	offset = strchr(priv->name,',');
 	if ( offset ) {
             priv->table = COPY_STRING(offset+1);
         } else {
@@ -1561,11 +1568,11 @@ nisplus_open(name, proto, retries, interval, statp, db, error)
 
     /* if stat required, just zero it out, there is nothing to put there */
     if (statp) {
-	(void) bzero((char *)statp, sizeof(*statp));
+	(void) memset((char *) statp, '\0', sizeof(*statp));
     }
 
     /* find offset to domain name, advance past the last comma */
-    if ( !(domain = rindex(priv->name,','))) {
+    if ( !(domain = strrchr(priv->name,','))) {
 	domain = priv->name;
     } else {
 	domain++;
@@ -1756,15 +1763,15 @@ nisplus_lookup(gdb, key, value, error)
 
 		/* assemble the return value from the columns */
 		objbuf = &(result->objects.objects_val[0]);
-		valbuf = xmalloc(objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len+1);
+		valbuf = xmalloc((size_t) objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len + 1);
 		strncpy(valbuf, objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_val,
-				objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len);
+				(size_t) objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len);
 		valsize = objbuf->zo_data.objdata_u.en_data.en_cols.en_cols_val[db->fieldnum].ec_value.ec_value_len+1;
 
 		/* back up over any trailing whitespace */
-		for (valsize--; valsize > 0 && isspace(valbuf[valsize]); valsize--);
+		for (valsize--; valsize > 0 && isspace((int) valbuf[valsize]); valsize--);
 		    valsize++;
-		    if (isspace(valbuf[valsize])) {
+		    if (isspace((int) valbuf[valsize])) {
 			valbuf[valsize] = '\0';
 		}
 
@@ -1851,94 +1858,420 @@ nisplus_lookup(gdb, key, value, error)
 
 
 
-#ifdef HAVE_BSD_NETWORKING
-
 /*
- * match_ip - match an ip against an allowed list
+ * match_re - match a string against a regular expression
  *
- * Takes a list of IP:IP:IP and compares against an IP string.
+ * NOTE:  If icase is non-zero then the goal is to do case-insensitive matching
+ * by default.  However this is not possible on non-P1003.2 systems.  Grrr....
  *
- * 'IP' is an IP number or IP/bits CIDR network spec.  IP may optionally be
- * followed by ';message text', which of course must not contain a ':'
+ * returns:
  *
- * The field separator show above as ':' is specified by the caller.  The
- * message sub-field separator show above as ';' is specified by the caller.
+ *	1	- the string matched the RE
  *
- * TODO:
- *	- add support for negation, must use last-match-wins!
+ *	0	- no match.
  *
- * WARNING: cannot diddle with pattern -- may be read-only string.
- *
- * Returns 1 if matched, else 0.
+ *	-1	- an error occured, error message written to paniclog
  */
 int
-match_ip(ip, patterns, fldsep, msgsep, reasonp)
-    char *ip;			/* ip address we are matching */
+match_re(key, exp, icase, start, beg, end, last)
+    char *key;					/* string to match */
+    char *exp;					/* RE to match against */
+    unsigned int icase;				/* ignore case? */
+    char **start;				/* optional ptr to beg of match */
+    char **beg;					/* optional ptr to beg of sub-expression */
+    char **end;					/* optional ptr to after sub-expression */
+    char **last;				/* optional ptr to after match */
+{
+#if defined(UNIX_SYS5_4)
+    char *rcstr;
+#else
+# if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
+    char expbuf[ESIZE];	
+    char *rcstr;
+# else /* P1003.2 / 4.4bsd / Spencer regex */
+    regex_t preg;
+    char re_errmsg[BUFSIZ];
+    int reg_icase = icase ? REG_ICASE : 0;
+    regmatch_t pmatch[2];
+# endif
+#endif
+    int  rc;
+
+    DEBUG3(DBG_DRIVER_HI, "match_re: matching '%s' against expression '%s'%s\n", key, exp, icase ? ", ignoring case" : "");
+
+#if defined(UNIX_SYS5_4)
+    if (!(rcstr = compile(exp, (char *) NULL, (char *) NULL))) {
+	write_log(WRITE_LOG_PANIC, "match_re: bad regular expression '%s' [regerrno = %d]",
+		  exp, regerrno);
+	return (-1);
+    }
+#else
+# if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
+    if (!(rcstr = compile(exp, expbuf, &expbuf[ESIZE], '\0'))) {
+	write_log(WRITE_LOG_PANIC, "match_re: bad regular expression '%s' [regerr = %d]",
+	       exp, regerr);
+	return (-1);
+    }
+# else /* P1003.2 / 4.4bsd / Spencer regex */
+    if ((rc = regcomp(&preg, exp, (REG_EXTENDED | reg_icase | ((beg || end) ? 0 : REG_NOSUB))))) {
+	(void) regerror(rc, &preg, re_errmsg, sizeof(re_errmsg));
+	write_log(WRITE_LOG_PANIC, "match_re: bad regular expression '%s': %s",
+	       exp, re_errmsg);
+	regfree(&preg);
+	return (-1);
+    }
+# endif
+#endif
+
+#if defined(UNIX_SYS5_4)
+    /* advance() forces leading '^', step() does not */
+    rc = step(key, rcstr);		/* non-zero indicates match */
+    if (rcstr) {
+	free(rcstr);			/* don't leak compile()'s storage */
+    }
+    if (rc && beg) {
+	if (nbra) {
+	    *beg = braslist[0];
+	} else {
+	    *beg = loc1;
+	}
+    }
+    if (rc && end) {
+	if (nbra) {
+	    *end = braelist[0];
+	} else {
+	    *end = loc2;
+	}
+    }
+#else
+# if defined(UNIX_SYS5_3) || defined(UNIX_SYS5_2) || defined(USE_REGEXP_H)
+    /* advance() forces leading '^', step() does not */
+    locs = NULL;    
+    rc = step(key, expbuf);		/* non-zero indicates match */
+    /*
+     * XXX bra?list[] are available SysVr3.2 (regexp.h 1.4.1.2), and in
+     * SysVr4.0 (1.9), but they are not documented.
+     */
+    if (rc && beg) {
+	if (nbra) {
+	    *beg = braslist[0];
+	} else {
+	    *beg = loc1;
+	}
+    }
+    if (rc && end) {
+	if (nbra) {
+	    *end = braelist[0];
+	} else {
+	    *end = loc2;
+	}
+    }
+# else /* P1003.2 / 4.4bsd / Spencer regex */
+    /* zero indicates match */
+    switch ((rc = regexec(&preg, key, (size_t) 2, pmatch, 0))) {
+    case 0:
+	rc = 1;				/* match */
+	/* if (pmatch[1].rm_so == -1) assert(pmatch[1].rm_eo == -1); */
+	if (start) {
+	    *start = key + pmatch[0].rm_so;
+	}
+	if (beg) {
+	    *beg = key + ((pmatch[1].rm_so == -1) ? pmatch[0].rm_so : pmatch[1].rm_so);
+	}
+	if (end) {
+	    *end = key + ((pmatch[1].rm_eo == -1) ? pmatch[0].rm_eo : pmatch[1].rm_eo);
+	}
+	if (last) {
+	    *last = key + pmatch[0].rm_eo;
+	}
+	break;
+    case REG_NOMATCH:
+	rc = 0;				/* no match */
+	break;
+    default:
+	(void) regerror(rc, &preg, re_errmsg, sizeof(re_errmsg));
+	write_log(WRITE_LOG_PANIC, "match_re: error matching '%s' with expression '%s': %s",
+		  exp, re_errmsg);
+	rc = -1;			/* error during matching */
+    }
+    regfree(&preg);
+# endif
+#endif
+
+    DEBUG3(DBG_DRIVER_HI, "match_re: '%s' %s expression '%s'\n", key, rc ? "matched" : "did not match", exp);
+
+    return rc;
+}
+
+
+/*
+ * match_hostname - match a hostname against a list of REs
+ *
+ * IMPORTANT NOTE:  hostnames are case insensitive.  However only on P1003.2
+ * systems will we be using RE matching code that can do case-insensitive
+ * matches.
+ *
+ * WARNING:  Dots in hostnames should be quoted with a backslash.
+ *
+ */
+int
+match_hostname(hostname, patterns, reasonp)
+    char *hostname;		/* hostname we are matching */
     char *patterns;		/* list of patterns we are using to match it */
-    int fldsep;			/* field separator character */
-    int msgsep;			/* optional message string separator */
+    char **reasonp;		/* optional pointer for returned message string */
+{
+    return match_re_list(hostname, patterns, 1, reasonp);
+}
+
+
+/*
+ * match_re_list - match a string against a list of REs
+ *
+ * Takes a list of RE:RE:RE and compares each against the string until a match
+ * is found or the list is exhausted.
+ *
+ * RE is a Regular Expression.
+ *
+ * RE may optionally be preceded by "![ \t]*" which will reverse the return
+ * value (i.e. immediately return 0 instead of 1) if the RE matches.
+ *
+ * RE may optionally be followed by ';message text', which of course must not
+ * contain a ':'
+ *
+ * WARNING: cannot diddle with "patterns" storage -- may be read-only string.
+ *
+ * Returns 1 if matched, else 0 (unless the matched RE was prefixed by '!', in
+ * which case it returns 0).
+ *
+ * if a match is found and if a sub-field accompanies the matching item then
+ * the *reasonp will be set to point to a copy of its value too.
+ */
+int
+match_re_list(str, patterns, icase, reasonp)
+    char *str;			/* string we are matching */
+    char *patterns;		/* list of patterns we are using to match it */
+    unsigned int icase;		/* ignore case? */
     char **reasonp;		/* optional pointer for returned message string */
 {
     char *pat_p;
     char *end_p;
-    unsigned int len;
+    size_t len;
     
+    if (reasonp) {
+	*reasonp = NULL;
+    }
     if (!patterns) {
 	return 0;
     }
 
     /* run through the pattern list */
     for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1) {
+	int matchresult = 1;
 
 	/* skip any spaces at front */
-	while (*pat_p && isspace(*pat_p)) {
-	       pat_p++;
+	while (*pat_p && isspace((int) *pat_p)) {
+	    pat_p++;
 	}
 	/* find the end of the next pattern */
 	end_p = pat_p;
-	while (*end_p && *end_p != fldsep && !isspace(*pat_p)) {
-	       end_p++;
+	while (*end_p && *end_p != ':' && !isspace((int) *pat_p)) {
+	    end_p++;
+	}
+	if (end_p > pat_p && *pat_p == '!') {
+	    pat_p++;
+	    matchresult = 0;		/* a match of a negated item... */
+	    /* skip any spaces after the '!'  */
+	    while (*pat_p && pat_p > end_p && isspace((int) *pat_p)) {
+		pat_p++;
+	    }
 	}
 	/* skip empty patterns */
 	len = end_p - pat_p;
 	if (len > 0) {
-	    char *curpat;
 	    char *p;
+	    static struct str exp;
+	    static int inited = FALSE;
 
-	    curpat = xmalloc(len + 1);
-	    strncpy(curpat, pat_p, len);
-	    curpat[len] = '\0';
-
-	    if (msgsep) {
-		if ((p = strchr(curpat, msgsep))) {
-		    *p = '\0';
-		    if (reasonp) {
-			*reasonp = COPY_STRING(p + 1);
-		    }
-		} else if (reasonp) {
-		    *reasonp = NULL;
+	    if (!inited) {
+		STR_INIT(&exp);
+		inited = TRUE;
+	    } else {
+		STR_LEN(&exp) = 0;
+	    }
+	    STR_NEXT(&exp, '^');		/* create a fully anchored RE pattern  */
+	    for (p = pat_p; len && *p != ';'; p++, len--) {
+		STR_NEXT(&exp, *p);
+	    }
+	    STR_CAT(&exp, "$");			/* NUL-terminates the string */
+	    switch (match_re(str, STR(&exp), icase, (char **) NULL, (char **) NULL, (char **) NULL, (char **) NULL)) {
+	    case 1:				/* ignore errors for now */
+		if (*p == ';' && reasonp) {
+		    /* note we're skipping the ';' itself */
+		    *reasonp = strncpy(xmalloc(len), p + 1, len);
+		    (*reasonp)[len - 1] = '\0';
 		}
+		DEBUG5(DBG_ADDR_HI,
+		       "string '%s' matches expression %s'%s'%s%s\n",
+		       str,
+		       matchresult ? "" : "!",
+		       STR(&exp),
+		       (reasonp && *reasonp) ? " with comment: " : "",
+		       (reasonp && *reasonp) ? *reasonp : "");
+		return matchresult;
+	    }
+	    /* continue with next pattern */
+	}
+	if (*end_p == '\0') {
+	    break;				/* if there is one */
+	}
+    }
+    DEBUG2(DBG_ADDR_HI, "string '%s' does not match in '%s'\n",
+	   str, patterns);
+    
+    return 0;
+}
+
+
+
+#ifdef HAVE_BSD_NETWORKING
+
+/*
+ * match_ip - match an ip against a list of host/net numbers
+ *
+ * Takes a list of IP:IP:IP and compares against an IP given in ASCII.
+ *
+ * 'IP' is an IP number or IP/bits CIDR network spec.
+ *
+ * IP may optionally be preceded by "![ \t]*" which will reverse the return
+ * value (i.e. immediately return 0 instead of 1) if the IP matches.
+ *
+ * IP may optionally be followed by ';message text', which of course must not
+ * contain a ':'
+ *
+ * Note that the field separator shown above as ':' is specified by the caller,
+ * as is message sub-field separator show above as ';'.  Normally the field
+ * separator will be either ':' or ',' and the sub-field separator will be ';'
+ * or '\0' (i.e. none).
+ *
+ * WARNING:  cannot diddle with pattern -- may be read-only string.
+ *
+ * RETURNS:  1 if matched, else 0 (unless the matched IP was prefixed by '!', in
+ * which case it returns 0).
+ *
+ * If a match is found and if a sub-field accompanies the matching item then
+ * the *reasonp will be set to point to a copy of its value too.
+ *
+ * WARNING:  There is no error reporting capability in this function's API.  If
+ * the pattern is mal-formed then in theory it can never be matched and in
+ * theory if a mal-formed pattern is negated with a '!' then 1 should be
+ * returned immediately.  However this is not currently the case since given
+ * the way this function is used such behaviour would most likely cause an
+ * uninentional wildcard match.
+ */
+int
+match_ip(ip, patterns, fldsep, msgsep, allow_wildcard, reasonp)
+    /*const*/ char *ip;		/* ip address we are matching */
+    /*const*/ char *patterns;	/* list of patterns we are using to match it */
+    int fldsep;			/* field separator character */
+    int msgsep;			/* optional message string separator */
+    int allow_wildcard;		/* allow 0/0? */
+    char **reasonp;		/* optional pointer for returned message string */
+{
+    /*const*/ char *pat_p;
+    /*const*/ char *end_p;
+    size_t len;
+    
+    if (reasonp) {
+	*reasonp = NULL;
+    }
+    if (!patterns) {
+	DEBUG1(DBG_ADDR_HI, "match_ip(): null patterns pointer, nothing to do for %s...\n", ip);
+	return 0;
+    }
+    DEBUG4(DBG_ADDR_HI, "match_ip(%s, %s, %c, %c): called...\n", ip, patterns, fldsep, msgsep);
+
+    /* run through the pattern list */
+    for (pat_p = patterns; /* NOTEST */ ; pat_p = end_p + 1) {
+	int matchresult = 1;
+
+	/* skip any spaces at front */
+	while (*pat_p && isspace((int) *pat_p)) {
+	    pat_p++;
+	}
+	/* find the end of the next pattern */
+	end_p = pat_p;
+	while (*end_p && *end_p != fldsep && !isspace((int) *pat_p)) {
+	    end_p++;
+	}
+	if (end_p > pat_p && *pat_p == '!') {
+	    pat_p++;
+	    matchresult = 0;		/* a match of a negated item... */
+	    /* skip any spaces after the '!'  */
+	    while (*pat_p && pat_p > end_p && isspace((int) *pat_p)) {
+		pat_p++;
+	    }
+	}
+	/* skip empty patterns */
+	len = end_p - pat_p;
+	if (len > 0) {
+	    char *p;
+	    static struct str exp;
+	    static int inited = FALSE;
+
+	    if (!inited) {
+		STR_INIT(&exp);
+		inited = TRUE;
+	    } else {
+		STR_LEN(&exp) = 0;
+	    }
+	    STR_NCAT(&exp, pat_p, len);
+	    STR_NEXT(&exp, '\0');
+	    p = NULL;
+	    if (msgsep && (p = strchr(STR(&exp), msgsep))) {
+		*p = '\0';
 	    }
 	    /* warn if we have a wild-card at end of the current pattern? */
 	    if (*(end_p - 1) == '*') {
-		write_log(WRITE_LOG_PANIC, "match_ip(): an unsupported wildcard pattern [%s] was found!", curpat);
+		write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC,
+			  "match_ip(): an unsupported wildcard pattern [%s] was found in '%s'!",
+			  STR(&exp), patterns);
 	    }
-	    if (smtp_local_net && strcmpic("localnet", curpat) == 0) {
+	    if (strcmpic("localnet", STR(&exp)) == 0) {
 		DEBUG2(DBG_ADDR_HI,
 		       "match_ip(): found 'localnet' in list...\n",
 		       ip,
 		       smtp_local_net);
 		/* our pattern will be the string in smtp_local_net */
-		if (match_ip_net(ip, smtp_local_net)) {
-		    DEBUG2(DBG_ADDR_MID,
-			   "ip [%s] matches localnet [%s]\n",
-			   ip, smtp_local_net);
-		    xfree(curpat);
-		    return 1;
+		if (!smtp_local_net) {
+		   DEBUG1(DBG_ADDR_HI, "match_ip(): nil smtp_local_net cannot match %s...\n", ip);
+		   /* continue with next pattern */
+		} else {
+		    switch (match_ip_net(ip, smtp_local_net, allow_wildcard)) {
+		    case 0:
+			break;		/* not this one! */
+		    case -1:
+			/* this should be almost impossible */
+			write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC,
+				  "match_ip(): invalid smtp_local_net [%s] was found for localnet in '%s'!",
+				  smtp_local_net, patterns);
+			break;
+		    default:
+			if (p && reasonp) {
+			    *reasonp = COPY_STRING(p + 1);
+			}
+			DEBUG5(DBG_ADDR_MID,
+			       "ip [%s] matches %slocalnet [%s]%s%s\n",
+			       ip,
+			       matchresult ? "" : "!",
+			       smtp_local_net,
+			       (reasonp && *reasonp) ? " with comment: " : "",
+			       (reasonp && *reasonp) ? *reasonp : "");
+			return matchresult;
+		    }
 		}
 		/* continue with next pattern */
 #ifdef HAVE_LIBWHOSON
-	    } else if (strcmpic("whoson", curpat) == 0) {
+	    } else if (strcmpic("whoson", STR(&exp)) == 0) {
 		int wso;
 		char retbuf[BUFSIZ];
 
@@ -1951,67 +2284,137 @@ match_ip(ip, patterns, fldsep, msgsep, reasonp)
 		    write_log(WRITE_LOG_PANIC, "wso_query(%s): whoson query failed: %s", ip, retbuf);
 		    /* continue with next pattern */
 		} else if (wso == 0) {
-		    DEBUG2(DBG_ADDR_MID, "ip [%s] found by whoson: %s.\n", ip, retbuf);
-		    xfree(curpat);
-		    return 1;
+		    DEBUG3(DBG_ADDR_MID, "ip [%s] found by whoson: %s%s.\n",
+			   ip,
+			   matchresult ? "" : "!",
+			   retbuf);
+		    return matchresult;
 		} else {
 		    DEBUG1(DBG_ADDR_MID, "ip [%s] NOT found by whoson\n", ip);
 		    /* continue with next pattern */
 		}
 #endif
 	    } else {
-		if (match_ip_net(ip, curpat)) {
-		    DEBUG2(DBG_ADDR_MID,
-			   "ip [%s] matches IP/network of [%s]\n",
-			   ip, curpat);
-		    xfree(curpat);
-		    return 1;
+		switch (match_ip_net(ip, STR(&exp), allow_wildcard)) {
+		case 0:
+		    break;	/* not this one! */
+		case -1:
+		    write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC,
+			      "match_ip(): invalid netspec [%s] was found in '%s'!",
+			      STR(&exp), patterns);
+		    /*
+		     * XXX  What if the only entry in a list is a mal-formed
+		     * pattern preceded by a '!'?
+		     */
+		    break;
+		default:
+		    if (p && reasonp) {
+			*reasonp = COPY_STRING(p + 1);
+		    }
+		    DEBUG5(DBG_ADDR_MID,
+			   "ip [%s] matches IP/network of %s[%s]%s%s\n",
+			   ip,
+			   matchresult ? "" : "!",
+			   STR(&exp),
+			   (reasonp && *reasonp) ? " with comment: " : "",
+			   (reasonp && *reasonp) ? *reasonp : "");
+		    return matchresult;
 		}
 		/* continue with next pattern */
 	    }
-	    xfree(curpat);
 	}
 	if (*end_p == '\0') {
 	    break;
 	}
     }
 
-    DEBUG2(DBG_ADDR_HI, "ip [%s] does not match any IP/network in '%s'\n",
+    DEBUG2(DBG_ADDR_HI, "match_ip(): [%s] does not match any IP/network in '%s'\n",
 	   ip, patterns);
     
     return 0;
 }
 
 int
-match_ip_net(ipaddr, netspec)
+match_ip_net(ipaddr, netspec, allow_wildcard)
     char *ipaddr;
     char *netspec;
+    int allow_wildcard;
 {
     unsigned long dest;
     unsigned long net;
     unsigned long mask;
-    int bits;
+    int bits = 0;
 
     DEBUG2(DBG_ADDR_HI, "match_ip_net([%s], [%s]) called...\n", ipaddr, netspec);
 
+#if 1 /* HAVE_INET_NET_PTON */
+    net = 0;			/* net must be pre-initialized */
     if ((bits = inet_net_pton(AF_INET, netspec, (void *) &net, sizeof(net))) == -1) {
 	DEBUG2(DBG_ADDR_LO, "match_ip_net(): [%s] is invalid: %s\n", netspec, strerror(errno));
 	return 0;
     }
+#else  /* BOGUS OLD CODE: */
+    char *slash;
+
+    /*
+     * inet_net_pton() is a lot smarter than this, properly handling inet(3)
+     * numbers in their full IPV4 (and IPV6) formats.
+     */
+    slash = strchr(netspec, '/');
+    if (slash) {
+	netspec = COPY_STRING(netspec);
+	slash = strchr(netspec, '/');		/* find again in copy */
+	*slash = '\0';
+	bits = atoi(slash + 1);
+    } else {
+	bits = 0;				/* compare whole addr. (XXX dumb) */
+    }
+    if ((net = inet_addr(netspec)) == INADDR_NONE) {
+	DEBUG1(DBG_ADDR_LO, "match_ip_net(): [%s] is invalid\n", netspec);
+	return 1;				/* malformed address */
+    }
+    if (slash) {
+	xfree(netspec);
+    }
+#endif
+    if (bits > 32 || bits < 0) {
+	panic(EX_CONFIG, "match_ip_net(): [%s] has invalid number of bits: %d\n", netspec, bits);
+	/* NOTREACHED */
+    }
     net = ntohl(net);
-    mask = 0xffffffffL << (32 - bits);
+    if (!bits && net) {
+	panic(EX_CONFIG, "match_ip_net(): wildcard [%s] has non-zero bits in network part\n", netspec);
+	/* NOTREACHED */
+    }
+    if (!bits && !allow_wildcard) {
+	return -1;
+    }
+    /* 
+     * Note: ((u_int32_t) << 32) is undefined according to the ANSI C spec.
+     * Hence the extra test for bits == 0.
+     */
+    mask = (bits == 0) ? 0 : (0xffffffffL << (32 - bits));
+
+    dest = 0;
     switch (inet_pton(AF_INET, ipaddr, &dest)) {
     case 0:
 	DEBUG1(DBG_ADDR_LO, "match_ip_net(): [%s] is not parsable.\n", ipaddr);
-	return 0;
+	return -1;
     case -1:
 	DEBUG2(DBG_ADDR_LO, "match_ip_net(): [%s] is invalid: %s\n", ipaddr, strerror(errno));
-	return 0;
+	return -1;
     }
     dest = ntohl(dest);
-#if 0 /* for heavy duty debugging only! */
+#if 0 /* for really heavy duty debugging only! */
     DEBUG4(DBG_ADDR_HI, "match_ip_net(): dest=0x%x, net=0x%x, bits=%d, mask=0x%x\n", dest, net, bits, mask);
 #endif
+    if (mask && (net & ~mask)) {
+#ifdef LOG_IP_NETSPEC_HAS_HOST_BITS
+	write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC, "match_ip_net(): warning: [%s] has non-zero bits in host part.\n", netspec);
+#else
+	DEBUG2(DBG_ADDR_MID, "match_ip_net(): warning: [%s] has non-zero bits in host part (0x%x).\n", netspec, net & ~mask);
+#endif
+    }
 
     return ((dest & mask) == (net & mask));
 }
@@ -2025,7 +2428,7 @@ flip_inet_addr(inaddr)
     unsigned long addr;		/* u_int32_t */
     char revaddr[(4*4)];
 
-    addr = inet_addr(inaddr);
+    addr = get_inet_addr(inaddr);
 
     addr = ntohl(addr);
 
@@ -2042,17 +2445,22 @@ flip_inet_addr(inaddr)
 
 #endif /* HAVE_BSD_NETWORKING */
 
+
 #ifdef STANDALONE
 
-int debug = 0;
+int debug = 1000;
 FILE *errfile = stderr;
 
-void
+char *smtp_local_net = "192.168.1/24";
+int return_to_sender = FALSE;
+
+char *program = "lookup";
+
+int
 main(argc, argv)
     int argc;
     char **argv;
 {
-    char *program = (--argc, *argv++);
     char *proto;
     char *name;
     char *db;
@@ -2061,6 +2469,8 @@ main(argc, argv)
     int retries = 0;
     int success;
 
+    argv++;				/* skip argv[0] */
+
     for (;;) {
 	if (*argv && EQ(*argv, "-r")) {
 	    argv++;
@@ -2079,22 +2489,23 @@ main(argc, argv)
 
     if (argc < 3) {
 	fprintf(stderr,
-		"Usage: %s [-r #retries] [-i interval] proto name key ...\n",
+		"Usage: %s [-r #retries] [-i interval] db-name db-proto key ...\n",
 		program);
 	exit(EX_USAGE);
     }
 
-    proto = *argv++;
     name = *argv++;
+    proto = *argv++;
 
     success = open_database(name, proto, retries, interval,
-			    (struct stat *)NULL, &db, &error);
+			    (struct stat *) NULL, &db, &error);
     switch (success) {
-    case DB_AGAIN:
+    case FILE_AGAIN:
 	fprintf(stderr, "%s: try again later: %s\n", program, error);
 	exit(EX_TEMPFAIL);
 
-    case DB_FAIL:
+    case FILE_FAIL:
+    case FILE_NOMATCH:
 	fprintf(stderr, "%s: open failed: %s\n", program, error);
 	exit(EX_UNAVAILABLE);
     }
@@ -2104,26 +2515,34 @@ main(argc, argv)
 
 	switch (lookup_database(db, *argv, &value, &error)) {
 	case DB_AGAIN:
-	    fprintf(stderr, "%s: %s: try again later: %s\n",
-		    program, *argv, error);
+	    fprintf(stderr, "%s: db error: %s: try again later: %s\n", program, *argv, error);
 	    break;
 
 	case DB_FAIL:
-	    fprintf(stderr, "%s: %s: failed: %s\n", program, *argv, error);
+	    fprintf(stderr, "%s: db error: %s: failed: %s\n", program, *argv, error);
 	    break;
 
 	case DB_NOMATCH:
-	    fprintf(stderr, "%s: %s: no match\n", program, *argv);
+	    fprintf(stderr, "%s: db error: %s: no match\n", program, *argv);
+	    break;
+
+	case FILE_AGAIN:
+	    fprintf(stderr, "%s: file error: %s: try again later: %s\n", program, *argv, error);
+	    break;
+
+	case FILE_FAIL:
+	    fprintf(stderr, "%s: file error: %s: failed: %s\n", program, *argv, error);
 	    break;
 
 	case DB_SUCCEED:
-	    printf("%s --> %s", *argv, value);
+	    printf("%s --> %s\n", *argv, value);
 	    break;
 	}
 	argv++;
     }
 
     close_database(db);
+
     exit(EX_OK);
 }
 
