@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:lookup.h,v 1.4 2000/09/03 16:26:31 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:lookup.h,v 1.10 2003/04/21 21:56:30 woods Exp"
  */
 
 /*
@@ -25,10 +25,15 @@
 #define FILE_AGAIN	(-5)		/* try using database later */
 #define FILE_NOMATCH	(-6)		/* no such file */
 
+/* WARNING: bindlib.c defines DB_NEGMATCH as (-8) */
+
 /* external functions defined in lookup.c */
 extern int open_database __P((char *, char *, int, unsigned int, struct stat *, char **, char **));
 extern void close_database __P((char *));
 extern int lookup_database __P((char *, char *, char **, char **));
-extern int match_ip __P((char *, char *, int, int, char **));
-extern int match_ip_net __P((char *, char *));
+extern int match_re __P((char *, char *, unsigned int, char **, char **, char **, char **));
+extern int match_re_list __P((char *, char *, unsigned int, char **));
+extern int match_hostname __P((char *, char *, char **));
+extern int match_ip __P((char *, char *, int, int, int, char **));
+extern int match_ip_net __P((char *, char *, int));
 extern char *flip_inet_addr __P((char *));
