@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:main.c,v 1.70 2001/07/29 16:10:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:main.c,v 1.91 2003/06/10 00:24:18 woods Exp"
  */
 
 /*
@@ -17,39 +17,80 @@
  *
  *	external functions: main, initialize_state, process_args
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <limits.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
 #include <pwd.h>
 #include <signal.h>
-#define NEED_SOCKETS			/* Required for _PATH_RESCONF from <resolv.h> */
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-#if defined(POSIX_OS) || !defined(UNIX_BSD)
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# define _GNU_SOURCE			/* to see decl. of strsignal() */
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
+# include <sys/wait.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# include <sys/time.h>
-#endif	/* UNIX_BSD */
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
 #if defined(HAVE_RLIMIT)
 # include <sys/resource.h>
+# ifndef RLIM_T_DECLARED
+typedef int rlim_t;
+# endif
 #endif	/* HAVE_RLIMIT */
+
 #ifdef	UNIX_AIX3
 # include <sys/id.h>
 #endif	/* UNIX_AIX3 */
-#if defined(POSIX_OS)
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
-#include "config.h"
+#endif
+
 #include "smail.h"
+#include "config.h"
+#include "smailsock.h"
+#include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "parse.h"
 #include "addr.h"
@@ -59,24 +100,14 @@
 #include "direct.h"
 #include "route.h"
 #include "transport.h"
+#include "smailwait.h"
 #include "child.h"
 #include "exitcodes.h"
 #include "smailconf.h"
-#include "alloc.h"
 #include "extern.h"
 #include "debug.h"
 #include "error.h"
-
-#ifdef HAVE_BSD_NETWORKING
-# ifndef OBSOLETE_RESOLVER
-extern int h_errno;			/* import h_errno; many systems don't define it in <netdb.h> */
-# endif
-# ifdef HAVE_BIND
-#  ifndef _PATH_RESCONF
-#   define _PATH_RESCONF	"/etc/resolv.conf"
-#  endif
-# endif
-#endif
+#include "smailport.h"
 
 /* exported variables */
 int islocal;				/* TRUE if mail originated locally */
@@ -103,6 +134,7 @@ struct addr *recipients;		/* list of recipient addresses */
 int num_recipients = 0;			/* number of entries in 'recipients' */
 char *primary_name;			/* primary local name from hostnames */
 FILE *errfile = NULL;			/* file to write debug messages to */
+int only_testing = 0;			/* avoid writing sys/panic logs */
 unsigned int real_uid;			/* saved real uid before ruid setup */
 unsigned int real_gid;			/* saved real gid before rgid setup */
 enum prog_type prog_type;		/* type of program we are running as */
@@ -201,21 +233,6 @@ main(argc, argv)
 	(void) setuidx(ID_LOGIN, 0);
 #endif	/* UNIX_AIX3 */
 
-#ifdef	HAVE_SETGROUPS
-    /* clear out all extra groups.  We don't want to have to deal with them */
-    {
-	gid_t dummy_grouplist[1];
-	dummy_grouplist[0] = 0;
-
-	if (setgroups(0, (gid_t *) NULL) != 0) {
-	    if (setgroups(1, dummy_grouplist) != 0) {
-		write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_PANIC : 0),
-			  "setgroups() failed: %s", strerror(errno));
-	    }
-	}
-    }
-#endif	/* HAVE_SETGROUPS */
-
     /* get rid of any limits that might affect operation */
 #if defined(HAVE_ULIMIT) && !defined(HAVE_RLIMIT) /* XXX HAVE_SETRLIMIT */
     /* kill limits on file size */
@@ -236,13 +253,13 @@ main(argc, argv)
     {
 	struct rlimit rl;
 
-	rl.rlim_cur = RLIM_INFINITY;
-	rl.rlim_max = RLIM_INFINITY;
+	rl.rlim_cur = (rlim_t) RLIM_INFINITY;
+	rl.rlim_max = (rlim_t) RLIM_INFINITY;
 	(void) setrlimit(RLIMIT_CPU, &rl);
 	(void) setrlimit(RLIMIT_FSIZE, &rl);
 #if defined(DATA_RLIMIT)			/* usually in conf/os/OS_TYPE */
 	rl.rlim_cur = DATA_RLIMIT;
-#elif defined (SMALL_MEMORY)
+#elif defined(SMALL_MEMORY)
 	rl.rlim_cur = (4 * 1024 * 1024);
 #else
 	rl.rlim_cur = (64 * 1024 * 1024);	/* generous, aren't we! */
@@ -250,7 +267,7 @@ main(argc, argv)
 	(void) setrlimit(RLIMIT_DATA, &rl);
 #if defined(STACK_RLIMIT)			/* usually in conf/os/OS_TYPE */
 	rl.rlim_cur = STACK_RLIMIT;
-#elif defined (SMALL_MEMORY)
+#elif defined(SMALL_MEMORY)
 	rl.rlim_cur = (1024 * 1024);
 #else
 	rl.rlim_cur = (8 * 1024 * 1024);	/* generous, aren't we! */
@@ -283,8 +300,11 @@ main(argc, argv)
     }
 #endif
 
-    /* we will always be supplying exactly the mode we want */
-    (void) umask(0);
+    /*
+     * we will always be supplying exactly the mode we want, but just in case
+     * we'd better not be giving away write permission anywhere....
+     */
+    (void) umask(022);			/* S_IWGRP | S_IWOTH */
 
     /* set the program type based on the program's basename */
     prog_type = PROG_SMAIL;
@@ -385,6 +405,21 @@ main(argc, argv)
 	}
     }
 
+#ifdef	HAVE_SETGROUPS
+    /* clear out all extra groups.  We don't want to have to deal with them */
+    {
+	gid_t dummy_grouplist[1];
+	dummy_grouplist[0] = 0;
+
+	if (setgroups(0, (gid_t *) NULL) != 0) {
+	    if (setgroups(1, dummy_grouplist) != 0) {
+		write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_PANIC : 0),
+			  "setgroups() failed: %s", strerror(errno));
+	    }
+	}
+    }
+#endif	/* HAVE_SETGROUPS */
+
     if (config_file != save_config_file || arg_second_config_file ||
 	arg_director_file || arg_router_file || arg_transport_file ||
 	arg_qualify_file || arg_retry_file || arg_smail_lib_dir ||
@@ -733,12 +768,12 @@ main(argc, argv)
 	verify_addresses();
 	break;
 
-    case SMTP_MODE:			/* read SMTP requests on stdin */
-	smtp_mode(stdin, stdout);
+    case SMTP_MODE:			/* read interactive SMTP requests on stdin/stdout */
+	smtp_mode(stdin, stdout, (void *) NULL);
 	break;
 
     case BATCH_SMTP_MODE:		/* batched SMTP requests on stdin */
-	smtp_mode(stdin, (FILE *)NULL);
+	smtp_mode(stdin, (FILE *) NULL, (void *) NULL);
 	break;
 
     case DAEMON_MODE:			/* be a daemon waiting for requests */
@@ -794,6 +829,7 @@ main(argc, argv)
 
     case REBUILD_ALIASES: {
 	int status;
+	int lastarg;
 
 	if (smail_util_dir == NULL) {
 	    if (errfile) {
@@ -801,26 +837,36 @@ main(argc, argv)
 	    }
 	    exit(EX_UNAVAILABLE);
 	}
-#ifdef SHELL_EXEC_PATH
+#if !defined(HAVE_HASH_BANG)
+# ifdef SHELL_EXEC_PATH
 	utilargs[0] = SHELL_EXEC_PATH;
-#else
+# else
 	utilargs[0] = "/bin/sh";
-#endif
+# endif
 	utilargs[1] = xprintf("%s/mkaliases", smail_util_dir);
+	lastarg = 2;
+#else  /* HAVE_HASH_BANG */
+	utilargs[0] = xprintf("%s/mkaliases", smail_util_dir);
+	utilargs[1] = NULL;
+	lastarg = 1;
+#endif
 	utilargs[2] = NULL;
 	utilargs[3] = NULL;
 	utilargs[4] = NULL;
 	if (debug) {
-	    static char debug_arg[30];	/* space for max int digits (2^(CHAR_BIT * sizeof(int)) = 10^digits) */
+	    static char debug_arg[MAXINT_B10_DIGITS + 4];
 
 	    sprintf(debug_arg, "-v %d", debug);
-	    utilargs[2] = debug_arg;
+	    utilargs[lastarg++] = debug_arg;
 	}
 	if (arg_alias_file) {
-	    utilargs[utilargs[2] ? 3 : 2] = arg_alias_file;
+	    utilargs[lastarg++] = arg_alias_file;
 	}
 	DEBUG4(DBG_MAIN_MID, "main: about to run %s %s %s %s\n",
-	       utilargs[0], utilargs[1], utilargs[2] ? utilargs[2] : "", utilargs[3] ? utilargs[3] : "");
+	       utilargs[0],
+	       utilargs[1] ? utilargs[1] : "",
+	       utilargs[2] ? utilargs[2] : "",
+	       utilargs[3] ? utilargs[3] : "");
 	child = open_child(utilargs, (char **) NULL, (FILE **) NULL, (FILE **) NULL, fileno(stderr),
 			   CHILD_MINENV, (unsigned int) getuid(), (unsigned int) getgid());
 	if (child == -1) {
@@ -832,19 +878,38 @@ main(argc, argv)
 	}
 	if ((status = close_child((FILE *) NULL, (FILE *) NULL, child)) != 0) {
 	    if (status == -1) {
-		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: failed to reap child process from %s: %s.",
-			  program, utilargs[0], strerror(errno));
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: failed to reap child process %d from %s: %s.",
+			  program, child, utilargs[0], strerror(errno));
 		exit(EX_OSERR);
-	    } else if (status & 0xff) {
-		char *errstr = xprintf("%s: %s killed by signal %d",
-				       program, utilargs[0], status & 0x7f);
-		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, errstr);
+	    } else if (WIFEXITED(status)) {
+		if (WEXITSTATUS(status) != 0) {
+		    write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: %s[%d] returned status %s (%d)",
+			      program, utilargs[0], child,
+			      strsysexit(WEXITSTATUS(status)), WEXITSTATUS(status));
+		    exit(EX_DATAERR);
+		}
+	    } else if (WIFSIGNALED(status)) {
+		char signm[SIG2STR_MAX];
+
+		if (sig2str(WTERMSIG(status), signm) == -1) {
+		    sprintf(signm, "#%d", WTERMSIG(status));
+		}
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: %s[%d] killed by signal SIG%s %s: %s",
+			  program, utilargs[0], child,
+			  signm, WCOREDUMP(status) ? "and dumped core" : "(no core)",
+			  strsignal(WTERMSIG(status)));
+		exit(EX_UNAVAILABLE);
+	    } else if (WIFSTOPPED(status)) {
+		char signm[SIG2STR_MAX];
+
+		/* in theory we'll hopefully never see stopped processes... */
+		if (sig2str(WTERMSIG(status), signm) == -1) {
+		    sprintf(signm, "#%d", WSTOPSIG(status));
+		}
+		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, "%s: %s[%d] stopped unexpectedly by signal SIG%s: %s",
+			  program, utilargs[0], child,
+			  signm, strsignal(WSTOPSIG(status)));
 		exit(EX_UNAVAILABLE);
-	    } else {
-		char *errstr = xprintf("%s: %s returned status %s (%d)",
-				       program, utilargs[0], strsysexit(status>>8), status>>8);
-		write_log(WRITE_LOG_SYS|WRITE_LOG_TTY, errstr);
-		exit(EX_DATAERR);
 	    }
 	}
 	exit(0);
@@ -862,14 +927,14 @@ main(argc, argv)
      */
     if (report_memory_usage) {
 	if (errfile) {
-	    (void) fprintf(errfile, "%s: sbrk(0) = %ld\n",
-			   program, (long)sbrk(0));
+#if 0
+	    assert(sizeof(unsigned long) >= sizeof(void *));
+#endif
+	    (void) fprintf(errfile, "%s: sbrk(0) = %lu\n",
+			   program, (unsigned long) ((char *) sbrk((size_t /* intptr_t */) 0)));
 	}
     }
-    if (force_zero_exitvalue) {
-	exit(0);
-    }
-    exit(exitvalue);
+    exit(force_zero_exitvalue ? 0 : exitvalue);
     /* NOTREACHED */
 }
 
@@ -945,7 +1010,8 @@ process_args(args, restrict)
     char *arg;				/* single string from args */
     char *error;			/* error message */
     char *newsender;			/* temp for process sender */
-    static char *end_arg = "";		/* swallow remaining chars in arg */
+    int do_options = TRUE;		/* set TRUE until we see '--' */
+    static char *end_arg = "";		/* trigger to swallow rest of arg */
 
     /*
      * go through the list of arguments in search of options and
@@ -954,12 +1020,183 @@ process_args(args, restrict)
     while ((arg = *args++)) {
 
 	/* option arguments begin with '-', of course */
-	if (arg[0] == '-') {
+	if (arg[0] == '-' && do_options == TRUE) {
 
 	    /* switch on each letter */
 	    arg++;
 	    while (*arg) switch (*arg++) {
 
+	    case '-':
+		do_options = FALSE;
+		if (arg[0]) {
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: '--' option must stand alone.\n", program);
+		    }
+		    exit(EX_USAGE);
+		}
+		arg = end_arg;
+		break;
+
+	    case 'C':			/* set config file name */
+		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+		    rmail_panic();
+		}
+		config_file = arg;
+		arg = end_arg;		/* terminate args in current argv */
+		/* if no string there, take next arg */
+		if (config_file[0] == '\0') {
+		    config_file = *args++;
+		    panic_if_null(config_file, "C");
+		}
+		break;
+
+	    case 'D':                   /* set debugging output file */
+		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+		    rmail_panic();
+		}
+		arg_debug_file = arg;
+		arg = end_arg;
+		if (arg_debug_file[0] == '\0') {
+		    arg_debug_file = *args++;
+		    panic_if_null(arg_debug_file, "D");
+		}
+		if (debug == 0)
+		    debug = 1;
+		break;
+
+	    case 'E':
+		scan_frozen = TRUE;
+		break;
+
+	    case 'I':			/* use hidden-dot protocol on input */
+		dot_usage = HIDDEN_DOTS;
+		break;
+
+	    /* XXX sendmail now uses '-N dsn' to set DNS condition */
+	    case 'N':			/* don't deliver message */
+		dont_deliver = TRUE;
+		break;
+
+	    case 'Q':
+		queue_only = TRUE;	/* spool but do not deliver, yet */
+		break;
+
+	    case 'V':
+		operation_mode = PRINT_VERSION;
+		break;
+
+	    case 'F':			/* set full name of sender */
+		if ((prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) && restrict) {
+		    rmail_panic();
+		}
+		sender_name = arg;
+		arg = end_arg;		/* terminate args in current argv */
+		/* if no string there, take next arg */
+		if (sender_name[0] == '\0') {
+		    sender_name = *args++;
+		    panic_if_null(sender_name, "F");
+		}
+		break;
+
+	    case 'b':			/* set operating mode */
+		if (*arg == '\0') {
+		    arg = *args++;
+		    panic_if_null(arg, "b");
+		}
+		switch (*arg) {
+
+		case 'P':
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = PRINT_VARS_MODE;
+		    break;
+
+	        case 'R':		/* rogue tombstone mode */
+		    operation_mode = ROGUE_MODE;
+		    break;
+
+		case 'S':		/* batched SMTP mode */
+		    operation_mode = BATCH_SMTP_MODE;
+		    break;
+
+		case 'T':		/* header test mode mode */
+#if 0
+		    operation_mode = TEST_MODE;
+		    test_headers = TRUE;
+#else
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: test headers mode not yet supported\n", program, arg);
+		    }
+		    exit(2);
+#endif
+		    break;
+
+		case 'V':
+		    operation_mode = PRINT_VERSION;
+		    break;
+
+		case 'c':		/* print COPYING file */
+		    operation_mode = COPYING_MODE;
+		    break;
+
+		case 'd':		/* operate as daemon */
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = DAEMON_MODE;
+		    break;
+
+		case 'i':		/* initialize aliases database */
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = REBUILD_ALIASES;
+		    break;
+
+		case 'm':		/* just deliver mail */
+		    operation_mode = DELIVER_MAIL;
+		    break;
+
+		case 'p':		/* print the queue */
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = PRINT_QUEUE;
+		    break;
+
+		case 's':		/* process smtp on input */
+		    operation_mode = SMTP_MODE;
+		    break;
+
+		case 't':		/* run in address test mode */
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = TEST_MODE;
+		    break;
+
+		case 'v':		/* verify addresses only */
+		    operation_mode = VERIFY_ADDRS;
+		    break;
+
+		case 'z':		/* freeze config file */
+		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+			rmail_panic();
+		    }
+		    operation_mode = FREEZE_CONFIG;
+		    break;
+
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-b' suboption\n", program, arg);
+		    }
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
+		}
+		arg = end_arg;
+		break;
+
 	    case 'd':			/* set debug level */
 	    case 'v':			/* verbose, same as debug */
 		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
@@ -983,46 +1220,34 @@ process_args(args, restrict)
 		}
 		break;
 
-	    case 'D':                   /* set debugging output file */
-		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-		    rmail_panic();
+	    case 'e':			/* what to do on errors */
+		if (arg[0] == '\0') {
+		    arg = *args++;
+		    panic_if_null(arg, "e");
 		}
-		arg_debug_file = arg;
-		arg = end_arg;
-		if (arg_debug_file[0] == '\0') {
-		    arg_debug_file = *args++;
-		    panic_if_null(arg_debug_file, "D");
-		}
-		if (debug == 0)
-		    debug = 1;
-		break;
-
-	    case 'E':
-		scan_frozen = TRUE;
-		break;
-
-	    case 'V':
-		operation_mode = PRINT_VERSION;
-		break;
-
-	    case 'F':			/* set full name of sender */
-		if ((prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) && restrict) {
-		    rmail_panic();
-		}
-		sender_name = arg;
-		arg = end_arg;		/* terminate args in current argv */
-		/* if no string there, take next arg */
-		if (sender_name[0] == '\0') {
-		    sender_name = *args++;
-		    panic_if_null(sender_name, "F");
-		}
-		break;
-
-	    case 'r':			/* set path to sender */
-		/* SCO Execmail '-r' option not applicable to smail */
-		if (prog_type == PROG_EXECMAIL)
+		switch (*arg) {
+		case 'e':		/* we don't support berkenet */
+		case 'm':		/* mail back errors */
+		    error_processing = MAIL_BACK;
 		    break;
-		/* FALLTHRU */
+		case 'p':		/* print errors on screen */
+		    error_processing = TERMINAL;
+		    break;
+		case 'q':		/* be quiet about errors */
+		    error_processing = DEV_NULL;
+		    break;
+		case 'w':		/* send via "write" */
+		    error_processing = WRITE_BACK;
+		    break;
+		default:
+		    if (errfile) {
+			(void) fprintf(errfile, "%s: %s: invalid '-e' suboption\n", program, arg);
+		    }
+		    exit(EX_USAGE);
+		    /* NOTREACHED */
+		}
+		arg = end_arg;		/* swallows complete argument */
+		break;
 
 	    case 'f':			/* set path to sender */
 		if ((prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) && restrict) {
@@ -1061,7 +1286,7 @@ process_args(args, restrict)
 
 			c = sender + strlen(sender) - 1;
 			if (*c == '>') {
-			    *c = '\0';
+			    *c = '\0';		/* XXX FIXME: is arg writable? */
 			    sender++;
 			}
 		    } else {
@@ -1082,34 +1307,6 @@ process_args(args, restrict)
 		}
 		break;
 
-	    case 'N':			/* don't deliver message */
-		dont_deliver = TRUE;
-		break;
-
-	    case 'C':			/* set config file name */
-		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-		    rmail_panic();
-		}
-		config_file = arg;
-		arg = end_arg;		/* terminate args in current argv */
-		/* if no string there, take next arg */
-		if (config_file[0] == '\0') {
-		    config_file = *args++;
-		    panic_if_null(config_file, "C");
-		}
-		break;
-
-	    case 'q':			/* check queue (at interval) */
-		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-		    rmail_panic();
-		}
-		process_queue = TRUE;
-		if (arg[0]) {
-		    queue_interval =  xivaltou(arg);
-		    arg = end_arg;	/* uses rest of argument */
-		}
-		break;
-
 	    case 'h':			/* hopcount, number is the count */
 	        {
 		    char *errptr = NULL;
@@ -1134,60 +1331,19 @@ process_args(args, restrict)
 		}
 		break;
 
-	    case 'n':			/* don't do aliasing */
-		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-		    rmail_panic();
-		}
-		do_aliasing = FALSE;
-		break;
-
-	    case 't':			/* read recipients from message */
-		extract_addresses = TRUE;
-		break;
-
 	    case 'i':			/* don't treat dots specially */
 		dot_usage = NO_DOT_PROTOCOL;
 		break;
 
-	    case 'I':			/* use hidden-dot protocol on input */
-		dot_usage = HIDDEN_DOTS;
-		break;
-
 	    case 'm':			/* author can be included in alias */
 		me_too = TRUE;
 		break;
 
-	    case 'Q':
-		queue_only = TRUE;	/* spool but do not deliver, yet */
-		break;
-
-	    case 'e':			/* what to do on errors */
-		if (arg[0] == '\0') {
-		    arg = *args++;
-		    panic_if_null(arg, "e");
+	    case 'n':			/* don't do aliasing */
+		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+		    rmail_panic();
 		}
-		switch (*arg) {
-		case 'e':		/* we don't support berkenet */
-		case 'm':		/* mail back errors */
-		    error_processing = MAIL_BACK;
-		    break;
-		case 'w':		/* send via "write" */
-		    error_processing = WRITE_BACK;
-		    break;
-		case 'p':		/* print errors on screen */
-		    error_processing = TERMINAL;
-		    break;
-		case 'q':		/* be quiet about errors */
-		    error_processing = DEV_NULL;
-		    break;
-		default:
-		    if (errfile) {
-			(void) fprintf(errfile, "%s: %s: invalid '-e' suboption\n", program, arg);
-		    }
-		    exit(EX_USAGE);
-		    /* NOTREACHED */
-		}
-		arg = end_arg;		/* swallows complete argument */
+		do_aliasing = FALSE;
 		break;
 
 	    case 'o':			/* set various option */
@@ -1273,12 +1429,13 @@ process_args(args, restrict)
 			panic_if_null(arg, "oM");
 		    }
 		    switch (*arg++) {
-		    case 's':
+
+		    case 'P':
 			if (*arg == '\0') {
 			    arg = *args++;
-			    panic_if_null(arg, "oMs");
+			    panic_if_null(arg, "oMP");
 			}
-			sender_host = arg;
+			sender_program = arg;
 			break;
 
 		    case 'a':
@@ -1297,6 +1454,14 @@ process_args(args, restrict)
 			sender_proto = arg;
 			break;
 
+		    case 's':
+			if (*arg == '\0') {
+			    arg = *args++;
+			    panic_if_null(arg, "oMs");
+			}
+			sender_host = arg;
+			break;
+
 		    case 'u':
 			if (*arg == '\0') {
 			    arg = *args++;
@@ -1313,13 +1478,6 @@ process_args(args, restrict)
 			ident_method = arg;
 			break;
 
-		    case 'P':
-			if (*arg == '\0') {
-			    arg = *args++;
-			    panic_if_null(arg, "oMP");
-			}
-			sender_program = arg;
-			break;
 		    default:
 			if (errfile) {
 			    (void) fprintf(errfile, "%s: %s: invalid '-oM' suboption\n", program, --arg);
@@ -1374,11 +1532,13 @@ process_args(args, restrict)
 		    break;
 
 		case 'U':
-		    fprintf(stderr, "warning: this option will be changed from -oU to something else in the next release.\n");
+		    if (errfile) {
+			fprintf(errfile, "warning: this option will be changed from -oU to something else in the next release.\n");
+		    }
 		    report_memory_usage = TRUE;
 		    break;
 
-#if notyet
+#ifdef notyet
 		case 'U':
 		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
 			rmail_panic();
@@ -1415,17 +1575,14 @@ process_args(args, restrict)
 		    case 'b':
 			deliver_mode = BACKGROUND;
 			break;
-
 		    case 'f':
 		    case 'i':		/* sendmail "interactive delivery" */
 			deliver_mode = FOREGROUND;
 			break;
-
 		    case 'd':		/* sendmail "defer map lookups as well as queue" */
 		    case 'q':
 			deliver_mode = QUEUE_MESSAGE;
 			break;
-
 		    default:
 			if (errfile) {
 			    (void) fprintf(errfile, "%s: %s: invalid '-od' suboption\n", program, arg);
@@ -1441,7 +1598,6 @@ process_args(args, restrict)
 			panic_if_null(arg, "oe");
 		    }
 		    switch (*arg) {
-
 		    case 'e':		/* used by 4.4bsd rmail for: no errors, just status */
 		    case 'm':
 			error_processing = MAIL_BACK;
@@ -1494,90 +1650,29 @@ process_args(args, restrict)
 		arg = end_arg;
 		break;
 
-	    case 'b':			/* set operating mode */
-		if (*arg == '\0') {
-		    arg = *args++;
-		    panic_if_null(arg, "b");
+	    case 'q':			/* check queue (at interval) */
+		if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
+		    rmail_panic();
 		}
-		switch (*arg) {
-
-		case 'd':		/* operate as daemon */
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = DAEMON_MODE;
-		    break;
-
-		case 'i':		/* initialize aliases database */
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = REBUILD_ALIASES;
-		    break;
-
-		case 'm':		/* just deliver mail */
-		    operation_mode = DELIVER_MAIL;
-		    break;
-
-		case 'p':		/* print the queue */
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = PRINT_QUEUE;
-		    break;
-
-		case 't':		/* run in address test mode */
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = TEST_MODE;
-		    break;
-
-		case 'v':		/* verify addresses only */
-		    operation_mode = VERIFY_ADDRS;
-		    break;
-
-		case 'V':
-		    operation_mode = PRINT_VERSION;
-		    break;
-
-		case 's':		/* process smtp on input */
-		    operation_mode = SMTP_MODE;
-		    break;
-
-		case 'S':		/* batched SMTP mode */
-		    operation_mode = BATCH_SMTP_MODE;
-		    break;
-
-		case 'z':		/* freeze config file */
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = FREEZE_CONFIG;
-		    break;
-
-	        case 'R':		/* rogue tombstone mode */
-		    operation_mode = ROGUE_MODE;
-		    break;
-
-		case 'c':		/* print COPYING file */
-		    operation_mode = COPYING_MODE;
-		    break;
-
-		case 'P':
-		    if (prog_type == PROG_RMAIL || prog_type == PROG_RSMTP) {
-			rmail_panic();
-		    }
-		    operation_mode = PRINT_VARS_MODE;
-		    break;
-		default:
-		    if (errfile) {
-			(void) fprintf(errfile, "%s: %s: invalid '-b' suboption\n", program, arg);
-		    }
-		    exit(EX_USAGE);
-		    /* NOTREACHED */
+		process_queue = TRUE;
+		if (arg[0]) {
+		    queue_interval =  xivaltou(arg);
+		    arg = end_arg;	/* uses rest of argument */
 		}
-		arg = end_arg;
+		break;
+
+	    case 'r':			/* set path to sender */
+		/* SCO Execmail '-r' option not applicable to smail */
+		if (prog_type == PROG_EXECMAIL)
+		    break;
+		/* FALLTHRU */
+
+	    case 's':
+		mailq_summary_only = TRUE;
+		break;
+
+	    case 't':			/* read recipients from message */
+		extract_addresses = TRUE;
 		break;
 
 	    case 'x':			/* special internal hacks */
@@ -1650,7 +1745,7 @@ xivaltou(arg)
 	}
 	exit(EX_USAGE);
     }
-    if (l > UINT_MAX) {			/* if sizeof(long) != sizeof(unsigned int) */
+    if ((unsigned int) l > UINT_MAX) { /* if sizeof(long) != sizeof(unsigned int) */
 	if (errfile) {
 	    (void) fprintf(errfile, "%s: %s: interval is too large\n", program, arg);
 	}
@@ -1668,20 +1763,22 @@ escape_newline(s)
     register struct str *sp = &str;
     register int c;
 
-    if (strchr(s, '\n') == NULL)
+    if (strchr(s, '\n') == NULL) {
 	return s;
+    }
     STR_INIT(sp);
 
     while ((c = *s++)) {
-	if (c == '\n')
+	if (c == '\n') {
 	    STR_CAT(sp, "\\n");
-	else
+	} else {
 	    STR_NEXT(sp, c);
+	}
     }
     STR_NEXT(sp, '\0');
     STR_DONE(sp);
 
-    return sp->p;
+    return STR(sp);
 }
 
 /*
@@ -1731,14 +1828,14 @@ parse_grade_range(range, min, max)
     mn = 0;
     mx = 255;
     if (range != NULL) {
-	if (isalnum(*range)) {
+	if (isalnum((int) *range)) {
 	    mn = *range++;
 	    mx = mn;
 	}
 	if (*range == '-') {
 	    range++;
 	    mx = 255;
-	    if (isalnum(*range)) {
+	    if (isalnum((int) *range)) {
 		mx = *range;
 	    }
 	}
