@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:mkdrivtab.sh,v 1.15 2000/09/03 16:35:06 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:mkdrivtab.sh,v 1.20 2003/06/11 20:56:54 woods Exp"
 
 # Script to create drivertab.c and the driver makefile directories
 # from a driver configuration file.  This script makes heavy use of
@@ -114,7 +114,7 @@ sed -e 's/#.*//' -e '/^[ 	]*$/d' |
 		echo "Build drivertab.c ..." 1>&2
 	fi
 	outfile=drivertab.c
-	cat <<EOF > $outfile
+	cat <<__EOF__ > $outfile
 /*
  * drivertab.c:
  *	define the available director, router, transport and lookup
@@ -134,16 +134,20 @@ sed -e 's/#.*//' -e '/^[ 	]*$/d' |
  * THIS FILE DIRECTLY.
  */
 
-#include <stdio.h>
 #include "defs.h"
+
+#include <sys/types.h>
+#include <stdio.h>
+
 #include "smail.h"
+#include "list.h"
 #include "parse.h"
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
 #include "transport.h"
 
-EOF
+__EOF__
 
 	while read a b; do
 		case "$a" in
@@ -155,7 +159,7 @@ EOF
 			else
 			    echo "Build $outfile ..." 1>&2
 			fi
-			cat <<EOF > $outfile
+			cat <<__EOF__ > $outfile
 # Makefile for the $b driver library in smail
 
 # THIS FILE IS GENERATED AUTOMATICALLY BY THE SCRIPT $0 FROM
@@ -163,13 +167,9 @@ EOF
 # MAKE CHANGES TO THE DRIVER CONF FILE AND REBUILD RATHER THAN EDITING
 
 SHELL=/bin/sh
-MAKE=make
-GET=sccs get
 AR=ar
 LINT=lint
 CC=cc
-CLEAN=sccs clean
-SRC_PREFIX=
 ROOT=../..
 MKDEPEND=\${ROOT}/conf/lib/mkdepend.sh
 MKDEFS=\${ROOT}/conf/lib/mkdefs.sh
@@ -179,64 +179,64 @@ DEFS_SH=defs.sh
 DEFS_H=defs.h
 DEFS_SED=defs.sed
 
-EOF
+__EOF__
 			;;
 		%makefile_end)
-			cat <<\EOF >> $outfile
+			cat <<\__EOF__ >> $outfile
 SRC=${CSRC} ${HSRC}
 
 .c.o:
 	@. ./${DEFS_SH}; ${XEXEC} $$CC $$CFLAGS $$INCLUDES -c $*.c
 
-all:	${TARGET}
+all:	${TARGET_LIB}
 
 lint:	${CSRC} ${DEFS_SH}
 	@. ./${DEFS_SH}; ${XEXEC} ${LINT} ${LINTFLAGS} ${CSRC}
 
-${TARGET}: ${OBJ} Makefile ${DEFS_SH}
-	rm -f ${TARGET}
-	${AR} cr ${TARGET} ${OBJ}
-	@. ./${DEFS_SH}; ${XEXEC} $$RANLIB ${TARGET}
+${TARGET_LIB}: ${OBJ} Makefile ${DEFS_SH}
+	rm -f ${TARGET_LIB}
+	${AR} cr ${TARGET_LIB} ${OBJ}
+	@. ./${DEFS_SH}; ${XEXEC} $$RANLIB ${TARGET_LIB}
 
-csrc:;	@echo ${CSRC}
-hsrc:;	@echo ${HSRC}
+csrc:
+	@echo ${CSRC}
+
+hsrc:
+	@echo ${HSRC}
 
 sources: ${SRC}
 
-${SRC}:
-	${GET} $@
-
 mkdefs ${DEFS_H} ${DEFS_SH} ${DEFS_SED}: ${ROOT}/conf/EDITME
 	ROOT=${ROOT} ${SHELL} ${MKDEFS}
 
 ${OBJ}:	${DEFS_SH}
 
-names:
-	@for i in ${SRC}; do echo ${SRC_PREFIX}$$i; done
-
 depend:	${SRC} check_defs
-	@. ./${DEFS_SH}; ${XEXEC} ${SHELL} ${MKDEPEND} $$CPPFLAGS $$INCLUDES \
-		Makefile ${CSRC}
-	. ./${DEFS_SH}; echo "$$DEFS_DEPEND" >> Makefile; \
-	chmod -w Makefile
+	@rm -f .depend
+	@. ./${DEFS_SH}; ${XEXEC} ${SHELL} ${MKDEPEND} $$CPPFLAGS $$INCLUDES ${CSRC}
+	. ./${DEFS_SH}; echo "$$DEFS_DEPEND" >> .depend
 
 check_defs:
 	SHELL=${SHELL} ROOT=${ROOT} ${SHELL} ${CHECKDEFS}
 
+${ROOT}/conf/EDITME: .MAKE
+	cd ${ROOT}/conf && ${MAKE} EDITME
+
 clean:
-	rm -f ${DEFS_SH} ${DEFS_H} ${DEFS_SED}
+	rm -f .${DEFS_SH} .${DEFS_H} .${DEFS_SED} .Makefile
 	rm -f a.out core ${OBJ}
-	rm -f ${TARGET}
+	rm -f ${TARGET_LIB}
 
 clobber: clean
-	rm -f .${DEFS_SH} .${DEFS_H} .${DEFS_SED} .Makefile
+	rm -f .depend
+	rm -f ${DEFS_SH} ${DEFS_H} ${DEFS_SED}
 	rm -f Makefile
 
-nuke:   clobber
-	-${CLEAN}
+.MAKE:
 
-# DO NOT REMOVE THIS LINE OR "make depend" WILL NOT WORK
-EOF
+.PHONY: all install local-install clean local-clean clobber local-clobber depend local-depend lint mkdefs check_defs tags TAGS
+.PHONY: .MAKE csrc hsrc sources
+__EOF__
 			;;
 		%drop)	echo "    $b" >> $outfile;;
 		*)	echo "$a $b" >> $outfile;;
