@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:notify.c,v 1.46 2001/08/03 13:50:29 woods Exp"
+ * #ident	"@(#)smail/src:RELEASE-3_2_0_115:notify.c,v 1.60 2003/06/10 18:19:59 woods Exp"
  */
 
 /*
@@ -19,16 +19,53 @@
  *			    error_delivery, process_msg_log, notify,
  *                          hash_predelivered_addresses
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <stdio.h>
+#include <errno.h>
 #include <ctype.h>
 #include <pwd.h>
 #include <time.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+#include <signal.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# define _GNU_SOURCE			/* to see decl. of strsignal() */
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
+# include <sys/wait.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #include "smail.h"
 #include "parse.h"
 #include "addr.h"
@@ -37,13 +74,17 @@
 #include "log.h"
 #include "direct.h"
 #include "route.h"
-#include "transport.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
+#include "transport.h"
+#include "smailwait.h"
 #include "child.h"
 #include "exitcodes.h"
 #include "extern.h"
 #include "debug.h"
+#include "smailport.h"
 
 /* exported variables */
 int send_to_postmaster;			/* set TRUE to mail to postmaster */
@@ -205,7 +246,7 @@ defer_delivery(addr, defer_errors)
 #if (SMAIL_LOG_STYLE == 2)
 	/* Default new log style (as of 3.1.29) */
 	/* This is SMAIL_LOG_STYLE=2 */
-	write_log((cur->error->info & ERR_CONFERR)? (WRITE_LOG_SYS|WRITE_LOG_TTY): WRITE_LOG_SYS,
+	write_log((cur->error->info & ERR_CONFERR) ? (WRITE_LOG_SYS | WRITE_LOG_TTY) : WRITE_LOG_SYS,
 		  "Deferred TO:%s%s%s%s%s%s%s%s%s ERROR:(ERR%03ld) %s",
 		  cur->next_addr ? cur->next_addr : cur->in_addr,
 		  top ? " ORIG-TO:" : "", top ? top->in_addr : "",
@@ -218,7 +259,7 @@ defer_delivery(addr, defer_errors)
 # if (!defined(SMAIL_LOG_STYLE)) || (SMAIL_LOG_STYLE == 1)
     /* Old log style (as used by 3.1.28) */
     /* This is SMAIL_LOG_STYLE=1 */
-	write_log((cur->error->info & ERR_CONFERR)? (WRITE_LOG_SYS|WRITE_LOG_TTY): WRITE_LOG_SYS,
+	write_log((cur->error->info & ERR_CONFERR) ? (WRITE_LOG_SYS | WRITE_LOG_TTY) : WRITE_LOG_SYS,
 		  "%s %s%s%s... deferred: (ERR_%03ld) %s",
 		  cur->next_addr ? cur->next_addr : cur->in_addr,
 		  top ? "(orig-to: " : "",
@@ -518,7 +559,7 @@ decode_x_line(line, address, parent)
     register char *p = line;		/* point to parts of the line */
 
     /* skip initial white-space */
-    while (isspace(*p)) {
+    while (isspace((int) *p)) {
 	p++;
     }
     /* ignore anything not of the proper form */
@@ -553,7 +594,7 @@ decode_x_line(line, address, parent)
     *p++ = '\0';			/* the last > is the end of address */
 
     /* scan for a possible parent address */
-    while (isspace(*p)) {		/* skip up until parent: */
+    while (isspace((int) *p)) {		/* skip up until parent: */
 	p++;
     }
     /* is there a parent address? */
@@ -565,7 +606,7 @@ decode_x_line(line, address, parent)
     p += sizeof("parent:") - 1;
 
     /* skip initial white-space before actual parent address */
-    while (isspace(*p)) {
+    while (isspace((int) *p)) {
 	p++;
     }
 
@@ -600,7 +641,7 @@ decode_x_line(line, address, parent)
     }
     *p++ = '\0';
 
-    while (isspace(*p)) {
+    while (isspace((int) *p)) {
 	p++;
     }
 
@@ -635,7 +676,7 @@ put_defer_error(list, message, address, parent)
     }
     message += sizeof(reason_pfx) - 1;
     p = message;
-    while (isdigit(*message)) {
+    while (isdigit((int) *message)) {
 	message++;
     }
     if (*message != ')') {
@@ -646,7 +687,7 @@ put_defer_error(list, message, address, parent)
     *message = 0;
     error = atoi(p);
     *message++ = ')';
-    while (isspace(*message)) {
+    while (isspace((int) *message)) {
 	message++;
     }
     new_list = (struct defer_addr *) xmalloc(sizeof(*new_list));
@@ -679,9 +720,14 @@ put_defer_error(list, message, address, parent)
 /*
  * notify - notify all users who are to receive a note about errors
  *
- * also, note any configuration errors and set call_freeze_message
- * if any are found.  If any addrs exist in the defer list, set
- * some_deferred_addrs to prevent main from calling unlink_spool().
+ * also, note any configuration errors for messages in the defer list and set
+ * call_freeze_message if any are found.  If any addrs exist in the defer list,
+ * set some_deferred_addrs to prevent main from calling unlink_spool().
+ *
+ * If multiple recipients are to receive bounces separate messages are created
+ * so as to simply error handling of bounce deliveries.  This is done through
+ * one smail child process by sending all the messages in one group via the
+ * Batched-SMTP protocol.
  */
 void
 notify(defer, fail, sent_errors)
@@ -722,8 +768,8 @@ notify(defer, fail, sent_errors)
     }
 
     /*
-     * for deferred addresses, don't return anything, though note config
-     * errors and defer the message if any were found.
+     * for deferred addresses, don't return anything, though note any
+     * configuration errors and freeze the message if any were found.
      */
     for (cur = defer; cur; cur = cur->succ) {
 	some_deferred_addrs = TRUE;	/* main should not call unlink_spool */
@@ -731,6 +777,7 @@ notify(defer, fail, sent_errors)
 	    panic(EX_SOFTWARE, "notify(): called for deferred message without error info (in_addr=%s)", cur->in_addr);
 	}
 	if (cur->error->info & ERR_CONFERR) {
+	    write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC, "notify: freezing a deferred bounce due to configuration error.");
 	    call_freeze_message = TRUE;	/* config error, freeze the message in error/ */
 	}
     }
@@ -793,7 +840,7 @@ notify(defer, fail, sent_errors)
 	    return_to_sender = FALSE;
 	}
 	/* turn it off, as well, for really low priority messages */
-	if (islower(msg_grade) && msg_grade >= 'n') {
+	if (islower((int) msg_grade) && msg_grade >= 'n') {
 	    return_to_sender = FALSE;
 	}
     }
@@ -810,6 +857,7 @@ notify(defer, fail, sent_errors)
      */
     if (error_sender) {
 	if (return_to_sender || send_to_postmaster || to_owner) {
+	    write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC, "notify: freezing a double-bounce message.");
 	    call_freeze_message = TRUE;
 	    return;
 	}
@@ -828,15 +876,17 @@ notify(defer, fail, sent_errors)
      * open a channel to a new smail process, operating in bsmtp mode,
      * and send it messages for all of the recipients.
      */
-    pid = open_child(mailargs, (char **) NULL, &f, (FILE **) NULL, -1,
-		     CHILD_MINENV|CHILD_RETRY, prog_euid, prog_egid);
-    if (pid < 0) {
-	DEBUG(DBG_NOTIFY_LO, "notify: open_child failed, cannot notify\n");
+    if ((pid = open_child(mailargs, (char **) NULL, &f, (FILE **) NULL, -1,
+			  CHILD_MINENV|CHILD_RETRY, prog_euid, prog_egid)) < 0) {
+	/* XXX should this be a panic() call? */
+	write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC, "notify(): open_child() failed: %s", strerror(errno));
 	/* if we can't notify, let humans interfere later */
 	call_freeze_message = TRUE;
 	return;
     }
 
+    DEBUG1(DBG_NOTIFY_MID, "notify(): initiating BSMTP session with child PID %d.\n", pid);
+
     /* start up the session */
     (void) fprintf(f, "HELO %s\n", primary_name);
 
@@ -905,7 +955,11 @@ notify(defer, fail, sent_errors)
 	(void) fputs(text_banner, f);
 	write_notify_header(f);
 	putc('\n', f);
-	write_body(f, (long)PUT_DOTS);
+	if (islower((int) msg_grade)) {
+	    fputs("[[ ... low-priority message, body not included ... ]]\n", f);
+	} else {
+	    write_body(f, (long) PUT_DOTS);
+	}
 	(void) fputs(".\n", f);		/* finish the message */
     }
 
@@ -918,7 +972,8 @@ notify(defer, fail, sent_errors)
 	    DEBUG1(DBG_NOTIFY_LO, "     ... also returning to sender: %s\n", sender);
 	    cc2 = sender;
 	}
-	notify_new_message(f, "postmaster", cc1, cc2, "sending to postmaster");
+	notify_new_message(f, "postmaster", cc1, cc2,
+			   (return_to_sender) ? "sending to postmaster and sender" : "sending to postmaster");
 	send_log(f, TRUE, log_banner);	/* include X lines from log */
 	/* write messages in to_other destined to the postmaster */
 	(void) fputs(addr_error_banner, f);
@@ -947,7 +1002,11 @@ notify(defer, fail, sent_errors)
 	(void) fputs(text_banner, f);
 	write_notify_header(f);
 	putc('\n', f);
-	write_body(f, (long)PUT_DOTS);
+	if (islower((int) msg_grade)) {
+	    fputs("[[ ... low-priority message, body not included ... ]]\n", f);
+	} else {
+	    write_body(f, (long) PUT_DOTS);
+	}
 	(void) fputs(".\n", f);		/* finish the message */
     }
 
@@ -960,7 +1019,8 @@ notify(defer, fail, sent_errors)
 	    DEBUG(DBG_NOTIFY_LO, "      ... also sending to postmaster\n");
 	    cc1 = "postmaster";
 	}
-	notify_new_message(f, sender, cc1, cc2, "returning to sender");
+	notify_new_message(f, sender, cc1, cc2,
+			   (send_to_postmaster) ? "returning to sender and to postmaster" : "returning to sender");
 	send_log(f, FALSE, log_banner);	/* don't include X lines from log */
 	/* write messages in to_other destined to the sender */
 	(void) fputs(addr_error_banner, f);
@@ -998,19 +1058,58 @@ notify(defer, fail, sent_errors)
 	 * Note that grades >= 'n' have the return_to_sender flag turned off
 	 * completely above.
 	 */
-	if (islower(msg_grade)) {
-	    fputs("[low-priority message, body not included]\n", f);
+	if (islower((int) msg_grade)) {
+	    fputs("[[ ... low-priority message, body not included ... ]]\n", f);
 	} else {
-	    write_body(f, (long)PUT_DOTS);
+	    write_body(f, (long) PUT_DOTS);
 	}
 	(void) fputs(".\n", f);		/* finish the message */
     }
 
+    DEBUG1(DBG_NOTIFY_MID, "notify(): sending BSMTP 'QUIT' and closing child PID %d.\n", pid);
+
     (void) fputs("QUIT\n", f);
 
     if ((status = close_child(f, (FILE *)NULL, pid)) != 0) {
-	/* XXX or maybe we should call panic()? */
-	write_log(WRITE_LOG_SYS|WRITE_LOG_PANIC, "notify(): child [%d] process returned failure: %d", pid, status);
+	/* XXX should these be a panic() calls? */
+	if (status == -1) {
+	    write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC,
+		      "notify(): unable to reap status for BSMTP child PID %d: %s",
+		      pid,
+		      strerror(errno));
+	} else if (WIFEXITED(status)) {
+	    if (WEXITSTATUS(status) != 0) {
+		write_log(WRITE_LOG_SYS,
+			  "notify(): BSMTP child PID %d exit status %s (%d)",
+			  pid,
+			  strsysexit(WEXITSTATUS(status)),
+			  WEXITSTATUS(status));
+	    }
+	} else if (WIFSIGNALED(status)) {
+	    char signm[SIG2STR_MAX];
+
+	    if (sig2str(WTERMSIG(status), signm) == -1) {
+		sprintf(signm, "#%d", WTERMSIG(status));
+	    }
+	    write_log(((WTERMSIG(status) != SIGTERM) ? WRITE_LOG_PANIC : 0) | WRITE_LOG_SYS,
+		      "notify(): BSMTP child PID %d killed by signal SIG%s %s: %s",
+		      pid,
+		      signm,
+		      WCOREDUMP(status) ? "and dumped core" : "(no core)",
+		      strsignal(WTERMSIG(status)));
+	} else if (WIFSTOPPED(status)) {
+	    char signm[SIG2STR_MAX];
+
+	    /* in theory we'll hopefully never see stopped processes... */
+	    if (sig2str(WTERMSIG(status), signm) == -1) {
+		sprintf(signm, "#%d", WSTOPSIG(status));
+	    }
+	    write_log(WRITE_LOG_SYS | WRITE_LOG_PANIC,
+		      "notify(): BSMTP child PID %d stopped unexpectedly by signal SIG%d: %s",
+		      pid,
+		      signm,
+		      strsignal(WSTOPSIG(status)));
+	}
 	/*
 	 * this isn't the best, but if we can't send the bounce, let humans
 	 * interfere later
@@ -1032,7 +1131,7 @@ notify(defer, fail, sent_errors)
  *
  * If postmaster or the sender is to receive an error, link into the
  * to_other list, setting send_to_postmaster and/or return_to_sender
- * appropriately.  If a config error is found, set call_freeze_message.
+ * appropriately.
  *
  * If we need to send to an owner, determine if an owner string is
  * defined and link an addr structure for that owner into the to_owner
@@ -1172,6 +1271,29 @@ notify_new_message(f, to, cc1, cc2, subject_to)
     char *cc1, *cc2;			/* show carbon copy addresses */
     char *subject_to;			/* message for inclusion in subject */
 {
+    int form;
+    char *error;			/* temp to store error message */
+    char *new_recip = NULL;
+
+    DEBUG5(DBG_NOTIFY_LO, "notify_new_message: sending to:%s%s%s%s%s\n",
+	  to,
+	  cc1 ? ", cc1:" : "", cc1 ? cc1 : "",
+	  cc2 ? ", cc2:" : "", cc2 ? cc2 : "");
+    /*
+     * Make sure the recipient has a proper RFC-821 form address.
+     */
+    form = parse_address(to, (char **) NULL, &error, (int *) NULL);
+    switch (form) {
+    case LOCAL:
+	/* qualify the address with visible_name */
+	new_recip = xprintf("%s@%s", to, visible_name);
+	break;
+    case MAILBOX:
+	break;				/* all's well */
+    default:
+	new_recip = xprintf("PostMaster@%s", visible_name);
+	break;
+    }
     /*
      * initiate a new message
      * NOTE:  <+> is a smail-internal address form that causes the
@@ -1181,15 +1303,21 @@ notify_new_message(f, to, cc1, cc2, subject_to)
     (void) fprintf(f, "MAIL FROM:<+>\n");
 
     /* state the recipient */
-    (void) fprintf(f, "RCPT TO:<%s>\n", to);
+    (void) fprintf(f, "RCPT TO:<%s>\n", new_recip ? new_recip : to);
 
-    /* start the message text, with the complete header */
-    (void) fprintf(f, "DATA\nFrom: <MAILER-DAEMON@%s>\n", visible_name);
+    /* start the message text */
+    (void) fprintf(f, "DATA");
+
+    /* write out a complete RFC-822 header */
+    (void) fprintf(f, "From: <MAILER-DAEMON@%s>\n", visible_name);
     if (to[0] == '@') {
 	/* watch out for route-addrs */
 	(void) fprintf(f, "To: <%s>\n", to);
     } else {
-	(void) fprintf(f, "To: %s\n", to);
+	(void) fprintf(f, "To: %s\n", new_recip ? new_recip : to);
+    }
+    if (new_recip) {
+	xfree(new_recip);
     }
     if (cc1 == NULL && cc2) {
 	cc1 = cc2;
@@ -1197,21 +1325,48 @@ notify_new_message(f, to, cc1, cc2, subject_to)
     }
     if (cc1) {
 	if (cc1[0] == '@') {
-	    fprintf(f, "Cc: <%s>", cc1);
+	    (void) fprintf(f, "Cc: <%s>", cc1);
 	} else {
-	    fprintf(f, "Cc: %s", cc1);
+	    (void) fprintf(f, "Cc: %s", cc1);
 	}
 	if (cc2) {
 	    if (cc2[0] == '@') {
-		fprintf(f, ", <%s>", cc2);
+		(void) fprintf(f, ", <%s>", cc2);
 	    } else {
-		fprintf(f, ", %s", cc2);
+		(void) fprintf(f, ", %s", cc2);
 	    }
 	}
-	putc('\n', f);
+	(void) putc('\n', f);
     }
-    (void) fprintf(f, "Subject: mail failed, %s\nReference: <%s@%s>\n\n",
-		   subject_to, message_id, primary_name);
+    switch (form) {
+    case LOCAL:
+    case MAILBOX:
+	(void) fprintf(f, "Subject: mail failed, %s\n", subject_to);
+	(void) fputs("Summary: message returned due to delivery errors.\n", f);
+	break;
+    case FAIL:
+	(void) fprintf(f, "Subject: mail failed (see summary), %s\n", subject_to);
+	(void) fprintf(f, "Summary: bounce message not sent to <%s>\n", to);
+	(void) fprintf(f, "         due to addressing parse error: %s.\n", error);
+	break;
+    default:
+	(void) fprintf(f, "Subject: mail failed (see summary), %s\n", subject_to);
+	(void) fprintf(f, "Summary: bounce message not sent to <%s>\n", to);
+	(void) fprintf(f, "         due to unsuitable addressing form %s [%d].\n",
+		       (form == RFC_ROUTE) ? "RFC_ROUTE" :
+		       (form == RFC_ENDROUTE) ? "RFC_ENDROUTE" :
+		       (form == UUCP_ROUTE) ? "UUCP_ROUTE" :
+		       (form == PCT_MAILBOX) ? "PCT_MAILBOX" :
+		       (form == BERKENET) ? "BERKENET" :
+		       (form == DECNET) ? "DECNET" : "<bad-form!>",
+		       form);
+	break;
+    }
+    (void) fprintf(f, "Reference: <%s@%s>\n", message_id, primary_name);
+    /* a blank line ends the main message header.... */
+    (void) putc('\n', f);
+
+    return;
 }
 
 /*
@@ -1272,8 +1427,8 @@ write_notify_header(f)
     struct transport tport;
     struct addr addr;
 
-    bzero(&tport, sizeof(tport));
-    bzero(&addr, sizeof(addr));
+    (void) memset((char *) &tport, '\0', sizeof(tport)); /* XXX hope NULLs are all 0 bits */
+    (void) memset((char *) &addr, '\0', sizeof(addr)); /* XXX hope NULLs are all 0 bits */
     addr.transport = &tport;
     tport.name = "error";
     tport.flags = PUT_RECEIVED | STRICT_TPORT;
@@ -1341,7 +1496,7 @@ hash_predelivered_addresses()
 				   &remainder,
 				   &parse_flag) != LOCAL) && !islocalhost(target)) {
 		    DEBUG2(DBG_NOTIFY_MID, "Hashing out %s (%s) ... previously processed.\n", address, preparsed_address);
-		    (void) add_to_hash(preparsed_address, (char *) NULL, 0, hit_table);
+		    (void) add_to_hash(preparsed_address, (char *) NULL, (size_t) 0, hit_table);
 		}
 	    }
 	}
