@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:parse.c,v 1.11 2001/07/29 23:11:32 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:parse.c,v 1.20 2003/04/02 22:21:39 woods Exp"
  */
 
 /*
@@ -23,6 +23,8 @@
  *	external functions:  parse_entry, parse_config, parse_table,
  *	read_entry.
  */
+
+#include "defs.h"
 #ifdef STANDALONE
 # define xmalloc malloc
 # define xrealloc realloc
@@ -33,10 +35,28 @@
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
 
-#ifdef HAVE_STDLIB_H
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
 #endif
 
 #include "smail.h"
@@ -45,10 +65,12 @@
 #include "addr.h"
 #include "log.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
 #include "debug.h"
 #include "extern.h"
+#include "smailport.h"
 
 /* variables exported from this file */
 char *on = "1";				/* boolean on attribute value */
@@ -92,7 +114,7 @@ parse_entry(entry, generic, driver, error)
      * by optional white space followed by a `:'
      */
     STR_INIT(&str);
-    while (*entry && !isspace(*entry) && *entry != ':' && *entry != '#') {
+    while (*entry && !isspace((int) *entry) && *entry != ':' && *entry != '#') {
 	STR_NEXT(&str, *entry++);
     }
     entry = skip_space(entry);
@@ -114,7 +136,7 @@ parse_entry(entry, generic, driver, error)
      * the entry
      */
     while (*entry) {
-	int i;				/* temp */
+	size_t i;			/* temp */
 
 	/* attribute name begins at next non-white space character */
 	entry = skip_space(entry);
@@ -138,7 +160,7 @@ parse_entry(entry, generic, driver, error)
 	    STR_NEXT(&str, *entry++);
 	}
 	i = STR_LEN(&str);
-	while (*entry && (isalnum(*entry) || *entry == '_' || *entry == '-')) {
+	while (*entry && (isalnum((int) *entry) || *entry == '_' || *entry == '-')) {
 	    STR_NEXT(&str, *entry++);
 	}
 	if (i == STR_LEN(&str)) {
@@ -170,6 +192,9 @@ parse_entry(entry, generic, driver, error)
 
 	entry = skip_space(entry + 1);
 
+	/*
+	 * XXX this code is very nearly a duplicate of the code below in parse_config()
+	 */
 	if (*entry == '"') {
 	    entry++;
 	    /*
@@ -204,8 +229,8 @@ parse_entry(entry, generic, driver, error)
 	     * character.
 	     */
 	    while (*entry && (*entry == '\\' ||
-			      index("!@$%^&*-_+~/?|<>:[]{}.`'", *entry) ||
-			      isalnum(*entry)))
+			      strchr("!@$%^&*-_+~/?|<>:[]{}.`'", *entry) ||
+			      isalnum((int) *entry)))
 	    {
 		if (*entry == '\\') {
 		    entry++;
@@ -298,12 +323,12 @@ c_quote(p, val)
 	 */
 	cnt = 3;
 	*val = 0;
-	while (cnt && (isdigit(c = *p++) ||
+	while (cnt && (isdigit((int) (c = *p++)) ||
 		       (c >= 'A' && c <= 'F') ||
 		       (c >= 'a' && c <= 'f')))
 	{
 	    *val = (*val * 16) +
-		(isdigit(c)? c - '0': isupper(c)? c - 'A': c - 'a');
+		(isdigit((int) c)? c - '0': isupper((int) c)? c - 'A': c - 'a');
 	}
 	return p - 1;
     case '0': case '1': case '2': case '3':
@@ -328,11 +353,7 @@ static char *
 eof_error(name)
     char *name;				/* name of attribute */
 {
-    static char *message = "unexpected end of string for attribute %s";
-    char *error = xmalloc(strlen(name) + sizeof(message));
-
-    (void) sprintf(error, message, name);
-    return error;
+    return xprintf("unexpected end of quoted string value for attribute %s", name);
 }
 
 /*
@@ -414,7 +435,7 @@ finish_parse(tokens, generic, driver)
  *
  * given a string, such as returned by read_entry, turn it into a single
  * attribute entry with the "name" field pointing at the start of the config
- * attribute name and with the "value" field pointing at the attribute's value.
+ * variable name and with the "value" field pointing at the variable's value.
  * Both are pointing at the same allocated storage block and can be freed by
  * passing the "name" pointer to xfree().
  *
@@ -445,17 +466,17 @@ parse_config(entry, error)
     }
 
     /*
-     * get the attribute name
+     * get the variable name
      */
-    while (*entry && (isalnum(*entry) || *entry == '_' || *entry == '-')) {
+    while (*entry && (isalnum((int) *entry) || *entry == '_' || *entry == '-')) {
 	STR_NEXT(&str, *entry++);
     }
-    STR_NEXT(&str, '\0');		/* termiante attribute name */
+    STR_NEXT(&str, '\0');		/* terminate variable name */
 
     entry = skip_space(entry);
 
     if (*entry == '\0') {
-	/* boolean attribute */
+	/* boolean variable */
 	STR_DONE(&str);
 	attr->name = STR(&str);
 	if (attr_type == '-') {
@@ -466,12 +487,12 @@ parse_config(entry, error)
 
 	return attr;
     } else if (*entry != '=') {
-	*error = xprintf("expected `=' after attribute name: %s", STR(&str));
+	*error = xprintf("expected `=' after variable name: %s", STR(&str));
 	return NULL;
     }
 
     if (attr_type != ' ') {
-	*error = xprintf("unexpected pattern:  `= value' follows boolean attribute: %s", STR(&str));
+	*error = xprintf("unexpected pattern:  `= value' follows boolean variable: %s", STR(&str));
 	return NULL;
     }
 
@@ -481,7 +502,13 @@ parse_config(entry, error)
 
     value_offset = STR_LEN(&str);
 
+    /*
+     * XXX this code is very nearly a duplicate of the code above in parse_entry()
+     */
     if (*entry == '"') {
+#if 0
+	DEBUG2(DBG_CONF_HI, "parse_config(): variable %s: about to slurp quoted value: '%s'\n", STR(&str), entry);
+#endif
 	entry++;
 	/* if a quote, skip to the closing quote (\ quotes next char) */
 	while (*entry && *entry != '"') {
@@ -499,25 +526,37 @@ parse_config(entry, error)
 	     * make sure that the string doesn't suddenly come
 	     * to an end at a funny spot
 	     */
-	    *error = xprintf("end of quoted attribute found before closing double-quote: %s", STR(&str));
+	    *error = eof_error(STR(&str));
 	    return NULL;
 	}
 	entry++;			/* skip the " */
     } else {
 	/*
-	 * not in double quotes, only a limited set of characters
-	 * are allowed in an unquoted string, though \ quotes any
-	 * character.
+	 * not in double quotes, only a limited set of characters are allowed
+	 * in an unquoted variable string (more than in an attribute string),
+	 * though \ quotes any character.
+	 *
+	 * Note too that in a config file we allow continuation of variable
+	 * values without a trailing backslash.
 	 */
-	while (*entry && (*entry == '\\' ||
-			  index("!@$%^&*-_+~/?|<>:[]{}.`'", *entry) ||
-			  isalnum(*entry)))
+#if 0
+	DEBUG2(DBG_CONF_HI, "parse_config(): variable %s: about to slurp unquoted value: '%s'\n", STR(&str), entry);
+#endif
+	while (*entry && (*entry == '\\' || *entry == '\n' ||
+			  strchr(" \t!@$%^&*-_+~/?|<>;:[]{},.`'", *entry) || /* XXX " \t:," added */
+			  isalnum((int) *entry)))
 	{
+	    if (*entry == '\n') {
+		entry = skip_space(entry);
+		if (*entry == '\0') {
+		    break;		/* normal end of entry... */
+		}
+	    }
 	    if (*entry == '\\') {
 		entry++;
 		if (*entry == '\0') {
 		    /* must have something after \ */
-		    *error = xprintf("unexpected end of attribute after backslash: %s", STR(&str));
+		    *error = xprintf("unexpected end of variable after backslash: %s", STR(&str));
 		    return NULL;
 		}
 	    }
@@ -525,13 +564,14 @@ parse_config(entry, error)
 	}
     }
     STR_NEXT(&str, '\0');		/* close off the value */
-    entry = skip_space(entry);
 
     /*
-     * make sure this is really the end of the entry
+     * make sure this is really the end of the entry, first eating any trailing
+     * whitespace that would be ignored anyway....
      */
+    entry = skip_space(entry);
     if (*entry != '\0') {
-	*error = xprintf("unexpected data after end of entry: %s", STR(&str));
+	*error = xprintf("unexpected data after end of entry: %s, remaining text: '%s'", STR(&str), entry);
 	return NULL;
     }
 
@@ -569,7 +609,7 @@ parse_table(entry, error)
     STR_INIT(&str);
 
     entry = skip_space(entry);
-    while (*entry && !isspace(*entry) && *entry != '#') {
+    while (*entry && !isspace((int) *entry) && *entry != '#') {
 	STR_NEXT(&str, *entry++);
     }
     STR_NEXT(&str, '\0');		/* terminate name of host */
@@ -582,7 +622,7 @@ parse_table(entry, error)
     }
 
     offset_transport = STR_LEN(&str);
-    while (*entry && !isspace(*entry) && *entry != '#') {
+    while (*entry && !isspace((int) *entry) && *entry != '#') {
 	STR_NEXT(&str, *entry++);
     }
     STR_NEXT(&str, '\0');		/* terminate name of transport */
@@ -617,7 +657,7 @@ skip_space(p)
 	    while (*p && *p != '\n') {
 		p++;
 	    }
-	} else if (!isspace(*p)) {
+	} else if (!isspace((int) *p)) {
 	    /* found something that isn't white space, return it */
 	    return p;
 	} else {
@@ -649,9 +689,11 @@ read_entry(f)
      * scan for the beginning of an entry, which begins at the first
      * non-white space, non-comment character
      */
-    while ((c = getc(f)) != EOF && (isspace(c) || c == '#')) {
+    while ((c = getc(f)) != EOF && (isspace((int) c) || c == '#')) {
 	if (c == '#') {
-	    while ((c = getc(f)) != EOF && c != '\n') ;
+	    while ((c = getc(f)) != EOF && c != '\n') {
+		;
+	    }
 	    if (c == EOF) {
 		break;
 	    }
@@ -704,7 +746,9 @@ read_entry(f)
 		break;
 	    }
 	    if (c == '\n') {
-		while ((c = getc(f)) == ' ' || c == '\t') ;
+		while ((c = getc(f)) == ' ' || c == '\t') {
+		    ;
+		}
 	    } else {
 		STR_NEXT(&str, '\\');
 	    }
@@ -716,7 +760,7 @@ read_entry(f)
      * that's the end of that entry
      */
     if (c != EOF) {
-	(void) ungetc(c, f);		/* first character for next time */
+	(void) ungetc(c, f);		/* first character for the next entry? */
     }
     STR_NEXT(&str, '\0');		/* end of the entry */
 
