@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:pathto.c,v 1.15 2000/09/03 17:00:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:pathto.c,v 1.21 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -18,30 +18,56 @@
  *	external functions: main, initialize_state, process_args,
  *	deliver_mail
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "route.h"
-#include "transport.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
+#include "transport.h"
 #include "exitcodes.h"
 #include "error.h"
 #include "extern.h"
+#include "debug.h"
+#include "smailport.h"
 
 /* variables local to this file */
 static int bang_paths_only;
@@ -200,8 +226,8 @@ print_pathto_addr(addr)
 /*ARGSUSED*/
 void
 optto(argc, argv)
-    int argc;
-    char **argv;			/* generally, argv from main */
+    int argc;				/* UNUSED */
+    char **argv;			/* UNUSED */
 {
     (void) fprintf(stderr, "%s: optto not currently supported\n", program);
     exit(EX_UNAVAILABLE);
@@ -248,7 +274,7 @@ uupath(argc, argv)
 		(void) fprintf(stderr, "%s: %s: %s\n",
 			       program, *argv, error);
 	    } else {
-		last_part = rindex(path, '!');
+		last_part = strrchr(path, '!');
 		if (last_part == NULL) {
 		    (void) printf("%s\n", addr->next_host);
 		} else {
