@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:pwcache.c,v 1.14 2000/09/11 01:17:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:pwcache.c,v 1.22 2003/06/10 00:24:18 woods Exp"
  */
 
 /*
@@ -20,20 +20,48 @@
  * files.
  */
 
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <pwd.h>
 #include <grp.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "hash.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "extern.h"
-
+#include "smailport.h"
 
 /* functions local to this file */
 static void fill_pw_cache __P((struct passwd *));
@@ -92,16 +120,17 @@ getpwbyuid(uid)
 {
     struct pw_cache *entp, ent;
     static struct passwd ret;
-    static char uid_buf[50];		/* big enough to hold a number */
+    static char uid_buf[MAXLONG_B10_DIGITS + 1];
 
     if (uid_cache == NULL) {
 	/* XXX - hash table should be associated with a block */
 	uid_cache = new_hash_table(N_CACHE,
-				   (struct block *)NULL,
+				   (struct block *) NULL,
 				   HASH_DEFAULT);
     }
-    (void) sprintf(uid_buf, "%d", uid);
-    if (lookup_in_hash(uid_buf, (char **)&entp, (int *)NULL,
+    /* XXX sign extension could cause ugly problems here.... we NEED uid_t! */
+    (void) sprintf(uid_buf, "%lu", (unsigned long int) uid);
+    if (lookup_in_hash(uid_buf, (char **) &entp, (size_t *) NULL,
 		       uid_cache) == ALREADY_HASHED)
     {
 	if (entp->pw_name[0] == BAD_NAME) {
@@ -120,13 +149,13 @@ getpwbyuid(uid)
 	if (pw == NULL) {
 	    ent.pw_uid = uid;
 	    ent.pw_name[0] = BAD_NAME;
-	    (void) add_to_hash(uid_buf, (char *)&ent, sizeof(ent), uid_cache);
+	    (void) add_to_hash(uid_buf, (char *) &ent, sizeof(ent), uid_cache);
 	    return NULL;
 	}
 	if (user_cache == NULL) {
 	    /* XXX - hash table should be associated with a block */
 	    user_cache = new_hash_table(N_CACHE,
-					(struct block *)NULL,
+					(struct block *) NULL,
 					HASH_DEFAULT);
 	}
 	fill_pw_cache(pw);
@@ -143,16 +172,17 @@ getgrbygid(gid)
 {
     struct gr_cache *entp, ent;
     static struct group ret;
-    static char gid_buf[50];		/* big enough to hold a number */
+    static char gid_buf[MAXLONG_B10_DIGITS + 1];
 
     if (gid_cache == NULL) {
 	/* XXX - hash table should be associated with a block */
 	gid_cache = new_hash_table(N_CACHE,
-				   (struct block *)NULL,
+				   (struct block *) NULL,
 				   HASH_DEFAULT);
     }
-    (void) sprintf(gid_buf, "%d", gid);
-    if (lookup_in_hash(gid_buf, (char **)&entp, (int *)NULL,
+    /* XXX sign extension could cause ugly problems here.... we NEED gid_t! */
+    (void) sprintf(gid_buf, "%lu", (unsigned long int) gid);
+    if (lookup_in_hash(gid_buf, (char **) &entp, (size_t *) NULL,
 		       gid_cache) == ALREADY_HASHED)
     {
 	if (entp->gr_name[0] == BAD_NAME) {
@@ -169,13 +199,13 @@ getgrbygid(gid)
 	if (gr == NULL) {
 	    ent.gr_gid = gid;
 	    ent.gr_name[0] = BAD_NAME;
-	    (void) add_to_hash(gid_buf, (char *)&ent, sizeof(ent), gid_cache);
+	    (void) add_to_hash(gid_buf, (char *) &ent, sizeof(ent), gid_cache);
 	    return NULL;
 	}
 	if (group_cache == NULL) {
 	    /* XXX - hash table should be associated with a block */
 	    group_cache = new_hash_table(N_CACHE,
-					 (struct block *)NULL,
+					 (struct block *) NULL,
 					 HASH_DEFAULT);
 	}
 	fill_gr_cache(gr);
@@ -199,10 +229,10 @@ getpwbyname(user)
     if (user_cache == NULL) {
 	/* XXX - hash table should be associated with a block */
 	user_cache = new_hash_table(N_CACHE,
-				    (struct block *)NULL,
+				    (struct block *) NULL,
 				    HASH_DEFAULT);
     }
-    if (lookup_in_hash(user, (char **)&entp, (int *)NULL,
+    if (lookup_in_hash(user, (char **) &entp, (size_t *) NULL,
 		       user_cache) == ALREADY_HASHED)
     {
 	if (entp->pw_uid == BAD_ID) {
@@ -217,25 +247,25 @@ getpwbyname(user)
 	register struct passwd *pw;
 	register char *p;
 
-	(void) strncpy(ent.pw_name, user, PW_SIZE - 1);
+	(void) strncpy(ent.pw_name, user, (size_t) (PW_SIZE - 1));
 	ent.pw_name[PW_SIZE - 1] = '\0';
 	for (p = ent.pw_name; *p; p++) {
-	    *p = lowercase((unsigned char) *p);
+	    *p = tolower((int) *p);
 	}
 
 	setpwent();
 	pw = getpwnam(ent.pw_name);
 	if (pw == NULL) {
-	    (void) strncpy(ent.pw_name, user, PW_SIZE - 1);
+	    (void) strncpy(ent.pw_name, user, (size_t) (PW_SIZE - 1));
 	    ent.pw_name[PW_SIZE - 1] = '\0';
 	    ent.pw_uid = BAD_ID;
-	    (void) add_to_hash(user, (char *)&ent, sizeof(ent), user_cache);
+	    (void) add_to_hash(user, (char *) &ent, sizeof(ent), user_cache);
 	    return NULL;
 	}
 	if (uid_cache == NULL) {
 	    /* XXX - hash table should be associated with a block */
 	    uid_cache = new_hash_table(N_CACHE,
-				       (struct block *)NULL,
+				       (struct block *) NULL,
 				       HASH_DEFAULT);
 	}
 	fill_pw_cache(pw);
@@ -259,10 +289,10 @@ getgrbyname(group)
     if (group_cache == NULL) {
 	/* XXX - hash table should be associated with a block */
 	group_cache = new_hash_table(N_CACHE,
-				     (struct block *)NULL,
+				     (struct block *) NULL,
 				     HASH_DEFAULT);
     }
-    if (lookup_in_hash(group, (char **)&entp, (int *)NULL,
+    if (lookup_in_hash(group, (char **) &entp, (size_t *) NULL,
 		       group_cache) == ALREADY_HASHED)
     {
 	if (entp->gr_gid == BAD_ID) {
@@ -275,25 +305,25 @@ getgrbyname(group)
 	register struct group *gr;
 	register char *p;
 
-	(void) strncpy(ent.gr_name, group, PW_SIZE - 1);
+	(void) strncpy(ent.gr_name, group, (size_t) (PW_SIZE - 1));
 	ent.gr_name[PW_SIZE - 1] = '\0';
 	for (p = ent.gr_name; *p; p++) {
-	    *p = lowercase((unsigned char) *p);
+	    *p = tolower((int) *p);
 	}
 
 	setgrent();
 	gr = getgrnam(ent.gr_name);
 	if (gr == NULL) {
-	    (void) strncpy(ent.gr_name, group, PW_SIZE - 1);
+	    (void) strncpy(ent.gr_name, group, (size_t) (PW_SIZE - 1));
 	    ent.gr_name[PW_SIZE - 1] = '\0';
 	    ent.gr_gid = BAD_ID;
-	    (void) add_to_hash(group, (char *)&ent, sizeof(ent), group_cache);
+	    (void) add_to_hash(group, (char *) &ent, sizeof(ent), group_cache);
 	    return NULL;
 	}
 	if (gid_cache == NULL) {
 	    /* XXX - hash table should be associated with a block */
 	    gid_cache = new_hash_table(N_CACHE,
-				       (struct block *)NULL,
+				       (struct block *) NULL,
 				       HASH_DEFAULT);
 	}
 	fill_gr_cache(gr);
@@ -307,23 +337,24 @@ fill_pw_cache(pw)
     register struct passwd *pw;
 {
     struct pw_cache ent;		/* fill this with pw info */
-    char uid_buf[50];			/* big enough to hold a number */
+    char uid_buf[MAXLONG_B10_DIGITS + 1];
     register char *p;
-    unsigned int size_pw_dir;
+    size_t size_pw_dir;
 
-    (void) strncpy(ent.pw_name, pw->pw_name, PW_SIZE - 1);
+    (void) strncpy(ent.pw_name, pw->pw_name, (size_t) (PW_SIZE - 1));
     ent.pw_name[PW_SIZE - 1] = '\0';
     for (p = ent.pw_name; *p; p++) {
-	*p = lowercase((unsigned char) *p);
+	*p = tolower((int) *p);
     }
     ent.pw_uid = pw->pw_uid;
     ent.pw_gid = pw->pw_gid;
     size_pw_dir = strlen(pw->pw_dir) + 1;
     ent.pw_dir = xmalloc(size_pw_dir);
     (void) strcpy(ent.pw_dir, pw->pw_dir);
-    (void) sprintf(uid_buf, "%lu", (long unsigned int) pw->pw_uid);
-    (void) add_to_hash(uid_buf, (char *)&ent, sizeof(ent), uid_cache);
-    (void) add_to_hash(ent.pw_name, (char *)&ent, sizeof(ent), user_cache);
+    /* XXX sign extension could cause ugly problems here.... we NEED uid_t! */
+    (void) sprintf(uid_buf, "%lu", (unsigned long int) pw->pw_uid);
+    (void) add_to_hash(uid_buf, (char *) &ent, sizeof(ent), uid_cache);
+    (void) add_to_hash(ent.pw_name, (char *) &ent, sizeof(ent), user_cache);
 }
 
 /* fill the gid and groupname caches from a group file entry */
@@ -332,16 +363,17 @@ fill_gr_cache(gr)
     register struct group *gr;
 {
     struct gr_cache ent;		/* fill this with gr info */
-    char gid_buf[50];			/* big enough to hold a number */
+    char gid_buf[MAXLONG_B10_DIGITS + 1];
     register char *p;
 
-    (void) strncpy(ent.gr_name, gr->gr_name, PW_SIZE - 1);
+    (void) strncpy(ent.gr_name, gr->gr_name, (size_t) (PW_SIZE - 1));
     ent.gr_name[PW_SIZE - 1] = '\0';
     for (p = ent.gr_name; *p; p++) {
-	*p = lowercase((unsigned char) *p);
+	*p = tolower((int) *p);
     }
     ent.gr_gid = gr->gr_gid;
-    (void)sprintf(gid_buf, "%lu", (long unsigned int) gr->gr_gid);
-    (void) add_to_hash(gid_buf, (char *)&ent, sizeof(ent), gid_cache);
-    (void) add_to_hash(ent.gr_name, (char *)&ent, sizeof(ent), group_cache);
+    /* XXX sign extension could cause ugly problems here.... we NEED gid_t! */
+    (void)sprintf(gid_buf, "%lu", (unsigned long int) gr->gr_gid);
+    (void) add_to_hash(gid_buf, (char *) &ent, sizeof(ent), gid_cache);
+    (void) add_to_hash(ent.gr_name, (char *) &ent, sizeof(ent), group_cache);
 }
