@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:qualify.c,v 1.7 2000/09/03 17:00:15 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:qualify.c,v 1.14 2003/04/02 22:21:34 woods Exp"
  */
 
 /*
@@ -19,19 +19,46 @@
  *	chip@ateng.ateng.com.  It has been modified.
  */
 
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "smailconf.h"
 #include "extern.h"
+#include "smailport.h"
 
 struct qualify {
     char    *host;                      /* host name */
@@ -39,8 +66,8 @@ struct qualify {
 };
 
 static struct qualify *domains = NULL;  /* Array of host/domain pairs */
-static int domain_count = 0;            /* Count of valid pairs */
-static int domain_max = 0;              /* Count of allocated pairs */
+static unsigned int domain_count = 0;	/* Count of valid pairs */
+static unsigned int domain_max = 0;	/* Count of allocated pairs */
 
 char *
 read_qualify_file()
@@ -66,7 +93,7 @@ read_qualify_file()
 	    return xprintf("cannot open %s: %s", qualify_file, strerror(errno));
 	}
 
-	add_config_stat(qualify_file, (struct stat *)NULL);
+	add_config_stat(qualify_file, (struct stat *) NULL);
 	return NULL;
     }
 
@@ -77,7 +104,7 @@ read_qualify_file()
 
     if (!domains) {
 	domain_max = 16;
-	domains = (struct qualify *) xmalloc((unsigned) (domain_max * sizeof(*domains)));
+	domains = (struct qualify *) xmalloc((size_t) (domain_max * sizeof(*domains)));
     }
     while ((entry = read_entry(f))) {
 	struct attribute *new;
@@ -90,7 +117,7 @@ read_qualify_file()
 	if (domain_count >= domain_max) {
 	    domain_max += 16;
 	    domains = (struct qualify *) xrealloc((char *) domains,
-						  (unsigned) (domain_max * sizeof(struct qualify)));
+						  (size_t) (domain_max * sizeof(struct qualify)));
 	}
 	domains[domain_count].host = new->name;
 	domains[domain_count].domain = new->value;
@@ -106,9 +133,9 @@ char *
 qualify_domain(s)
     char *s;				/* domain to fully qualify */
 {
-    int i;
+    unsigned int i;
 
-    if (index(s, '.') != NULL) {
+    if (strchr(s, '.') != NULL) {
 	return NULL;
     }
 
@@ -127,7 +154,7 @@ void
 dump_qualify_config(f)
      FILE * f;
 {
-    int i;
+    unsigned int i;
 
     fputs("#\n# -- qualify configuration\n#\n", f);
     for (i = 0; i < domain_count; ++i) {
