@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:queue.c,v 1.48 2001/07/29 16:10:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:queue.c,v 1.63 2003/04/02 22:21:25 woods Exp"
  */
 
 /*
@@ -20,53 +20,84 @@
  *			    swallow_smtp
  */
 
+#include "defs.h"
+
 #include <sys/types.h>
+#include <sys/param.h>			/* for MAXLOGNAME if avail... */
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
+#include <limits.h>
 #include <pwd.h>
 #include <grp.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
 
 #include "smail.h"
+#include "smailarch.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "log.h"
 #include "field.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
+#include "header.h"
 #include "spool.h"
 #include "exitcodes.h"
 #include "debug.h"
 #include "extern.h"
+#include "smailport.h"
 
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
+/* variables exported from this file */
+unsigned long msg_body_size;		/* size of message body */
 
 /* variables local to this file */
 static off_t start_msg;			/* where message body begins */
-long msg_body_size;			/* size of message body */
 
-#define MAXUSER 16			/* maximum length of user name */
+/*
+ * maximum length of a user name
+ */
+#ifndef MAXUSER
+# if defined(LOGIN_NAME_MAX)
+#  define MAXUSER	(LOGIN_NAME_MAX-1)
+# elif defined(MAXLOGNAME)
+#  define MAXUSER	MAXLOGNAME
+# else
+#  define MAXUSER	16			/* default for NetBSD anyway.... */
+# endif
+#endif
+
+#if (MAXUSER >= BUFSIZ)
+# include "Something is wrong: MAXUSER >= BUFSIZ!" /* compile-time error! */
+#endif
 
 /* variables exported from this file */
 int msg_grade;				/* grade level for this message */
@@ -190,17 +221,22 @@ queue_envelope(list)
 {
     register struct addr *cur;		/* current recipient addr to process */
     char *p;				/* temp */
-    char buf[128];			/* temp storage (> MAXUSER and > "%d %d" */
+    char buf[BUFSIZ];			/* temp storage (> sizeof("%d %d")) */
+
+#if (MAXUSER >= BUFSIZ)
+# include "ERROR: (MAXUSER >= BUFSIZ)!!!"
+#endif
 
     if (local_sender) {
-	if (put_line(local_sender) == EOF)
+	if (put_line(local_sender) == EOF) {
 	    return FAIL;
-    }
-    else {
-	(void) memset(buf, ' ', MAXUSER);
+	}
+    } else {
+	(void) memset(buf, ' ', (size_t) MAXUSER);
 	buf[MAXUSER] = '\0';
-	if (put_line(buf) == EOF)
+	if (put_line(buf) == EOF) {
 	    return FAIL;
+	}
     }
 
     /*
@@ -505,8 +541,9 @@ fixup_login_user()
 {
     register int i;
 
-    if (local_sender)
+    if (local_sender) {
 	return SUCCEED;
+    }
 
     compute_local_sender();
 
@@ -515,8 +552,9 @@ fixup_login_user()
      * larger user name
      */
     i = (int)strlen(local_sender);
-    if (i > MAXUSER)
+    if (i > MAXUSER) {
 	i = MAXUSER;
+    }
 
     /*
      * write the name to disk
@@ -595,9 +633,9 @@ put_line(s)
 char **
 read_message()
 {
-    int a = 32;				/* vectors allocated thus far */
-    int i;				/* temp */
-    register int c;			/* character from spool file */
+    size_t a = 32;			/* vectors allocated thus far */
+    size_t i;				/* temp index */
+    register int c;			/* character from spool file or EOF */
     register char **argv;		/* computed argument vector */
     char *p;
     char ls[MAXUSER + 1];		/* local sender from file */
@@ -616,8 +654,9 @@ read_message()
     first = 1;
     while ((c = GETSPOOL()) != EOF && c != READ_FAIL &&
 	   c != '\n' && c != ' ' && i < MAXUSER) {
-	if (c != '!' || ! first)
+	if (c != '!' || ! first) {
 	    ls[i++] = c;
+	}
 	first = 0;
     }
     /* read to the end of the first line */
@@ -628,16 +667,17 @@ read_message()
 
     /* read the real uid and effective gid under which the mailer was exec'd */
     if (c != READ_FAIL && c != EOF) {
-	char buf[128];			/* temp input buffer */
+	char buf[BUFSIZ];		/* temp input buffer */
 	char *p2 = buf;			/* temp for scanning buf */
 
-	while ((c = GETSPOOL()) != '\n' && c != EOF && c != READ_FAIL && (p2-buf) < sizeof(buf)) {
+	while ((c = GETSPOOL()) != '\n' && c != EOF && c != READ_FAIL && (size_t) (p2-buf) < sizeof(buf)) {
 	    *p2++ = c;
 	}
 	*p2 = '\0';
 	p2 = buf;
-	if (*p2 == '!')
+	if (*p2 == '!') {
 	    p2++;
+	}
 	(void) sscanf(p2, "%d %d", &real_uid, &prog_egid);
     }
     if (c == READ_FAIL) {
@@ -674,6 +714,7 @@ read_message()
 
 	for (p = strcolon(trusted_groups); p; p = strcolon((char *)NULL)) {
 	    struct group *gr = getgrbyname(p);
+
 	    if (gr && prog_egid == gr->gr_gid) {
 		break;
 	    }
@@ -683,9 +724,9 @@ read_message()
     if (p == NULL && !just_trust) {
 	/* user is not verified, sender is the login user */
 	DEBUG1(DBG_QUEUE_MID, "sender is not trusted\n", real_uid);
-	sender = xmalloc(strlen(ls) + 1);
+	sender = xmalloc((size_t) strlen(ls) + 1);
 	strcpy(sender, ls);
-	islocal = TRUE;
+	islocal = TRUE;			/* sender appears to be local */
 	sender_is_trusted = FALSE;
     }
 
@@ -697,13 +738,13 @@ read_message()
     if (sender && sender_is_trusted) {
 	char *error;			/* temp to store error message */
 
-	islocal = parse_address(sender, (char **)0, &error, (int *)0) == LOCAL;
+	islocal = parse_address(sender, (char **) NULL, &error, (int *) NULL) == LOCAL;
     }
 
     /*
      * read the argument vectors stored in the spool file
      */
-    DEBUG1(DBG_QUEUE_HI, "read arg vectors from spool file (at %ld)\n", (long) tell_spool());
+    DEBUG1(DBG_QUEUE_HI, "read arg vectors from spool file (at %lu)\n", (unsigned long) tell_spool());
     argv = (char **)xmalloc(a * sizeof(char *));
     i = 0;
     first = 1;
@@ -716,8 +757,9 @@ read_message()
 	do {
 	    if (first) {
 		first = 0;
-		if (c == '!')
+		if (c == '!') {
 		    continue;
+		}
 	    }
 	    if (c == '\\') {
 		c = GETSPOOL();
@@ -742,12 +784,12 @@ read_message()
 	 */
 	STR_NEXT(sp, '\0');
 	STR_DONE(sp);
-	DEBUG1(DBG_QUEUE_HI, "read vector <%s> from spool file\n", sp->p);
-	argv[i++] = sp->p;
+	DEBUG1(DBG_QUEUE_HI, "read vector '%s' from spool file\n", STR(sp));
+	argv[i++] = STR(sp);
 	/* do we need a larger vector? */
 	if (i >= a) {
 	    a += 32;
-	    argv = (char **) xrealloc((char *) argv, (unsigned) a * sizeof(char *));
+	    argv = (char **) xrealloc((char *) argv, a * sizeof(char *));
 	}
 	first = 1;
     }
@@ -772,7 +814,7 @@ read_message()
      */
     if (sender == NULL) {
 	if (ls[0]) {
-	    sender = xmalloc(strlen(ls) + 1);
+	    sender = xmalloc((size_t) strlen(ls) + 1);
 	    strcpy(sender, ls);
 	} else {
 	    /* love to gang up on that postmaster */
@@ -786,7 +828,7 @@ read_message()
     /*
      * now grab the header lines.
      */
-    DEBUG1(DBG_QUEUE_HI, "calling read_header (offset=%ld)\n", (long) tell_spool());
+    DEBUG1(DBG_QUEUE_HI, "calling read_header (offset=%lu)\n", (unsigned long) tell_spool());
     if (read_header() == FAIL) {
 	/* read_header logged the error */
 	return NULL;
@@ -796,7 +838,7 @@ read_message()
     start_msg = tell_spool();
     msg_body_size = msg_size - start_msg;
 
-    DEBUG2(DBG_SPOOL_HI, "body starts at %ld, body size: %ld\n", (long) start_msg, (long) msg_body_size);
+    DEBUG2(DBG_SPOOL_HI, "body starts at %lu, body size: %lu\n", (unsigned long) start_msg, msg_body_size);
 
     /* all done, return the arguments */
     return argv;
@@ -825,7 +867,7 @@ read_from_lines()
     char *mark;				/* mark point in line */
     off_t line_mark;			/* seek position for start of line */
 
-    DEBUG1(DBG_SPOOL_HI, "read_from_lines called (offset=%d)\n", tell_spool());
+    DEBUG1(DBG_SPOOL_HI, "read_from_lines called (offset=%lu)\n", (unsigned long) tell_spool());
     /* if we already know the sender, then don't bother computing it */
     if (sender == NULL) {
 	islocal = TRUE;			/* set FALSE if we find otherwise */
@@ -840,7 +882,7 @@ read_from_lines()
      */
     for (;;) {
 	/* read one complete line */
-	line->i = 0;
+	STR_CLEAR(line);
 	line_mark = tell_spool();
 	while ((c = GETSPOOL()) != EOF && c != READ_FAIL && c != '\n') {
 	    STR_NEXT(line, c);
@@ -853,8 +895,8 @@ read_from_lines()
 	    STR_NEXT(line, c);
 	}
 	STR_NEXT(line, '\0');
-	lp = line->p;
-	if (line->i < 20) {
+	lp = STR(line);
+	if (STR_LEN(line) < 20) {
 	    /* less than 20 chars? couldn't possibly be a From_ line */
 	    break;
 	}
@@ -863,7 +905,7 @@ read_from_lines()
 	    lp++;
 	}
 	/* do we have a From line? */
-	if (strncmp(lp, "From", 4) != 0 || !isspace(lp[4])) {
+	if (strncmp(lp, "From", (size_t) 4) != 0 || !isspace((int) lp[4])) {
 	    /* no, end of From_ lines */
 	    break;
 	}
@@ -872,43 +914,43 @@ read_from_lines()
 	}
 	/* extract the user */
 	lp += 5;
-	while (isspace(*lp)) {	/* scan for start of user name */
+	while (isspace((int) *lp)) {	/* scan for start of user name */
 	    lp++;
 	}
 	mark = lp;			/* mark the start of the user name */
 	/* skip over the user name token */
 	lp = forward_token(lp);
 	*lp = '\0';
-	user->i = 0;
+	STR_CLEAR(user);
 	STR_CAT(user, mark);
 
 	/* search for a remote from host */
-	lp = line->p + line->i - 2;
-	while (isspace(*lp)) {
+	lp = STR(line) + STR_LEN(line) - 2;
+	while (isspace((int) *lp)) {
 	    lp--;
 	}
 	lp[1] = '\0';			/* terminate end of potential host */
-	lp = backward_token(line->p, lp); /* back to start of host (?) */
+	lp = backward_token(STR(line), lp); /* back to start of host (?) */
 	mark = lp;			/* host may be here */
-	lp = backward_token(line->p, lp-1); /* back to start of from (?) */
-	if (strncmp(lp, "from", 4) != 0 || !isspace(lp[4])) {
-	    path->i = 0;		/* no remote from host, toss path */
-	    DEBUG1(DBG_SPOOL_HI, "no remote from host in <%s>, toss path\n",
-		   line->p);
+	lp = backward_token(STR(line), lp-1); /* back to start of from (?) */
+	if (strncmp(lp, "from", (size_t) 4) != 0 || !isspace((int) lp[4])) {
+	    STR_CLEAR(path);		/* no remote from host, toss path */
+	    DEBUG1(DBG_SPOOL_HI, "no remote from host in '%s', toss path\n",
+		   STR(line));
 	    continue;			/* next From_ line */
 	}
-	lp = backward_token(line->p, lp-1); /* back to start of remote (?) */
-	if (strncmp(lp, "remote", 6) != 0 || !isspace(lp[6])) {
-	    path->i = 0;		/* no remote from host, toss path */
-	    DEBUG1(DBG_SPOOL_HI, "no remote from host in <%s>, toss path\n",
-		   line->p);
+	lp = backward_token(STR(line), lp-1); /* back to start of remote (?) */
+	if (strncmp(lp, "remote", (size_t) 6) != 0 || !isspace((int) lp[6])) {
+	    STR_CLEAR(path);		/* no remote from host, toss path */
+	    DEBUG1(DBG_SPOOL_HI, "no remote from host in '%s', toss path\n",
+		   STR(line));
 	    continue;			/* next From_ line */
 	}
 
 	/*
 	 * found a remote_from host, add it to the sender path
 	 */
-	if (path->i > 0) {
+	if (STR_LEN(path) > 0) {
 	    STR_NEXT(path, '!');	/* ! is separator */
 	}
 	STR_CAT(path, mark);
@@ -927,7 +969,7 @@ read_from_lines()
     if (sender == NULL) {
 	STR_NEXT(path, '\0');		/* terminate path and user */
 	STR_NEXT(user, '\0');
-	finish_sender(path->p, user->p);
+	finish_sender(STR(path), STR(user));
 	STR_FREE(path);		/* free the storage used */
 	STR_FREE(user);
     }
@@ -957,7 +999,7 @@ forward_token(p)
 {
     register int inquote = FALSE;	/* not in a quote */
 
-    while (isspace(*p))			/* scan to start of token */
+    while (isspace((int) *p))		/* scan to start of token */
 	p++;
 
     /* loop exits by return when done */
@@ -998,7 +1040,7 @@ backward_token(s,p)
     register char *p;			/* start search here */
 {
     /* scan before end of token */
-    while (isspace(*p)) {
+    while (isspace((int) *p)) {
 	if (p == s) {
 	    return p;
 	}
@@ -1006,7 +1048,7 @@ backward_token(s,p)
     }
 
     /* scan to start of token */
-    while (*p && !isspace(*p)) {
+    while (*p && !isspace((int) *p)) {
 	if (p == s) {
 	    return p;
 	}
@@ -1110,7 +1152,7 @@ write_body(f, flags)
     FILE *f;				/* write to this file */
     long flags;				/* transport flags */
 {
-    if (seek_spool((off_t) start_msg) == FAIL) {
+    if (seek_spool(start_msg) == FAIL) {
 	write_log(WRITE_LOG_SYS, "seek failed: %s/%s: %s",
 		  spool_dir, input_spool_fn, strerror(errno));
 	return READ_FAIL;
@@ -1129,7 +1171,7 @@ write_body(f, flags)
 void
 check_grade()
 {
-    struct list *hq;			/* temp for scanning headers */
+    charplist_t *hq;			/* temp for scanning headers */
 
     /* grab the current grade */
     msg_grade = message_id[strlen(message_id) - 1];
@@ -1138,7 +1180,7 @@ check_grade()
     for (hq = header; hq; hq = hq->succ) {
 	if (HDREQ("precedence", hq->text)) {
 	    /* found the precedence header */
-	    int grade = parse_precedence(index(hq->text, ':') + 1);
+	    int grade = parse_precedence(strchr(hq->text, ':') + 1);
 
 	    if (grade && msg_grade != grade) {
 		/*
@@ -1162,7 +1204,7 @@ check_grade()
 void
 log_incoming()
 {
-    struct list *hq;			/* temp for scanning headers */
+    charplist_t *hq;			/* temp for scanning headers */
     char *old_id = NULL;		/* previous message-id */
     char *trim_old_id = NULL;		/* trimmed previous message-id */
     char *resent_mid = NULL;		/* point to Resent-Message-Id field */
@@ -1179,9 +1221,9 @@ log_incoming()
 	}
     }
     if (resent_mid) {
-	old_id = index(resent_mid, ':');
+	old_id = strchr(resent_mid, ':');
     } else if (mid) {
-	old_id = index(mid, ':');
+	old_id = strchr(mid, ':');
     }
 
     if (sender_host) {
@@ -1211,9 +1253,9 @@ log_incoming()
     if (old_id) {
 	register char *p;
 
-	trim_old_id = xmalloc(strlen(old_id));
+	trim_old_id = xmalloc((size_t) strlen(old_id));
 	for (p = trim_old_id, old_id++; *old_id; old_id++) {
-	    if (!isspace(*old_id)) {
+	    if (!isspace((int) *old_id)) {
 		*p++ = *old_id;
 	    }
 	}
@@ -1224,20 +1266,20 @@ log_incoming()
 #if (SMAIL_LOG_STYLE == 2)
     /* Default new log style (as of 3.1.29) */
     /* This is SMAIL_LOG_STYLE=2 */
-    write_log(WRITE_LOG_SYS, "Received%s%s%s%s%s%s%s%s%s%s%s%ld%s%s%s%s",
+    write_log(WRITE_LOG_SYS, "Received%s%s%s%s%s%s%s%s%s%s%s%lu%s%s%s%s",
 	      " FROM:", sender,
 	      host_string? " HOST:": "", host_string? host_string: "",
 	      sender_proto? " PROTOCOL:": "", sender_proto? sender_proto: "",
 	      " PROGRAM:", program,
 	      trim_old_id? " ORIG-ID:": "", trim_old_id? trim_old_id: "",
-	      " SIZE:", (long) msg_size,
+	      " SIZE:", (unsigned long) msg_size,
 	      ident_sender? " IDENT:": "", ident_sender? ident_sender: "",
 	      ident_method? " ID-METHOD:": "", ident_method? ident_method: "");
 #else
 # if (!defined(SMAIL_LOG_STYLE)) || (SMAIL_LOG_STYLE == 1)
     /* Old log style (as used by 3.1.28) */
     /* This is SMAIL_LOG_STYLE=1 */
-    write_log(WRITE_LOG_SYS, "received%s%s%s%s%s%s%s%s%s%s%s%ld%s%s%s%s%s",
+    write_log(WRITE_LOG_SYS, "received%s%s%s%s%s%s%s%s%s%s%s%lu%s%s%s%s%s",
 	      "\n|\t     from: ", sender,
 		host_string?
 	      "\n|\t     host: ": "", host_string? host_string: "",
@@ -1246,7 +1288,7 @@ log_incoming()
 	      "\n|\t  program: ", program,
 		trim_old_id?
 	      "\n|\t  orig-id: ": "", trim_old_id? trim_old_id: "",
-	      "\n|\t     size: ", (long) msg_size, " bytes",
+	      "\n|\t     size: ", (unsigned long) msg_size, " bytes",
 	      ident_sender? 
 	      "\n|\t    ident: ": "", ident_sender? ident_sender: "",
 	      ident_method? 
@@ -1277,15 +1319,15 @@ scan_spool_dirs(frozen)
     int frozen;				/* look for frozen messages instead? */
 {
     static char **mv = NULL;		/* vector of messages */
-    unsigned int mc = 0;		/* count of messages */
-    static int mv_size = 32;		/* allocated vector entries */
+    size_t mc = 0;			/* count of messages */
+    static unsigned int mv_size = 32;	/* allocated vector entries */
     char *dn;				/* current directory to scan */
     static struct str str;		/* storage for filenames */
-    register int i;			/* temp */
+    register size_t i;			/* temp index */
 
     /* get the initial vector and string storage allocation */
     if (mv == NULL) {
-	mv = (char **)xmalloc(mv_size * sizeof(*mv));
+	mv = (char **) xmalloc((size_t) mv_size * sizeof(*mv));
 	STR_INIT(&str);
     } else {
 	STR_CHECK(&str);
@@ -1303,7 +1345,7 @@ scan_spool_dirs(frozen)
 	    int entry_grade;
 
 	    /* see if the file matches the form of a spool file */
-	    if (! isdigit(fn[0]) || strlen(fn) != SPOOL_FN_LEN) {
+	    if (! isdigit((int) fn[0]) || strlen(fn) != SPOOL_FN_LEN) {
 		DEBUG1(DBG_QUEUE_MID, "scan_spool_dirs() ignoring %s\n", fn);
 		continue;		/* nope */
 	    }
@@ -1322,7 +1364,7 @@ scan_spool_dirs(frozen)
 		    msgid = ptr->in_addr;
 		    if (*msgid == 'm') /* Skip leading m if specified */
 			msgid++;
-		    if (*msgid == '/') /* use basename if full path given */
+		    if (strchr(msgid, '/')) /* use basename if path specified */
 			msgid = strrchr(msgid, '/') + 1;
 		    if ((seen = EQ(fn, msgid)))
 			break;
@@ -1348,7 +1390,7 @@ scan_spool_dirs(frozen)
 	    if (mc >= mv_size - 1) {
 		/* note: we allow, in advance, for the last NULL entry */
 		mv_size += 32;		/* get more entries */
-		mv = (char **) xrealloc((char *)mv, (unsigned) (mv_size * sizeof(*mv)));
+		mv = (char **) xrealloc((char *) mv, (size_t) (mv_size * sizeof(*mv)));
 	    }
 	    /* use offsets for now, to be converted to (char *) later */
 	    set_ptr(mv[mc++], STR_LEN(&str));
@@ -1379,8 +1421,8 @@ queue_compare(x, y)
     const void *x;
     const void *y;
 {
-    char *a = *((char **) x);
-    char *b = *((char **) y);
+    const char *a = *((const char * const *) x);
+    const char *b = *((const char * const *) y);
     int a_grade = a[strlen(a) - 1];
     int b_grade = b[strlen(b) - 1];
 
