@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:resolve.c,v 1.27 2000/10/02 02:06:41 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:resolve.c,v 1.37 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -16,32 +16,56 @@
  *
  *	external functions: resolve_addr_list, islocalhost
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
-#define NEED_SOCKETS			/* grumble */
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
+#include "smailsock.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "hash.h"
 #include "direct.h"
 #include "route.h"
-#include "transport.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "log.h"
+#include "transport.h"
 #include "exitcodes.h"
 #include "extern.h"
 #include "debug.h"
 #include "error.h"
-
-#ifndef INADDR_NONE
-# define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
-#endif
-#ifndef INADDR_LOOPBACK
-# define INADDR_LOOPBACK	((unsigned long) 0x7F000001)
-#endif
+#include "smailport.h"
 
 /* exported variables */
 struct hash_table *hit_table = NULL;	/* table to recognize address hits */
@@ -127,7 +151,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 	     */
 	    if (cur->flags & ADDR_DONTHASH) {
 		DEBUG1(DBG_RESOLVE_MID, "%s: not checking for dups because of ADDR_DONTHASH.\n", cur->work_addr);
-	    } else if (hash_addrs && add_to_hash(cur->work_addr, (char *) NULL, 0, hit_table) == ALREADY_HASHED) {
+	    } else if (hash_addrs && add_to_hash(cur->work_addr, (char *) NULL, (size_t) 0, hit_table) == ALREADY_HASHED) {
 		DEBUG1(DBG_RESOLVE_LO, "%s: has already been seen -- skipping.\n", cur->work_addr);
 		continue;
 	    }
@@ -136,7 +160,6 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 				 &cur->remainder, &cur->parseflags);
 	    switch (form) {
 	    case FAIL:
-	    case PARSE_ERROR:
 		/*
 		 * ERR_111 - address parse error
 		 *
@@ -172,7 +195,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 	    default:			/* anything else is a remote-form address */
 		/* determine if the target host is actually a local host */
 		if (islocalhost(cur->target)) {
-		    DEBUG1(DBG_RESOLVE_HI, "target %s was actually a local host, will reparse remainder.\n", cur->target);
+		    DEBUG2(DBG_RESOLVE_LO, "target domain %s is a local host, will reparse remainder (%s).\n", cur->target, cur->remainder);
 		    /* remember the local name */
 		    cur->local_name = COPY_STRING(cur->target);/* XXX will leak? */
 		    /* it is a local host, but save the name for virtual host processing */
@@ -220,7 +243,7 @@ resolve_addr_list(in, out, defer, fail, hash_addrs)
 			      cur->next_addr,
 			      cur->next_host ? cur->next_host : "(localhost)",
 			      cur->transport->name);
-	    if (add_to_hash(hashval, (char *) NULL, 0, hit_out_table) == ALREADY_HASHED) {
+	    if (add_to_hash(hashval, (char *) NULL, (size_t) 0, hit_out_table) == ALREADY_HASHED) {
 		DEBUG1(DBG_RESOLVE_LO, "%s: has already been seen -- skipping during rescan.\n", hashval);
 		/* we always get around the loop at least once without entering
 		 * this side of the 'if', so 'prev' will never be NULL
@@ -292,7 +315,8 @@ islocalhost(target)
 	 * Do a reverse query to get the hostname.  This is a bit bogus as it
 	 * requires that the lookup work for the check to function properly.
 	 */
-	if ((hostentp = gethostbyaddr((char *) &(inet_s.s_addr), sizeof(inet_s.s_addr), AF_INET))) {
+	if ((hostentp = gethostbyaddr((char *) &(inet_s.s_addr), /* XXX (socklen_t) */ sizeof(inet_s.s_addr), AF_INET))) {
+	    DEBUG2(DBG_RESOLVE_MID, "islocalhost: address literal target %s resolves to '%s'\n", target, hostentp->h_name);
 	    if (islocalhost(hostentp->h_name)) {
 	        return TRUE;
 	    } else {
@@ -300,6 +324,7 @@ islocalhost(target)
 		int i;
 		
 		for (i = 0; (p = (hostentp->h_aliases)[i]); i++) {
+		    DEBUG2(DBG_RESOLVE_MID, "islocalhost: address literal target %s has alias '%s'\n", target, p);
 		    if (islocalhost(p)) {
 		        return TRUE;
 		    }
