@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:retry.c,v 1.47 2000/10/02 02:03:19 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:retry.c,v 1.57 2003/04/02 22:21:04 woods Exp"
  */
 
 /*
@@ -15,22 +15,43 @@
  *			    retry_host_lock, retry_host_unlock
  */
 
-#include <stdio.h>
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+#include <setjmp.h>
+#include <signal.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
 # include <fcntl.h>
 #else
@@ -38,6 +59,23 @@
 #  include <sys/file.h>
 # endif
 #endif
+#ifdef __GLIBC__
+# include <sys/file.h>
+#endif
+#ifdef HAVE_SYS_LOCKING_H
+# include <sys/locking.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
 #include "smail.h"
 #include "main.h"
@@ -45,17 +83,20 @@
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
-#include "transport.h"
 #include "log.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "lookup.h"
 #include "jump.h"
 #include "spool.h"
+#include "transport.h"
 #include "smailconf.h"
 #include "extern.h"
 #include "debug.h"
 #include "error.h"
+#include "smailport.h"
 
 #define RETRY_MODE   0644		/* file creation modes */
 
@@ -65,25 +106,6 @@
 # define O_SWYM 0
 #endif
 
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
-
-#ifdef ANSI_C
-# define P_(x)		x
-# define VOLATILE	static volatile
-#else
-# define P_(x)		()
-# define VOLATILE	/* most compilers won't optimize global variables */
-#endif
-
 struct retry {
     struct retry *succ;			/* next in chain */
     char    *domains;			/* target domain, or "*" */
@@ -98,20 +120,20 @@ static char *locked_path = NULL;
 static struct retry *locked_retry = NULL;
 static JUMP_ENVBUF tryfile_jmp;
 
-static struct retry *default_retry P_((void));
-static int match_retry_domain P_((char *, char *));
-static int allow_now P_((int, char *, time_t, time_t, struct retry *,
+static struct retry *default_retry __P((void));
+static int match_retry_domain __P((char *, char *));
+static int allow_now __P((int, char *, time_t, time_t, struct retry *,
 			 struct error **));
-static int allow_later P_((time_t, time_t, struct retry *, struct error **));
-static char *tryfile_name P_((struct transport *, char *, int));
-static int tryfile_cat P_((struct str *, char *, unsigned, int, int, int));
-static int tryfile_mkdir P_((struct str *));
-static int tryfile_lock_wait P_((int, long));
-static int tryfile_lock P_((int));
-static void tryfile_unlock P_((int));
-static void tryfile_alarm P_((int));
-static void write_error_or_unlink P_((int, char *, struct error *));
-static void read_error P_((int, char *, struct error *));
+static int allow_later __P((time_t, time_t, struct retry *, struct error **));
+static char *tryfile_name __P((struct transport *, char *, int));
+static int tryfile_cat __P((struct str *, char *, size_t, int, int, int));
+static int tryfile_mkdir __P((struct str *));
+static int tryfile_lock_wait __P((int, long));
+static int tryfile_lock __P((int));
+static void tryfile_unlock __P((int));
+static void tryfile_alarm __P((int));
+static void write_error_or_unlink __P((int, char *, struct error *));
+static void read_error __P((int, char *, struct error *));
 
 /*
  * read_retry_file - read and remember contents of retry file
@@ -226,7 +248,7 @@ read_retry_file()
 
 	/* assign new structure's domain */
 	/* shrink the storage from parse_table() to toss the "value" part */
-	r->domains = xrealloc(new->name, (unsigned) (strlen(new->name) + 1));
+	r->domains = xrealloc(new->name, (size_t) (strlen(new->name) + 1));
 
 	/* assign new structure's limits */
 	r->interval = interval;
@@ -294,9 +316,9 @@ dump_retry_config(f)
 	    fputs("0/", f);
 	    comment = "#\tnote retry_interval of zero uses runtime (queue_interval * 2)\n";
 	} else {
-	    fprintf(f, "%s/", (value = ltoival(rp->interval)) ? value : (comment = "#\tinvalid interval in entry above\n", "-1"));
+	    fprintf(f, "%s/", (value = ltoival((long) rp->interval)) ? value : (comment = "#\tinvalid interval in entry above\n", "-1"));
 	}
-	fprintf(f, "%s\n", (value = ltoival(rp->duration)) ? value : (comment = "#\tinvalid duration in entry above\n", "-1"));
+	fprintf(f, "%s\n", (value = ltoival((long) rp->duration)) ? value : (comment = "#\tinvalid duration in entry above\n", "-1"));
 	if (comment) {
 	    fputs(comment, f);
 	}
@@ -365,7 +387,7 @@ retry_addr_before(in, defer, fail)
 		    break;
 		}
 
-		DEBUG1(DBG_RETRY_LO, "examine %s\n", tryname);
+		DEBUG1(DBG_RETRY_LO, "retry_addr_before(): examining %s\n", tryname);
 
 		/*
 		 * examine the retry file: lock and call allow_now() and then unlock;
@@ -376,12 +398,13 @@ retry_addr_before(in, defer, fail)
 
 		    if ((fd = open(tryname, O_RDWR|O_SWYM)) == -1) {
 			time(&started);
+			DEBUG2(DBG_RETRY_LO, "retry_addr_before(): open of %s failed: %s\n", tryname, strerror(errno));
 			may_deliver = TRUE;
 			break;
 		    }
 		    lk = tryfile_lock_wait(fd, 0L);
 		    if (lk == DB_FAIL) {
-			DEBUG1(DBG_RETRY_LO, "tryfile_lock_wait() failed: %s", strerror(errno));
+			DEBUG2(DBG_RETRY_LO, "retry_addr_before(): lock of %s failed: %s\n", tryname, strerror(errno));
 			/* XXX should this be written to the system log instead? */
 			(void) close(fd);
 			(void) unlink(tryname);	/* XXX really??? */
@@ -497,7 +520,7 @@ retry_addr_finished(addr)
 		continue;
 	    }
 
-	    DEBUG2(DBG_RETRY_LO, "%s %s\n",
+	    DEBUG2(DBG_RETRY_LO, "retry_addr_finished(): %s %s\n",
 		   error ? "writing error message in" : "removing", tryname);
 
 	    /* write to, or remove, retry file */
@@ -507,17 +530,19 @@ retry_addr_finished(addr)
 		if ((fd = open(tryname, O_WRONLY | (error ? O_CREAT : 0),
 			       RETRY_MODE)) == -1)
 		{
+		    /* worth noting if we were not going to remove it... */
 		    if (error) {
-		        write_log(WRITE_LOG_SYS, "tryfile_lock_wait(): open(%s) failed: %s",
+		        write_log(WRITE_LOG_SYS, "retry_addr_finished(): open(%s) failed: %s",
 				  tryname, strerror(errno));
 		    }
 		    break;
 		}
 		lk = tryfile_lock_wait(fd, 0L);
 		if (lk == DB_FAIL) {
-		    write_log(WRITE_LOG_SYS, "tryfile_lock_wait() lock of %s failed: %s %s",
+		    write_log(WRITE_LOG_SYS, "retry_addr_finished(): lock of %s failed: %s%s %s",
 			      tryname,
 			      strerror(errno),
+			      error ? "," : "",
 			      error ? error->message : "(would have removed it)");
 		    (void) close(fd);
 		    break;
@@ -579,7 +604,7 @@ retry_host_lock(transport, host, defer_failure, error)
 
     /* calculate path of retry file, creating directories */
     if ((tryname = tryfile_name(transport, host, TRUE)) == NULL) {
-	*error = note_error(ERR_175, xprintf("cannot find retry file for %s/%s", transport, host));
+	*error = note_error(ERR_175, xprintf("cannot create retry file directory for %s/%s", transport, host));
 	*defer_failure = TRUE;
 	return FAIL;
     }
@@ -690,7 +715,7 @@ retry_host_lock(transport, host, defer_failure, error)
 /* ARGSUSED */
 void
 retry_host_unlock(started, error)
-    time_t started;			/* started not used! */
+    time_t started;			/* UNUSED */
     struct error *error;
 {
     if (locked_fd < 0) {
@@ -862,7 +887,7 @@ tryfile_name(transport, host, create_dirs)
     }
 
     if ((int)strlen(host) <= pathmax) {
-	if (tryfile_cat(&path, host, (unsigned) strlen(host), TRUE, FALSE,
+	if (tryfile_cat(&path, host, strlen(host), TRUE, FALSE,
 			create_dirs) == FAIL)
 	{
 	    return NULL;
@@ -889,7 +914,7 @@ tryfile_name(transport, host, create_dirs)
 		}
 		--p;
 	    }
-	    if (tryfile_cat(&path, p, (unsigned) (q - p), first, partial,
+	    if (tryfile_cat(&path, p, (size_t) (q - p), first, partial,
 			    create_dirs) == FAIL)
 	    {
 		return NULL;
@@ -906,12 +931,14 @@ static int
 tryfile_cat(sp, p, len, first, partial, create_dirs)
     struct str *sp;
     char *p;
-    unsigned len;
-    int first, partial, create_dirs;
+    size_t len;
+    int first;
+    int partial;
+    int create_dirs;
 {
-    unsigned i;
+    size_t i;
 
-    if (!first && sp->i && sp->p[sp->i - 1] != '_') {
+    if (!first && STR_LEN(sp) && STR(sp)[STR_LEN(sp) - 1] != '_') {
 	STR_NEXT(sp, '.');
     }
 
@@ -921,11 +948,7 @@ tryfile_cat(sp, p, len, first, partial, create_dirs)
 
     STR_NEXT(sp, '/');
     for (i = 0; i < len; ++i) {
-	int c = *(p + i) & 0xFF;
-	if (isupper(c)) {
-	    c = tolower(c);
-	}
-	STR_NEXT(sp, c);
+	STR_NEXT(sp, tolower((int) (*(p + i) & 0xFF))); /*  hostnames are case insensitive */
     }
     if (partial) {
 	STR_NEXT(sp, '_');
@@ -941,22 +964,28 @@ tryfile_mkdir(sp)
     struct stat statbuf;
     int ret = SUCCEED;
 
-    STR_NEXT(sp, '\0');
-    if (stat(sp->p, &statbuf) == -1) {
-	DEBUG1(DBG_RETRY_LO, "make directory %s\n", sp->p);
-	(void) mkdir(sp->p, auto_mkdir_mode);
-	if (stat(sp->p, &statbuf) == -1) {
-	    write_log(WRITE_LOG_SYS, "can't create directory %s", sp->p);
+    STR_NEXT(sp, '\0');				/* NUL terminate */
+    if (stat(STR(sp), &statbuf) == -1) {
+	DEBUG1(DBG_RETRY_LO, "make directory %s\n", STR(sp));
+	(void) mkdir(STR(sp), auto_mkdir_mode);
+	if (stat(STR(sp), &statbuf) == -1) {
+	    write_log(WRITE_LOG_SYS, "can't create directory %s", STR(sp));
 	    ret = FAIL;
 	}
     }
-    sp->i--;
+    STR_PREV(sp);				/* drop the NUL just added */
 
     return ret;
 }
 
-VOLATILE JUMPSIG old_sigalrm;
-VOLATILE unsigned int old_alarm;
+#if ((__STDC__ - 0) <= 0)
+/* most traditional compilers won't optimize global variables */
+JUMPSIG old_sigalrm;
+unsigned int old_alarm;
+#else
+static volatile JUMPSIG old_sigalrm;
+static volatile unsigned int old_alarm;
+#endif
 
 /*
  * tryfile_lock_wait - lock an open retry file, waiting until an exclusive lock
@@ -1041,7 +1070,7 @@ tryfile_unlock(fd)
 /* ARGSUSED */
 static void
 tryfile_alarm(sig)
-     int sig;
+     int sig;					/* UNUSED */
 {
     JUMP_LONGJMP(tryfile_jmp, 1);
 }
@@ -1086,25 +1115,25 @@ write_error_or_unlink(fd, tryfile, error)
 static void
 read_error(fd, tryfile, error)
     int fd;
-    char *tryfile;				/* UNUSED! */
+    char *tryfile;				/* UNUSED */
     struct error *error;
 {
     struct stat statbuf;
     char *buf, *errmsg, *p;
     long errcode;
-    unsigned int len;
+    size_t len;
 
     /* assume that we can't read the file */
     error->info = ERR_173;
     error->message = "retry time expired";
 
     /* read the whole file */
-    if (fstat(fd, &statbuf) == -1 || statbuf.st_size > 0x7FFF) {
+    if (fstat(fd, &statbuf) == -1 || statbuf.st_size > 0x7FFF) { /* XXX ??? */
 	return;
     }
-    len = (int)statbuf.st_size;
+    len = statbuf.st_size;
     buf = xmalloc(len + 1);
-    if (read(fd, buf, (size_t) (len + 1)) != len) {
+    if ((size_t) read(fd, buf, (size_t) (len + 1)) != len) {
 	xfree(buf);
 	return;
     }
@@ -1115,7 +1144,7 @@ read_error(fd, tryfile, error)
     errmsg = NULL;
 
     p = buf;
-    while (isdigit(*p)) {
+    while (isdigit((int) *p)) {
 	++p;
     }
     if (*p == ' ') {
