@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:route.c,v 1.38 2000/09/03 17:00:17 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:route.c,v 1.51 2003/04/02 22:20:57 woods Exp"
  */
 
 /*
@@ -22,14 +22,42 @@
  *			    read_method_file dump_router_config
  */
 
-#define NEED_SOCKETS
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
+#include "smailsock.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
@@ -38,16 +66,19 @@
 #  define USE_MX_HINTS
 #  include "bindsmtpth.h"
 #endif /* HAVE_BSD_NETWORKING */
-#include "transport.h"
 #include "field.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "log.h"
 #include "exitcodes.h"
+#include "transport.h"
 #include "smailconf.h"
 #include "extern.h"
 #include "debug.h"
 #include "error.h"
+#include "smailport.h"
 
 /* variables exported from this file */
 int cached_routers = FALSE;		/* TRUE if cache_routers() called */
@@ -64,11 +95,10 @@ static char *router_driv_function __P((char *, struct attribute *));
 
 /* Config table */
 static struct attr_table router_generic[] = {
-    { "driver", t_string, NULL, NULL, OFFSET(router, driver) },
-    { "transport", t_string, NULL, NULL,
-	  OFFSET(router, default_transport) },
-    { "method", t_proc, NULL, (tup *)router_read_method, 0 },
-    { "always", t_boolean, NULL, NULL, USE_ALWAYS },
+    { "driver", t_string, 0, NULL, NULL, OFFSET(router, driver) },
+    { "transport", t_string, 0, NULL, NULL, OFFSET(router, default_transport) },
+    { "method", t_proc, 0, NULL, (tup *) router_read_method, 0 },
+    { "always", t_boolean, 0, NULL, NULL, USE_ALWAYS },
 };
 
 
@@ -174,7 +204,7 @@ route_remote_addrs(in, out, retry, defer, fail)
 	     *      message was in error, he or she should send mail to the
 	     *      postmaster.
 	     */
-	    cur->error = note_error(ERR_NSOWNER|ERR_101, "unknown host");
+	    cur->error = note_error(ERR_NSOWNER|ERR_101, "unknown target host domain name");
 	    cur->succ = *fail;
 	    *fail = cur;
 	    continue;
@@ -199,6 +229,7 @@ route_remote_addrs(in, out, retry, defer, fail)
 				 (char *) NULL, &list, F_ALIAS, &field_error);
 	    if (field_error) {
 		DEBUG1(DBG_ROUTE_HI, "     cannot parse field: %s\n", field_error);
+		exitvalue = EX_DATAERR;		/* set exit status */
 		/*
 		 * ERR_111 - malformed address(es) returned by the router
 		 *
@@ -258,6 +289,7 @@ route_remote_addrs(in, out, retry, defer, fail)
 	}
 	if (cur->error) {
 	    /* got an error, fail the address */
+	    exitvalue = EX_NOHOST;	/* set exit status */
 	    cur->succ = *fail;
 	    *fail = cur;
 	} else {
@@ -475,7 +507,7 @@ route_driver_finish(rp, addr, match_count, next_host, route, transport, hints)
 {
     DEBUG1(DBG_DRIVER_HI, "route_driver_finish() called for driver %s.\n", rp->driver);
 
-    if (addr->flags&ADDR_FINISHED) {
+    if (addr->flags & ADDR_FINISHED) {
 	DEBUG1(DBG_DRIVER_HI, "%s already finished.\n", addr->in_addr);
 	/*
 	 * a router erroneously found a new route for a finished addr,
@@ -496,25 +528,27 @@ route_driver_finish(rp, addr, match_count, next_host, route, transport, hints)
 	 * consider the address to have been matched completely
 	 */
 
-	if (strlen(addr->target) == match_count ||
+	if (strlen(addr->target) == (size_t) match_count ||
 	    (addr->target[0] == '.' &&
-	     strlen(addr->target) == match_count - 1))
+	     strlen(addr->target) == (size_t) match_count - 1))
 	{
 	    DEBUG2(DBG_DRIVER_HI, "%s: full match found to %s.\n", addr->in_addr, addr->target);
 
-	    addr->flags |= ADDR_FINISHED|ADDR_FULLMATCH;
+	    addr->flags |= (ADDR_FINISHED | ADDR_FULLMATCH);
 	} else {
 	    if (next_host == NULL) {
 		/*
 		 * the local host cannot be matched partially, so
 		 * ignore any attempts to do so
 		 */
-		DEBUG1(DBG_DRIVER_HI, "%s: partial match found, but next_host is null.\n", addr->in_addr);
+		DEBUG2(DBG_DRIVER_HI, "%s: partial match found for %s, but next_host is NULL.\n",
+		       addr->in_addr, addr->target);
 
 		return;
 	    }
-	    if (rp->flags&USE_ALWAYS) {
-		DEBUG1(DBG_DRIVER_HI, "%s: partial match found, but USE_ALWAYS so marking finished.\n", addr->in_addr);
+	    if (rp->flags & USE_ALWAYS) {
+		DEBUG4(DBG_DRIVER_HI, "%s: partial match found for '%s' (%d != %d), but USE_ALWAYS so marking finished.\n",
+		       addr->in_addr, addr->target, strlen(addr->target), match_count);
 		addr->flags |= ADDR_FINISHED;
 	    }
 	}
@@ -559,6 +593,8 @@ route_driver_finish(rp, addr, match_count, next_host, route, transport, hints)
 		   (addr->parent) ? addr->parent->in_addr : addr->in_addr,
 		   addr->remainder, addr->router->name);
 	}
+    } else {
+	DEBUG2(DBG_DRIVER_HI, "%s: no match found by %s\n", addr->in_addr, rp->driver);
     }
 }
 
@@ -688,9 +724,9 @@ next_addr_basic(addr, inet)
 	     * route-addr or !-route out of it
 	     */
 
-	    if (route == NULL)
+	    if (route == NULL) {
 		addr->next_addr = COPY_STRING(remainder);
-	    else {
+	    } else {
 		if (inet) {
 		    r_route = route_to_route_addr(route);
 		    addr->next_addr = xprintf("%s:%s", r_route, remainder);
@@ -725,8 +761,8 @@ next_addr_basic(addr, inet)
 	     *      The address owner or sender should fix the address
 	     */
 	    addr->error = note_error(ERR_NSOWNER|ERR_179, 
-				    xprintf("malformed user part in remote address - %s",
-					    new_remainder));
+				     xprintf("malformed user part in remote address - %s",
+					     new_remainder));
 	    return;
 	}
 	if (route && strchr(route, '!') == NULL) {
@@ -1090,6 +1126,7 @@ read_router_file()
     FILE *f;				/* open router file */
     char *error;			/* error from read_standard_file() */
     struct stat statbuf;
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_router_generic = ENDTABLE(router_generic);
     static struct router router_template = {
 	NULL,				/* name */
@@ -1148,6 +1185,7 @@ dump_router_config(f)
      FILE * f;
 {
     struct router *rp;			/* temp for stepping thru routers */
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_router_generic = ENDTABLE(router_generic);
     char * name;			/* name of router */
     long flags;				/* flags for router */
@@ -1197,9 +1235,9 @@ router_read_method(struct_p, attr)
     struct router *rp = (struct router *)struct_p;
     char *error;
 
-    if (attr->value == off) {
+    if (EQ(attr->value, off)) {
 	rp->method = NULL;
-    } else if (attr->value == on) {
+    } else if (EQ(attr->value, on)) {	/* XXX this seems WRONG! */
 	return xprintf("%s: boolean form for non-boolean attribute",
 		       attr->name);
     } else {
@@ -1272,12 +1310,12 @@ read_method_file(fn, error)
 	mgrade = strchr(new->name, '/');
 	if (mgrade) {
 	    *mgrade++ = '\0';
-	    if (isalnum(mgrade[0])) {
+	    if (isalnum((int) mgrade[0])) {
 		methods[ct].mingrade = mgrade[0];
 		methods[ct].maxgrade = mgrade[0];
 	    }
 	    if (mgrade[1] == '-') {
-		if (isalnum(mgrade[2])) {
+		if (isalnum((int) mgrade[2])) {
 		     methods[ct].maxgrade = mgrade[2];
 		} else {
 		     methods[ct].maxgrade = 255;
