@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:bind.c,v 1.29 2000/09/03 17:00:22 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:bind.c,v 1.35 2003/04/02 22:15:05 woods Exp"
  */
 
 /*
@@ -26,27 +26,58 @@
  * Specifications for this router driver can be found in "bindlib.c".
  *
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/param.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "bind.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 
@@ -151,7 +182,7 @@ static int
 bind_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
@@ -166,5 +197,13 @@ bind_lookup(rp, addr, fl, rt_info, error_p)
 		    what, rt_info, error_p);
     xfree(what);
 
+    switch (ret) {
+    case DB_SUCCEED:
+	DEBUG6(DBG_DRIVER_MID, "router %s: driver %s: '%s' is reachable as mail exchanger '%s'%s%s\n",
+	       rp->name, rp->driver, addr->target, rt_info->next_host,
+	       rt_info->route ? " using route: " : "", rt_info->route ? rt_info->route : "");
+	break;
+    }
+
     return ret;
 }
