@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:gethost.c,v 1.22 2000/10/02 02:06:42 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:gethost.c,v 1.36 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -75,43 +75,58 @@
  *		[192.257]		- might return [192.0.1.1]
  */
 
-#define NEED_SOCKETS
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
+#include "../smailsock.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "gethost.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
-
-#ifndef INADDR_NONE
-# define INADDR_NONE		((unsigned long) (-1)) /* XXX 64bit too??? */
-#endif
-#ifndef INADDR_LOOPBACK
-# define INADDR_LOOPBACK	((unsigned long) 0x7F000001)
-#endif
-
-/*
- * import h_errno; many systems don't define it in <netdb.h>
- */
-
-#ifndef OBSOLETE_RESOLVER
-extern int h_errno;
-#endif
+#include "../smailport.h"
 
 /* functions local to this file */
 static int gethostbyname_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
@@ -119,16 +134,16 @@ static int gethostbyaddr_lookup __P((struct router *, struct addr *, int, struct
 static int bad_form __P((struct router *, char *, struct error **));
 
 static struct attr_table gethostbyname_attributes[] = {
-    { "domain", t_string, NULL, NULL, OFFSET(gethostbyname_private, domain) },
-    { "required", t_string, NULL, NULL, OFFSET(gethostbyname_private, required) },
-    { "only_local_domain", t_boolean, NULL, NULL, GETHOST_ONLY_LOCAL },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(gethostbyname_private, domain) },
+    { "required", t_string, 0, NULL, NULL, OFFSET(gethostbyname_private, required) },
+    { "only_local_domain", t_boolean, 0, NULL, NULL, GETHOST_ONLY_LOCAL },
 };
 static struct attr_table *end_gethostbyname_attributes =
     ENDTABLE(gethostbyname_attributes);
 
 static struct attr_table gethostbyaddr_attributes[] = {
-    { "fail_if_error", t_boolean, NULL, NULL, GETHOST_FAIL_IFERR },
-    { "check_for_local", t_boolean, NULL, NULL, GETHOST_CHECK_LOCAL },
+    { "fail_if_error", t_boolean, 0, NULL, NULL, GETHOST_FAIL_IFERR },
+    { "check_for_local", t_boolean, 0, NULL, NULL, GETHOST_CHECK_LOCAL },
 };
 static struct attr_table *end_gethostbyaddr_attributes =
     ENDTABLE(gethostbyaddr_attributes);
@@ -216,25 +231,46 @@ static int
 gethostbyname_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
-    struct error **error_p;		/* return lookup error here */
+    struct error **error_p;		/* UNUSED return lookup error here */
 {
     struct hostent *hostentp;		/* host file entry */
     char *copy_target;			/* copy of the target */
     char *free_target;			/* free from this point */
     register char *p;			/* temp */
+    unsigned long tmpaddr;
     struct gethostbyname_private *priv;
 
-    priv = (struct gethostbyname_private *)rp->private;
+    priv = (struct gethostbyname_private *) rp->private;
+
+    DEBUG3(DBG_DRIVER_HI, "router %s: driver %s: gethostbyname_lookup called for %s.\n",
+	   rp->name, rp->driver, addr->target);
 
     if (priv->required) {
 	if (match_end_domain(priv->required, addr->target) == NULL) {
-	    /* did not end in a required domain */
+	    DEBUG4(DBG_DRIVER_HI, "router %s: driver %s: target '%s' did not match required domain %s.\n",
+		   rp->name, rp->driver, addr->target, priv->required);
 	    return DB_NOMATCH;
 	}
     }
 
+    /*
+     * modern(?) gethostbyname() will "work" (i.e. allow a match) if given an
+     * address in the form accepted by inet_pton() so we first run it through
+     * inet_pton() and return DB_NOMATCH if it is a valid IP address.  If the
+     * postmaster wants to match address literals then they really should only
+     * be accepted in the proper format (i.e. with surrounding square brackets,
+     * because that's the only way they'll likely work for delivery to any
+     * remote SMTP server, for one) and that's done by the gethostbyaddr router
+     * driver.
+     */
+    if (inet_pton(AF_INET, addr->target, &tmpaddr) > 0) {
+	DEBUG3(DBG_DRIVER_HI, "router %s: driver %s: target '%s' is a bare IP address, not a hostname.\n",
+	       rp->name, rp->driver, addr->target);
+	return DB_NOMATCH;
+    }
+
     free_target = copy_target = COPY_STRING(addr->target);
     if (copy_target[0] == '.') {
 	/* ignore initial dot */
@@ -246,6 +282,7 @@ gethostbyname_lookup(rp, addr, fl, rt_info, error_p)
      */
     if (priv->domain) {
 	char *domain_part = match_end_domain(priv->domain, copy_target);
+
 	if (domain_part) {
 	    DEBUG1(DBG_DRIVER_HI, "strip \"%s\"\n", domain_part);
 	    *domain_part = '\0';
@@ -253,15 +290,17 @@ gethostbyname_lookup(rp, addr, fl, rt_info, error_p)
     }
 
     /* if we only want local names and there is any dot left, punt it */
-    if ((rp->flags & GETHOST_ONLY_LOCAL) && index(copy_target, '.') != NULL) {
+    if ((rp->flags & GETHOST_ONLY_LOCAL) && strchr(copy_target, '.') != NULL) {
+	DEBUG3(DBG_DRIVER_HI, "router %s: driver %s: target '%s' is not a local-area hostname.\n",
+	       rp->name, rp->driver, copy_target);
 	xfree(free_target);
 	return DB_NOMATCH;
     }
 
     /* look for a match, mapping upper case to lower case */
     for (p = copy_target; *p; p++) {
-	if (isupper(*p)) {
-	    *p = tolower(*p);
+	if (isupper((int) *p)) {
+	    *p = tolower((int) *p);
 	}
     }
 
@@ -269,9 +308,17 @@ gethostbyname_lookup(rp, addr, fl, rt_info, error_p)
     xfree(free_target);
 
     if (hostentp) {
+	/* XXX Do we have to set the parent address here to!?!?!? */
+	/*
+	 * the following is necessary to tell higher level software to re-parse
+	 * the address.
+	 */
 	rt_info->next_host = hostentp->h_name;
 	rt_info->matchlen = strlen(addr->target);
 
+	DEBUG4(DBG_DRIVER_MID, "router %s: driver %s: '%s' is reachable via hostname '%s'\n",
+	       rp->name, rp->driver, copy_target, rt_info->next_host);
+
 	return DB_SUCCEED;
     }
 
@@ -378,7 +425,7 @@ gethostbyaddr_lookup(rp, addr, fl, rt_info, error_p)
 	return bad_form(rp, addr->target, error_p);
     }
 
-    p = index(p, ']');
+    p = strchr(p, ']');
     if (!p || *(p+1) != '\0') {
 	return bad_form(rp, addr->target, error_p);
     }
@@ -396,6 +443,11 @@ gethostbyaddr_lookup(rp, addr, fl, rt_info, error_p)
      */
     rt_info->next_host = xprintf("[%s]", inet_ntoa(inet_s));
 
+    /*
+     * Tell the router that we've matched the whole thing....
+     */
+    rt_info->matchlen = strlen(addr->target);
+
     if (fl & RT_VERIFY) {
 	/* Only verifying, this is far enough */
 	return DB_SUCCEED;
@@ -403,33 +455,27 @@ gethostbyaddr_lookup(rp, addr, fl, rt_info, error_p)
 
     if (rp->flags & GETHOST_CHECK_LOCAL) {
 	if (islocalhost(rt_info->next_host)) {
-	    struct addr *new;
+	    struct addr *new_parent = alloc_addr();
 
-	    DEBUG2(DBG_DRIVER_LO, "router %s: '%s' is a local host, will force reparse.\n", rp->name, rt_info->next_host);
 	    /*
-	     * It seems like a good idea to record the fact that the old
-	     * address is the parent of the new address (mostly so that the
-	     * parent address will appear in the logs).  The usual place where
-	     * that happens is in a director (not a router), and the usual way
-	     * of doing that is to allocate a new addr structure, fill the
-	     * newly allocated struct with the new info, and adjust pointers to
-	     * link the new struct into an appropriate list and leave the old
-	     * struct in limbo.  But we don't have access to the linked list
-	     * pointers, so we cheat, taking advantage of the way we know we
-	     * will be called from rtd_standard, which does have access to the
-	     * lists.  What we do is allocate a new addr structure, copy the
-	     * old information to the newly allocated structure where it is not
-	     * on any of the linked lists, and modify the current addr
-	     * structure to contain the new information.  After that, the old
-	     * information is in limbo and the new information is linked into a
-	     * list, which is what we want.
+	     * It is necessary to record the fact that the old address is the
+	     * parent of the new address (mostly so that the rewrite driver can
+	     * be used to forward to an internal mail server without triggering
+	     * a remote relay violation, but also so that the parent address
+	     * will appear in the logs).
 	     */
-	    new = alloc_addr();
-	    /* new = addr; */
-	    (void) memcpy((char *) new, (char *) addr, sizeof(*addr));
-	    addr->parent = new;
-	    rt_info->matchlen = strlen(addr->target);
-	    rt_info->next_host = NULL; /* force reparse of remainder */
+	    /* new_parent = addr; */
+	    (void) memcpy((char *) new_parent, (char *) addr, sizeof(*addr));
+	    addr->parent = new_parent;
+
+	    /*
+	     * the following is necessary to tell higher level software to
+	     * re-parse the address.
+	     */
+	    rt_info->next_host = NULL;
+
+	    DEBUG2(DBG_DRIVER_LO, "router %s: '%s' is a local host, will force reparse.\n",
+		   rp->name, rt_info->next_host);
 	}
     }
 
