@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:pathalias.c,v 1.14 2001/02/05 23:27:11 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:pathalias.c,v 1.26 2003/04/02 22:18:32 woods Exp"
  */
 
 /*
@@ -18,9 +18,11 @@
  * Specifications for the pathalias routing driver:
  *
  *	associated transports:
+ *
  *	    No specific transport is set.
  *
  *	private attribute data:
+ *
  *	    file (string):  the name of the file which contains the
  *		host to route association database.  In the case of
  *		the dbm protocol this will actually be the basename
@@ -31,8 +33,10 @@
  *		the database.  Can be one of:
  *
  *		lsearch - performs a linear search.
+ *
  *		bsearch - performs a straightforward binary search
  *		    on a sorted file (default).
+ *
  *		dbm - uses the Berkeley DBM routines.  If NDBM is
  *		    available, it is used instead.  If only the
  *		    older routines are available, exactly one DBM
@@ -44,18 +48,37 @@
  *		before the database is searched.  A target containing
  *		only the domain (e.g., .uucp) will remain .uucp.
  *
+ *	    required (list of domains):  specifies a list of domains which
+ *		targets are required to be within for a match to be successfull
+ *
+ *	    retries, interval (numbers):  Specify the number of attempts a
+ *		failed attempt to open the database will be retried, and the
+ *		interval in seconds between each retry.
+ *
  *	private attribute flags:
+ *
  *	    reopen:  if set, then reopen the database for each call
  *		to the routing driver, and close before each return.
  *		This is necessary for systems that would not otherwise
  *		have a sufficient number of available file descriptors
  *		for all of their routing and directing needs.
- *	    optional:  if set, then if the open fails, assume an empty
- *		paths file.
- *	    tryagain:  if set, then if the open fails, try again on a
- *		later spool directory queue run.
+ *
+ *	    optional:  If set, then if the open fails, assume an empty
+ *		paths file.  If not set, and if 'tryagain' is also not set,
+ *		then freeze the message in the error queue and tag it with a
+ *		configuration error.
+ *
+ *	    tryagain:  if set, then if the open fails, try again on a later
+ *		spool directory queue run.  If not set, and if 'optional' is
+ *		also not set, then freeze the message in the error queue and
+ *		tag it with a configuration error.
+ *
+ *	    Note: the optional and tryagain flags are exclusive (since they
+ *	    affect the behaviour under the same condition) and cannot be used
+ *	    together.
  *
  *	algorithm:
+ *
  *	    For pathalias routing, given a target use the following
  *	    strategy:
  *
@@ -81,49 +104,83 @@
  *		   Also remember that the target string passed must
  *		   be left intact for higher-level software.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "pathalias.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static int pathalias_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
+static void set_parent __P((struct router *, struct addr *, struct rt_info *));
 static void close_if_reopen __P((struct router *));
 static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
 static char *strip_costs __P((char *));
-static struct error *bad_entry __P((struct router *, char *));
+static struct error *bad_entry __P((struct router *, char *, char *));
 static struct error *open_failed __P((struct router *, char *, char *));
 static struct error *lookup_error __P((struct router *, char *, char *));
 
 static struct attr_table pathalias_attributes[] = {
-    { "file", t_string, NULL, NULL, OFFSET(pathalias_private, file) },
-    { "proto", t_string, NULL, NULL, OFFSET(pathalias_private, proto) },
-    { "domain", t_string, NULL, NULL, OFFSET(pathalias_private, domain) },
-    { "required", t_string, NULL, NULL,
+    { "file", t_string, 0, NULL, NULL, OFFSET(pathalias_private, file) },
+    { "proto", t_string, 0, NULL, NULL, OFFSET(pathalias_private, proto) },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(pathalias_private, domain) },
+    { "required", t_string, 0, NULL, NULL,
 	  OFFSET(pathalias_private, required) },
-    { "retries", t_int, NULL, NULL, OFFSET(pathalias_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(pathalias_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, PA_REOPEN },
-    { "tryagain", t_boolean, NULL, NULL, PA_TRYAGAIN },
-    { "optional", t_boolean, NULL, NULL, PA_OPTIONAL },
+    { "retries", t_int, 0, NULL, NULL, OFFSET(pathalias_private, retries) },
+    { "interval", t_int, 0, NULL, NULL, OFFSET(pathalias_private, interval) },
+    { "reopen", t_boolean, 0, NULL, NULL, PA_REOPEN },
+    { "tryagain", t_boolean, 0, NULL, NULL, PA_TRYAGAIN },
+    { "optional", t_boolean, 0, NULL, NULL, PA_OPTIONAL },
 };
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+/* point to end of attribute table */
 static struct attr_table *end_pathalias_attributes =
     ENDTABLE(pathalias_attributes);
 
@@ -145,6 +202,8 @@ rtd_pathalias(rp, in, out, defer, fail)
 {
     rtd_standard(rp, in, out, defer, fail, pathalias_lookup);
     close_if_reopen(rp);
+
+    return;
 }
 
 /*
@@ -166,30 +225,32 @@ rtb_pathalias(rp, attrs)
 #else	/* not HAVE_RENAME */
 	1,				/* retries */
 #endif	/* not HAVE_RENAME */
-	2,				/* interval */
+	10,				/* interval */
 	NULL,				/* database -- for internal use */
+	NULL,				/* error_text -- for internal use */
     };
     struct pathalias_private *priv;	/* new pathalias_private structure */
 
     /* copy the template private data */
-    priv = (struct pathalias_private *)xmalloc(sizeof(*priv));
-    (void) memcpy((char *)priv, (char *)&pathalias_template, sizeof(*priv));
+    priv = (struct pathalias_private *) xmalloc(sizeof(*priv));
+    (void) memcpy((char *) priv, (char *) &pathalias_template, sizeof(*priv));
+    rp->private = (char *) priv;
 
-    rp->private = (char *)priv;
     /* fill in the attributes of the private data */
-    error = fill_attributes((char *)priv,
-			    attrs,
-			    &rp->flags,
-			    pathalias_attributes,
-			    end_pathalias_attributes);
-
-    if (error) {
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+    if ((error = fill_attributes((char *) priv,
+				 attrs,
+				 &rp->flags,
+				 pathalias_attributes,
+				 end_pathalias_attributes))) {
 	return error;
-    } else {
-	return NULL;
     }
-}
+    if ((rp->flags & PA_TRYAGAIN) && (rp->flags & PA_OPTIONAL)) {
+	return xprintf("%s: `optional' and `tryagain' attributes are incompatible", rp->name);
+    }
 
+    return NULL;
+}
 
 /*
  * rtp_pathalias - dump the configuration attributes
@@ -199,16 +260,16 @@ rtp_pathalias(f, rp)
      FILE * f;
      struct router *rp;
 {
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     (void) dump_standard_config(f,
 				rp->private,
 				rp->name,
 				rp->flags,
 				pathalias_attributes,
 				end_pathalias_attributes);
+
+    return;
 }
-
-
-
 
 /*
  * pathalias_lookup - lookup a host in a pathalias database
@@ -238,34 +299,39 @@ static int
 pathalias_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
-    int success;
+    int result;
     char *p;
     char *raw_path;
     char *match;
+    struct pathalias_private *priv;
 
-    success = find_domain(rp, addr, &raw_path, &match, error_p);
+    priv = (struct pathalias_private *) rp->private;
 
-    switch (success) {
+    result = find_domain(rp, addr, &raw_path, &match, error_p);
+
+    switch (result) {
     case FILE_NOMATCH:
-	if ((rp->flags & PA_OPTIONAL) == 0) {
-	    struct pathalias_private *priv;
-
-	    priv = (struct pathalias_private *)rp->private;
+	if (rp->flags & PA_TRYAGAIN) {
+	    return FILE_AGAIN;		/* translate to deferal */
+	} else if (!(rp->flags & PA_OPTIONAL)) {
+	    /* This isn't really necessary, but gives a more standard message */
 	    *error_p = open_failed(rp, priv->file, "Database not found");
-	    success = FILE_FAIL;
+
+	    return  FILE_FAIL;		/* translate to an unrecov. error */
 	}
-	/* FALLTHRU */
+	/* else the optional bit _is_ set.... */
+	return FILE_NOMATCH;		/* leave as-is */
 
     case FILE_FAIL:
     case FILE_AGAIN:
     case DB_NOMATCH:
     case DB_FAIL:
     case DB_AGAIN:
-	return success;
+	return result;
     }
 
     /* common cases for rt_info values */
@@ -275,15 +341,19 @@ pathalias_lookup(rp, addr, fl, rt_info, error_p)
 
     /* first form, local host */
     if (EQ(raw_path, "%s")) {
+	set_parent(rp, addr, rt_info);
+
 	return DB_SUCCEED;
     }
 
-    p = index(raw_path, '!');
+    p = strchr(raw_path, '!');
     if (p) {
 	*p++ = '\0';
 	rt_info->next_host = raw_path;
 	if (EQ(p, "%s")) {
 	    /* second form, host!%s, one hop to target */
+	    set_parent(rp, addr, rt_info);
+
 	    return DB_SUCCEED;
 	} else {
 	    /* strip off trailing !%s */
@@ -293,6 +363,8 @@ pathalias_lookup(rp, addr, fl, rt_info, error_p)
 		/* third form, hop then route to target */
 		*q = '\0';
 		rt_info->route = p;
+		set_parent(rp, addr, rt_info);
+
 		return DB_SUCCEED;
 	    }
 	}
@@ -300,10 +372,42 @@ pathalias_lookup(rp, addr, fl, rt_info, error_p)
     if (p) {
 	*p = '!';
     }
-    *error_p = bad_entry(rp, raw_path);
+    *error_p = bad_entry(rp, priv->file, raw_path);
+
     return DB_AGAIN;
 }
 
+/*ARGSUSED*/
+static void
+set_parent(rp, routed, rt_info)
+    struct router *rp;				/* UNUSED */
+    struct addr *routed;
+    struct rt_info *rt_info;
+{
+     struct addr *new_parent = alloc_addr();
+
+    /*
+     * It is necessary to record the fact that the old address is the parent of
+     * the new address (mostly so this driver can be used to forward to an
+     * internal mail server without triggering a remote relay violation, but
+     * also so that the parent address will appear in the logs).
+     */
+     /* new_parent = addr; */
+     (void) memcpy((char *) new_parent, (char *) routed, sizeof(*routed));
+     routed->parent = new_parent;
+
+    DEBUG6(DBG_DRIVER_MID, "pathalias_lookup: routing %s --> %s%s%s%s%s\n",
+	   routed->parent->in_addr,
+	   rt_info->next_host ? rt_info->next_host : "",
+	   rt_info->next_host ? "!" : "",
+	   rt_info->route ? rt_info->route : "",
+	   rt_info->route ? "!" : "",
+	   routed->remainder);
+
+    return;
+}
+
+
 /*
  * close_if_reopen - close the database if the reopen flag is set.
  */
@@ -358,8 +462,8 @@ strip_costs(raw_path)
 {
     register char *cp, *cptab;
 
-    cp = index(raw_path, ' ');
-    if ((cptab = index(raw_path, '\t'))) {
+    cp = strchr(raw_path, ' ');
+    if ((cptab = strchr(raw_path, '\t'))) {
 	if (cp == NULL || cptab < cp) {
 	    cp = cptab;
 	}
@@ -395,8 +499,8 @@ find_domain(rp, addr, raw_path, match, error_p)
     struct pathalias_private *priv;	/* private data */
     char *savedomain = NULL;		/* saved position of removed domain */
     char *target = addr->target;	/* target being searched for */
-    unsigned int len = strlen(target);	/* total length of target */
-    int success;			/* return from function calls */
+    size_t len = strlen(target);	/* total length of target */
+    int result;			/* return from function calls */
     char *error_text = NULL;		/* error messages from subroutines */
 
     priv = (struct pathalias_private *)rp->private;
@@ -420,7 +524,7 @@ find_domain(rp, addr, raw_path, match, error_p)
 
 	if (fstr && *fstr) {
 	    tdbname = COPY_STRING(fstr);
-	    success = open_database(tdbname, priv->proto, priv->retries,
+	    result = open_database(tdbname, priv->proto, priv->retries,
 				    priv->interval, (struct stat *) NULL,
 				    &priv->database, &error_text);
 	    /* XXX could check if expand_string() did anything since we may not
@@ -434,22 +538,22 @@ find_domain(rp, addr, raw_path, match, error_p)
 	    DEBUG1(DBG_DRIVER_LO,
 		   "pathalias: find_domain: error expanding dbname %s.",
 		   priv->file);
-	    success = FILE_FAIL;
+	    result = FILE_FAIL;
 	    error_text = "Database file name not valid";
 	}
 #else
-	success = open_database(priv->file, priv->proto, priv->retries,
+	result = open_database(priv->file, priv->proto, priv->retries,
 				priv->interval, (struct stat *) NULL,
 				&priv->database, &error_text);
 #endif
-	if (success != FILE_SUCCEED) {
+	if (result != FILE_SUCCEED) {
 	    if (priv->error_text) {
 		xfree(priv->error_text);
 	    }
 	    priv->error_text = COPY_STRING(error_text);
 	    *error_p = open_failed(rp, priv->file, error_text);
 
-	    return success;
+	    return result;
 	}
     }
 
@@ -470,40 +574,44 @@ find_domain(rp, addr, raw_path, match, error_p)
     /*
      * lookup the target as is
      */
-    success = lookup_database(priv->database, target, raw_path, &error_text);
-    if (success != DB_NOMATCH) {
+    result = lookup_database(priv->database, target, raw_path, &error_text);
+    if (result != DB_NOMATCH) {
 	if (savedomain) {
 	    *savedomain = '.';		/* restore the target */
 	}
 	*match = target;		/* return the match and the path */
-	if (success == DB_SUCCEED) {
-	    while (isspace(**raw_path)) (*raw_path)++;
+	if (result == DB_SUCCEED) {
+	    while (isspace((int) **raw_path)) {
+		(*raw_path)++;
+	    }
 	    (void) strip_costs(*raw_path); /* strip the associated costs */
 	} else {
 	    *error_p = lookup_error(rp, target, error_text);
 	}
 
-	return success;
+	return result;
     }
     if (target[0] == '.') {
 	/*
 	 * if it starts with a `.', look it up without the dot
 	 */
-	success = lookup_database(priv->database, target + 1,
+	result = lookup_database(priv->database, target + 1,
 				  raw_path, &error_text);
-	if (success != DB_NOMATCH) {
+	if (result != DB_NOMATCH) {
 	    if (savedomain) {
 		*savedomain = '.';
 	    }
 	    *match = target + 1;
-	    if (success == DB_SUCCEED) {
-		while (isspace(**raw_path)) (*raw_path)++;
+	    if (result == DB_SUCCEED) {
+		while (isspace((int) **raw_path)) {
+		    (*raw_path)++;
+		}
 		(void) strip_costs(*raw_path); /* strip the associated costs */
 	    } else {
 		*error_p = lookup_error(rp, target, error_text);
 	    }
 
-	    return success;
+	    return result;
 	}
     } else {
 	/*
@@ -514,9 +622,9 @@ find_domain(rp, addr, raw_path, match, error_p)
 	char *p = xmalloc(len + 2);
 
 	(void) sprintf(p, ".%s", target);
-	success = lookup_database(priv->database, p, raw_path, &error_text);
+	result = lookup_database(priv->database, p, raw_path, &error_text);
 	xfree(p);
-	if (success != DB_NOMATCH) {
+	if (result != DB_NOMATCH) {
 	    static char *new_raw_path = NULL;
 	    int path_len;
 
@@ -524,8 +632,10 @@ find_domain(rp, addr, raw_path, match, error_p)
 		*savedomain = '.';
 	    }
 	    *match = target;
-	    if (success == DB_SUCCEED) {
-		while (isspace(**raw_path)) (*raw_path)++;
+	    if (result == DB_SUCCEED) {
+		while (isspace((int) **raw_path)) {
+		    (*raw_path)++;
+		}
 		(void) strip_costs(*raw_path); /* strip the associated costs */
 		path_len = strlen(*raw_path);
 		if (path_len > 3 && EQ(*raw_path + path_len - 3, "!%s")) {
@@ -534,14 +644,14 @@ find_domain(rp, addr, raw_path, match, error_p)
 		    }
 		    new_raw_path = xmalloc(strlen(target) + path_len + 2);
 		    (*raw_path)[path_len - 2] = '\0';
-		    (void)sprintf(new_raw_path, "%s%s!%%s", *raw_path, target);
+		    (void) sprintf(new_raw_path, "%s%s!%%s", *raw_path, target);
 		    *raw_path = new_raw_path;
 		}
 	    } else {
 		*error_p = lookup_error(rp, target, error_text);
 	    }
 
-	    return success;
+	    return result;
 	}
     }
 
@@ -556,18 +666,20 @@ find_domain(rp, addr, raw_path, match, error_p)
 	}
 
 	/* advance to the next dot */
-	target = index(target, '.');
+	target = strchr(target, '.');
 	if (target) {
 	    /* if there is anything left, look it up */
-	    success = lookup_database(priv->database, target,
+	    result = lookup_database(priv->database, target,
 				      raw_path, &error_text );
-	    if (success != DB_NOMATCH) {
+	    if (result != DB_NOMATCH) {
 		if (savedomain) {
 		    *savedomain = '.';
 		}
 		*match = target;
-		if (success == DB_SUCCEED) {
-		    while (isspace(**raw_path)) (*raw_path)++;
+		if (result == DB_SUCCEED) {
+		    while (isspace((int) **raw_path)) {
+			(*raw_path)++;
+		    }
 		    (void) strip_costs(*raw_path); /* strip the costs */
 		    if (EQ(*raw_path, "%s")) {
 			/*
@@ -585,7 +697,7 @@ find_domain(rp, addr, raw_path, match, error_p)
 		    *error_p = lookup_error(rp, target, error_text);
 		}
 
-		return success;
+		return result;
 	    }
 	}
     }
@@ -604,8 +716,9 @@ find_domain(rp, addr, raw_path, match, error_p)
  */
 
 static struct error *
-bad_entry(rp, raw_path)
+bad_entry(rp, file, raw_path)
     struct router *rp;
+    char *file;
     char *raw_path;
 {
     char *error_text;
@@ -623,8 +736,8 @@ bad_entry(rp, raw_path)
      * RESOLUTION
      *      The postmaster should correct the pathalias database entry.
      */
-    error_text = xprintf("router %s: bad entry in pathalias database: %s",
-			 rp->name, raw_path);
+    error_text = xprintf("router %s: driver %s: bad entry in database %s: %s",
+			 rp->name, rp->driver, file, raw_path);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_CONFERR|ERR_124, error_text);
@@ -652,8 +765,8 @@ open_failed(rp, file, open_error)
      *      The postmaster should check the director entry against the
      *      database he wishes to use.
      */
-    error_text = xprintf("router %s: path database %s, open failed: %s",
-			 rp->name, file, open_error);
+    error_text = xprintf("router %s: driver %s: path database %s, open failed: %s",
+			 rp->name, rp->driver, file, open_error);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_124, error_text);
@@ -680,8 +793,8 @@ lookup_error(rp, target, lookup_error_text)
      * RESOLUTION
      *      Unspecified.
      */
-    error_text = xprintf("router %s: target %s, lookup failed: %s",
-			 rp->name, target, lookup_error_text);
+    error_text = xprintf("router %s: driver %s: target %s, lookup failed: %s",
+			 rp->name, rp->driver, target, lookup_error_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_159, error_text);
