@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:queryprog.c,v 1.10 2000/09/03 17:00:23 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:queryprog.c,v 1.18 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -40,29 +40,60 @@
  *	notes:
  *	    The match status values are cached.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../hash.h"
 #include "../child.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "queryprog.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static int queryprogram_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
@@ -72,12 +103,12 @@ static struct error *temp_command_error __P((struct router *));
 static struct error *output_parse_error __P((struct router *, char *));
 
 static struct attr_table queryprogram_attributes[] = {
-    { "cmd", t_string, NULL, NULL, OFFSET(queryprogram_private, cmd) },
-    { "domain", t_string, NULL, NULL, OFFSET(queryprogram_private, domain) },
-    { "required", t_string, NULL, NULL, OFFSET(queryprogram_private, required) },
-    { "hash_table_len", t_int, NULL, NULL, OFFSET(queryprogram_private, hash_table_len) },
-    { "read_path", t_boolean, NULL, NULL, QP_READ_PATH },
-    { "read_transport", t_boolean, NULL, NULL, QP_READ_TRANSPORT },
+    { "cmd", t_string, 0, NULL, NULL, OFFSET(queryprogram_private, cmd) },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(queryprogram_private, domain) },
+    { "required", t_string, 0, NULL, NULL, OFFSET(queryprogram_private, required) },
+    { "hash_table_len", t_int, 0, NULL, NULL, OFFSET(queryprogram_private, hash_table_len) },
+    { "read_path", t_boolean, 0, NULL, NULL, QP_READ_PATH },
+    { "read_transport", t_boolean, 0, NULL, NULL, QP_READ_TRANSPORT },
 };
 static struct attr_table *end_queryprogram_attributes =
     ENDTABLE(queryprogram_attributes);
@@ -177,7 +208,7 @@ static int
 queryprogram_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
@@ -256,7 +287,8 @@ queryprogram_lookup(rp, addr, fl, rt_info, error_p)
 
 	if (output && output[0] != '\0') {
 	    if (decode_query_output(rp, output, rt_info, &error) == FAIL) {
-		(void) close_child((FILE *)NULL, f, pid);
+		/* XXX should probably log the close_child status, but it's complex.... */
+		(void) close_child((FILE *) NULL, f, pid);
 		*error_p = output_parse_error(rp, error);
 		return DB_FAIL;
 	    }
@@ -264,9 +296,11 @@ queryprogram_lookup(rp, addr, fl, rt_info, error_p)
     }
 
     /* get the child's status */
-    status = close_child((FILE *)NULL, f, pid);
+    status = close_child((FILE *) NULL, f, pid);
 
-    return status==0? DB_SUCCEED: DB_NOMATCH;
+    /* XXX should probably log the close_child status, but it's complex.... */
+
+    return ((status == 0) ? DB_SUCCEED : DB_NOMATCH);
 }
 
 /*
@@ -290,9 +324,13 @@ decode_query_output(rp, output, rt_info, error)
 	char *route;
 
 	/* look for a path as first space-delimited field */
-	while (isspace(*p)) p++;
+	while (isspace((int) *p)) {
+	    p++;
+	}
 	path = p;
-	while (*p && !isspace(*p)) p++;
+	while (*p && !isspace((int) *p)) {
+	    p++;
+	}
 	if (*p) {
 	    *p++ = '\0';
 	}
@@ -317,9 +355,13 @@ decode_query_output(rp, output, rt_info, error)
 	char *transport;
 
 	/* look for a transport as the next field */
-	while (isspace(*p)) p++;
+	while (isspace((int) *p)) {
+	    p++;
+	}
 	transport = p;
-	while (*p && !isspace(*p)) p++;
+	while (*p && !isspace((int) *p)) {
+	    p++;
+	}
 	*p = '\0';
 	if (transport[0]) {
 	    struct transport *tp;
