@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:reroute.c,v 1.13 2001/02/05 23:27:11 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:reroute.c,v 1.22 2003/04/02 22:18:18 woods Exp"
  */
 
 /*
@@ -194,50 +194,81 @@
  *	    needs lots of time and determination.  Please restrain yourself,
  *	    for the sake of fast and flawless mail delivery.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "reroute.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static int reroute_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
 static void close_if_reopen __P((struct router *));
 static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
-static struct error *bad_entry __P((struct router *, char *));
+static struct error *bad_entry __P((struct router *, char *, char *));
 static struct error *open_failed __P((struct router *, char *, char *));
 static struct error *lookup_error __P((struct router *, char *, char *));
 
 static struct attr_table reroute_attributes[] = {
-    { "file", t_string, NULL, NULL, OFFSET(reroute_private, file) },
-    { "proto", t_string, NULL, NULL, OFFSET(reroute_private, proto) },
-    { "domain", t_string, NULL, NULL, OFFSET(reroute_private, domain) },
-    { "required", t_string, NULL, NULL,
+    { "file", t_string, 0, NULL, NULL, OFFSET(reroute_private, file) },
+    { "proto", t_string, 0, NULL, NULL, OFFSET(reroute_private, proto) },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(reroute_private, domain) },
+    { "required", t_string, 0, NULL, NULL,
 	  OFFSET(reroute_private, required) },
-    { "retries", t_int, NULL, NULL, OFFSET(reroute_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(reroute_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, RR_REOPEN },
-    { "tryagain", t_boolean, NULL, NULL, RR_TRYAGAIN },
-    { "optional", t_boolean, NULL, NULL, RR_OPTIONAL },
-    { "matchall", t_boolean, NULL, NULL, RR_MATCHALL },
-    { "matchlocal", t_boolean, NULL, NULL, RR_MATCHLOCAL },
-    { "matchdb", t_boolean, NULL, NULL, RR_MATCHDB },
-    { "bounceonly", t_boolean, NULL, NULL, RR_BOUNCEONLY },
+    { "retries", t_int, 0, NULL, NULL, OFFSET(reroute_private, retries) },
+    { "interval", t_int, 0, NULL, NULL, OFFSET(reroute_private, interval) },
+    { "reopen", t_boolean, 0, NULL, NULL, RR_REOPEN },
+    { "tryagain", t_boolean, 0, NULL, NULL, RR_TRYAGAIN },
+    { "optional", t_boolean, 0, NULL, NULL, RR_OPTIONAL },
+    { "matchall", t_boolean, 0, NULL, NULL, RR_MATCHALL },
+    { "matchlocal", t_boolean, 0, NULL, NULL, RR_MATCHLOCAL },
+    { "matchdb", t_boolean, 0, NULL, NULL, RR_MATCHDB },
+    { "bounceonly", t_boolean, 0, NULL, NULL, RR_BOUNCEONLY },
 };
 static struct attr_table *end_reroute_attributes =
     ENDTABLE(reroute_attributes);
@@ -283,6 +314,7 @@ rtb_reroute(rp, attrs)
 #endif	/* not HAVE_RENAME */
 	2,				/* interval */
 	NULL,				/* database -- for internal use */
+	NULL,				/* rror_text -- for internal use */
     };
     struct reroute_private *priv;	/* new reroute_private structure */
 
@@ -353,26 +385,32 @@ static int
 reroute_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
     int success;
     char *new_path;
     char *flag;
+    struct reroute_private *priv;
+
+    priv = (struct reroute_private *)rp->private;
 
     success = find_domain(rp, addr, &new_path, &flag, error_p);
 
     switch (success) {
     case FILE_NOMATCH:
-	if ((rp->flags & RR_OPTIONAL) == 0) {
-	    struct reroute_private *priv;
-
-	    priv = (struct reroute_private *)rp->private;
+	if (rp->flags & RR_TRYAGAIN) {
+	    return FILE_AGAIN;		/* translate to deferal */
+	} else if (!(rp->flags & RR_OPTIONAL)) {
+	    /* This isn't really necessary, but gives a more standard message */
 	    *error_p = open_failed(rp, priv->file, "Database not found");
-	    success = FILE_FAIL;
+
+	    return  FILE_FAIL;		/* translate to an unrecov. error */
 	}
-	/* FALLTHRU */
+
+	/* else the optional bit _is_ set.... */
+	return FILE_NOMATCH;		/* leave as-is */
 
     case FILE_FAIL:
     case FILE_AGAIN:
@@ -384,16 +422,45 @@ reroute_lookup(rp, addr, fl, rt_info, error_p)
 
     /* check the reroute flag */
     if (flag[0] == '+') {
-	DEBUG1(DBG_DRIVER_MID, "reroute_lookup: rerouted: address=%s\n",
-	       new_path);
+	struct addr *new = alloc_addr();
+	/*
+	 * It is necessary to record the fact that the old address is the parent of
+	 * the new address (mostly so that the rewrite driver can be used to
+	 * forward to an internal mail server without triggering a remote relay
+	 * violation, but also so that the parent address will appear in the logs).
+	 *
+	 */
+	(void) memcpy((char *) new, (char *) addr, sizeof(*addr));
+	addr->parent = new;
+	/*
+	 * all the following is necessary to tell higher level software to
+	 * re-parse the address passed back in addr->remainder.
+	 *
+	 * The usual place where this kind of thing happens is in a director
+	 * (not a router).  Conveniently we have already effectivley saved a
+	 * copy of the original structure in the newly allocated parent
+	 * structure above.  All we have to do now is to modify the current
+	 * addr structure to contain the new input address and to tell the
+	 * routing engine that we've matched the whole length of the target
+	 * address (which will cause the ADDR_FINISHED and ADDR_FULLMATCH flags
+	 * to be set in route_driver_finish() thus claiming this address for
+	 * ourselves).
+	 */
+	addr->in_addr = COPY_STRING(new_path);	/* XXX rewrite did this too */
+	addr->remainder = new_path;
 	rt_info->next_host = NULL;
 	rt_info->route = NULL;
+
 	rt_info->matchlen = strlen(addr->target);
-	addr->remainder = new_path;
+
+	DEBUG3(DBG_DRIVER_MID, "router %s: reroute_lookup forcing reparse: rerouted %s -> %s\n",
+	       rp->name, addr->parent->in_addr, addr->in_addr);
+
 	return DB_SUCCEED;	/* reparse */
     }
 
-    *error_p = bad_entry(rp, flag);
+    *error_p = bad_entry(rp, priv->file, flag);
+
     return DB_AGAIN;
 }
 
@@ -744,7 +811,7 @@ find_domain(rp, addr, new_path, flag, error_p)
 	    }
 
 	    /* advance to the next dot */
-	    target = index(target, '.');
+	    target = strchr(target, '.');
 	    if (target) {
 		/* if there is anything left, look it up */
 		success = lookup_database(priv->database, target,
@@ -785,8 +852,9 @@ find_domain(rp, addr, new_path, flag, error_p)
  */
 
 static struct error *
-bad_entry(rp, raw_path)
+bad_entry(rp, file, raw_path)
     struct router *rp;
+    char *file;
     char *raw_path;
 {
     char *error_text;
@@ -805,8 +873,8 @@ bad_entry(rp, raw_path)
      * RESOLUTION
      *      The postmaster should correct the reroute database entry.
      */
-    error_text = xprintf("router %s: bad entry in reroute database: %s",
-			 rp->name, raw_path);
+    error_text = xprintf("router %s: driver %s: bad entry in database %s: %s",
+			 rp->name, rp->driver, file, raw_path);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_CONFERR|ERR_181, error_text);
@@ -834,8 +902,8 @@ open_failed(rp, file, open_error)
      *      The postmaster should check the director entry against the
      *      database he wishes to use.
      */
-    error_text = xprintf("router %s: reroute database %s, open failed: %s",
-			 rp->name, file, open_error);
+    error_text = xprintf("router %s: driver %s: database %s, open failed: %s",
+			 rp->name, rp->driver, file, open_error);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_182, error_text);
@@ -862,8 +930,8 @@ lookup_error(rp, target, lookup_error_text)
      * RESOLUTION
      *      Unspecified.
      */
-    error_text = xprintf("router %s: target %s, lookup failed: %s",
-			 rp->name, target, lookup_error_text);
+    error_text = xprintf("router %s: driver %s: target %s, lookup failed: %s",
+			 rp->name, rp->driver, target, lookup_error_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_183, error_text);
