@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:rewrite.c,v 1.18 2001/02/05 23:27:12 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:rewrite.c,v 1.30 2003/04/02 22:18:11 woods Exp"
  */
 /*
  * rewrite.c
@@ -32,17 +32,67 @@
  * Specifications for the rewrite driver:
  *
  *	associated transports:
+ *
  *	    No specific transport is set.
  *
  *	private attribute data:
- *	    file, proto, domain, required, retries, interval:
- *		same as for pathalias driver.
+ *
+ *	    file (string):  the name of the file which contains the
+ *		host to route association database.  In the case of
+ *		the dbm protocol this will actually be the basename
+ *		for the two associated DBM files, ending in .pag and
+ *		.dir.
+ *
+ *	    proto (name):  specifies the protocol used in accessing
+ *		the database.  Can be one of:
+ *
+ *		lsearch - performs a linear search.
+ *
+ *		bsearch - performs a straightforward binary search
+ *		    on a sorted file (default).
+ *		dbm - uses the Berkeley DBM routines.  If NDBM is
+ *		    available, it is used instead.  If only the
+ *		    older routines are available, exactly one DBM
+ *		    database can be opened by smail.
+ *
+ *	    domain (list of domains):  specifies the default domain for hosts
+ *		in the database.  Targets ending in this domain will have the
+ *		domain stripped (including a preceding dot) before the database
+ *		is searched.  A target containing only the domain (e.g., .uucp)
+ *		will remain .uucp.
+ *
+ *	    required (list of domains):  specifies a list of domains which
+ *		targets are required to be within for a match to be successfull
+ *
+ *	    retries, interval (numbers):  Specify the number of attempts a
+ *		failed attempt to open the database will be retried, and the
+ *		interval in seconds between each retry.
+ *
  *
  *	private attribute flags:
- *	    reopen, tryagain, optional, cacheopen:
- *		same as for pathalias driver.
+ *
+ *	    reopen:  if set, then reopen the database for each call
+ *		to the routing driver, and close before each return.
+ *		This is necessary for systems that would not otherwise
+ *		have a sufficient number of available file descriptors
+ *		for all of their routing and directing needs.
+ *
+ *	    optional:  If set, then if the open fails, assume an empty
+ *		paths file.  If not set, and if 'tryagain' is also not set,
+ *		then freeze the message in the error queue and tag it with a
+ *		configuration error.
+ *
+ *	    tryagain:  if set, then if the open fails, try again on a later
+ *		spool directory queue run.  If not set, and if 'optional' is
+ *		also not set, then freeze the message in the error queue and
+ *		tag it with a configuration error.
+ *
+ *	    Note: the optional and tryagain flags are exclusive (since they
+ *	    affect the behaviour under the same condition) and cannot be used
+ *	    together.
  *
  *	algorithm:
+ *
  *	    Given a target (host or domain name), look it up in the
  *	    database, using the same search strategy as the pathalias
  *	    driver.  The best match wins, as with pathalias.
@@ -68,8 +118,10 @@
  *		name beginning with a `.'.
  *
  *	    flag: One of the following:
+ *
  *		`-'	Rewriting should not be done.  The router then
  *			behaves as if the target had not been matched.
+ *
  *		`+'	Rewriting should be done, using the specified
  *			format if the remainder was a simple mailbox
  *			(user name), and leaving the remainder unchanged
@@ -81,68 +133,126 @@
  *		remainder and $host refers to the target.
  *
  *	    examples:
+ *
  *		.special.com	+	special-$user
- *			This rewrites "username@subdomain.special.com"
- *			and "username@special.com" to
- *			"special-username".  For this to be useful,
- *			some other part of the smail configuration
- *			should recognise "special-username" as a
- *			local alias.  Also, don't forget to install a
- *			local alias for "special-postmaster"!  This
- *			also rewrites "user%elsewhere@special.com" to
- *			"user%elsewhere", which will later be treated
- *			like "user@elsewhere".
+ *
+ *			This rewrites "username@subdomain.special.com" and
+ *			"username@special.com" to "special-username".  For this
+ *			to be useful, some other part of the smail
+ *			configuration should recognise "special-username" as a
+ *			local alias.  Also, don't forget to install a local
+ *			alias for "special-postmaster"!  This also rewrites
+ *			"user%elsewhere@special.com" to "user%elsewhere", which
+ *			will later be treated like "user@elsewhere".
+ *
  *		special.com	-
+ *
  *			This prevents rewriting of "anything@special.com",
- *			overriding the effect of the above example.  When
- *			both this and the above example are used together,
+ *			overriding the effect of the above example.  When both
+ *			this and the above example are used together,
  *			"username@special.com" will not be rewritten, but
  *			"username@subdomain.special.com" will be rewritten.
+ *
  *		.foo.org	+	$user-$host
- *			This rewrites "username@foo.org" to
- *			"username-foo.org", and rewrites
- *			"username@subdomain.foo.org" to
+ *
+ *			This rewrites "username@foo.org" to "username-foo.org",
+ *			and rewrites "username@subdomain.foo.org" to
  *			"username-subdomain.foo.org".
+ *
  *		frobozz.com	+	${lookup:user:lsearch{frobozz-aliases}{$value}{postmaster}}
- *			This rewrites "username@frobozz.com" by
- *			searching for "username" in the file
- *			"frobozz-aliases" (in the smail lib
- *			directory).  If the search fails, it rewrites
- *			to "postmaster". Note that the format of the
- *			file searched by the ${lookup...} expansion is a
- *			lot more restricted than the format of a normal
- *			alias file, and that each input must map to
- *			exactly one output in such a file.
+ *
+ *			This rewrites "username@frobozz.com" by searching for
+ *			"username" in the file "frobozz-aliases" (in the smail
+ *			lib directory).  If the search fails, it rewrites to
+ *			"postmaster".
+ *
+ *			Note that the format of the file searched by the
+ *			${lookup...} expansion is a lot more restricted than
+ *			the format of a normal alias file, and that each input
+ *			must map to exactly one target address value in such a
+ *			file.
+ *
+ *		frobozz.com	+	${lookup:user:lsearch{frobozz-aliases}{$value}{"$user@$host"}}
+ *
+ *			As for the previous case, except that if the search
+ *			fails, it rewrites to "username@frobozz.com" (note the
+ *			quotes), and this will have the effect of bouncing the
+ *			message with an error message that says (in a bounce):
+ *
+ *			"username@frobozz.com" ... not deliverable: no such user or mailbox
+ *
+ *			or in SMTP:
+ *
+ *			550-5.1.1 reject sending to address '<username@frobozz.com>'.
+ *			550-5.1.1 The address <username@frobozz.com> was not accepted.
+ *			550-5.1.1 Reason given was: (ERR_100) no such user or mailbox.
+ *			550 5.1.1 Permanent failure logged.
+ *
  *		frobozz.com	+	${lookup:user:lsearch{frobozz-aliases}{$value}{$input_addr}}
- *			As for the previous case, except that if
- *			the file lookup fails it leaves the address
- *			unchanged.  The unchanged address will be passed
- *			on to subsequent routers, which might be able to
- *			route it or might fail it as un unknown host.
- *			XXX: There should probably be a way to fine-tune
- *			the error handling.
+ *
+ *			As for the previous case, except that if the file
+ *			lookup fails it leaves the address unchanged.  The
+ *			unchanged address will be passed on to subsequent
+ *			routers, which might be able to route it or might fail
+ *			it as an unknown host.
+ *
+ *			Note that quoting 
+ *
+ *		XXX: There should probably be a way to fine-tune
+ *		the error handling.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "rewrite.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /*
  * XXX: these error codes should go in error.h, or should be shared with
@@ -158,23 +268,28 @@
 static int rewrite_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
 static void close_if_reopen __P((struct router *));
 static int find_domain __P((struct router *, struct addr *, char **, char **, struct error **));
-static struct error *bad_entry __P((struct router *, char *));
+static struct error *bad_entry __P((struct router *, char *, char *));
 static struct error *open_failed __P((struct router *, char *, char *));
 static struct error *lookup_error __P((struct router *, char *, char *));
 static struct error *expand_error __P((struct router *, char *, char *));
 
 static struct attr_table rewrite_attributes[] = {
     /* note that many of these are shared with pathalias */
-    { "file", t_string, NULL, NULL, OFFSET(rewrite_private, file) },
-    { "proto", t_string, NULL, NULL, OFFSET(rewrite_private, proto) },
-    { "domain", t_string, NULL, NULL, OFFSET(rewrite_private, domain) },
-    { "required", t_string, NULL, NULL, OFFSET(rewrite_private, required) },
-    { "retries", t_int, NULL, NULL, OFFSET(rewrite_private, retries) },
-    { "interval", t_int, NULL, NULL, OFFSET(rewrite_private, interval) },
-    { "reopen", t_boolean, NULL, NULL, RW_REOPEN },
-    { "tryagain", t_boolean, NULL, NULL, RW_TRYAGAIN },
-    { "optional", t_boolean, NULL, NULL, RW_OPTIONAL },
+    { "file", t_string, 0, NULL, NULL, OFFSET(rewrite_private, file) },
+    { "proto", t_string, 0, NULL, NULL, OFFSET(rewrite_private, proto) },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(rewrite_private, domain) },
+    { "required", t_string, 0, NULL, NULL, OFFSET(rewrite_private, required) },
+    { "retries", t_int, 0, NULL, NULL, OFFSET(rewrite_private, retries) },
+    { "interval", t_int, 0, NULL, NULL, OFFSET(rewrite_private, interval) },
+    { "reopen", t_boolean, 0, NULL, NULL, RW_REOPEN },
+    { "tryagain", t_boolean, 0, NULL, NULL, RW_TRYAGAIN },
+    { "optional", t_boolean, 0, NULL, NULL, RW_OPTIONAL },
+#ifdef FIXME
+    { NULL, 0, NULL, NULL, NULL }
+#endif
 };
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+/* point to end of attribute table */
 static struct attr_table *end_rewrite_attributes =
     ENDTABLE(rewrite_attributes);
 
@@ -225,8 +340,9 @@ rtb_rewrite(rp, attrs)
 #else	/* not HAVE_RENAME */
 	1,				/* retries */
 #endif	/* not HAVE_RENAME */
-	2,				/* interval */
+	10,				/* interval */
 	NULL,				/* database -- for internal use */
+	NULL,				/* error_text -- for internal use */
     };
     struct rewrite_private *priv;	/* new rewrite_private structure */
 
@@ -236,6 +352,7 @@ rtb_rewrite(rp, attrs)
     rp->private = (char *) priv;
 
     /* fill in the attributes of the private data */
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     if ((error = fill_attributes((char *) priv,
 				 attrs,
 				 &rp->flags,
@@ -243,6 +360,9 @@ rtb_rewrite(rp, attrs)
 				 end_rewrite_attributes))) {
 	return error;
     }
+    if ((rp->flags & RW_TRYAGAIN) && (rp->flags & RW_OPTIONAL)) {
+	return xprintf("%s: `optional' and `tryagain' attributes are incompatible", rp->name);
+    }
 
     return NULL;
 }
@@ -255,6 +375,7 @@ rtp_rewrite(f, rp)
      FILE * f;
      struct router *rp;
 {
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     (void) dump_standard_config(f,
 				rp->private,
 				rp->name,
@@ -262,8 +383,6 @@ rtp_rewrite(f, rp)
 				rewrite_attributes,
 				end_rewrite_attributes);
 }
-
-
 
 /*
  * rewrite_lookup - lookup a host in a rewrite database
@@ -297,7 +416,7 @@ static int
 rewrite_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
@@ -311,20 +430,26 @@ rewrite_lookup(rp, addr, fl, rt_info, error_p)
     char *new_address;	/* rewritten address */
     char *new_target = NULL; /* target of rewritten address */
     char *new_remainder = NULL; /* remainder of rewritten address */
+    struct rewrite_private *priv;
+
+    priv = (struct rewrite_private *) rp->private;
 
     /* is the target in the database? */
     result = find_domain(rp, addr, &raw_data, &match, error_p);
 
     switch (result) {
-    case FILE_NOMATCH:
-	if ((rp->flags & RW_OPTIONAL) == 0) {
-	    struct rewrite_private *priv;
-
-	    priv = (struct rewrite_private *)rp->private;
+    case FILE_NOMATCH:			/* the file did not exist */
+	if (rp->flags & RW_TRYAGAIN) {
+	    return FILE_AGAIN;		/* translate to deferal */
+	} else if (!(rp->flags & RW_OPTIONAL)) {
+	    /* This isn't really necessary, but gives a more standard message */
 	    *error_p = open_failed(rp, priv->file, "Database not found");
-	    result = FILE_FAIL;
+
+	    return  FILE_FAIL;		/* translate to an unrecov. error */
 	}
-	/* FALLTHRU */
+
+	/* else the optional bit _is_ set.... */
+	return FILE_NOMATCH;		/* leave as-is */
 
     case FILE_FAIL:
     case FILE_AGAIN:
@@ -337,32 +462,40 @@ rewrite_lookup(rp, addr, fl, rt_info, error_p)
     /* parse raw_data into flag and format */
     p = raw_data;
     if (p) {
-	while (*p && isspace(*p)) p++;
+	while (*p && isspace((int) *p)) {
+	    p++;
+	}
     }
     if (p == NULL || *p == '\0') {
-	*error_p = bad_entry(rp, raw_data);
+	*error_p = bad_entry(rp, priv->file, raw_data);
 	return DB_AGAIN;
     }
     flag = p;
-    while (*p && !isspace(*p)) {
+    while (*p && !isspace((int) *p)) {
 	p++;				/* skip the flags */
     }
-    while (*p && isspace(*p)) {
+    while (*p && isspace((int) *p)) {
 	p++;				/* and move to the next token */
     }
     format = p; /* might be empty string.  check later if we care */
 
     /* does the database say we should rewrite this address? */
     switch (*flag) {
-    case '-':	return DB_NOMATCH;
-    case '+':	if (*format) {
-		    break; /* rewrite it */
-		} else {
-		    *error_p = bad_entry(rp, raw_data);
-		    return DB_AGAIN;
-		}
-    default:	*error_p = bad_entry(rp, raw_data);
-		return DB_AGAIN;
+
+    case '-':
+	return DB_NOMATCH;
+
+    case '+':
+	if (*format) {
+	    break; /* rewrite it */
+	} else {
+	    *error_p = bad_entry(rp, priv->file, raw_data);
+	    return DB_AGAIN;
+	}
+
+    default:
+	*error_p = bad_entry(rp, priv->file, raw_data);
+	return DB_AGAIN;
     }
 
     /*
@@ -419,39 +552,43 @@ rewrite_lookup(rp, addr, fl, rt_info, error_p)
     }
 
     /*
-     * It seems like a good idea to record the fact that the old address is the
-     * parent of the new address (mostly so that the parent address will appear
-     * in the logs).  The usual place where that happens is in a director (not
-     * a router), and the usual way of doing that is to allocate a new addr
-     * structure, fill the newly allocated struct with the new info, and adjust
-     * pointers to link the new struct into an appropriate list and leave the
-     * old struct in limbo.  But we don't have access to the linked list
-     * pointers, so we cheat, taking advantage of the way we know we will be
-     * called from rtd_standard, which does have access to the lists.  What we
-     * do is allocate a new addr structure, copy the old information to the
-     * newly allocated structure where it is not on any of the linked lists,
-     * and modify the current addr structure to contain the new information.
-     * After that, the old information is in limbo and the new information is
-     * linked into a list, which is what we want.
+     * It is necessary to record the fact that the old address is the parent of
+     * the new address (mostly so that the rewrite driver can be used to
+     * forward to an internal mail server without triggering a remote relay
+     * violation, but also so that the parent address will appear in the logs).
+     *
      */
     {
 	struct addr *new = alloc_addr();
+
+	/* new = addr; */
 	(void) memcpy((char *) new, (char *) addr, sizeof(*addr));
 	addr->parent = new;
-	addr->in_addr = COPY_STRING(new_address);
-	addr->remainder = new_address;
     }
 
-    DEBUG2(DBG_DRIVER_MID, "rewrite_lookup: rewrote %s --> %s\n",
-	   addr->parent->in_addr, new_address);
-
     /*
-     * all the following is necessary to tell higher level software
-     * to re-parse the address passed back in addr->remainder.
+     * all the following is necessary to tell higher level software to re-parse
+     * the address passed back in addr->remainder.
+     *
+     * The usual place where this kind of thing happens is in a director (not a
+     * router).  Conveniently we have already effectivley saved a copy of the
+     * original structure in the newly allocated parent structure above.  All
+     * we have to do now is to modify the current addr structure to contain the
+     * new input address and to tell the routing engine that we've matched the
+     * whole length of the target address (which will cause the ADDR_FINISHED
+     * and ADDR_FULLMATCH flags to be set in route_driver_finish() thus
+     * claiming this address for ourselves).
      */
+    addr->in_addr = COPY_STRING(new_address);	/* XXX must we do this too? */
+    addr->remainder = new_address;		/* XXX reroute only did this */
     rt_info->next_host = NULL;
     rt_info->route = NULL;
+
     rt_info->matchlen = strlen(addr->target);
+
+    DEBUG3(DBG_DRIVER_MID, "router %s: rewrite_lookup forcing reparse: rewrote %s --> %s\n",
+	   rp->name, addr->parent->in_addr, addr->in_addr);
+
     return DB_SUCCEED;
 }
 
@@ -508,7 +645,7 @@ find_domain(rp, addr, raw_data, match, error_p)
     struct rewrite_private *priv;	/* private data */
     char *savedomain = NULL;		/* saved position of removed domain */
     char *target = addr->target;	/* target being searched for */
-    unsigned int len = strlen(target);	/* total length of target */
+    size_t len = strlen(target);	/* total length of target */
     int result;				/* return from function calls */
     char *error_text = NULL;		/* error messages from subroutines */
 
@@ -589,7 +726,9 @@ find_domain(rp, addr, raw_data, match, error_p)
 	}
 	*match = target;		/* return the match and the data */
 	if (result == DB_SUCCEED) {
-	    while (isspace(**raw_data)) (*raw_data)++;
+	    while (isspace((int) **raw_data)) {
+		(*raw_data)++;
+	    }
 	} else {
 	    *error_p = lookup_error(rp, target, error_text);
 	}
@@ -607,7 +746,9 @@ find_domain(rp, addr, raw_data, match, error_p)
 	    }
 	    *match = target + 1;
 	    if (result == DB_SUCCEED) {
-		while (isspace(**raw_data)) (*raw_data)++;
+		while (isspace((int) **raw_data)) {
+		    (*raw_data)++;
+		}
 	    } else {
 		*error_p = lookup_error(rp, target, error_text);
 	    }
@@ -630,7 +771,9 @@ find_domain(rp, addr, raw_data, match, error_p)
 	    }
 	    *match = target;
 	    if (result == DB_SUCCEED) {
-		while (isspace(**raw_data)) (*raw_data)++;
+		while (isspace((int) **raw_data)) {
+		    (*raw_data)++;
+		}
 	    } else {
 		*error_p = lookup_error(rp, target, error_text);
 	    }
@@ -649,7 +792,7 @@ find_domain(rp, addr, raw_data, match, error_p)
 	}
 
 	/* advance to the next dot */
-	target = index(target, '.');
+	target = strchr(target, '.');
 	if (target) {
 	    /* if there is anything left, look it up */
 	    result = lookup_database(priv->database, target,
@@ -660,7 +803,9 @@ find_domain(rp, addr, raw_data, match, error_p)
 		}
 		*match = target;
 		if (result == DB_SUCCEED) {
-		    while (isspace(**raw_data)) (*raw_data)++;
+		    while (isspace((int) **raw_data)) {
+			(*raw_data)++;
+		    }
 		} else {
 		    *error_p = lookup_error(rp, target, error_text);
 		}
@@ -683,8 +828,9 @@ find_domain(rp, addr, raw_data, match, error_p)
  */
 
 static struct error *
-bad_entry(rp, raw_data)
+bad_entry(rp, file, raw_data)
     struct router *rp;
+    char *file;
     char *raw_data;
 {
     char *error_text;
@@ -701,8 +847,8 @@ bad_entry(rp, raw_data)
      * RESOLUTION
      *      The postmaster should correct the rewrite database entry.
      */
-    error_text = xprintf("router %s: bad entry in rewrite database: %s",
-			 rp->name, raw_data);
+    error_text = xprintf("router %s: driver %s: bad entry in database %s: %s",
+			 rp->name, rp->driver, file, raw_data);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_CONFERR|ERR_1201, error_text);
@@ -731,12 +877,11 @@ open_failed(rp, file, open_error)
      *      The postmaster should check the router entry against the
      *      database he wishes to use.
      */
-    error_text = xprintf("router %s: rewrite database %s, open failed: %s",
-			 rp->name, file, open_error);
+    error_text = xprintf("router %s: driver %s: database '%s', open failed: %s",
+			 rp->name, rp->driver, file, open_error);
     DEBUG2(DBG_DRIVER_LO, "%s: %s\n", error_text, (rp->flags & RW_TRYAGAIN) ? "(will defer)" : "Configuration error!");
 
-    return note_error((rp->flags & RW_TRYAGAIN) ? ERR_1202 : ERR_CONFERR|ERR_1202,
-		      error_text);
+    return note_error((rp->flags & RW_TRYAGAIN) ? ERR_1202 : (ERR_CONFERR | ERR_1202), error_text);
 }
 
 static struct error *
@@ -760,8 +905,8 @@ lookup_error(rp, target, lookup_error_text)
      * RESOLUTION
      *      Unspecified.
      */
-    error_text = xprintf("router %s: target %s, lookup failed: %s",
-			 rp->name, target, lookup_error_text);
+    error_text = xprintf("router %s: driver %s: target %s, lookup failed: %s",
+			 rp->name, rp->driver, target, lookup_error_text);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_1203, error_text);
@@ -790,11 +935,11 @@ expand_error(rp, format, infop)
      *	    rewrite database.
      */
     if (!infop) {
-	error_text = xprintf("router %s: expansion of \"%s\" failed",
-			     rp->name, format);
+	error_text = xprintf("router %s: driver %s: expansion of \"%s\" failed",
+			     rp->name, rp->driver, format);
     } else {
-	error_text = xprintf("router %s: expansion of \"%s\" failed: %s",
-			     rp->name, format, infop);
+	error_text = xprintf("router %s: driver %s: expansion of \"%s\" failed: %s",
+			     rp->name, rp->driver, format, infop);
     }
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
     return note_error(ERR_CONFERR|ERR_1204, error_text);
