@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:rtlib.c,v 1.13 2000/09/03 17:00:23 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:rtlib.c,v 1.21 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -16,24 +16,57 @@
  *	can depend on these routines rather than having to duplicate
  *	this intelligence.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../exitcodes.h"
+#include "../transport.h"
 #include "rtlib.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /*
  * rtd_standard - standard function for routing to a target
@@ -70,29 +103,30 @@
  *
  *	DB_NOMATCH	The target was not matched by the router.  The
  *			addr structure will be linked to the 'out'
- *			or 'retry' list without modification.
+ *			list without modification.
  *
  *	DB_FAIL		An unrecoverable error in routing to the
  *			target.  This will cause the addr structure to
- *			be linked into the 'fail' output list, tagged
+ *			be linked into the 'fail' output list, and tagged
  *			with the 'error' structure.
  *
  *	DB_AGAIN	A temporary error in routing to the target.
  *			This will cause the addr structure to be
- *			linked into the 'defer' output list, tagged
+ *			linked into the 'defer' output list, and tagged
  *			with the 'error' structure.
  *
  *	FILE_NOMATCH	The router considers itself optional and
- *			disabled.  This and all subsequent unfinished
+ *			disabled (its database doesn't exist, for
+ *			example).  This and all subsequent unfinished
  *			addr structures will be linked into the 'out'
- *			or 'retry' list unchanged.
+ *			list unchanged.
  *
  *	FILE_FAIL	There was an unrecoverable error in the
  *			router.  This is treated as a configuration
- *			error, so the ERR_CONFERR flag is set in the
+ *			error, the ERR_CONFERR flag will be set in the
  *			'error' structure.  This and all subsequent
  *			unfinished addrs will be linked into the
- *			'defer' list, tagged with the 'error'
+ *			'defer' list, and tagged with the 'error'
  *			structure.
  *
  *	FILE_AGAIN	There was a temporary error in the router.
@@ -160,7 +194,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 
 	    case FILE_FAIL:
 	    case FILE_AGAIN:
-		DEBUG1(DBG_DRIVER_HI, "%s: router failed, moving to error & defer lists.\n", cur->in_addr);
+		DEBUG1(DBG_DRIVER_HI, "%s: router failed, moving to both error & defer lists.\n", cur->in_addr);
 		cur->error = error;	/* use error from last lookup */
 		cur->succ = *defer;
 		*defer = cur;
@@ -176,6 +210,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 	     * look it up.
 	     */
 	    DEBUG1(DBG_DRIVER_HI, "%s: clearing rt_info and preparing to call lookup().\n", cur->in_addr);
+	    rt_info.matchlen = -1;
 	    rt_info.next_host = NULL;
 	    rt_info.route = NULL;
 	    rt_info.transport = NULL;
@@ -207,6 +242,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 
 	case DB_FAIL:
 	    /* The address should be failed, with an error of some kind. */
+	    exitvalue = EX_NOHOST;	/* set exit status */
 	    cur->error = error;
 	    cur->succ = *fail;
 	    *fail = cur;
@@ -214,6 +250,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 
 	case DB_AGAIN:
 	    /* Routing for this address should be reattempted later. */
+	    exitvalue = EX_TEMPFAIL;	/* set exit status */
 	    cur->error = error;
 	    cur->succ = *defer;
 	    *defer = cur;
@@ -228,6 +265,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 
 	case FILE_FAIL:
 	    /* Permanent router error, this is a configuration error. */
+	    exitvalue = EX_UNAVAILABLE;	/* set exit status */
 	    error->info |= ERR_CONFERR;
 	    cur->error = error;
 	    cur->succ = *defer;
@@ -237,6 +275,7 @@ rtd_standard(rp, in, out, defer, fail, lookup)
 
 	case FILE_AGAIN:
 	    /* Temporary router database error, retry all addresses. */
+	    exitvalue = EX_TEMPFAIL;	/* set exit status */
 	    cur->error = error;
 	    cur->succ = *defer;
 	    *defer = cur;
