@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:smarthost.c,v 1.13 2000/09/03 17:00:24 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:smarthost.c,v 1.20 2003/04/02 22:17:52 woods Exp"
  */
 
 /*
@@ -51,24 +51,55 @@
  *	  software is expected to be able to allow for secondary passes
  *	  through the routers to resolve the transport to the smarthost.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "smarthost.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static struct error *no_smarthost_info __P((struct router *));
@@ -76,9 +107,9 @@ static struct error *smarthost_parse_error __P((struct router *, char *, char *)
 static int match_auth_domains __P((char *));
 
 static struct attr_table smarthost_attributes[] = {
-    { "path", t_string, NULL, NULL, OFFSET(smarthost_private, path) },
-    { "host", t_string, NULL, NULL, OFFSET(smarthost_private, host) },
-    { "route", t_string, NULL, NULL, OFFSET(smarthost_private, route) },
+    { "path", t_string, 0, NULL, NULL, OFFSET(smarthost_private, path) },
+    { "host", t_string, 0, NULL, NULL, OFFSET(smarthost_private, host) },
+    { "route", t_string, 0, NULL, NULL, OFFSET(smarthost_private, route) },
 };
 static struct attr_table *end_smarthost_attributes =
     ENDTABLE(smarthost_attributes);
@@ -94,7 +125,7 @@ rtd_smarthost(rp, in, out, defer, fail)
     struct addr *in;			/* input addr structures */
     struct addr **out;			/* non-failed addr structures */
     struct addr **defer;		/* addrs to defer to a later time */
-    struct addr **fail;			/* unresolvable addrs */
+    struct addr **fail;			/* UNUSED unresolvable addrs */
 {
     register struct addr *cur;		/* current addr being processed */
     struct addr *next;			/* next addr to process */
@@ -181,6 +212,7 @@ rtb_smarthost(rp, attrs)
     static struct smarthost_private smarthost_template = {
 	NULL,				/* host */
 	NULL,				/* route */
+	NULL,				/* internal -- transport_ptr */
     };
     struct smarthost_private *priv;	/* new smarthost_private structure */
 
@@ -261,8 +293,8 @@ no_smarthost_info(rp)
      *      define either a smart_host/smart_route pair
      *      or a smart_path attribute.
      */
-    error_text = xprintf("router %s: smart_host or smart_path required",
-			 rp->name);
+    error_text = xprintf("router %s: driver %s: smart_host or smart_path required",
+			 rp->name, rp->driver);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_125, error_text);
@@ -292,8 +324,8 @@ smarthost_parse_error(rp, path, parse_error)
      *      The smart_path attribute in the router file
      *      should be corrected to be a valid !-path.
      */
-    error_text = xprintf("router %s: error parsing %s: %s",
-			 rp->name, path, parse_error);
+    error_text = xprintf("router %s: driver %s: error parsing %s: %s",
+			 rp->name, rp->driver, path, parse_error);
     DEBUG1(DBG_DRIVER_LO, "%s\n", error_text);
 
     return note_error(ERR_CONFERR|ERR_126, error_text);
