@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/routers:RELEASE-3_2_0_114:uuname.c,v 1.10 2000/09/03 17:00:24 woods Exp"
+#ident	"@(#)smail/src/routers:RELEASE-3_2_0_115:uuname.c,v 1.18 2003/04/02 22:17:38 woods Exp"
  */
 
 /*
@@ -43,27 +43,69 @@
  *	notes:
  *	    The output of the command is cached.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
-#include "defs.h"
+#include <signal.h>
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# define _GNU_SOURCE			/* to see decl. of strsignal() */
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
+# include <sys/wait.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #include "../smail.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../route.h"
-#include "../transport.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
 #include "../lookup.h"
+#include "../smailwait.h"
 #include "../child.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "rtlib.h"
 #include "uuname.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static int uuname_lookup __P((struct router *, struct addr *, int, struct rt_info *, struct error **));
@@ -73,10 +115,10 @@ static struct error *open_error __P((struct router *, char *));
 static struct error *scan_error __P((struct router *, char *));
 
 static struct attr_table uuname_attributes[] = {
-    { "cmd", t_string, NULL, NULL, OFFSET(uuname_private, cmd) },
-    { "domain", t_string, NULL, NULL, OFFSET(uuname_private, domain) },
-    { "required", t_string, NULL, NULL, OFFSET(uuname_private, required) },
-    { "statfile", t_string, NULL, NULL, OFFSET(uuname_private, statfile) },
+    { "cmd", t_string, 0, NULL, NULL, OFFSET(uuname_private, cmd) },
+    { "domain", t_string, 0, NULL, NULL, OFFSET(uuname_private, domain) },
+    { "required", t_string, 0, NULL, NULL, OFFSET(uuname_private, required) },
+    { "statfile", t_string, 0, NULL, NULL, OFFSET(uuname_private, statfile) },
 };
 static struct attr_table *end_uuname_attributes =
     ENDTABLE(uuname_attributes);
@@ -142,7 +184,7 @@ rtc_uuname(rp)
 	    return;
 	}
 	if (rp->flags & UU_CACHED) {
-	    if (statbuf.st_ino == priv->stat_ino &&
+	    if ((int) statbuf.st_ino == priv->stat_ino &&
 		statbuf.st_mtime == priv->stat_mtime)
 	    {
 		/* stat file has not changed */
@@ -188,6 +230,11 @@ rtb_uuname(rp, attrs)
 	NULL,				/* domain */
 	NULL,				/* required */
 	NULL,				/* cmd_output - for internal use */
+	NULL,				/* file associated with command */
+	NULL,				/* internal - end of cached output */
+	NULL,				/* internal - processing error */
+	0,				/* internal - st_ino from stat */
+	0,				/* internal - st_mtime from stat */
     };
     struct uuname_private *priv;	/* new uuname_private structure */
 
@@ -255,7 +302,7 @@ static int
 uuname_lookup(rp, addr, fl, rt_info, error_p)
     struct router *rp;			/* router table entry */
     struct addr *addr;			/* addr structure */
-    int fl;				/* flags from rt[dv]_standard */
+    int fl;				/* UNUSED flags from rt[dv]_standard */
     struct rt_info *rt_info;		/* return route info here */
     struct error **error_p;		/* return lookup error here */
 {
@@ -382,24 +429,30 @@ read_command(priv, error)
     FILE *f;
     struct str str;			/* string region */
     register int c;			/* character from uuname command */
-    static char *argv[] = {		/* args for shell */
-#ifdef SHELL_EXEC_PATH
-	SHELL_EXEC_PATH,
-#else
-	"/bin/sh",
-#endif
-	"-c",
-	NULL,
-	NULL,
-    };
     int pid;				/* pid of child process */
     int status;				/* exit status from child */
     int ferrorret;			/* return value from ferror() */
+    static char *argv[] = {		/* args for shell */
+#if !defined(HAVE_HASH_BANG)		/* this might not be a good idea anyway... */
+# ifdef SHELL_EXEC_PATH
+	SHELL_EXEC_PATH,
+# else
+	"/bin/sh",
+# endif
+	"-c",
+#endif /* HAVE_HASH_BANG */
+	NULL,
+	NULL,
+    };
 
     DEBUG1(DBG_DRIVER_MID, "scanning %s output ...", priv->cmd);
+#if defined(HAVE_HASH_BANG)
+    argv[0] = priv->cmd;
+#else
     argv[2] = priv->cmd;
-    pid = open_child(argv, (char **)NULL, (FILE **)NULL, &f,
-		     errfile? fileno(errfile): -1,
+#endif
+    pid = open_child(argv, (char **) NULL, (FILE **) NULL, &f,
+		     errfile ? fileno(errfile) : -1,
 		     CHILD_MINENV, nobody_uid, nobody_gid);
     if (pid == EOF) {
 	*error = xprintf("process creation for `%s': %s",
@@ -421,24 +474,60 @@ read_command(priv, error)
     }
     STR_DONE(&str);
     ferrorret = ferror(f);
-    status = close_child((FILE *)NULL, f, pid);
+    status = close_child((FILE *) NULL, f, pid);
     if (status != 0) {
 	STR_FREE(&str);
-	*error = xprintf("command `%s' returned exit status %s",
-			 priv->cmd, strsysexit(status));
-	DEBUG(DBG_DRIVER_MID, "error\n");
+	if (status == -1) {
+	    *error = xprintf("command `%s' PID=%d: unable to reap exit status: %s",
+			     priv->cmd,
+			     pid,
+			     strerror(errno));
+	} else if (WIFEXITED(status)) {
+	    if (WEXITSTATUS(status) != 0) {
+		*error = xprintf("command `%s' PID=%d: exit status: %s (%d)",
+				 priv->cmd,
+				 pid,
+				 strsysexit(WEXITSTATUS(status)),
+				 WEXITSTATUS(status));
+	    }
+	} else if (WIFSIGNALED(status)) {
+	    char signm[SIG2STR_MAX];
+
+	    if (sig2str(WTERMSIG(status), signm) == -1) {
+		sprintf(signm, " %d", WTERMSIG(status));
+	    }
+	    *error = xprintf("command `%s' PID=%d: killed by signal SIG%s %s: %s",
+			     priv->cmd,
+			     pid,
+			     signm,
+			     WCOREDUMP(status) ? "and dumped core" : "(no core)",
+			     strsignal(WTERMSIG(status)));
+	} else if (WIFSTOPPED(status)) {
+	    char signm[SIG2STR_MAX];
+
+	    /* in theory we'll hopefully never see stopped processes... */
+	    if (sig2str(WTERMSIG(status), signm) == -1) {
+		sprintf(signm, " %d", WSTOPSIG(status));
+	    }
+	    *error = xprintf("command `%s' PID=%d: stopped unexpectedly by signal SIG%s: %s",
+			     priv->cmd,
+			     pid,
+			     signm,
+			     strsignal(WSTOPSIG(status)));
+	}
+	DEBUG1(DBG_DRIVER_MID, "error: %s\n", *error);
 	return FILE_FAIL;
     }
     if (ferrorret) {
 	STR_FREE(&str);
 	*error = xprintf("read error in output from `%s'", priv->cmd);
-	DEBUG(DBG_DRIVER_MID, "error\n");
+	DEBUG1(DBG_DRIVER_MID, "error: %s\n", *error);
 	return FILE_AGAIN;
     }
     if (STR_LEN(&str) > 0 && STR(&str)[STR_LEN(&str) - 1] != '\0') {
 	STR_FREE(&str);
 	*error = xprintf("output of `%s' does not end in newline", priv->cmd);
-	DEBUG(DBG_DRIVER_MID, "error\n");
+	DEBUG1(DBG_DRIVER_MID, "error: %s\n", *error);
 	return FILE_FAIL;
     }
 
