@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:silly.c,v 1.10 2000/09/09 20:52:31 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:silly.c,v 1.16 2003/04/02 22:15:06 woods Exp"
  */
 
 /*
@@ -19,28 +19,48 @@
  *
  *	external functions: silly
  */
-#include <stdio.h>
-#include <sys/types.h>
+
 #include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#include <sys/types.h>
+#include <stdio.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
 #ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
 # include <string.h>
 #endif
-#if defined(POSIX_OS)
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif  /* not POSIX_OS */
+#endif
 
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
+#include "smailport.h"
 
 #ifdef STANDALONE
 char *local_sender;
@@ -61,7 +81,7 @@ void
 silly()
 {
 	char name[TOMBNAME+1];		/* name to put on tombstone */
-	unsigned int sender_len;	/* length of the sender string */
+	size_t sender_len;		/* length of the sender string */
 	int pad_len;			/* spaces on left for centering */
 	int i;				/* index */
 
