@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:smailconf.c,v 1.57 2001/07/29 23:11:32 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:smailconf.c,v 1.78 2003/06/17 19:21:23 woods Exp"
  */
 
 /*
@@ -14,20 +14,44 @@
  * smailconf.c:
  *	loading and processing for the configuration files
  *
- *	external functions: read_config_file, read_standard_file,
- *			    fill_attributes, find_attribute,
- *			    add_config_stat, is_newconf, make_lib_fn
+ *	external functions: read_config_file, quote_string_value,
+ *			    format_attribute, print_config_variable,
+ *			    read_standard_file, fill_attributes,
+ *			    find_config_attribute, add_config_stat,
+ *			    is_newconf
  */
 
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
 
-#ifdef HAVE_STDLIB_H
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
 #endif
 
 #include "smail.h"
@@ -37,21 +61,24 @@
 #include "addr.h"
 #include "direct.h"
 #include "route.h"
-#include "transport.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "version.h"
+#include "transport.h"
 #include "smailconf.h"
 #include "exitcodes.h"
 #include "extern.h"
 #include "debug.h"
+#include "smailport.h"
 
 /* These 2 values are used in handling int values as suffix multipliers */
 #define K_MULTIPLIER	1024
 #define M_MULTIPLIER	(1024*1024)
 
 /* functions local to this file */
-static int format_attribute __P((struct attr_table *, char *, int, char **, char **, char *, long));
+static struct attr_table *find_attribute __P((char *, struct attr_table *, struct attr_table *));
 static void print_config_help __P((void));
 static void print_config_all __P((void));
 
@@ -61,6 +88,7 @@ static void print_config_all __P((void));
  * determine if any configuration files have changed.  Optional config
  * files which do not exist have a 0 stored in the mtime field.
  */
+/* FIXME: move this declaration to smailconf.h */
 struct config_stat {
     struct config_stat *succ;
     char    *fn;			/* name of config file */
@@ -69,130 +97,151 @@ struct config_stat {
     ino_t   ino;			/* inode number for file */
 } *config_stat_list = NULL;
 
-/* define the attributes which may be in the config file */
+/*
+ * define the attributes which may be in the config file
+ *
+ * NOTE: keep these in sorted order, except always give duplicate/deprecated
+ * names (i.e. those which point to the same variable) immediately after the
+ * canonical name so that they will be skipped by print_config_all();
+ * (i.e. "smail -bP all").
+ */
 static struct attr_table conf_attributes[] = {
-    { "auth_domains", t_string, NULL, (tup *)&auth_domains, 0 },
-    { "auth_domain", t_string, NULL, (tup *)&auth_domains, 0 },
-    { "auto_mkdir", t_boolean, NULL, (tup *)&auto_mkdir, 0 },
-    { "auto_mkdir_mode", t_mode, NULL, (tup *)&auto_mkdir_mode, 0 },
-    { "config_file", t_string, NULL, (tup *)&config_file, 0 },
-    { "console", t_string, NULL, (tup *)&cons_fn, 0 },
-    { "copying_file", t_string, NULL, (tup *)&copying_file, 0 },
-    { "date_field", t_string, NULL, (tup *)&date_field, 0 },
-    { "delivery_grades", t_string, NULL, (tup *)&delivery_grades, 0 },
-    { "delivery_mode", t_string, NULL, (tup *)&delivery_mode_string, 0 },
-    { "director_file", t_string, NULL, (tup *)&director_file, 0 },
-    { "domains", t_string, NULL, (tup *)&visible_domains, 0 },
-    { "domain", t_string, NULL, (tup *)&visible_domains, 0 },
-    { "visible_domains", t_string, NULL, (tup *)&visible_domains, 0 },
-    { "visible_domain", t_string, NULL, (tup *)&visible_domains, 0 },
-    { "error_copy_postmaster", t_boolean, NULL,
-      (tup *)&error_copy_postmaster, 0 },
-    { "flock_mailbox", t_boolean, NULL, (tup *)&flock_mailbox, 0 },
-    { "fnlock_interval", t_int, NULL, (tup *)&fnlock_interval, 0 },
-    { "fnlock_mode", t_mode, NULL, (tup *)&fnlock_mode, 0 },
-    { "fnlock_retries", t_int, NULL, (tup *)&fnlock_retries, 0 },
-    { "from_field", t_string, NULL, (tup *)&from_field, 0 },
-    { "grades", t_string, NULL, (tup *)&grades, 0 },
-    { "hit_table_len", t_int, NULL, (tup *)&hit_table_len, 0 },
-    { "host_lock_timeout", t_interval, NULL, (tup *)&host_lock_timeout, 0 },
-    { "hostnames", t_string, NULL, (tup *)&hostnames, 0 },
-    { "hostname", t_string, NULL, (tup *)&hostnames, 0 },
-    { "listen_name", t_string, NULL, (tup *)&listen_name, 0 },
-    { "sending_name", t_string, NULL, (tup *)&sending_name, 0 },
-    { "lock_by_name", t_boolean, NULL, (tup *)&lock_by_name, 0 },
-    { "lock_mode", t_mode, NULL, (tup *)&lock_mode, 0 },
-    { "log_mode", t_mode, NULL, (tup *)&log_mode, 0 },
-    { "logfile", t_string, NULL, (tup *)&log_fn, 0 },
-    { "max_hop_count", t_int, NULL, (tup *)&max_hop_count, 0 },
-    { "max_load_ave", t_double, NULL, (tup *)&max_load_ave, 0 },
-    { "max_message_size", t_long, NULL, (tup *)&max_message_size, 0 },
-    { "message_buf_size", t_int, NULL, (tup *)&message_bufsiz, 0 },
-    { "message_id_field", t_string, NULL, (tup *)&message_id_field, 0 },
-    { "message_log_mode", t_mode, NULL, (tup *)&message_log_mode, 0 },
-    { "method_dir", t_string, NULL, (tup *)&method_dir, 0 },
-    { "more_hostnames", t_string, NULL, (tup *)&more_hostnames, 0 },
-    { "gateway_names", t_string, NULL, (tup *)&more_hostnames, 0 },
-    { "nobody", t_string, NULL, (tup *)&nobody, 0 },
-    { "open_interval", t_int, NULL, (tup *)&open_interval, 0 },
-    { "open_retries", t_int, NULL, (tup *)&open_retries, 0 },
-    { "paniclog", t_string, NULL, (tup *)&panic_fn, 0 },
-    { "postmaster_address", t_string, NULL, (tup *)&postmaster_address, 0 },
-    { "postmaster", t_string, NULL, (tup *)&postmaster_address, 0 },
-    { "primary_name", t_string, NULL, (tup *)&primary_name, 0 },
-    { "qualify_file", t_string, NULL, (tup *)&qualify_file, 0 },
-    { "queue_only", t_boolean, NULL, (tup *)&queue_only, 0 },
-    { "received_field", t_string, NULL, (tup *)&received_field, 0 },
-    { "resolve_timeout", t_interval, NULL, (tup *)&resolve_timeout, 0 },
-    { "require_configs", t_boolean, NULL, (tup *)&require_configs, 0 },
-    { "retry_duration", t_interval, NULL, (tup *)&retry_duration, 0 },
-    { "retry_file", t_string, NULL, (tup *)&retry_file, 0 },
-    { "retry_interval", t_interval, NULL, (tup *)&retry_interval, 0 },
-    { "return_path_field", t_string, NULL, (tup *)&return_path_field, 0 },
-    { "router_file", t_string, NULL, (tup *)&router_file, 0 },
-    { "runq_grades", t_string, NULL, (tup *)&runq_grades, 0 },
-    { "second_config_file", t_string, NULL, (tup *)&second_config_file, 0 },
-    { "sender_env_variable", t_string, NULL, (tup *)&sender_env_variable, 0 },
-    { "smail", t_string, NULL, (tup *)&smail, 0 },
-    { "smail_lib_dir", t_string, NULL, (tup *)&smail_lib_dir, 0 },
-    { "smail_util_dir", t_string, NULL, (tup *)&smail_util_dir, 0 },
-    { "smart_path", t_string, NULL, (tup *)&smart_path, 0 },
-    { "smart_transport", t_string, NULL, (tup *)&smart_transport, 0 },
-    { "smart_user", t_string, NULL, (tup *)&smart_user, 0 },
-    { "smtp_accept_max", t_int, NULL, (tup *)&smtp_accept_max, 0 },
-    { "smtp_accept_queue", t_int, NULL, (tup *)&smtp_accept_queue, 0 },
-    { "smtp_banner", t_string, NULL, (tup *)&smtp_banner, 0 },
-    { "smtp_allow_debug", t_boolean, NULL, (tup *)&smtp_allow_debug, 0 },
-    { "smtp_debug", t_boolean, NULL, (tup *)&smtp_allow_debug, 0 },
-    { "smtp_allow_expn", t_boolean, NULL, (tup *)&smtp_allow_expn, 0 },
-    { "smtp_info", t_boolean, NULL, (tup *)&smtp_allow_expn, 0 },
-    { "smtp_receive_command_timeout", t_interval, NULL,
-      (tup *)&smtp_receive_command_timeout, 0 },
-    { "smtp_receive_message_timeout", t_interval, NULL,
-      (tup *)&smtp_receive_message_timeout, 0 },
-    { "smtp_recipient_no_verify", t_string, NULL, (tup *)&smtp_recipient_no_verify, 0 },
-    { "smtp_remote_allow", t_string, NULL, (tup *)&smtp_remote_allow, 0 },
-    { "smtp_hello_verify", t_boolean, NULL, (tup *)&smtp_hello_verify, 0 },
-    { "smtp_hello_broken_allow", t_string, NULL, (tup *)&smtp_hello_broken_allow, 0 },
-    { "smtp_hello_reject_dns_paranoid", t_boolean, NULL, (tup *)&smtp_hello_reject_dns_paranoid, 0 },
-    { "smtp_hello_reject_hosts", t_string, NULL, (tup *)&smtp_hello_reject_hosts, 0 },
-    { "smtp_hello_verify_literal", t_boolean, NULL, (tup *)&smtp_hello_verify_literal, 0 },
-    { "smtp_hello_verify_ptr", t_boolean, NULL, (tup *)&smtp_hello_verify_ptr, 0 },
-    { "smtp_rbl_domains", t_string, NULL, (tup *)&smtp_rbl_domains, 0 },
-    { "smtp_rbl_except", t_string, NULL, (tup *)&smtp_rbl_except, 0 },
-    { "smtp_sender_no_verify", t_string, NULL, (tup *)&smtp_sender_no_verify, 0 },
-    { "smtp_sender_rhsbl_domains", t_string, NULL, (tup *)&smtp_sender_rhsbl_domains, 0 },
-    { "smtp_sender_rhsbl_except", t_string, NULL, (tup *)&smtp_sender_rhsbl_except, 0 },
-    { "smtp_sender_verify_mx_only", t_boolean, NULL, (tup *)&smtp_sender_verify_mx_only, 0 },
-    { "smtp_max_recipients", t_int, NULL, (tup *)&smtp_max_recipients, 0 },
-    { "smtp_expn_delay", t_int, NULL, (tup *)&smtp_expn_delay, 0 },
-    { "smtp_vrfy_delay", t_int, NULL, (tup *)&smtp_vrfy_delay, 0 },
-    { "smtp_error_delay", t_int, NULL, (tup *)&smtp_error_delay, 0 },
-    { "debug_pause_for_smtp_connections", t_int, NULL, (tup *)&debug_pause_for_smtp_connections, 0},
-    { "spool_dirs", t_string, NULL, (tup *)&spool_dirs, 0 },
-    { "spool_dir", t_string, NULL, (tup *)&spool_dir, 0 },
-    { "spool_grade", t_char, NULL, (tup *)&spool_grade, 0 },
-    { "spool_mode", t_mode, NULL, (tup *)&spool_mode, 0 },
-    { "switch_percent_and_bang", t_boolean, NULL,
-      (tup *)&switch_percent_and_bang, 0 },
-    { "rfc1413_query_timeout", t_interval, NULL, (tup *)&rfc1413_query_timeout, 0 },
-    { "transport_file", t_string, NULL, (tup *)&transport_file, 0 },
-    { "trusted_users", t_string, NULL, (tup *)&trusted, 0 },
-    { "trusted_user", t_string, NULL, (tup *)&trusted, 0 },
-    { "trusted", t_string, NULL, (tup *)&trusted, 0 },
-    { "trusted_groups", t_string, NULL, (tup *)&trusted_groups, 0 },
-    { "trusted_group", t_string, NULL, (tup *)&trusted_groups, 0 },
-    { "uucp_name", t_string, NULL, (tup *)&uucp_name, 0 },
-    { "version", t_infoproc, NULL, (tup *)version, 0 },
-    { "visible_name", t_string, NULL, (tup *)&visible_name, 0 },
+    { "allow_one_mx_target_cname_hack", t_boolean, 0, FALSE, (tup *)&allow_one_mx_target_cname_hack, 0 },
+    { "auth_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&auth_domains, 0 },
+    { "auth_domain", t_string, FA_FMT_IS_LIST, NULL, (tup *)&auth_domains, 0 },	/* dup */
+    { "auto_mkdir", t_boolean, 0, FALSE, (tup *)&auto_mkdir, 0 },
+    { "auto_mkdir_mode", t_mode, 0, 0, (tup *)&auto_mkdir_mode, 0 },
+    { "config_file", t_string, 0, NULL, (tup *)&config_file, 0 },
+    { "console", t_string, 0, NULL, (tup *)&cons_fn, 0 },
+    { "copying_file", t_string, 0, NULL, (tup *)&copying_file, 0 },
+    { "daemon_pidfile", t_string, 0, NULL, (tup *)&daemon_pidfile, 0 },
+    { "date_field", t_string, 0, NULL, (tup *)&date_field, 0 },
+    { "delivery_grades", t_string, 0, NULL, (tup *)&delivery_grades, 0 },
+    { "delivery_mode", t_string, 0, NULL, (tup *)&delivery_mode_string, 0 },
+    { "director_file", t_string, 0, NULL, (tup *)&director_file, 0 },
+    { "domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&visible_domains, 0 },
+    { "domain", t_string, FA_FMT_IS_LIST, NULL, (tup *)&visible_domains, 0 }, /* dup of domains */
+    { "visible_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&visible_domains, 0 }, /* deprecated(?) */
+    { "visible_domain", t_string, FA_FMT_IS_LIST, NULL, (tup *)&visible_domains, 0 }, /* deprecated(?) dup of visible_domains */
+    { "error_copy_postmaster", t_boolean, 0, FALSE, (tup *)&error_copy_postmaster, 0 },
+    { "flock_mailbox", t_boolean, 0, FALSE, (tup *)&flock_mailbox, 0 },
+    { "fnlock_interval", t_int, 0, 0, (tup *)&fnlock_interval, 0 },
+    { "fnlock_mode", t_mode, 0, 0, (tup *)&fnlock_mode, 0 },
+    { "fnlock_retries", t_int, 0, 0, (tup *)&fnlock_retries, 0 },
+    { "from_field", t_string, 0, NULL, (tup *)&from_field, 0 },
+    { "grades", t_string, 0, NULL, (tup *)&grades, 0 },
+    { "hit_table_len", t_int, 0, 0, (tup *)&hit_table_len, 0 },
+    { "host_lock_timeout", t_interval, 0, 0, (tup *)&host_lock_timeout, 0 },
+    { "hostnames", t_string, FA_FMT_IS_LIST, NULL, (tup *)&hostnames, 0 },
+    { "hostname", t_string, FA_FMT_IS_LIST, NULL, (tup *)&hostnames, 0 }, /* dup of hostnames */
+    { "listen_name", t_string, 0, NULL, (tup *)&listen_name, 0 },
+    { "lock_by_name", t_boolean, 0, FALSE, (tup *)&lock_by_name, 0 },
+    { "lock_mode", t_mode, 0, 0, (tup *)&lock_mode, 0 },
+    { "log_mode", t_mode, 0, 0, (tup *)&log_mode, 0 },
+    { "logfile", t_string, 0, NULL, (tup *)&log_fn, 0 },
+    { "max_hop_count", t_int, 0, 0, (tup *)&max_hop_count, 0 },
+    { "max_load_ave", t_double, 0.0, NULL, (tup *)&max_load_ave, 0 },
+    { "max_message_size", t_long, 0, 0L, (tup *)&max_message_size, 0 },
+    { "message_buf_size", t_int, 0, 0, (tup *)&message_bufsiz, 0 },
+    { "message_id_field", t_string, 0, NULL, (tup *)&message_id_field, 0 },
+    { "message_log_mode", t_mode, 0, 0, (tup *)&message_log_mode, 0 },
+    { "method_dir", t_string, 0, NULL, (tup *)&method_dir, 0 },
+    { "more_hostnames", t_string, FA_FMT_IS_LIST, NULL, (tup *)&more_hostnames, 0 },
+    { "gateway_names", t_string, FA_FMT_IS_LIST, NULL, (tup *)&more_hostnames, 0 }, /* semi-deprecated */
+    { "nobody", t_string, 0, NULL, (tup *)&nobody, 0 },
+    { "open_interval", t_int, 0, 0, (tup *)&open_interval, 0 },
+    { "open_retries", t_int, 0, 0, (tup *)&open_retries, 0 },
+    { "paniclog", t_string, 0, NULL, (tup *)&panic_fn, 0 },
+    { "postmaster_address", t_string, 0, NULL, (tup *)&postmaster_address, 0 },
+    { "postmaster", t_string, 0, NULL, (tup *)&postmaster_address, 0 }, /* deprecated */
+    { "primary_name", t_string, 0, NULL, (tup *)&primary_name, 0 },
+    { "qualify_file", t_string, 0, NULL, (tup *)&qualify_file, 0 },
+    { "queue_only", t_boolean, 0, FALSE, (tup *)&queue_only, 0 },
+    { "received_field", t_string, 0, NULL, (tup *)&received_field, 0 },
+    { "require_configs", t_boolean, 0, FALSE, (tup *)&require_configs, 0 },
+    { "resolve_timeout", t_interval, 0, 0, (tup *)&resolve_timeout, 0 },
+    { "retry_duration", t_interval, 0, 0, (tup *)&retry_duration, 0 },
+    { "retry_file", t_string, 0, NULL, (tup *)&retry_file, 0 },
+    { "retry_interval", t_interval, 0, 0, (tup *)&retry_interval, 0 },
+    { "return_path_field", t_string, 0, NULL, (tup *)&return_path_field, 0 },
+    { "rfc1413_query_timeout", t_interval, 0, 0, (tup *)&rfc1413_query_timeout, 0 },
+    { "router_file", t_string, 0, NULL, (tup *)&router_file, 0 },
+    { "runq_grades", t_string, 0, NULL, (tup *)&runq_grades, 0 },
+    { "second_config_file", t_string, 0, NULL, (tup *)&second_config_file, 0 },
+    { "sender_env_variable", t_string, 0, NULL, (tup *)&sender_env_variable, 0 },
+    { "sending_name", t_string, 0, NULL, (tup *)&sending_name, 0 },
+    { "smail", t_string, 0, NULL, (tup *)&smail, 0 },
+    { "smail_lib_dir", t_string, 0, NULL, (tup *)&smail_lib_dir, 0 },
+    { "smail_util_dir", t_string, 0, NULL, (tup *)&smail_util_dir, 0 },
+    { "smart_path", t_string, 0, NULL, (tup *)&smart_path, 0 },
+    { "smart_transport", t_string, 0, NULL, (tup *)&smart_transport, 0 },
+    { "smart_user", t_string, 0, NULL, (tup *)&smart_user, 0 },
+    { "smtp_accept_max", t_int, 0, 0, (tup *)&smtp_accept_max, 0 },
+    { "smtp_accept_queue", t_int, 0, 0, (tup *)&smtp_accept_queue, 0 },
+    { "smtp_allow_debug", t_boolean, 0, FALSE, (tup *)&smtp_allow_debug, 0 },
+    { "smtp_allow_expn", t_boolean, 0, FALSE, (tup *)&smtp_allow_expn, 0 },
+    { "smtp_debug", t_boolean, 0, FALSE, (tup *)&smtp_allow_debug, 0 }, /* deprecated */
+    { "smtp_debug_pause", t_int, 0, 0, (tup *)&smtp_debug_pause, 0},
+    { "smtp_info", t_boolean, 0, FALSE, (tup *)&smtp_allow_expn, 0 }, /* deprecated */
+    { "smtp_bad_mx_targets", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_bad_mx_targets, 0 },
+    { "smtp_banner", t_string, 0, NULL, (tup *)&smtp_banner, 0 },
+    { "smtp_error_delay", t_int, 0, 0, (tup *)&smtp_error_delay, 0 },
+    { "smtp_expn_delay", t_int, 0, 0, (tup *)&smtp_expn_delay, 0 },
+    { "smtp_hello_broken_allow", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_hello_broken_allow, 0 },
+    { "smtp_hello_dnsbl_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_hello_dnsbl_domains, 0 },
+    { "smtp_hello_dnsbl_except", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_hello_dnsbl_except, 0 },
+    { "smtp_hello_reject_dns_paranoid", t_boolean, 0, FALSE, (tup *)&smtp_hello_reject_dns_paranoid, 0 },
+    { "smtp_hello_reject_hostnames", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_hello_reject_hostnames, 0 },
+    { "smtp_hello_verify", t_boolean, 0, FALSE, (tup *)&smtp_hello_verify, 0 },
+    { "smtp_hello_verify_literal", t_boolean, 0, FALSE, (tup *)&smtp_hello_verify_literal, 0 },
+    { "smtp_hello_verify_ptr", t_boolean, 0, FALSE, (tup *)&smtp_hello_verify_ptr, 0 },
+    { "smtp_host_dnsbl_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_host_dnsbl_domains, 0 },
+    { "smtp_host_dnsbl_except", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_host_dnsbl_except, 0 },
+    { "smtp_host_reject_hostnames", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_host_reject_hostnames, 0 },
+    { "smtp_local_sender_allow", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_local_sender_allow, 0 },
+    { "smtp_local_sender_restrict", t_boolean, 0, FALSE, (tup *)&smtp_local_sender_restrict, 0 },
+    { "smtp_max_recipients", t_int, 0, 0, (tup *)&smtp_max_recipients, 0 },
+    { "smtp_max_bounce_recipients", t_int, 0, 0, (tup *)&smtp_max_bounce_recipients, 0 },
+    { "smtp_permit_mx_backup", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_permit_mx_backup, 0},
+    { "smtp_rbl_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_rbl_domains, 0 },
+    { "smtp_rbl_except", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_rbl_except, 0 },
+    { "smtp_receive_command_timeout", t_interval, 0, 0, (tup *)&smtp_receive_command_timeout, 0 },
+    { "smtp_receive_message_timeout", t_interval, 0, 0, (tup *)&smtp_receive_message_timeout, 0 },
+    { "smtp_recipient_no_verify", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_recipient_no_verify, 0 },
+    { "smtp_reject_hosts", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_reject_hosts, 0 },
+    { "smtp_hello_reject_hosts", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_reject_hosts, 0 }, /* deprecated name (in 3.2.0.112 - 3.2.0.114) */
+    { "smtp_remote_allow", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_remote_allow, 0 },
+    { "smtp_sender_no_verify", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_sender_no_verify, 0 },
+    { "smtp_sender_reject", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_sender_reject, 0 },
+    { "smtp_sender_reject_db", t_string, 0, NULL, (tup *)&smtp_sender_reject_db, 0 },
+    { "smtp_sender_reject_db_proto", t_string, 0, NULL, (tup *)&smtp_sender_reject_db_proto, 0 },
+    { "smtp_sender_reject_hostnames", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_sender_reject_hostnames, 0 },
+    { "smtp_sender_rhsbl_domains", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_sender_rhsbl_domains, 0 },
+    { "smtp_sender_rhsbl_except", t_string, FA_FMT_IS_LIST, NULL, (tup *)&smtp_sender_rhsbl_except, 0 },
+    { "smtp_sender_verify_mx_only", t_boolean, 0, FALSE, (tup *)&smtp_sender_verify_mx_only, 0 },
+    { "smtp_vrfy_delay", t_int, 0, 0, (tup *)&smtp_vrfy_delay, 0 },
+    { "spool_dirs", t_string, FA_FMT_IS_LIST, NULL, (tup *)&spool_dirs, 0 },
+    { "spool_dir", t_string, 0, NULL, (tup *)&spool_dir, 0 }, /* set at runtime from spool_dirs */
+    { "spool_grade", t_char, 0, '\0', (tup *)&spool_grade, 0 },
+    { "spool_mode", t_mode, 0, 0, (tup *)&spool_mode, 0 },
+    { "switch_percent_and_bang", t_boolean, 0, FALSE, (tup *)&switch_percent_and_bang, 0 },
+    { "transport_file", t_string, 0, NULL, (tup *)&transport_file, 0 },
+    { "trusted_users", t_string, 0, NULL, (tup *)&trusted, 0 },
+    { "trusted_user", t_string, 0, NULL, (tup *)&trusted, 0 }, /* dup */
+    { "trusted", t_string, 0, NULL, (tup *)&trusted, 0 }, /* deprecated */
+    { "trusted_groups", t_string, 0, NULL, (tup *)&trusted_groups, 0 },
+    { "trusted_group", t_string, 0, NULL, (tup *)&trusted_groups, 0 }, /* dup */
+    { "uucp_name", t_string, 0, NULL, (tup *)&uucp_name, 0 },
+    { "version", t_infoproc, 0, NULL, (tup *)version, 0 },
+    { "visible_name", t_string, 0, NULL, (tup *)&visible_name, 0 },
+#ifdef FIXME
+    { NULL, 0, 0, NULL, NULL, 0 }
+#endif
 };
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+/* point to the end of the configuration attributes table */
 struct attr_table *end_conf_attributes = ENDTABLE(conf_attributes);
 
-/* Flag values used by format_attribute */
-#define FA_USE_PREFIX	1
-#define FA_USE_VALUE	2
-#define FA_IS_COMMENT	4
-
 
 
 /*
@@ -274,22 +323,24 @@ read_config_file(fn)
 	if (attr == NULL) {
 	    return xprintf("%s: parse error: %s", fn, error);
 	}
-	conf_attr = find_attribute(attr->name, conf_attributes,
-				   end_conf_attributes);
+	conf_attr = find_config_attribute(attr->name);
 	if (conf_attr == NULL) {
 	    return xprintf("%s: unknown variable: %s",
 			   fn, attr->name);
 	}
 	conf_attr->value = attr->value;
 
+#if 0 /* XXX this code is mostly duplicated in fill_attributes() */
+	check_type(conf_add->type, value, (char **) &(sp + conf_attr->offset), "variable");
+#else
 	if (conf_attr->type == t_boolean) {
-	    /* make sure boolean types are given as booleans */
-	    if (attr->value != on && attr->value != off) {
-		return xprintf("%s: boolean variable `%s' has non-boolean form",
-			       fn, attr->name);
+	    /* make sure boolean types are given only boolean values */
+	    if (!EQ(attr->value, on) && !EQ(attr->value, off)) {
+		return xprintf("%s: boolean variable `%s' has non-boolean form of '%s'",
+			       fn, attr->name, attr->value);
 	    }
 	} else {
-	    /* make sure non-boolean types aren't given as booleans */
+	    /* make sure non-boolean types aren't given with '+'/'-' prefixes */
 	    if ((attr->value == on &&
 		 conf_attr->type != t_proc) ||
 		(attr->value == off &&
@@ -305,13 +356,14 @@ read_config_file(fn)
 			       fn, attr->name);
 	    }
 	}
+#endif /* !0: duplicated code */
 
 #if 0 /* XXX this code is mostly duplicated in fill_attributes() */
 	convert_value(conf_attr->type, value, (char **) conf_attr->uptr, "variable");
 #else
 	switch (conf_attr->type) {
 	case t_string:
-	    if (attr->value == off) {
+	    if (attr->value == off) {		/* special case, '-' prefix */
 		conf_attr->uptr->v_string = NULL;
 	    } else {
 		conf_attr->uptr->v_string = attr->value;
@@ -392,22 +444,31 @@ read_config_file(fn)
 	default:
 	    return xprintf("%s: variable %s: unknown variable type", fn, attr->name);
 	}
-#endif
+#endif /* !0: duplicated code */
     }
 
     return NULL;
 }
 
 
-
+/*
+ * quote_string_value - format a string as a quoted string
+ *
+ * unprintable chars and whitespace are convered to C-style escapes
+ *
+ * Lists of colon-separated items can be displayed with one item per line using
+ * backslash-escaped continuation newlines.
+ */
 char *
-quote_string_value(str)
+quote_string_value(str, is_list)
      char * str;
+     int is_list;
 {
     static struct str val;
     static int initialised = FALSE;
     char s;
     char buf[5];		/* must hold '\\', '2', '3', '4', '\0' */
+    int in_curly = 0;
 
     if (!initialised) {
 	STR_INIT(&val);
@@ -420,44 +481,60 @@ quote_string_value(str)
     STR_NEXT(&val, '\"');
     while ((s = *str++)) {
 	switch (s) {
-	  case '\t':
+	case '\t':
 	    STR_NEXT(&val, '\\');
 	    STR_NEXT(&val, 't');
 	    break;
-	  case '\r':
+	case '\r':
 	    STR_NEXT(&val, '\\');
 	    STR_NEXT(&val, 'r');
 	    break;
-	  case '\n':
+	case '\n':
 	    STR_NEXT(&val, '\\');
 	    STR_NEXT(&val, 'n');
 	    break;
-	  case '\\':
+	case '\\':
 	    STR_NEXT(&val, '\\');
 	    STR_NEXT(&val, '\\');
 	    break;
-	  case '\"':
+	case '\"':
 	    STR_NEXT(&val, '\\');
 	    STR_NEXT(&val, '\"');
 	    break;
-	  case ' ':
+	case ' ':
 	    STR_NEXT(&val, s);
 	    break;
-	  default:
-	    if (iscntrl(s)) {
+	case ':':
+	    if (is_list && !in_curly && (*str != ':')) {
+		STR_NEXT(&val, '\\');
+		STR_NEXT(&val, '\n');
+	    }
+	    STR_NEXT(&val, ':');
+	    break;
+	case '{':
+	    in_curly++;
+	    STR_NEXT(&val, s);
+	    break;
+	case '}':
+	    in_curly--;
+	    STR_NEXT(&val, s);
+	    break;
+	default:
+	    if (iscntrl((int) s)) {
 		sprintf(buf, "\\%03o", (int) s);
 		STR_CAT(&val, buf);
 	    } else {
 		STR_NEXT(&val, s);
 	    }
+	    break;
 	}
     }
     STR_CAT(&val, "\"");	/* STR_CAT used here since it terminates the string */
+
     return STR(&val);
 }
 
 
-
 /*
  * format_attribute - given a attribute, return a formatted value
  *
@@ -475,8 +552,7 @@ quote_string_value(str)
  *
  * Return value is bit mask which items are valid
  */
-
-static int
+int
 format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
      struct attr_table * conf_attr;	/* The attriute info */
      char * name;			/* The conf variable name */
@@ -492,10 +568,12 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
     int ret;				/* return mask */
     long lval;				/* int/long conversion var */
 
-    /* NB This routine depends on the minimum length of val (which should
-     * be STR_BUMP) being reasonable - ie greater than 20, and not being
-     * shrunk.  If someone tampers behind the scenes expect the whole lot
-     * to fall down :-(
+    /*
+     * NB:  This routine depends on the minimum length of val (which should be
+     * STR_BUMP) being reasonable - i.e. greater than 20, and not being shrunk,
+     * because it uses sprintf() to insert certain types of values directly
+     * into the allocated storage for val.  If someone tampers behind the
+     * scenes expect the whole lot to fall down :-(
      */
     if (!initialised) {
 	STR_INIT(&val);
@@ -511,8 +589,9 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
     ret = 0;
 
     if (conf_attr == NULL) {
-	if (errfile)
+	if (errfile) {
 	    (void) fprintf(errfile, "%s: unknown attribute: %s\n", program, name);
+	}
 	return ret;
     }
 
@@ -523,55 +602,52 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
     }
 
     switch (conf_attr->type) {
-      case t_string:
-	{
-	    if (vptr->v_string) {
-		*value = quoted ? quote_string_value(vptr->v_string) : vptr->v_string;
-		ret = FA_USE_VALUE;
-	    } else {
-		*prefix = "-";
-		ret = FA_USE_PREFIX;
-	    }
-	}
-	break;
-
-      case t_boolean:
-	{
-	    int v2;
-	    /* Booleans are handled a bit differently in the drivers */
-	    if (conf_attr->offset != 0) {
-		/* If offset is non-zero, then use it as a mask on flags */
-		v2 = flags & conf_attr->offset;
-	    } else {
-		v2 = vptr->v_boolean;
-	    }
-
-	    if (v2) {
-		*value = "true";
-		*prefix = "+";
-	    } else {
-		*value = "false";
-		*prefix = "-";
-	    }
+    case t_string:
+	if (vptr->v_string) {
+	    *value = quoted ? quote_string_value(vptr->v_string, conf_attr->fmt_hints & FA_FMT_IS_LIST) : vptr->v_string;
+	    ret = FA_USE_VALUE;
+	} else {
+	    *prefix = "-";
 	    ret = FA_USE_PREFIX;
 	}
 	break;
 
-      case t_char:
+    case t_boolean: {
+	int v2;
+	/* Booleans are handled a bit differently in the drivers */
+	if (conf_attr->offset != 0) {
+	    /* If offset is non-zero, then use it as a mask on flags */
+	    v2 = flags & conf_attr->offset;
+	} else {
+	    v2 = vptr->v_boolean;
+	}
+
+	if (v2) {
+	    *value = "true";
+	    *prefix = "+";
+	} else {
+	    *value = "false";
+	    *prefix = "-";
+	}
+	ret = FA_USE_PREFIX;
+	break;
+    }
+
+    case t_char:
 	STR_NEXT(&val, vptr->v_char);
 	STR_NEXT(&val, 0);
 	*value = STR(&val);
 	ret = FA_USE_VALUE;
 	break;
-
-      case t_int:
-      case t_long:
+	
+    case t_int:
+    case t_long:
 	if (conf_attr->type == t_int) {
 	    lval = vptr->v_int;
 	} else {
 	    lval = vptr->v_long;
 	}
-
+	
 	if (lval == 0) {
 	    *prefix = "-";
 	    ret = FA_USE_PREFIX;
@@ -588,7 +664,7 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
 	}
 	break;
 
-      case t_mode:
+    case t_mode:
 	if (vptr->v_int == 0) {
 	    *prefix = "-";
 	    ret = FA_USE_PREFIX;
@@ -599,7 +675,7 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
 	}
 	break;
 
-      case t_interval:
+    case t_interval:
 	if (vptr->v_long == 0) {
 	    *prefix = "-";
 	    ret = FA_USE_PREFIX;
@@ -612,37 +688,36 @@ format_attribute(conf_attr, name, quoted, prefix, value, ptr, flags)
 	}
 	break;
 
-      case t_double:
+    case t_double:
 	(void) sprintf(STR(&val), "%g", vptr->v_double);
 	*value = STR(&val);
 	ret = FA_USE_VALUE;
 	break;
 
-      case t_proc:
+    case t_proc:
 	if (errfile) {
 	    (void) fprintf(errfile, "%s: %s: cannot print proc attributes\n",
 			   program, name);
 	}
 	break;
 
-      case t_infoproc:
-	*value = quote_string_value((*((char *(*) __P((char *)))vptr))(name));
+    case t_infoproc:
+	*value = quote_string_value((*((char *(*) __P((char *)))vptr))(name), FALSE);
 	*prefix = "#";
 	ret = FA_USE_VALUE|FA_USE_PREFIX|FA_IS_COMMENT;
 	break;
 
-      default:
+    default:
 	if (errfile) {
 	    (void) fprintf(errfile, "%s: variable %s: unknown type\n", program, name);
 	}
 	break;
     }
-    return(ret);
 
+    return(ret);
 }
 
 
-
 /*
  * print_config_variable - write the value for a config variable on stdout
  */
@@ -694,35 +769,40 @@ print_config_variable(name)
 	dump_retry_config(stdout);
 	return;
     }
-
-    conf_attr = find_attribute(name, conf_attributes, end_conf_attributes);
-    ret = format_attribute(conf_attr, name, debug, &prefix, &value, (char *) NULL, 0L);
-
-    if (ret) {
-	if (debug) {
-	    printf("%s%s%s%s\n",
-		   ((ret & FA_USE_PREFIX) ? prefix : ""),
-		   name,
-		   ((ret & FA_USE_VALUE) ? "=" : ""),
-		   ((ret & FA_USE_VALUE) ? value : ""));
+    if (*name != '$') {
+	conf_attr = find_config_attribute(name);
+	if (conf_attr == NULL) {
+	    (void) printf("unknown configuration attribute: %s\n", name);
 	} else {
-	    (void) printf("%s\n", value ? value : "(null)");
+	    ret = format_attribute(conf_attr, name, debug, &prefix, &value, (char *) NULL, 0L);
+	    if (ret) {
+		if (debug) {
+		    printf("%s%s%s%s\n",
+			   ((ret & FA_USE_PREFIX) ? prefix : ""),
+			   name,
+			   ((ret & FA_USE_VALUE) ? "=" : ""),
+			   ((ret & FA_USE_VALUE) ? value : ""));
+		} else {
+		    (void) printf("%s\n", value ? value : "(null-value)");
+		}
+	    }
 	}
-	return;
-    }
-
-    if (debug) {
+    } else {
 	value = expand_string(name, (struct addr *) NULL, (char *) NULL, (char *) NULL);
 	if (value) {
-	    printf("%s = %s.\n", name, value);
+	    if (debug) {
+		printf("%s = %s\n", (name+1), value);
+	    } else {
+		printf("%s\n", value);
+	    }
 	    xfree(value);
 	}
     }
+    return;
 }
 
 
 
-
 /*
  * print_config_help - output a listing of all config attributes, with type
  */
@@ -732,6 +812,7 @@ print_config_help()
     register struct attr_table *t;
     register tup *last_uptr = NULL;
 
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     for (t = conf_attributes; t < end_conf_attributes; t++) {
 	register char *typename;
 
@@ -767,9 +848,10 @@ print_config_all()
     int save_debug = debug;
 
     debug = 1;
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     for (t = conf_attributes; t < end_conf_attributes; t++) {
 	if (t->uptr == last_uptr) {
-	    continue;
+	    continue;			/* skip dup/deprecated names */
 	}
 	last_uptr = t->uptr;
 	switch(t->type) {
@@ -834,6 +916,7 @@ print_config_all()
  *	structure list will be stored at the location pointed to by
  *	`head'.
  */
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 char *
 read_standard_file(f, template, struct_size,
 		   name_offset, flags_offset, succ_offset,
@@ -841,7 +924,7 @@ read_standard_file(f, template, struct_size,
 		   driv_function, head)
     FILE *f;				/* input file */
     register char *template;		/* template structure */
-    unsigned int struct_size;		/* size of template structure */
+    size_t struct_size;			/* size of template structure */
     int name_offset;			/* name field offset in structure */
     int flags_offset;			/* where to store boolean flags */
     int succ_offset;			/* where to store succ pointer */
@@ -890,6 +973,7 @@ read_standard_file(f, template, struct_size,
 	 * fill in the generic attributes, calling attr_function for
 	 * attributes not in attr_table
 	 */
+	/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 	error = fill_attributes(cur, generic_attrs,
 				(long *)(cur + flags_offset),
 				attr_table, end_attr_table);
@@ -917,6 +1001,7 @@ read_standard_file(f, template, struct_size,
  * The reverse of read_standard_file - this prints out something
  * like the original config file!
  */
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 void
 dump_standard_config(f, config, name, flags, attr_table, end_attr_table)
     FILE *f;				/* output file */
@@ -931,6 +1016,7 @@ dump_standard_config(f, config, name, flags, attr_table, end_attr_table)
     char * prefix;			/* returned prefix */
     char * value;			/* returned value */
 
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     for (attr = attr_table; (attr < end_attr_table); attr++) {
 	if (attr->type != t_proc) {
 	    ret = format_attribute(attr, name, 1, &prefix, &value, config, flags);
@@ -966,6 +1052,7 @@ dump_standard_config(f, config, name, flags, attr_table, end_attr_table)
  * output:
  *	returns either an error message, or NULL if everything went okay.
  */
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 char *
 fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
     char *sp;
@@ -982,20 +1069,24 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 	register char *value = attrs->value;
 
 	/* find the attribute in the list of known attributes */
+	/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
 	t = find_attribute(attrs->name, attr_table, end_attr_table);
 	if (t == NULL) {
 	    /* didn't find the attribute */
 	    return xprintf("%s: unknown attribute", attrs->name);
 	}
 
-	/* what do with this attribute? */
+#if 0 /* XXX this code is mostly duplicated in fill_attributes() */
+	check_type(t->type, value, (char **) &(sp + t->offset), "attribute");
+#else
 	if (t->type == t_boolean) {
-	    /* make sure boolean types are given as booleans */
-	    if (attrs->value != on && attrs->value != off) {
-		return xprintf("boolean attribute `%s' has non-boolean form",
-			       attrs->name);
+	    /* make sure boolean types are given boolean values */
+	    if (!EQ(attrs->value, on) && !EQ(attrs->value, off)) {
+		return xprintf("boolean attribute `%s' has non-boolean form of '%s'",
+			       attrs->name, attrs->value);
 	    }
 	} else {
+	    /* make sure non-boolean types aren't given with '+'/'-' prefixes */
 	    if ((attrs->value == on &&
 		 t->type != t_proc) ||
 		(attrs->value == off &&
@@ -1012,6 +1103,7 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 			       attrs->name);
 	    }
 	}
+#endif /* 0: duplicated code */
 
 	/* fill in the attribute */
 #if 0 /* XXX this code is mostly duplicated in read_config_file() */
@@ -1019,7 +1111,7 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 #else
 	switch (t->type) {
 	case t_string:
-	    if (value == off) {
+	    if (value == off) {			/* special case '-' prefix? */
 		*(char **)(sp + t->offset) = NULL;
 	    } else {
 		*(char **)(sp + t->offset) = value;
@@ -1027,7 +1119,7 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 	    break;
 
 	case t_boolean:
-	    if (value == on) {
+	    if (EQ(value, on)) {
 		*flags |= t->offset;
 	    } else {
 		*flags &= ~t->offset;
@@ -1071,7 +1163,7 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 		}
 		*(long *)(sp + t->offset) = val;
 	    }
-	    break;	/* FIXME: ???? was fallthru */
+	    break;
 
 	case t_double:
 	    {
@@ -1100,7 +1192,7 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 	    return xprintf("%s: unknown attribute type", attrs->name);
 
 	}
-#endif
+#endif /* 0: duplicated code */
     }
 
     return NULL;
@@ -1109,12 +1201,14 @@ fill_attributes(sp, attrs, flags, attr_table, end_attr_table)
 /*
  * find_attribute - find an attribute within an attribute table
  */
-struct attr_table *
+/* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+static struct attr_table *
 find_attribute(name, table, end)
     register char *name;		/* name to search for */
     register struct attr_table *table;	/* table to search in */
     register struct attr_table *end;	/* end of table */
 {
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     while (table < end) {
 	if (EQ(name, table->name)) {
 	    return table;
@@ -1124,6 +1218,14 @@ find_attribute(name, table, end)
     return NULL;
 }
 
+struct attr_table *
+find_config_attribute(name)
+    char *name;
+{
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
+    return find_attribute(name, conf_attributes, end_conf_attributes);
+}
+
 
 /*
  * add_config_stat - add a new config file to config_stat_list
@@ -1186,29 +1288,6 @@ is_newconf()
     return FALSE;
 }
 
-/*
- * make_lib_fn - build a filename relative to the smail lib directory
- *
- * If the filename begins with '/' or is "-" return it unmodified.
- * Return NULL if smail_lib_dir is undefined, or if the given
- *  filename is NULL.
- */
-char *
-make_lib_fn(fn)
-    char *fn;				/* filename to expand */
-{
-    if (fn == NULL) {
-	return NULL;
-    }
-    if (fn[0] == '/' || (fn[0] == '-' && fn[1] == '\0')) {
-	return COPY_STRING(fn);
-    }
-    if (smail_lib_dir == NULL) {
-	return NULL;
-    }
-    return xprintf("%s/%s", smail_lib_dir, fn);
-}
-
 
 #ifdef	STANDALONE
 
