@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:smailconf.h,v 1.9 2001/07/30 02:41:24 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:smailconf.h,v 1.14 2003/02/19 22:25:15 woods Exp"
  */
 
 /*
@@ -33,6 +33,7 @@ struct attr_table {
 	t_proc,				/* attribute handled by procedure */
 	t_infoproc			/* procedure which displays info */
     } type;
+    int fmt_hints;			/* flags for formatting hints */
     char *value;			/* value to put in config file */
     union u_attr {
 	char *v_string;			/* string variable */
@@ -50,14 +51,22 @@ struct attr_table {
 /* convenience typedef for use in initializing conf_form tables */
 typedef union u_attr tup;
 
+/* Flag values used for fmt_hints */
+#define FA_FMT_IS_LIST	0x0001		/* show as list (escaped newline before colons) */
+
+/* Flag values used by format_attribute */
+#define FA_USE_PREFIX	0x0001		/* show prefix */
+#define FA_USE_VALUE	0x0002		/* show value */
+#define FA_IS_COMMENT	0x0004		/* show result as a comment */
+
 /* external functions defined in smailconf.c */
 extern char *read_config_file __P((char *));
-extern char *quote_string_value __P((char *));
+extern char *quote_string_value __P((char *, int));
+extern int format_attribute __P((struct attr_table *, char *, int, char **, char **, char *, long));
 extern void print_config_variable __P((char *));
-extern char *read_standard_file __P((FILE *, char *, unsigned int, int, int, int, struct attr_table *, struct attr_table *, char *(*)(char *, struct attribute *), char **));
+extern char *read_standard_file __P((FILE *, char *, size_t, int, int, int, struct attr_table *, struct attr_table *, char *(*)(char *, struct attribute *), char **));
 extern void dump_standard_config __P((FILE *, char *, char *, long, struct attr_table *, struct attr_table *));
 extern char *fill_attributes __P((char *, struct attribute *, long *, struct attr_table *, struct attr_table *));
-extern struct attr_table *find_attribute __P((char *, struct attr_table *, struct attr_table *));
+extern struct attr_table *find_config_attribute __P((char *));
 extern void add_config_stat __P((char *, struct stat *));
 extern int is_newconf __P((void));
-extern char *make_lib_fn __P((char *));
