@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:spool.c,v 1.38 2001/08/03 13:38:30 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:spool.c,v 1.55 2003/06/10 00:24:18 woods Exp"
  */
 
 /*
@@ -29,32 +29,46 @@
  *			     log_spool_errors, new_grade, freeze_message,
  *			     message_date
  */
-#include <stdio.h>
-#include <ctype.h>
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#include <stdio.h>
+#include <ctype.h>
 #include <errno.h>
 #include <signal.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
-#endif
-#include "smail.h"
-#include "main.h"
-#include "parse.h"
-#include "addr.h"
-#include "spool.h"
-#include "direct.h"
-#include "route.h"
-#include "transport.h"
-#include "log.h"
-#include "alloc.h"
-#include "dys.h"
-#include "exitcodes.h"
-#include "debug.h"
-#include "extern.h"
+#include <limits.h>
 
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(UNIX_SYS5_4)
+# include <sys/sysmacros.h>			/* for MIN() & MAX() */
+#endif
 #ifdef HAVE_STATVFS
 # include <sys/statvfs.h>			/* SysVr4 & SUSv2 */
 #else /* assume HAVE_STATFS */
@@ -69,10 +83,15 @@
 # endif
 #endif
 
-#if defined(POSIX_OS) || !defined(UNIX_BSD)
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# include <sys/time.h>
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
 
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
@@ -82,14 +101,41 @@
 #  include <sys/file.h>
 # endif
 #endif
+#ifdef __GLIBC__
+# include <sys/file.h>
+#endif
+#ifdef HAVE_SYS_LOCKING_H
+# include <sys/locking.h>
+#endif
 
-#if defined(POSIX_OS)
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
+#endif
+
+#include "smail.h"
+#include "smailarch.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
+#include "spool.h"
+#include "direct.h"
+#include "route.h"
+#include "log.h"
+#include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
+#include "dys.h"
+#include "transport.h"
+#include "exitcodes.h"
+#include "debug.h"
+#include "extern.h"
+#include "smailport.h"
+
+#if !defined(PATH_MAX) && defined(_POSIX_PATH_MAX)
+# define PATH_MAX	_POSIX_PATH_MAX
+#elif !defined(PATH_MAX)
+# define PATH_MAX	255		/* maybe conservative, but safe. */
+#endif
 
 #ifdef STANDALONE
 # define xmalloc malloc
@@ -110,7 +156,7 @@ char *msg_ptr;				/* read placeholder in msg_buf */
 char *msg_max;				/* last valid char in msg_buf */
 char *end_msg_buf;			/* end of msg_buf */
 off_t msg_foffset;			/* file offset for msg_buf contents */
-off_t msg_size;				/* size of spool file */
+unsigned long msg_size;			/* size of spool file */
 
 /* types local to this file */
 struct log_msgs {
@@ -140,28 +186,6 @@ static void failed_write __P((void));
 static int copy_old_spool __P((void));
 static void log_message __P((char *));
 
-/* trap undefined open flags */
-#ifndef	O_RDONLY
-# define O_RDONLY	0
-#endif
-#ifndef O_WRONLY
-# define O_WRONLY	1
-#endif
-#ifndef O_RDWR
-# define O_RDWR		2
-#endif
-
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
-
 
 /*
  * creat_spool - create a spool file
@@ -239,7 +263,7 @@ int_creat_spool()
     spool_dir = NULL;
 
     /* get the first "alternate" directory, which is the primary directory */
-    if (set_alt_spool((char *)NULL) == FAIL) {
+    if (set_alt_spool((char *) NULL) == FAIL) {
 	return FAIL;
     }
 
@@ -260,36 +284,36 @@ int_creat_spool()
 	 * to the user.
 	 */
 #ifdef O_EXCL
-#if defined(O_SYNC) && !defined(HAVE_FSYNC) && !defined(NO_SPOOL_O_SYNC)
+# if defined(O_SYNC) && !defined(HAVE_FSYNC) && !defined(NO_SPOOL_O_SYNC)
 	spoolfile = open(temp_fn, O_RDWR|O_CREAT|O_EXCL|O_SYNC, spool_mode);
-#else
+# else
 	spoolfile = open(temp_fn, O_RDWR|O_CREAT|O_EXCL, spool_mode);
-#endif	/* O_SYNC && !HAVE_FSYNC && !NO_SPOOL_O_SYNC */
-#else	/* O_EXCL */
+# endif /* O_SYNC && !HAVE_FSYNC && !NO_SPOOL_O_SYNC */
+#else /* O_EXCL */
 	/*
 	 * if we must use creat, then the mode should not allow writing.
 	 * As well, we will have to reopen for reading later on in
 	 * read_spool, if we try to read on the file without closing
 	 * and then opening again.
 	 */
-	spoolfile = creat(temp_fn, spool_mode&(~0222));
+	spoolfile = creat(temp_fn, spool_mode & (~0222));
 	if (spoolfile < 0 && errno == EACCES) {
 	    /* we assume that EACCES does not represent a directory error */
 	    errno = EEXIST;
 	}
 #endif	/* O_EXCL */
 	if (spoolfile < 0 && errno == ENOENT) {
-	    /* NOENT means the directory did not exist, try to make it */
+	    /* ENOENT means the directory did not exist, try to make it */
 	    DEBUG1(DBG_SPOOL_LO, "make directory %s/input\n", spool_dir);
 	    (void) mkdir("input", auto_mkdir_mode);
 #ifdef O_EXCL
-#if defined(O_SYNC) && !defined(HAVE_FSYNC) && !defined(NO_SPOOL_O_SYNC)
+# if defined(O_SYNC) && !defined(HAVE_FSYNC) && !defined(NO_SPOOL_O_SYNC)
 	    spoolfile = open(temp_fn, O_RDWR|O_CREAT|O_EXCL|O_SYNC, spool_mode);
-#else
+# else
 	    spoolfile = open(temp_fn, O_RDWR|O_CREAT|O_EXCL, spool_mode);
-#endif	/* O_SYNC && !HAVE_FSYNC && !NO_SPOOL_O_SYNC */
-#else
-	    spoolfile = creat(temp_fn, spool_mode&(~0222));
+# endif /* O_SYNC && !HAVE_FSYNC && !NO_SPOOL_O_SYNC */
+#else /* O_EXCL */
+	    spoolfile = creat(temp_fn, spool_mode & (~0222));
 	    if (spoolfile < 0 && errno == EACCES) {
 		errno = EEXIST;
 	    }
@@ -301,6 +325,11 @@ int_creat_spool()
 		DEBUG2(DBG_SPOOL_MID,
 		       "int_creat_spool: %s/input/%s exists, unlinking\n",
 		       spool_dir, temp_fn);
+		if (unlink(temp_fn) < 0) {
+		    DEBUG3(DBG_SPOOL_MID,
+			   "int_creat_spool: unlink(%s/input/%s): %s\n",
+			   spool_dir, temp_fn, strerror(errno));
+		}
 		attempts++;		/* only attempt unlink once */
 		continue;
 	    }
@@ -314,8 +343,10 @@ int_creat_spool()
 	}
 
 	/*
-	 * we have an open temp file.  Now create a lock file
-	 * and move the temp file to its more permanent name.
+	 * we have an open temp file.  Now create a lock file (to prevent other
+	 * smail processes from trying to access this spool file while the
+	 * message is still being spooled) and move the temp file to its more
+	 * permanent name.
 	 */
 	build_spool_fn(spool_grade);
 #ifndef lock_fd
@@ -344,19 +375,33 @@ int_creat_spool()
 	}
 #endif /* lock_fd */
 	if (rename(temp_fn, input_spool_fn) < 0) {
-	    /* why would rename fail? */
+	    int oerrno = errno;
+
+	    /*
+	     * XXX this is actually quite bad!  It would suggest the postmaster
+	     * has copied the spool directory causing the old queue names to
+	     * become invalid as they now no longer match their inode numbers!
+	     */
 	    if (lock_by_name) {
 		(void) unlink(lock_fn);
 	    }
 	    (void) close(spoolfile);
-	    (void) unlink(temp_fn);
-	    if (set_alt_spool("failed to rename temp file") == FAIL) {
+	    if (oerrno != EEXIST) {	/* otherwise keep temp_fn as a placeholder */
+		(void) unlink(temp_fn);
+	    }
+	    if (set_alt_spool("failed to rename temp file to spool file") == FAIL) {
 		return FAIL;
 	    }
 	    continue;
 	}
+	/*
+	 * XXX we should probably check to be sure there's also no "error" file
+	 * of the same name, and then if so then:
+	 *
+	 * 	panic("You evil postmaster!  You seem to have copied your spool!")
+	 */
 
-	/* we have a valid locked spool file */
+	/* we have a valid, locked, spool file */
 	return SUCCEED;
     }
 }
@@ -386,7 +431,7 @@ build_spool_fn(grade_char)
 
     /* get the inode number in base 62 information */
     (void) fstat(spoolfile, &statbuf);
-    (void) strcpy(a_inode, base62((unsigned long)statbuf.st_ino));
+    (void) strcpy(a_inode, base62((unsigned long) statbuf.st_ino));
 
     /* this is 14 chars which should work on all UN*X systems */
     spool_fn = fn;
@@ -654,8 +699,8 @@ static int
 lock_creat()
 {
     int lfd;
-    char apid[16];			/* holds the pid as decimal# + \0 */
-    int ct;
+    char apid[MAXLONG_B10_DIGITS + 1];	/* holds the pid as decimal# + \0 */
+    long ct;
 
 #ifdef O_EXCL
     lfd = open(lock_fn, O_RDWR|O_CREAT|O_EXCL, lock_mode);
@@ -672,9 +717,9 @@ lock_creat()
 	/* we failed to creat the lock the file */
 	return FAIL;
     }
-    (void) sprintf(apid, "%lu\n", (long unsigned int) getpid());
-    ct = strlen(apid);
-    if (write(lfd, apid, (unsigned int) ct) < ct) {
+    (void) sprintf(apid, "%lu\n", (unsigned long int) getpid());
+    ct = (long) strlen(apid);
+    if (write(lfd, apid, (size_t) ct) < ct) {
 	/* we failed to write the pid into the lock file, quit */
 	(void) unlink(lock_fn);
 	(void) close(lfd);
@@ -868,7 +913,7 @@ write_spool()
     }
     msg_size += msg_max - msg_buf;
 
-    if (accepted_msg_size >= 0 && msg_size >= accepted_msg_size) {
+    if (accepted_msg_size >= 0 && msg_size >= (unsigned long) accepted_msg_size) {
 	DEBUG1(DBG_SPOOL_LO,
 	      "write_spool: message grew bigger than allowed [%ld]\n", accepted_msg_size);
 	return FAIL;
@@ -926,7 +971,7 @@ copy_old_spool()
      */
     (void) lseek(old_spoolfile, (off_t) 0L, SEEK_SET);
     while (copied < msg_size) {
-	register int ct;
+	register long ct;
 
 	/* fill up as much of the temp buffer as reasaonble */
 	if (msg_size - copied < message_bufsiz) {
@@ -935,7 +980,7 @@ copy_old_spool()
 	    ct = message_bufsiz;
 	}
 
-	ct = read(old_spoolfile, tempbuf, (unsigned int) ct);
+	ct = read(old_spoolfile, tempbuf, (size_t) ct);
 	if (ct <= 0) {
 	    failed_write();
 	    log_message("read failed in copying to new file");
@@ -943,7 +988,7 @@ copy_old_spool()
 	    (void) close(old_spoolfile);
 	    return READ_FAIL;		/* read error - can't recover */
 	} else {
-	    if (write(spoolfile, tempbuf, (unsigned int) ct) < ct) {
+	    if (write(spoolfile, tempbuf, (size_t) ct) < ct) {
 		failed_write();
 		return WRITE_FAIL;	/* write error - maybe can recover */
 	    }
@@ -1187,31 +1232,26 @@ freeze_message()
 {
     char error_fn[sizeof("error/") + SPOOL_FN_LEN];
     int success;
-    char *save_input_spool_fn = input_spool_fn;
-    char *save_spool_fn = spool_fn;
 
     (void) sprintf(error_fn, "error/%s", spool_fn);
-    spool_fn = NULL;
-    success = link(save_input_spool_fn, error_fn);
+    success = rename(input_spool_fn, error_fn);
     if (success < 0 && auto_mkdir && errno == ENOENT) {
-	DEBUG1(DBG_SPOOL_LO, "make directory %s/error\n", spool_dir);
-	(void) mkdir("error", auto_mkdir_mode);
-	success = link(save_input_spool_fn, error_fn);
+	DEBUG1(DBG_SPOOL_LO, "freeze_message(): make directory %s/error\n", spool_dir);
+	if (mkdir("error", auto_mkdir_mode) < 0) {
+	    DEBUG1(DBG_SPOOL_LO, "freeze_message(): mkdir(error): %s\n", strerror(errno));
+	}
+	success = rename(input_spool_fn, error_fn);
     }
     if (success < 0) {
 	write_log(WRITE_LOG_PANIC|WRITE_LOG_SYS,
 		  "freeze_message: error linking %s/%s to %s/%s: %s",
-		  spool_dir, save_input_spool_fn, spool_dir, error_fn,
+		  spool_dir, input_spool_fn, spool_dir, error_fn,
 		  strerror(errno));
     } else {
-	if (errfile) {
-	    /* XXX for some reason this appears in the paniclog too! */
-	    write_log(WRITE_LOG_TTY|WRITE_LOG_SYS, "mail moved to %s/%s",
-		      spool_dir, error_fn);
-	}
-	input_spool_fn = save_input_spool_fn;
-	spool_fn = save_spool_fn;
-	unlink_spool();
+	/* XXX for some reason this appears in the paniclog too! */
+	write_log(WRITE_LOG_TTY|WRITE_LOG_SYS, "mail moved to %s/%s",
+		  spool_dir, error_fn);
+	unlink_spool();				/* just in case.... */
     }
 }
 
@@ -1242,7 +1282,7 @@ seek_spool(offset)
     }
 
     /* compute the beginning of the block containing the offset */
-    msg_foffset = offset - offset%message_bufsiz;
+    msg_foffset = offset - (offset % message_bufsiz);
 
     /* actually read in the message */
     (void) lseek(spoolfile, msg_foffset, SEEK_SET);
@@ -1291,9 +1331,9 @@ send_spool(f, flags)
     register char *p;
     int eof = FALSE;			/* TRUE if found end of file */
     register int last_c = '\n';
-    long dot = flags & PUT_DOTS;
-    long crlf = flags & PUT_CRLF;
-    long uucp_hack = flags & UNIX_FROM_HACK;
+    int dot = ((flags & PUT_DOTS) != 0);
+    int crlf = ((flags & PUT_CRLF) != 0);
+    int uucp_hack = ((flags & UNIX_FROM_HACK) != 0);
 
     DEBUG(DBG_SPOOL_HI, "send_spool called\n");
     while (!eof) {
@@ -1303,21 +1343,21 @@ send_spool(f, flags)
 		    putc('.', f);		/* hidden dot algorithm */
 		}
 		if (uucp_hack && *p == 'F') {
-		    int ct = msg_max - p;
+		    size_t ct = msg_max - p;
 
 		    /* look ahead to see if this line begins with From */
 		    if (ct > sizeof("From ") - 1) {
 			if (strncmp("From ", p, sizeof("From ") - 1) == 0) {
 			    putc('>', f);
 			}
-		    } else if (strncmp("From ", p, (unsigned int) ct) == 0) {
+		    } else if (strncmp("From ", p, ct) == 0) {
 			/*
 			 * we have to look ahead to the next block to
 			 * determine if this is a From, but make sure
 			 * there is another block, first.
 			 */
 			if (msg_foffset + (msg_max - msg_buf) < msg_size) {
-			    int i;
+			    unsigned int i;
 
 			    msg_ptr = p;
 			    if (read_spool() == FAIL) {
@@ -1379,7 +1419,7 @@ send_spool(f, flags)
 int
 read_spool()
 {
-    register int ct;
+    register long ct;
 
     /* advance the current file offset by the amount previously read */
     msg_foffset += msg_max - msg_buf;
@@ -1406,7 +1446,7 @@ read_spool()
     (void) lseek(spoolfile, msg_foffset, SEEK_SET);
 
     /* read in a buffer */
-    ct = read(spoolfile, msg_buf, (unsigned int) ct);
+    ct = read(spoolfile, msg_buf, (size_t) ct);
     if (ct < 0) {
 	DEBUG1(DBG_SPOOL_LO, "read_spool: read of spool file failed: %s\n",
 	       strerror(errno));
@@ -1454,7 +1494,7 @@ log_message(m)
     char *m;				/* message */
 {
     char *p, *q;			/* temp */
-    register unsigned a;		/* how much to alloc */
+    register size_t a;		/* how much to alloc */
     char *save;
 
     a = sizeof(*log_msgs) + strlen(m);
@@ -1506,7 +1546,7 @@ static int
 set_alt_spool(m)
     char *m;				/* message to put in log */
 {
-    static char altname[SIZE_FILNAM];
+    static char altname[PATH_MAX];
     static char *altlist = NULL;
     char *p;
 
@@ -1526,14 +1566,21 @@ set_alt_spool(m)
     /* loop until we have a valid alternate spool directory, or none left */
     while (altlist && *altlist) {
 	/* copy a directory name into altname, may end in ':' or nul */
-	for (p = altname; *altlist && *altlist != ':'; *p++ = *altlist++) ;
+	for (p = altname; *altlist && *altlist != ':'; *p++ = *altlist++) {
+	    if ((size_t) (p - altname) >= (sizeof(altname) - 1)) {
+		log_message("config error: alt directory name too long");
+		break;
+	    }
+	}
 	if (*altlist == ':') {
 	    /* next try gets the next directory */
 	    altlist++;
+	} else if (*altlist) {
+	    continue;			/* altname over-run above... */
 	}
 	*p++ = '\0';			/* terminate directory name */
 
-	if (altname[0] != '/') {
+	if (*altname != '/') {
 	    /* directory must begin with '/', ignore it */
 	    log_message("config error: alt directory doesn't begin with /");
 	    continue;
@@ -1676,11 +1723,11 @@ message_date()
 	return clck;
     }
     for (p = message_id + 1; *p; p++) {
-	if (isdigit(*p)) {
+	if (isdigit((int) *p)) {
 	    digit = *p - '0';
-	} else if (isupper(*p)) {
+	} else if (isupper((int) *p)) {
 	    digit = *p - ('A' - 10);
-	} else if (islower(*p)) {
+	} else if (islower((int) *p)) {
 	    digit = *p - ('a' - 36);
 	} else {
 	    break;
@@ -1693,25 +1740,25 @@ message_date()
 #if defined(HAVE_STATFS) || defined(HAVE_STATVFS)
 
 /*
- * Return the maximum space free on any of spool_dirs, in KB
+ * Return the maximum space free on any of spool_dirs, in Kbytes (but never
+ * return more than (LONG_MAX / 1024) Kbytes, because it will be multiplied out
+ * to bytes eventually and stored in a long).
+ *
+ * If we can't figure it out for any reason just return (-1).
  *
  * Note we don't really care about the total space available on all spool dirs
  * because we're only going to use this to calculate the available space for a
  * single message.
- *
- * (using KB gives us support for filesystems up to 2^41 bytes (2 terabytes)
- * without having to get into system-dependent checks for 64-bit integer
- * support)
  */
 long
 spool_max_free_space()
 {
     char *dirs = spool_dirs;
-    char dirname[SIZE_FILNAM];
+    char dirname[PATH_MAX];
     char *p;
-    unsigned long max_free_kbytes = 0;
-    unsigned long free_kbytes;		/* free space in a given spool_dir */
-    unsigned long reserved;		/* free space to reserve (kb) */
+    long max_free_kbytes = -1;		/* max free space in any spoo_dir */
+    long free_kbytes;			/* free space in a given spool_dir */
+    long reserved_kbytes;		/* free space to reserve (kb) */
     unsigned long blocks_free;
     unsigned long block_size;
     int result;				/* stat*fs() return code */
@@ -1723,7 +1770,10 @@ spool_max_free_space()
 
     if (max_message_size > 0 && smtp_accept_max > 0) {
 	/* leave 2MB for logs, etc., plus max incoming messages */
-	reserved = (2 * 1024) + (smtp_accept_max * max_message_size);
+	reserved_kbytes = (2 * 1024) + (smtp_accept_max * max_message_size / 1024);
+	DEBUG3(DBG_SPOOL_MID,
+	       "spool_max_free_space: reserving %ld kbytes for [2MB plus %ld max message bytes * %d accepts]...\n",
+	       reserved_kbytes, max_message_size, smtp_accept_max);
     } else if (smtp_accept_max > 0) {
 	/*
 	 * just round out to a conservative 2MB plus 1MB per potential incoming
@@ -1731,34 +1781,57 @@ spool_max_free_space()
 	 * but to do that we'd have to add another function to first calculate
 	 * the total size of all the spool_dirs filesystems)
 	 */
-	reserved = (2 + smtp_accept_max) * 1024;
+	reserved_kbytes = (2 + smtp_accept_max) * 1024;
+	DEBUG2(DBG_SPOOL_LO,
+	       "spool_max_free_space: max_message_size == 0, defaulting to reserve of %ld kbytes for [2MB plus 1MB * %d accepts]\n",
+	       reserved_kbytes, smtp_accept_max);
     } else {
 	/*
 	 * just go for 10MB -- this is really lame (maybe we should warn when
 	 * smtp_accept_max isn't set to something > 0?)
 	 */
-	reserved = 10 * 1024;
+	reserved_kbytes = 10 * 1024;
+	DEBUG1(DBG_SPOOL_LO,
+	       "spool_max_free_space: max_message_size == 0 && smtp_accept_max == 0, defaulting to reserve of %ld kbytes\n",
+	       reserved_kbytes);
     }
     while (dirs && *dirs) {
 	for (p = dirname; *dirs && *dirs != ':'; *p++ = *dirs++) {
-	    ;
+	    if ((size_t) (p - dirname) >= (sizeof(dirname) - 1)) {
+		log_message("config error: directory name too long!");
+		break;
+	    }
 	}
 	if (*dirs == ':') {
 	    /* next try gets the next directory */
 	    dirs++;
+	} else if (*dirs) {
+	    continue;				/* dirname over-run above... */
 	}
 	*p++ = '\0';				/* terminate directory name */
 # ifdef HAVE_STATVFS
+#  define STATFS_FN	"statvfs"
 	result = statvfs(dirname, &buf);
 # else
 #  ifdef BSD					/* 4.4 only? */
+#  define STATFS_FN	"bsd statfs"
 	result = statfs(dirname, &buf);
 #  else
+#  define STATFS_FN	"sysv statfs"
 	/* AT&T UNIX SysVr3 and clones */
 	result = statfs(dirname, &buf, sizeof(buf), 0);
 #  endif
 # endif
 	if (result == -1) {
+	    /*
+	     * XXX should this be write_log(SYS &| PANIC)?
+	     *
+	     * NOTE: The first time ever this is called on a new system which
+	     * doesn't yet have the firt of spool_dirs created this will always
+	     * fail with ENOENT....
+	     */
+	    DEBUG3(DBG_SPOOL_LO, "spool_max_free_space: %s: %s() failed: %s\n",
+		   dirname, STATFS_FN, strerror(errno));
 	    continue;
 	}
 # ifdef HAVE_STATVFS
@@ -1773,22 +1846,46 @@ spool_max_free_space()
 	block_size= buf.f_bsize;
 #  endif
 # endif
+	DEBUG3(DBG_SPOOL_MID,
+	       "spool_max_free_space: %s: [bfree=%lu, bsize=%lu]\n",
+	       dirname, blocks_free, block_size);
 	/*
-	 * XXX we should probably panic if there's more than 2^41 bytes free!
+	 * try desparately to avoid overflowing ulong, and never return more
+	 * than (LONG_MAX / 1024) Kbytes (since it will be multiplied out to
+	 * bytes eventually)
+	 *
+	 * [Do I ever wish C had the option of integer overflow detection!]
 	 */
-	if (block_size >= 1024) {		/* avoid overflowing ulong */
-	    block_size = block_size / 1024;	/* just ignore any remainder */
-	    free_kbytes = block_size * blocks_free;
+	if (block_size == 0 || blocks_free == 0) {
+	    continue;				/* avoid divide by zero */
+	}
+	if (block_size >= 1024 || blocks_free >= 1024) {
+	    unsigned long little1 = MIN(block_size, blocks_free);
+	    unsigned long big1 = MAX(block_size, blocks_free);
+
+	    if ((little1 * (big1 / 1024)) / little1 != (big1 / 1024)) {
+		free_kbytes = LONG_MAX / 1024;	/* even Kbytes overflowed, just go for LONG_MAX */
+	    } else {
+	        free_kbytes = MIN((little1 * (big1 / 1024)), (LONG_MAX / 1024));
+	    }
 	} else {
-	    free_kbytes = (block_size * blocks_free) / 1024;
+	    free_kbytes = MIN(((block_size * blocks_free) / 1024), (LONG_MAX / 1024));
 	}
 	if (free_kbytes > max_free_kbytes) {
 	    max_free_kbytes = free_kbytes;
 	}
+	DEBUG3(DBG_SPOOL_MID,
+	       "spool_max_free_space: %s has %lu free kbytes [max is %ld]\n",
+	       dirname, free_kbytes, max_free_kbytes);
     }
-    max_free_kbytes -= reserved;
+    if (max_free_kbytes >= reserved_kbytes) {
+	max_free_kbytes -= reserved_kbytes;
+    } else if (max_free_kbytes >= 0) {
+	max_free_kbytes = 0;
+    }
+    DEBUG1(DBG_SPOOL_HI, "spool_max_free_space: returning %ld kbytes\n", max_free_kbytes);
 
-    return ((max_free_kbytes <= 0) ? 0 : max_free_kbytes);
+    return max_free_kbytes;
 }
 
 #else /* not HAVE_STAT*FS */
@@ -1938,8 +2035,8 @@ main(argc, argv)
 	if (seek_spool((off_t) 0L) != SUCCEED) {
 	    (void) fprintf(stderr, "seek_spool failed\n");
 	}
-	if (send_spool(stdout, TRUE, FALSE) != SUCCEED) {
-	    (void) fprintf(stderr, "seek_spool failed\n");
+	if (send_spool(stdout, (PUT_DOTS | UNIX_FROM_HACK)) != SUCCEED) {
+	    (void) fprintf(stderr, "send_spool failed\n");
 	}
     }
     if (dounspool) {
