@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:sysdep.c,v 1.82 2000/09/29 21:20:08 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:sysdep.c,v 1.107 2003/06/18 07:01:56 woods Exp"
  */
 
 /*
@@ -20,36 +20,59 @@
  *			    compute_hostname, compute_domain,
  *                          open_child, close_child,
  *			    close_all, scan_dir, fcntl_rd_lock, touch,
- *			    fsetsize, hstrerror
+ *			    fsetsize, set_sigchld_handler
  */
-#define NEED_SOCKETS
-#include <stdio.h>
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <signal.h>
+#include <sys/param.h>
+#include <stdio.h>
 #include <ctype.h>
-#include <errno.h>
 #include <pwd.h>
-#include "defs.h"
-#include "smail.h"
-#include "main.h"
-#include "parse.h"
-#include "addr.h"
-#include "child.h"
-#include "alloc.h"
-#include "dys.h"
-#include "log.h"
-#include "route.h"
-#include "bindlib.h"
-#include "exitcodes.h"
+#include <signal.h>
+#include <errno.h>
+#include <limits.h>
 
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
 
-#if	defined(UNIX_SYS5) || defined(INCLUDE_SYS_PARAM)
-# include <sys/param.h>
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
+# include <sys/wait.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
 
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
@@ -59,43 +82,27 @@
 #  include <sys/file.h>
 # endif
 #endif
-
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
+#ifdef __GLIBC__
+# include <sys/file.h>
 #endif
-#if defined(POSIX_OS)
-# include <limits.h>
+#ifdef HAVE_SYS_LOCKING_H
+# include <sys/locking.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_LIMITS_H)
-#  include <limits.h>
-# endif
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
-
-#ifndef STDERR_FILENO
-# define STDIN_FILENO	0	/* standard input file descriptor */
-#endif
-#ifndef STDOUT_FILENO
-# define STDOUT_FILENO	1	/* standard output file descriptor */
-#endif
-#ifndef STDERR_FILENO
-# define STDERR_FILENO	2	/* standard error file descriptor */
 #endif
 
-#ifdef	HAVE_UNAME
+#ifdef HAVE_UNAME
 # include <sys/utsname.h>
 #endif
-
-#if     defined(UNIX_BSD) && !defined(POSIX_OS)
+#if defined(UNIX_BSD) && !defined(POSIX_OS)
 # include <sys/dir.h>
 #else
-# if	defined(UNIX_XENIX) && !defined(POSIX_OS)
+# if defined(UNIX_XENIX) && !defined(POSIX_OS)
 #  include <sys/ndir.h>
 # else
-#  if	defined(HAVE_READDIR)
+#  if defined(HAVE_READDIR)
 #   include <dirent.h>
 #   define direct dirent
 #  else
@@ -104,12 +111,6 @@
 # endif	/* UNIX_XENIX && !POSIX_OS */
 #endif	/* UNIX_BSD && !POSIX_OS */
 
-#if (defined(POSIX_OS) || !defined(UNIX_BSD)) && !defined(ISC_SOCKET_TIME_BUG)
-# include <time.h>
-#else
-# include <sys/time.h>
-#endif
-
 #if defined(POSIX_OS) || defined(HAVE_UTIME_H)
 # include <utime.h>
 #else
@@ -130,33 +131,49 @@
 # include <vfork.h>
 #endif
 
-#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
-# include <sys/wait.h>
-#endif
+#include "smail.h"
+#include "smailarch.h"
+#include "smailsock.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
+#include "child.h"
+#include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
+#include "smailwait.h"
+#include "dys.h"
+#include "log.h"
+#include "route.h"
+#include "bindlib.h"
+#include "exitcodes.h"
+#include "extern.h"
+#include "debug.h"
+#include "smailport.h"
 
-#if (defined(UNIX_BSD) || defined(WAIT_USE_UNION)) && !defined(NO_WAIT_USE_UNION)
-# define STATUS_TYPE	union wait
-#else
-# define STATUS_TYPE	int
+/*
+ * double-check some assumptions....
+ *
+ * XXX can CPP deal with "the usual conversions"?
+ */
+#if (MAXINT_B10_DIGITS == MAXLONG_B10_DIGITS && UINT_MAX != ULONG_MAX)
+# include "ERROR:  ARCH_TYPE assumptions about MAX*_B10_DIGITS are wrong!"
 #endif
-
-/* XXX maybe these should each be wrapped with #ifndef ? */
-#if !defined(UNIX_BSD) && !defined(WAIT_USE_UNION) && !defined(POSIX_OS)
-# ifdef UNIX_AIX3				/* XXX is this true for non-rs6k? */
-#  define WIFSTOPPED(w)	((w) & 0100)		/* borrowed from GNU GDB */
-# else
-#  define WIFSTOPPED(w)	(((w) & 0377) == 0177)
+#if (__STDC__ - 0) > 0
+# if (MAXINT_B10_DIGITS <= 5 && UINT_MAX > 0xffffU)
+#  include "ERROR:  ARCH_TYPE (16bit?) assumptions about MAXINT_B10_DIGITS are wrong!"
+# endif
+# if (MAXINT_B10_DIGITS <= 10 && UINT_MAX > 0xffffffffU)
+#  include "ERROR:  ARCH_TYPE (32bit?) assumptions about MAXINT_B10_DIGITS are wrong!"
+# endif
+# if (MAXLONG_B10_DIGITS <= 10 && ULONG_MAX > 0xffffffffU)
+#  include "ERROR:  ARCH_TYPE assumptions about MAXLONG_B10_DIGITS are wrong!"
+# endif
+# ifdef HAVE_LONG_LONG
+#  if (MAXLONG_B10_DIGITS <= 20 && 0xffffffffffffffffULL)
+#   include "ERROR:  ARCH_TYPE assumptions about MAXLONG_B10_DIGITS are wrong!"
+#  endif
 # endif
-# define WIFSIGNALED(w)	(((w) & 0377) != 0177 && ((w) & ~0377) == 0)
-# define WIFEXITED(w)	(((w) & 0377) == 0)
-
-# define WEXITSTATUS(w)	(((w) >> 8) & 0377)
-# define WRETCODE(w)	WEXITSTATUS(w)
-# define WSTOPSIG(w)	(((w) >> 8) & 0377)
-# define WTERMSIG(w)	((w) & 0x177)
-#endif
-#if !defined(WCOREDUMP)
-#  define WCOREDUMP(w)	(((w) & 0200) != 0)	/* not required by POSIX.1 */
 #endif
 
 /*
@@ -218,43 +235,6 @@ extern struct tm *localtime();
 extern int errno;
 #endif
 
-/* define these, if they aren't already */
-#ifndef O_RDONLY
-# define O_RDONLY	0
-#endif
-#ifndef	O_WRONLY
-# define O_WRONLY	1
-#endif
-#ifndef O_RDWR
-# define O_RDWR		2
-#endif
-
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
-
-/* mode values for access(2) */
-#ifndef F_OK
-# define F_OK		0	/* does file exist */
-#endif
-#ifndef X_OK
-# define X_OK		1	/* is it executable by caller */
-#endif
-#ifndef W_OK
-# define W_OK		2	/* is it writable by caller */
-#endif
-#ifndef R_OK
-# define R_OK		4	/* is it readable by caller */
-#endif
-
-
 
 /*
  * time_stamp - return a time stamp string in the form:
@@ -345,7 +325,7 @@ get_time_zone(gtm, ltm)
     } else {
 	if (ltm->tm_yday != gtm->tm_yday) {
 	        tzoff +=  (ltm->tm_yday > gtm->tm_yday) ? 1440 : -1440;
-	    }
+	}
     }
 
     if (tzoff >= 0) {
@@ -423,8 +403,9 @@ compute_local_sender()
 {
     struct passwd *pw;
 
-    if (local_sender)
+    if (local_sender) {
 	return;
+    }
 
     islocal = TRUE;			/* only local sender determined here */
 
@@ -442,7 +423,7 @@ compute_local_sender()
     }
 
     if ((pw = getpwuid(real_uid)) != NULL) {
-	local_sender = xmalloc((unsigned)(strlen(pw->pw_name) + 1));
+	local_sender = xmalloc((size_t) (strlen(pw->pw_name) + 1));
 	(void) strcpy(local_sender, pw->pw_name);
 	return;
     }
@@ -485,12 +466,12 @@ fullname_from_gcos(gecos)
     char *cp;
 
     STR_INIT(sp);
-    if (isdigit(*gecos) && (cp = strchr(gecos, '-')) != NULL)
+    if (isdigit((int) *gecos) && (cp = strchr(gecos, '-')) != NULL)
 	gecos = cp + 1;			/* skip USG-style 0000-Name junk */
     while (*gecos && *gecos != ',' && *gecos != '(') {
 	/* name may end with a comma or USG-style (0000) junk */
 	if (*gecos == '&') {		/* & means copy sender, capitalized */
-	    STR_NEXT(sp, uppercase((unsigned char) sender[0]));
+	    STR_NEXT(sp, toupper((int) sender[0]));
 	    STR_CAT(sp, sender+1);
 	} else {
 	    STR_NEXT(sp, *gecos);
@@ -499,9 +480,36 @@ fullname_from_gcos(gecos)
     }
     STR_NEXT(sp, '\0');
     STR_DONE(sp);
-    sender_name = sp->p;
+    sender_name = STR(sp);
 }
 
+#ifdef HAVE_BSD_NETWORKING
+/*
+ * get_inet_addr - call inet_addr()
+ *
+ * believe it or not, but inet_addr() is not portable!!!!  :-(
+ */
+unsigned long
+get_inet_addr(ipaddr)
+    char *ipaddr;
+{
+    unsigned long inet;			/* internet address */
+# if defined(INET_ADDR_USE_STRUCT)
+    struct in_addr s_inet;		/* Ultrix(?) internet address */
+# endif
+
+# ifdef INET_ADDR_USE_STRUCT
+    s_inet = inet_addr(ipaddr);
+    inet = s_inet.s_addr;
+# else
+    inet = inet_addr(ipaddr);
+# endif
+
+    return inet;
+}
+#endif /* HAVE_BSD_NETWORKING */
+
+
 /*
  * fopen_as_user - call fopen(3s) within the context of a specific uid/gid
  *
@@ -514,7 +522,7 @@ fullname_from_gcos(gecos)
  *
  * NOTE:  the mode argument is only required if type is "a" or "w".
  */
-/* XXX FIXME: this doesn't belong in sysdep.c, does it? */
+/* XXX FIXME: this doesn't belong in sysdep.c, does it?  [yes, if we add the socketpair() file-descriptor passing version!] */
 FILE *
 fopen_as_user(fn, type, uid, gid, mode)
     char *fn;				/* name of file to open */
@@ -716,8 +724,12 @@ fopen_as_user(fn, type, uid, gid, mode)
 	    /* wait for the child process to die */
 	    while ((i = wait((STATUS_TYPE *) &status)) != pid) {
 		if (i < 0) {
+		    int oerrno = errno;
+
 		    DEBUG2(DBG_DRIVER_MID, "fopen_as_user(): test child wait() for %d: %s.\n",
 			   pid, strerror(errno));
+		    close(fd);
+		    errno = oerrno;
 		    return NULL;
 		}
 	    }
@@ -726,25 +738,31 @@ fopen_as_user(fn, type, uid, gid, mode)
 		    DEBUG1(WEXITSTATUS(status) - 1 ? DBG_DRIVER_LO : DBG_DRIVER_HI,
 			   "fopen_as_user(): test child exited with %d.\n",
 			   WEXITSTATUS(status) - 1);
+		    close(fd);
 		    errno = WEXITSTATUS(status) - 1;
 		    return NULL;
 		}
 	    } else if (WIFSIGNALED(status)) {
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] signalled %d.\n",
 		       pid, WTERMSIG(status));
+		close(fd);
 		errno = EINTR;
 		return NULL;
 	    } else if (WIFSTOPPED(status)) {
 		DEBUG2(DBG_DRIVER_LO, "fopen_as_user(): test child[%d] stopped %d.\n",
 		       pid, WSTOPSIG(status));
+		close(fd);
 		errno = EINTR;
 		return NULL;
 	    }
         }	
     }
     if (!(fp = fdopen(fd, type))) {
+	int oerrno = errno;
 	DEBUG3(DBG_DRIVER_LO, "fopen_as_user(): fdopen(%s, %s): %s.\n",
 	       fn, type, strerror(errno));
+	close(fd);
+	errno = oerrno;
 	return NULL;
     }
 
@@ -929,7 +947,7 @@ static void
 check_stale(l_fn)
     char *l_fn;				/* name of lock file */
 {
-    char buf[50];			/* for contents of lock file (a PID) */
+    char buf[MAXLONG_B10_DIGITS + 1];	/* for contents of lock file (a PID) */
     int ct;
     int fd;
     int pid = 0;
@@ -1121,10 +1139,11 @@ lockfile_name(fn, plfn)
 	    char *dir;
 
 	    p = base;
-	    while (p > fn && *(p - 1) == '/')
+	    while (p > fn && *(p - 1) == '/') {
 		--p;
-	    dir = xmalloc((unsigned int) ((p - fn) + 1));
-	    (void) memcpy(dir, fn, (unsigned int) (p - fn));
+	    }
+	    dir = xmalloc((size_t) ((p - fn) + 1));
+	    (void) memcpy(dir, fn, (size_t) (p - fn));
 	    dir[p - fn] = '\0';
 	    no_trunc_flag = pathconf(dir, _PC_NO_TRUNC);
 	    if (! no_trunc_flag) {
@@ -1180,23 +1199,18 @@ compute_hostname()
 {
     static char *hostname;
 # ifdef	GETHOSTNAME_USE_PTR
-    int len;
+    size_t len;
 # endif
 
-    /*
-     * My man page says that 255 chars (plus nul byte) is the limit
-     * on length of the local host name.  There appears to be no
-     * #define for it in 4.2BSD.
-     */
-    hostname = xmalloc(256);
+    hostname = xmalloc((size_t) MAXHOSTNAMELEN);
 # ifdef GETHOSTNAME_USE_PTR
-    len = 256;
+    len = MAXHOSTNAMELEN;
     (void) gethostname(hostname, &len);
 # else
-    (void) gethostname(hostname, 256);
+    (void) gethostname(hostname, (size_t) MAXHOSTNAMELEN);
 # endif
 
-    hostname = xrealloc(hostname, strlen(hostname) + 1);
+    hostname = xrealloc(hostname, (size_t) strlen(hostname) + 1);
 
     return hostname;
 }
@@ -1324,7 +1338,7 @@ compute_domain ( hostname )
  *		  variable and with the read-end tied to the stdin of
  *		  the child process.
  *	out	- a pointer to a FILE* variable.  If non-NULL, a pipe
- *		  is created, with thre read-end returned in this
+ *		  is created, with the read-end returned in this
  *		  variable and with the write-end tied to the stdout
  *		  of the child process.
  *	errfd	- if errfd >= 0 then it sepecifies a file descriptor
@@ -1411,20 +1425,17 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 	return EOF;
     }
 
-    /* keep trying to create a fork until we succeed or retries == 0 */
-    while (
-#if (defined(UNIX_BSD) || defined(HAVE_VFORK)) && !defined(DONT_USE_VFORK)
-	   (pid = vfork()) < 0 &&
-#else	/* not UNIX_BSD && not HAVE_VFORK */
-	   (pid = fork()) < 0 &&
-#endif	/* UNIX_BSD || HAVE_VFORK */
-	   errno == EAGAIN &&
-	   --retries >= 0)
-    {
-	(void) sleep(FORK_INTERVAL);	/* sleep between retries */
+    /*
+     * keep trying to create a new child process until either we succeed, or we
+     * run out of retries
+     *
+     * Do not use vfork() -- we muck with far too much in here!
+     */
+    while ((pid = fork()) < 0 && errno == EAGAIN && --retries >= 0) {
+	(void) sleep(FORK_INTERVAL);	/* sleep a bit between retries */
     }
     if (pid < 0) {
-	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+	write_log(WRITE_LOG_TTY | WRITE_LOG_SYS | WRITE_LOG_PANIC,
 		  "open_child(%s): fork() failed after %d: %s", argv[0], FORK_RETRIES, strerror(errno));
 	/* free resources */
 	if (in) {
@@ -1496,7 +1507,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 		(void) close(STDIN_FILENO);
 		if (open("/dev/null", O_RDONLY) < 0) { /* XXX assumes will re-open on just closed fd!!!! */
 		    /* uggh! failed to open /dev/null, quit without flush */
-		    write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+		    write_log(WRITE_LOG_TTY | WRITE_LOG_SYS | WRITE_LOG_PANIC,
 			      "open_child(%s): open(/dev/null, O_RDONLY) failed: %s", argv[0], strerror(errno));
 		    _exit(EX_OSFILE);
 		}
@@ -1531,7 +1542,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 		(void) close(STDOUT_FILENO);
 		if (open("/dev/null", O_WRONLY) < 0) { /* XXX assumes will re-open on just closed fd!!!! */
 		    /* uggh! failed to open /dev/null, quit without flush */
-		    write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+		    write_log(WRITE_LOG_TTY | WRITE_LOG_SYS | WRITE_LOG_PANIC,
 			      "open_child(%s): open(/dev/null, O_WRONLY) failed: %s", argv[0], strerror(errno));
 		    _exit(EX_OSFILE);
 		}
@@ -1574,12 +1585,12 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 
 	/* change uid and gid */
 	if (setgid(gid) == -1) {
-	    write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_PANIC : 0),
+	    write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_SYS | WRITE_LOG_PANIC : 0),
 		      "open_child(%s): setgid(%d) failed (possible local configuration error): %s", argv[0], gid, strerror(errno));
 	    _exit(EX_NOPERM);
 	}
 	if (setuid(uid) == -1) {
-	    write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_PANIC : 0),
+	    write_log(WRITE_LOG_TTY | ((geteuid() == 0) ? WRITE_LOG_SYS | WRITE_LOG_PANIC : 0),
 		      "open_child(%s): setuid(%d) failed (possible local configuration error): %s", argv[0], uid, strerror(errno));
 	    _exit(EX_NOPERM);
 	}
@@ -1619,7 +1630,7 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
 	}
 
 	/* Oh well... all this work and we can't even exec the file! */
-	write_log(WRITE_LOG_TTY | WRITE_LOG_PANIC,
+	write_log(WRITE_LOG_TTY | WRITE_LOG_SYS | WRITE_LOG_PANIC,
 		  "open_child(): exec(%s) failed (possible local configuration error): %s",
 		  argv[0], strerror(oerrno));
 	_exit(EX_OSFILE);
@@ -1665,8 +1676,8 @@ open_child(argv, envp, in, out, errfd, flags, uid, gid)
  *
  * NOTE:  we do not keep the exit status of processes other than the
  *	  one we are waiting.  Thus, only one process should be open
- *	  at a time by open_child, unless the caller wishes to handle
- *	  the performing the waits itself.
+ *	  at a time by open_child, unless the caller wishes to do
+ *	  the wait() calls itself.
  *
  * return EOF, on error, or exit status.
  */
@@ -1688,12 +1699,14 @@ close_child(in, out, pid)
     }
 
     /* wait for the child process to die */
-    while ((i = wait((STATUS_TYPE *)&status)) != pid && i >= 0) ;
-
+    while ((i = wait((STATUS_TYPE *) &status)) != pid && i >= 0) {
+	;
+    }
     if (i < 0) {
 	/* failed to find the child process */
 	return FAIL;
     }
+
     return status;			/* everything went okay */
 }
 
@@ -1710,7 +1723,10 @@ close_all()
 {
     register int i;
     int max_nfd;
-    /* so just how many file descriptors do we have? */
+
+    /*
+     * so, just how many file descriptors do we have?
+     */
 #ifdef POSIX_OS
     max_nfd = sysconf(_SC_OPEN_MAX);
 #else	/* not POSIX_OS */
@@ -1725,8 +1741,20 @@ close_all()
 # endif
 #endif
 
-    for (i = 3; i  < max_nfd; i++)
+    for (i = 3; i  < max_nfd; i++) {
+	if (errfile && i == fileno(errfile)) {
+	    continue;
+	}
+	if (logfile && i == fileno(logfile)) {
+	    continue;
+	}
+	if (panicfile && i == fileno(panicfile)) {
+	    continue;
+	}
 	(void) close(i);
+    }
+
+    return;
 }
 
 #ifndef	HAVE_RENAME
@@ -1790,7 +1818,7 @@ mkdir(dir, mode)
 	NULL,
     };
     /* set the umask so that the directory will have the correct perms */
-    int oumask = umask((~mode)&0777);
+    unsigned int oumask = umask((~mode) & 0777);
     int pid;
 
     mkdir_argv[1] = dir;
@@ -1904,6 +1932,25 @@ fcntl_rd_lock(fd)
 {
     struct flock l;
 
+    l.l_type = F_RDLCK;
+    l.l_whence = 0;
+    l.l_start = 0L;
+    l.l_len = 0L;
+    if (fcntl(fd, F_SETLK, &l) < 0) {
+	return FAIL;
+    }
+    return SUCCEED;
+}
+
+/*
+ * fcntl_rd_lock_wait - get a shared lock with blocking using a System V-style fcntl.
+ */
+int
+fcntl_rd_lock_wait(fd)
+    int fd;
+{
+    struct flock l;
+
     l.l_type = F_RDLCK;
     l.l_whence = 0;
     l.l_start = 0L;
@@ -1926,8 +1973,8 @@ touch(fname, atime, mtime)
     struct utimbuf times;
     time_t cur_time;
 
-    bzero((char *)&times, sizeof(times));
-    cur_time = time(NULL);
+    (void) memset((char *) &times, '\0', sizeof(times));
+    cur_time = time((time_t *) NULL);
     times.actime  = atime ? atime : cur_time;
     times.modtime = mtime ? mtime : cur_time;
     return utime(fname, &times);
@@ -1943,7 +1990,7 @@ int
 fsetsize(fd, wantsize, oversize)
     int fd;			/* fd to adjust */
     off_t wantsize;		/* desired size */
-    off_t oversize;		/* bytes not to overwrite UNUSED ifndef F_FREESP */
+    off_t oversize;		/* bytes not to overwrite UNUSED #ifndef F_FREESP */
 {
     off_t cursize;
 
@@ -1954,7 +2001,7 @@ fsetsize(fd, wantsize, oversize)
 
     /* maybe lengthen... */
     if (cursize < wantsize) {
-	if (lseek(fd, wantsize - 1, 0) == -1 || write(fd, "", 1) == -1) {
+	if (lseek(fd, wantsize - 1, 0) == -1 || write(fd, "", (size_t) 1) == -1) {
 	    return -1;
 	}
 	return 0;
@@ -2025,38 +2072,303 @@ fsetsize(fd, wantsize, oversize)
     return 0;
 }
 
-#ifdef NEED_HSTRERROR
-const char *
-hstrerror(h_err)
-    int h_err;
+#ifndef HAVE_VFPRINTF
+/*
+ * vfprintf - a hacked version of vfprintf() for sites that don't have it
+ *
+ * XXX - will _doprnt() work here?
+ */
+int
+vfprintf(file, fmt, ap)
+    FILE *file;
+    char *fmt;
+    va_list ap;
 {
-    /* XXX Should use h_errlist if it's available */
-    if (h_err < 0)
-	return "Resolver internal error";
-    switch (h_err) {
-    case 0:
-	return "Resolver Error 0 (no error)";
+    int a,b,c,d,e,f,g,h,i,j,k,l,m,n,o;
 
-    case HOST_NOT_FOUND:
-	return "Unknown host";
+    a = va_arg(ap, int);
+    b = va_arg(ap, int);
+    c = va_arg(ap, int);
+    d = va_arg(ap, int);
+    e = va_arg(ap, int);
+    f = va_arg(ap, int);
+    g = va_arg(ap, int);
+    h = va_arg(ap, int);
+    i = va_arg(ap, int);
+    j = va_arg(ap, int);
+    k = va_arg(ap, int);
+    l = va_arg(ap, int);
+    m = va_arg(ap, int);
+    n = va_arg(ap, int);
+    o = va_arg(ap, int);
 
-    case TRY_AGAIN:
-	return "Hostname lookup failure";
-
-    case NO_RECOVERY:
-	return "Unknown server error";
-
-    case NO_DATA:
-	return "No DNS data associated with name";
-
-#if defined(NO_ADDRESS) && ((NO_ADDRESS - 0) != (NO_DATA))
-    case NO_ADDRESS:
-	return "No address associated with name";
-#endif
-
-    default:
-	return "Unknown resolver error";
-    }
-    /* NOTREACHED */
+    return fprintf(file, fmt, a,b,c,d,e,f,g,h,i,j,k,l,m,n,o);
 }
+#endif	/* HAVE_VFPRINTF */
+
+#ifndef HAVE_STRERROR
+/*
+ * strerror -  return a string representing the error given in errnum.
+ */
+char *
+strerror(errnum)
+    int errnum;
+{
+    static char misc_err[sizeof("Unknown error: ") + MAXINT_B10_DIGITS + 1]; /* used when sprintf must be used */
+#if 0
+    extern const char *const sys_errlist[];	/* 4.4BSD */
+#else
+    extern char *sys_errlist[];		/* list of error strings */
 #endif
+    extern int sys_nerr;		/* number of entries in sys_errlist */
+
+    if (errnum > sys_nerr || errnum < 0) {
+	/* there is no entry for it in sys_errlist, build one */
+	(void) sprintf(misc_err, "Unknown error: %d", errnum);
+	return misc_err;
+    } else {
+	/* there is an entry in sys_errlist, use it */
+	return sys_errlist[errnum];
+    }
+}
+#endif /* HAVE_STRERROR */
+
+/*
+ * strsysexit - return a string corresponding to an exit code.
+ *
+ * XXX as-is this should be sysexit2name() or something
+ */
+char *
+strsysexit(exitcode)
+    int exitcode;
+{
+    static char buf[sizeof("EX_") + MAXINT_B10_DIGITS + 1]; /* buffer for generating message */
+
+    switch (exitcode) {
+    case EX_USAGE:
+	return "EX_USAGE";
+    case EX_DATAERR:
+	return "EX_DATAERR";
+    case EX_NOINPUT:
+	return "EX_NOINPUT";
+    case EX_NOUSER:
+	return "EX_NOUSER";
+    case EX_NOHOST:
+	return "EX_NOHOST";
+    case EX_UNAVAILABLE:
+	return "EX_UNAVAILABLE";
+    case EX_SOFTWARE:
+	return "EX_SOFTWARE";
+    case EX_OSERR:
+	return "EX_OSERR";
+    case EX_OSFILE:
+	return "EX_OSFILE";
+    case EX_CANTCREAT:
+	return "EX_CANTCREAT";
+    case EX_IOERR:
+	return "EX_IOERR";
+    case EX_TEMPFAIL:
+	return "EX_TEMPFAIL";
+    case EX_PROTOCOL:
+	return "EX_PROTOCOL";
+    case EX_NOPERM:
+	return "EX_NOPERM";
+    default:
+	(void) sprintf(buf, "EX_%d", exitcode);
+	return buf;
+    }
+}
+
+/* NetBSD gained sys_nsig sometime just prior to 1.4 */
+#if defined(__NetBSD_Version__) && (__NetBSD_Version__ >= 104000000)
+# define HAVE_SYS_NSIG		1
+# define SYS_NSIG_DECLARED	1
+#endif
+
+/* FreeBSD gained sys_nsig sometime just prior to 4.0 */
+#if defined(__FreeBSD_version) && (__FreeBSD_version >= 400017)
+# define HAVE_SYS_NSIG		1
+# define SYS_NSIG_DECLARED	1
+#endif
+
+#if !defined(HAVE_STRSIGNAL) || (!defined(HAVE_SIG2STR) && defined(HAVE_SYS_SIGNAME))
+# if !defined(SYS_NSIG_DECLARED) && !defined(__SYS_NSIG_DECLARED)
+#  if defined(HAVE_SYS_NSIG)
+extern int sys_nsig;
+#  else
+#   define sys_nsig	NSIG
+#  endif
+# endif
+#endif
+
+#ifndef HAVE_STRSIGNAL
+/*
+ * strsignal -  return a pointer to a string describing the signal given in signum.
+ */
+char *
+strsignal(signum)
+    int signum;
+{
+# if !defined(SYS_SIGLIST_DECLARED) && !defined(__SYS_SIGLIST_DECLARED)
+#  if 0		/* 4.4BSD, GNU/Linux, SCO5, SunOS-5, if they needed it  */
+    extern const char *const sys_siglist[];
+#  else
+    extern char *sys_siglist[];
+#  endif
+# endif
+
+#if defined(SIGRTMIN) && defined(SIGRTMAX)
+    if (signum >= sys_nsig && signum >= SIGRTMIN && signum <= SIGRTMAX) {
+	static char rt_descr[sizeof("Real time signal: ") + MAXINT_B10_DIGITS + 1];
+
+	/*
+	 * Signum is out of range but is within SIGRTMIN..SIGRTMAX so we build
+	 * our own description for it.
+	 */
+	(void) sprintf(rt_descr, "Real time signal: %d", signum);
+	return rt_descr;
+    } else
+#endif
+    if (signum >= sys_nsig || signum < 0) {
+	char signame[SIG2STR_MAX];
+	static char misc_err[sizeof("Unknown signal: SIG []") + MAXINT_B10_DIGITS + SIG2STR_MAX + 1];
+
+	/*
+	 * Signum is out of range so there's no entry for it in sys_siglist, so
+	 * we build our own description.
+	 *
+	 * Even though the SunOS-5 manual says "The strsignal() function
+	 * returns NULL if sig is not a valid signal number.", it's easy enough
+	 * to assume signum is valid and instead that sys_siglist is
+	 * incomplete.  The behaviour of this implementation matches that of
+	 * 4.4BSD.
+	 */
+	if (sig2str(signum, signame) == -1) {
+	    (void) sprintf(signame, "#%d", signum);
+	}
+	(void) sprintf(misc_err, "Unknown signal: SIG%s [%d]", signame, signum);
+	return misc_err;
+    }
+
+    /* there is an entry in sys_siglist, use it */
+    return sys_siglist[signum];
+}
+#endif /* HAVE_STRSIGNAL */
+
+#ifndef HAVE_SIG2STR
+# if defined(HAVE_SYS_SIGNAME)
+#  define SYS_NSIGNAME	sys_nsig
+# else
+#  define SYS_NSIGNAME	sys_nsigname	/* can't be sure gen'ed one has sys_nsig entries */
+extern const unsigned int sys_nsigname;	/* from ../compat/signame.c */
+# endif
+/*
+ * sig2str - pass back via signame a string naming the signal given by signum.
+ *
+ * Returns -1 if no name is known for the signal.
+ *
+ * NOTE: Unfortunately this most horrible API is "standard" in SunOS-5.x
+ */
+int
+sig2str(signum, signame)
+    int signum;
+    char *signame;			/* ptr to at least SIG2STR_MAX bytes */
+{
+# if !defined(SYS_SIGNAME_DECLARED) && !defined(__SYS_SIGNAME_DECLARED)
+#  if 0					/* 4.4BSD, Linux, etc. */
+    extern const char *const sys_signame[];
+#  else
+    extern char *sys_signame[];
+#  endif
+# endif
+
+# if defined(SIGRTMIN) && defined(SIGRTMAX)
+    if (signum >- SYS_NSIGNAME && signum >= SIGRTMIN && signum <= SIGRTMAX) {
+	/*
+	 * Signum is out of range but is within SIGRTMIN..SIGRTMAX so we build
+	 * our own description for it.
+	 *
+	 * XXX any str2sig() implementation should try to account for this
+	 * hack, just as the SunOS-5 implementation does, though perhaps not
+	 * with the "RTMAX-1" weirdness.
+	 */
+#  if (SIG2STR_MAX < (MAXINT_B10_DIGITS + 7))
+#   include "ERROR: SIG2STR_MAX is too small!"
+#  endif
+	(void) sprintf(signame, "RTMIN+%d", (signum - SIGRTMIN));
+	return 0;
+    } else 
+# endif
+    if (signum >= SYS_NSIGNAME || signum < 0) {
+	/* XXX Maybe we dould format "#%d" in here anyway? */
+	return (-1);
+    }
+    strncpy(signame, sys_signame[signum], (size_t) SIG2STR_MAX);
+    signame[SIG2STR_MAX - 1] = '\0';
+
+    return 0;
+}
+#endif /* HAVE_SIG2STR */
+
+/*
+ * Unless the target system has POSIX.4 queued signals we really need reliable
+ * signals in order to safely handle SIGCHLD.  This is because only with
+ * reliable signals are we free to ignore problems with lost signals (which
+ * would adversely affect our ability to reliably count remaining active child
+ * processes) and free to avoid worrying about creating a loop should we
+ * re-instate the signal handler before all exited child processes have been
+ * reaped.
+ *
+ * (except under old SysVr2.x with signal(SIGCLD) where the right thing
+ * always happens -- the signal hander is always re-entered if any SIGCLD
+ * signals arrive while it is executing, and with only wait(2) the signal
+ * handler must reap dead child processes).
+ *
+ * XXX it is basically impossible to deal with SIGCLD on SysVr3.x because it's
+ * not queued, and it is reset, and there's no non-blocking alternative to
+ * wait(2).
+ *
+ * XXX on BSD4.3 or BSD4.2 we will need to use yucky old sigvec(2).
+ *
+ * For all others (i.e. for POSIX_OS) we use sigaction(2).
+ *
+ * Rumour (in the form of code in Postfix) has it that on HP/UX (and perhaps
+ * others) the handler will have to muck with the signal context in order to
+ * get select() or accept() to return EINTR instead of restarting.
+ */
+
+#ifdef SA_NOCLDSTOP			/* aka HAVE_SIGACTION aka POSIX_OS */
+
+int
+set_sigchld_handler(handler)
+    void (*handler) __P((int));
+{
+    struct sigaction action;
+
+    sigemptyset(&action.sa_mask);
+    action.sa_flags = 0;
+
+# ifdef SA_RESTART
+    /* NOTE: we do not intend accept() or select() to restart! */
+    action.sa_flags |= SA_RESTART;
+# endif
+    action.sa_flags |= SA_NOCLDSTOP;
+    action.sa_handler = handler;
+
+    return sigaction(SIGCHLD, &action, (struct sigaction *) NULL);
+}
+
+#else /* SA_NOCLDSTOP */
+# ifdef UNIX_SYS5_2
+
+int
+set_sigchld_handler(handler)
+    void (*handler)(int);
+{
+	return (signal(SIGCLD, handler) == SIG_ERR) ? -1 : 0;
+}
+
+# else
+#  include "ERROR: smail not currently supported on this platform without sigaction(2)"
+# endif	/* UNIX_SYS5_2 */
+#endif /* SA_NOCLDSTOP */
