@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:transport.c,v 1.35 2001/06/23 03:43:48 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:transport.c,v 1.47 2003/04/02 22:15:54 woods Exp"
  */
 
 /*
@@ -22,42 +22,70 @@
  *			     write_message, remote_from_line, local_from_line,
  *			     find_transport, find_transport_driver
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
+#include <sys/param.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <sys/param.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-#if (defined(POSIX_OS) || !defined(UNIX_BSD)) && !defined(ISC_SOCKET_TIME_BUG)
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# include <sys/time.h>
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "hash.h"
-#include "transport.h"
 #include "log.h"
 #include "spool.h"
 #include "alloc.h"
+#include "list.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
+#include "transport.h"
 #include "smailconf.h"
-#ifndef DEPEND
-# include "extern.h"
-# include "debug.h"
-# include "error.h"
-#endif
-
-#ifdef	UNIX_BSD
-# include <sys/file.h>
-#endif
+#include "extern.h"
+#include "debug.h"
+#include "error.h"
+#include "smailport.h"
 
 /* variables exported from this file */
 char *path_to_sender = NULL;		/* uucp-style route to sender */
@@ -77,32 +105,32 @@ static char *inet_from_line __P((struct transport *));
 
 
 static struct attr_table transport_generic[] = {
-    { "driver", t_string, NULL, NULL, OFFSET(transport, driver) },
-    { "max_addrs", t_int, NULL, NULL, OFFSET(transport, max_addrs) },
-    { "max_hosts", t_int, NULL, NULL, OFFSET(transport, max_hosts) },
-    { "max_chars", t_int, NULL, NULL, OFFSET(transport, max_chars) },
-    { "retry_dir", t_string, NULL, NULL, OFFSET(transport, retry_dir) },
-    { "shadow", t_string, NULL, NULL, OFFSET(transport, shadow) },
-    { "error_transport", t_string, NULL, NULL,
+    { "driver", t_string, 0, NULL, NULL, OFFSET(transport, driver) },
+    { "max_addrs", t_int, 0, NULL, NULL, OFFSET(transport, max_addrs) },
+    { "max_hosts", t_int, 0, NULL, NULL, OFFSET(transport, max_hosts) },
+    { "max_chars", t_int, 0, NULL, NULL, OFFSET(transport, max_chars) },
+    { "retry_dir", t_string, 0, NULL, NULL, OFFSET(transport, retry_dir) },
+    { "shadow", t_string, 0, NULL, NULL, OFFSET(transport, shadow) },
+    { "error_transport", t_string, 0, NULL, NULL,
 	  OFFSET(transport, error_transport) },
-    { "strict", t_boolean, NULL, NULL, STRICT_TPORT },
-    { "uucp", t_boolean, NULL, NULL, UUCP_XFORM },
-    { "inet", t_boolean, NULL, NULL, INET_XFORM },
-    { "received", t_boolean, NULL, NULL, PUT_RECEIVED },
-    { "return_path", t_boolean, NULL, NULL, PUT_RETURNPATH },
-    { "from", t_boolean, NULL, NULL, PUT_FROM },
-    { "local", t_boolean, NULL, NULL, LOCAL_TPORT },
-    { "local_xform", t_boolean, NULL, NULL, LOCAL_XFORM },
-    { "crlf", t_boolean, NULL, NULL, PUT_CRLF },
-    { "bsmtp", t_boolean, NULL, NULL, BSMTP_TPORT },
-    { "hbsmtp", t_boolean, NULL, NULL, HBSMTP_TPORT },
-    { "dots", t_boolean, NULL, NULL, PUT_DOTS },
-    { "debug", t_boolean, NULL, NULL, DEBUG_TPORT },
-    { "unix_from_hack", t_boolean, NULL, NULL, UNIX_FROM_HACK },
-    { "uucp_from_hack", t_boolean, NULL, NULL, UNIX_FROM_HACK },
-    { "remove_header", t_proc, NULL, (tup *)tp_remove_header, 0 },
-    { "insert_header", t_proc, NULL, (tup *)tp_insert_header, 0 },
-    { "append_header", t_proc, NULL, (tup *)tp_append_header, 0 },
+    { "strict", t_boolean, 0, NULL, NULL, STRICT_TPORT },
+    { "uucp", t_boolean, 0, NULL, NULL, UUCP_XFORM },
+    { "inet", t_boolean, 0, NULL, NULL, INET_XFORM },
+    { "received", t_boolean, 0, NULL, NULL, PUT_RECEIVED },
+    { "return_path", t_boolean, 0, NULL, NULL, PUT_RETURNPATH },
+    { "from", t_boolean, 0, NULL, NULL, PUT_FROM },
+    { "local", t_boolean, 0, NULL, NULL, LOCAL_TPORT },
+    { "local_xform", t_boolean, 0, NULL, NULL, LOCAL_XFORM },
+    { "crlf", t_boolean, 0, NULL, NULL, PUT_CRLF },
+    { "bsmtp", t_boolean, 0, NULL, NULL, BSMTP_TPORT },
+    { "hbsmtp", t_boolean, 0, NULL, NULL, HBSMTP_TPORT },
+    { "dots", t_boolean, 0, NULL, NULL, PUT_DOTS },
+    { "debug", t_boolean, 0, NULL, NULL, DEBUG_TPORT },
+    { "unix_from_hack", t_boolean, 0, NULL, NULL, UNIX_FROM_HACK },
+    { "uucp_from_hack", t_boolean, 0, NULL, NULL, UNIX_FROM_HACK },
+    { "remove_header", t_proc, 0, NULL, (tup *)tp_remove_header, 0 },
+    { "insert_header", t_proc, 0, NULL, (tup *)tp_insert_header, 0 },
+    { "append_header", t_proc, 0, NULL, (tup *)tp_append_header, 0 },
 };
 
 
@@ -124,7 +152,7 @@ assign_transports(list)
     register struct assign_transport *asn; /* list of assigned transports */
     register struct addr **aap;		/* points to array for sorting */
     register int i;			/* index */
-    unsigned int ct;			/* count of items in input list */
+    size_t ct;				/* count of items in input list */
     int host_ct = 0;			/* count of hosts in instance */
     int addr_ct = 0;			/* count of addrs in instance */
     int char_ct = 0;			/* count of chars in instance */
@@ -139,12 +167,12 @@ assign_transports(list)
 
     DEBUG1(DBG_REMOTE_HI, "assign_transports called with %d addrs\n", ct);
     /* create an array and copy remote information to it for sorting */
-    aap = (struct addr **) xmalloc((unsigned) (ct * sizeof(*aap)));
+    aap = (struct addr **) xmalloc((size_t) (ct * sizeof(*aap)));
     for (aq = list, i = 0; aq; aq = aq->succ, i++) {
 	aap[i] = aq;
     }
 
-    qsort((char *)aap, ct, sizeof(*aap), assign_compare);
+    qsort((char *) aap, ct, sizeof(*aap), assign_compare);
 
     /*
      * we now have an array sorted by transport and host.
@@ -152,8 +180,8 @@ assign_transports(list)
      * specific transport instances, taking into account
      * max_addr and max_hosts for the transports.
      */
-    last_host=(char*)0;
-    last_owner=(char *)(-1);     /* can't use 0 as owner can be unset */
+    last_host = NULL;
+    last_owner = (char *) (-1);		/* can't use 0 as owner can be unset */
     for (asn = NULL, i = ct-1; i >= 0; --i) {
 	/* possible reasons to switch to a new instance */
 	if (
@@ -224,8 +252,8 @@ assign_compare(x, y)
     const void *x;
     const void *y;
 {
-    struct addr *a = *((struct addr **) x);
-    struct addr *b = *((struct addr **) y);
+    const struct addr *a = *((const struct addr * const *) x);
+    const struct addr *b = *((const struct addr * const *) y);
     register int ret = strcmp(a->transport->name, b->transport->name);
 
     if (ret) {
@@ -812,7 +840,6 @@ get_sender_addr(tp)
     case RFC_ROUTE:
     case RFC_ENDROUTE:
     case MAILBOX:
-    case PARSE_ERROR:
 	return sender;
 
     default:
@@ -986,6 +1013,7 @@ read_transport_file()
     char *error;			/* error from read_standard_file() */
     struct transport **tpp;
     struct stat statbuf;
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_transport_generic = ENDTABLE(transport_generic);
     static struct transport transport_template = {
 	NULL,				/* name */
@@ -995,6 +1023,10 @@ read_transport_file()
 	1,				/* max_addrs */
 	1,				/* max_hosts */
 	2000,				/* max_chars, about half of NCARGS? */
+	NULL,				/* headers to remove */
+	NULL,				/* headers to insert */
+	NULL,				/* headers to append */
+	NULL,				/* name of directory for retry files */
 	NULL,				/* shadow */
 	NULL,				/* error_transport */
 	NULL,				/* private */
@@ -1051,11 +1083,12 @@ dump_transport_config(f)
      FILE * f;
 {
     struct transport *tp;		/* temp for stepping thru transports */
+    /* FIXME:  avoid this ugly stuff -- use null-entry termination semantics... */
     struct attr_table *end_transport_generic = ENDTABLE(transport_generic);
     char * name;			/* name of transport */
     long flags;				/* flags for transport */
     struct transport_driver *driver;	/* drivers for transport */
-    struct list *lp;			/* list pointer for header stuff */
+    charplist_t *lp;			/* list pointer for header stuff */
     struct hash_table * tp_hash;	/* hash table to find multiple defs */
 
     DEBUG(DBG_ROUTE_HI, "dump transport configs called\n");
@@ -1070,7 +1103,7 @@ dump_transport_config(f)
     for (tp = transports; tp; tp = tp->succ) {
 	name = tp->name;
 	flags = tp->flags;
-	if (add_to_hash(name, NULL, 0, tp_hash) == ALREADY_HASHED) {
+	if (add_to_hash(name, (char *) NULL, (size_t) 0, tp_hash) == ALREADY_HASHED) {
 	    fprintf(f, "#\n# %s: a transport of this name has already been dumped\n",
 		    name);
 	} else {
@@ -1083,11 +1116,11 @@ dump_transport_config(f)
 					end_transport_generic);
 	    /* The dump config routine cannot handle t_proc, so we do that here */
 	    for(lp = tp->hdrremove; (lp); lp = lp->succ)
-		fprintf(f, "\tremove_header=%s,\n", quote_string_value(lp->text));
+		fprintf(f, "\tremove_header=%s,\n", quote_string_value(lp->text, FALSE));
 	    for(lp = tp->hdrinsert; (lp); lp = lp->succ)
-		fprintf(f, "\tinsert_header=%s,\n", quote_string_value(lp->text));
+		fprintf(f, "\tinsert_header=%s,\n", quote_string_value(lp->text, FALSE));
 	    for(lp = tp->hdrappend; (lp); lp = lp->succ)
-		fprintf(f, "\tappend_header=%s,\n", quote_string_value(lp->text));
+		fprintf(f, "\tappend_header=%s,\n", quote_string_value(lp->text, FALSE));
 	    fputs("\t;\n", f);
 	    driver = find_transport_driver(tp->driver);
 	    if (driver->dumper)
@@ -1104,9 +1137,9 @@ tp_remove_header(struct_p, attr)
     struct attribute *attr;		/* attribute from transports file */
 {
     struct transport *tp = (struct transport *)struct_p;
-    struct list *lp;
+    charplist_t *lp;
 
-    lp = (struct list *)xmalloc(sizeof(*lp));
+    lp = (charplist_t *) xmalloc(sizeof(*lp));
     lp->text = attr->value;
     lp->succ = tp->hdrremove;
     tp->hdrremove = lp;
@@ -1119,9 +1152,9 @@ tp_insert_header(struct_p, attr)
     struct attribute *attr;		/* attribute from transports file */
 {
     struct transport *tp = (struct transport *)struct_p;
-    struct list *lp;
+    charplist_t *lp;
 
-    lp = (struct list *)xmalloc(sizeof(*lp));
+    lp = (charplist_t *) xmalloc(sizeof(*lp));
     lp->text = attr->value;
     lp->succ = tp->hdrinsert;
     tp->hdrinsert = lp;
@@ -1134,9 +1167,9 @@ tp_append_header(struct_p, attr)
     struct attribute *attr;		/* attribute from transports file */
 {
     struct transport *tp = (struct transport *)struct_p;
-    struct list *lp;
+    charplist_t *lp;
 
-    lp = (struct list *)xmalloc(sizeof(*lp));
+    lp = (charplist_t *) xmalloc(sizeof(*lp));
     lp->text = attr->value;
     lp->succ = tp->hdrappend;
     tp->hdrappend = lp;
