@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:transport.h,v 1.11 2000/09/03 16:29:02 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:transport.h,v 1.12 2002/03/07 20:46:11 woods Exp"
  */
 
 /*
@@ -27,9 +27,9 @@ struct transport {
     int max_addrs;			/* maximum addresses per instance */
     int max_hosts;			/* maximum hosts per instance */
     int max_chars;			/* max chars of address per instance */
-    struct list *hdrremove;		/* headers to remove */
-    struct list *hdrinsert;		/* headers to insert */
-    struct list *hdrappend;		/* headers to append */
+    charplist_t *hdrremove;		/* headers to remove */
+    charplist_t *hdrinsert;		/* headers to insert */
+    charplist_t *hdrappend;		/* headers to append */
     char *retry_dir;			/* name of directory for retry files */
     char *shadow;			/* name of shadow transport */
     char *error_transport;		/* use this transport on errors */
