@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_114:appendfile.c,v 1.39 2001/07/29 23:11:32 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_115:appendfile.c,v 1.49 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -65,18 +65,41 @@
  *	answer is:  Don't use NFS.  Do local delivery on the file server.
  */
 
-#define NEED_SOCKETS
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/stat.h>
 
-#include "defs.h"
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
 
-#if defined(HAVE_PATHS_H) || defined (UNIX_BSD)
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_PATHS_H)
 # include <paths.h>
 #endif
 
@@ -89,46 +112,32 @@
 #ifndef HAVE_BSD_NETWORKING
 # undef HAVE_COMSAT
 #endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
 
-#include "../config.h"
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #include "../smail.h"
+#include "../smailsock.h"
+#include "../config.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../log.h"
 #include "../spool.h"
 #include "../child.h"
-#include "../transport.h"
 #include "../exitcodes.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "appendfile.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
-
-/* whence values for lseek(2) */
-#ifndef SEEK_SET
-# define SEEK_SET	0	/* set file offset to offset */
-#endif
-#ifndef SEEK_CUR
-# define SEEK_CUR	1	/* set file offset to current plus offset */
-#endif
-#ifndef SEEK_END
-# define SEEK_END	2	/* set file offset to EOF plus offset */
-#endif
-
-#ifndef _PATH_DEVNULL
-# define _PATH_DEVNULL	"/dev/null"
-#endif
+#include "../smailport.h"
 
 /* functions local to this file */
 static void get_appendfile_ugid __P((struct transport *, struct addr *, int *, int *));
@@ -139,17 +148,17 @@ static void notify_comsat __P((struct transport *, char *, off_t));
 #endif
 
 static struct attr_table appendfile_attributes[] = {
-    { "file", t_string, NULL, NULL, OFFSET(appendfile_private, file) },
-    { "dir", t_string, NULL, NULL, OFFSET(appendfile_private, dir) },
-    { "user", t_string, NULL, NULL, OFFSET(appendfile_private, user) },
-    { "group", t_string, NULL, NULL, OFFSET(appendfile_private, group) },
-    { "prefix", t_string, NULL, NULL, OFFSET(appendfile_private, prefix) },
-    { "suffix", t_string, NULL, NULL, OFFSET(appendfile_private, suffix) },
-    { "mode", t_mode, NULL, NULL, OFFSET(appendfile_private, mode) },
-    { "append_as_user", t_boolean, NULL, NULL, APPEND_AS_USER },
-    { "expand_user", t_boolean, NULL, NULL, APPEND_EXPAND_USER },
-    { "check_user", t_boolean, NULL, NULL, APPEND_CHECK_USER },
-    { "notify_comsat", t_boolean, NULL, NULL, APPEND_COMSAT },
+    { "file", t_string, 0, NULL, NULL, OFFSET(appendfile_private, file) },
+    { "dir", t_string, 0, NULL, NULL, OFFSET(appendfile_private, dir) },
+    { "user", t_string, 0, NULL, NULL, OFFSET(appendfile_private, user) },
+    { "group", t_string, 0, NULL, NULL, OFFSET(appendfile_private, group) },
+    { "prefix", t_string, 0, NULL, NULL, OFFSET(appendfile_private, prefix) },
+    { "suffix", t_string, 0, NULL, NULL, OFFSET(appendfile_private, suffix) },
+    { "mode", t_mode, 0, NULL, NULL, OFFSET(appendfile_private, mode) },
+    { "append_as_user", t_boolean, 0, NULL, NULL, APPEND_AS_USER },
+    { "expand_user", t_boolean, 0, NULL, NULL, APPEND_EXPAND_USER },
+    { "check_user", t_boolean, 0, NULL, NULL, APPEND_CHECK_USER },
+    { "notify_comsat", t_boolean, 0, NULL, NULL, APPEND_COMSAT },
 };
 
 static struct attr_table *end_appendfile_attributes = ENDTABLE(appendfile_attributes);
@@ -239,7 +248,7 @@ tpd_appendfile(addr, succeed, defer, fail)
 	 * make sure the username can't cause an expansion to move to
 	 * another directory
 	 */
-	if (index(addr->next_addr, '/')) {
+	if (strchr(addr->next_addr, '/')) {
 	    /*
 	     * ERR_129 - username contains /
 	     *
@@ -801,9 +810,9 @@ restore_atime(fname, atime)
 /* ARGSUSED */
 static void
 notify_comsat(tp, mbox, offset)
-struct transport *tp;				/* UNUSED! */
-char *mbox;
-off_t offset;
+    struct transport *tp;			/* UNUSED! */
+    char *mbox;
+    off_t offset;
 {
     static char me[] = "notify_comsat";
     struct sockaddr_in sa;
@@ -842,10 +851,10 @@ off_t offset;
     msg = xprintf("%s@%d\n", base, offset);
     DEBUG2(DBG_DRIVER_HI, "%s: sending msg: %s", me, msg);
     /* XXX some systems have arg-3 as size_t, others as int !!!! */
-#if (defined(__svr4__) && defined(__sun__)) || defined(HAVE_SENDTO_WITH_3RD_INT)
-    ret = sendto(fd, msg, (int) strlen(msg) + 1, 0, (struct sockaddr *)&sa, sizeof(sa));
+#if defined(HAVE_SENDTO_WITH_3RD_PARAM_INT)
+    ret = sendto(fd, msg, (int) (strlen(msg) + 1), 0, (struct sockaddr *) &sa, sizeof(sa));
 #else
-    ret = sendto(fd, msg, strlen(msg) + 1, 0, (struct sockaddr *)&sa, sizeof(sa));
+    ret = sendto(fd, msg, (size_t) (strlen(msg) + 1), 0, (struct sockaddr *) &sa, /* XXX (socklen_t) */ sizeof(sa));
 #endif
     xfree(msg);
     if (ret == -1) {
