@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_114:pipe.c,v 1.30 2001/07/29 23:11:32 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_115:pipe.c,v 1.41 2003/06/10 00:21:53 woods Exp"
  */
 
 /*
@@ -52,6 +52,9 @@
  *		is non-zero, and the ignore_status flag is NOT set, then
  *		report back to the sender instead of the postmaster.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -60,13 +63,41 @@
 #include <ctype.h>
 #include <pwd.h>
 #include <grp.h>
-#include "defs.h"
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# define _GNU_SOURCE			/* to see decl. of strsignal() */
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
-#ifdef HAVE_STDLIB_H
-# include <stdlib.h>
+
+#if defined(POSIX_OS) || defined(UNIX_BSD) || defined(WAIT_USE_UNION)
+# include <sys/wait.h>
 #endif
+
 #include "../config.h"
 #include "../smail.h"
 #include "../main.h"
@@ -74,34 +105,38 @@
 #include "../addr.h"
 #include "../log.h"
 #include "../spool.h"
+#include "../smailwait.h"
 #include "../child.h"
-#include "../transport.h"
 #include "../exitcodes.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "pipe.h"
 #include "../extern.h"
 #include "../debug.h"
 #include "../error.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 static char **get_pipe_env __P((struct transport *, struct addr *));
 static void get_pipe_ugid __P((struct transport *, struct addr *, int *, int *));
 
 static struct attr_table pipe_attributes[] = {
-    { "cmd", t_string, NULL, NULL, OFFSET(pipe_private, cmd) },
-    { "user", t_string, NULL, NULL, OFFSET(pipe_private, user) },
-    { "group", t_string, NULL, NULL, OFFSET(pipe_private, group) },
-    { "umask", t_mode, NULL, NULL, OFFSET(pipe_private, umask) },
-    { "pipe_as_user", t_boolean, NULL, NULL, PIPE_AS_USER },
-    { "pipe_as_sender", t_boolean, NULL, NULL, PIPE_AS_SENDER },
-    { "ignore_status", t_boolean, NULL, NULL, PIPE_IGNORE_STATUS },
-    { "log_output", t_boolean, NULL, NULL, PIPE_LOG_OUTPUT },
-    { "parent_env", t_boolean, NULL, NULL, PIPE_PARENT_ENV },
-    { "ignore_write_errors", t_boolean, NULL, NULL, PIPE_IGNORE_WRERRS },
-    { "defer_child_errors", t_boolean, NULL, NULL, PIPE_DEFER_ERRORS },
-    { "status_to_sender", t_boolean, NULL, NULL, PIPE_STATUS_2SENDER },
+    { "cmd", t_string, 0, NULL, NULL, OFFSET(pipe_private, cmd) },
+    { "user", t_string, 0, NULL, NULL, OFFSET(pipe_private, user) },
+    { "group", t_string, 0, NULL, NULL, OFFSET(pipe_private, group) },
+    { "umask", t_mode, 0, NULL, NULL, OFFSET(pipe_private, umask) },
+    { "pipe_as_user", t_boolean, 0, NULL, NULL, PIPE_AS_USER },
+    { "pipe_as_sender", t_boolean, 0, NULL, NULL, PIPE_AS_SENDER },
+    { "ignore_status", t_boolean, 0, NULL, NULL, PIPE_IGNORE_STATUS },
+    { "log_output", t_boolean, 0, NULL, NULL, PIPE_LOG_OUTPUT },
+    { "parent_env", t_boolean, 0, NULL, NULL, PIPE_PARENT_ENV },
+    { "ignore_write_errors", t_boolean, 0, NULL, NULL, PIPE_IGNORE_WRERRS },
+    { "defer_child_errors", t_boolean, 0, NULL, NULL, PIPE_DEFER_ERRORS },
+    { "status_to_sender", t_boolean, 0, NULL, NULL, PIPE_STATUS_2SENDER },
 };
 static struct attr_table *end_pipe_attributes = ENDTABLE(pipe_attributes);
 
@@ -128,6 +163,7 @@ tpd_pipe(addr, succeed, defer, fail)
     unsigned int gid;			/* gid for child process */
     int errfd;				/* child process output file */
     char *error;			/* error from build_cmd_line() */
+    char *errstr = NULL;		/* error after close_child() */
     struct error *write_error = NULL;	/* pipe write error message */
     unsigned int save_umask;		/* saved value from umask() */
 
@@ -175,7 +211,7 @@ tpd_pipe(addr, succeed, defer, fail)
 	/* XXX perhaps we should just use substitute()???? */
 	ourcmd = expand_string(priv->cmd, addr, (char *) NULL, (char *) NULL);
     } else {
-	ourcmd = priv->cmd;		/* should be ``/bin/sh -c ${shquote:user}'' */
+	ourcmd = priv->cmd;		/* without HAVE_HASH_BANG should be ``/bin/sh -c ${shquote:user}'' */
     }
     /* get the argument vectors for the execv() call */
     argv = build_cmd_line(ourcmd, addr, "", &error);
@@ -270,7 +306,7 @@ tpd_pipe(addr, succeed, defer, fail)
 	    for (cp = *argp; (c = *cp++); ) {
 		if (c == '\\' || c == '"') {
 		    fprintf(errfile, "\\%c", c);
-		} else if (isprint(c)) {
+		} else if (isprint((int) c)) {
 		    putc(c, errfile);
 		} else {
 		    fprintf(errfile, "\\%03o", c & 0xff);
@@ -395,7 +431,8 @@ tpd_pipe(addr, succeed, defer, fail)
 	}
     }
 
-    status = close_child(child, (FILE *)NULL, pid);
+    status = close_child(child, (FILE *) NULL, pid);
+
     if (status == -1 && (tp->flags & PIPE_IGNORE_STATUS) == 0) {
 	/*
 	 * ERR_142 - failed to reap child process
@@ -421,50 +458,9 @@ tpd_pipe(addr, succeed, defer, fail)
 	    insert_addr_list(addr, fail, er);
 	    return;
 	}
-    } else if (status != 0) {
-	if (status & 0xff) {
-	    /*
-	     * ERR_143 - process killed by signal
-	     *
-	     * DESCRIPTION
-	     *      The exit status returned by close_child() revealed that
-	     *      the child process was killed by a signal.  This could be
-	     *      due to the machine being brought down, if the signal is
-	     *      SIGTERM.  Other signals may represent genuine problems.
-	     *
-	     * ACTIONS
-	     *      If the child was killed with SIGTERM, defer the input
-	     *      addresses, otherwise fail the addresses and notify the
-	     *      postmaster.  This is probably not a problem that address
-	     *      owners should have to deal with.  If ignore_status is
-	     *      set, then the problem is ignored (except for SIGTERM).
-	     *
-	     * RESOLUTION
-	     *      Time to use your skill in tracking down unusual
-	     *      problems, exept in the case of SIGTERM.
-	     */
-	    char *errstr = xprintf("transport %s: process killed by signal %d",
-				  tp->name, status & 0x7f);
-	    if ((status & 0x7f) == SIGTERM) {
-		insert_addr_list(addr, defer, note_error(ERR_143, errstr));
-		return;
-	    } if (tp->flags & PIPE_IGNORE_STATUS) {
-		/*
-		 * log errors in the system log file, but don't do anything
-		 * about them, if ignore_status is set
-		 */
-		struct addr *cur;
-
-		for (cur = addr; cur; cur = cur->succ) {
-		    write_log(WRITE_LOG_SYS, "note: %s ... %s", cur->in_addr, errstr);
-		}
-	    } else {
-		insert_addr_list(addr,
-				 (tp->flags&PIPE_DEFER_ERRORS)? defer: fail,
-				 note_error(ERR_NPOSTMAST|ERR_143, errstr));
-		return;
-	    }
-	} else {
+    }
+    if (WIFEXITED(status)) {
+	if (WEXITSTATUS(status) != 0) {
 	    /*
 	     * ERR_144 - process returned non-zero status
 	     *
@@ -503,9 +499,8 @@ tpd_pipe(addr, succeed, defer, fail)
 	     *      shell will not be involved and the exit code will most
 	     *      definitely be from the command given.
 	     */
-	    char *errstr =
-		xprintf("transport %s: child returned status %s (%d)",
-			tp->name, strsysexit(status>>8), status>>8);
+	    errstr = xprintf("transport %s: child returned status %s (%d)",
+			     tp->name, strsysexit(WEXITSTATUS(status)), WEXITSTATUS(status));
 
 	    if ((tp->flags & (PIPE_IGNORE_STATUS | PIPE_STATUS_2SENDER)) == PIPE_IGNORE_STATUS) {
 		/*
@@ -524,6 +519,67 @@ tpd_pipe(addr, succeed, defer, fail)
 		return;
 	    }
 	}
+    } else if (WIFSIGNALED(status)) {
+	char signm[SIG2STR_MAX];
+
+	if (sig2str(WTERMSIG(status), signm) == -1) {
+	    sprintf(signm, " %d", WTERMSIG(status));
+	}
+	errstr = xprintf("transport %s: child process %d killed by signal SIG%s %s: %s",
+			 tp->name, pid, signm,
+			 WCOREDUMP(status) ? "and dumped core" : "(no core)",
+			 strsignal(WTERMSIG(status)));
+    } else if (WIFSTOPPED(status)) {
+	char signm[SIG2STR_MAX];
+
+	/* in theory we'll hopefully never see stopped processes... */
+	if (sig2str(WTERMSIG(status), signm) == -1) {
+	    sprintf(signm, " %d", WSTOPSIG(status));
+	}
+	errstr = xprintf("transport %s: child process %d stopped unexpectedly by signal SIG%s: %s",
+			 tp->name, pid, signm, strsignal(WSTOPSIG(status)));
+    }
+    if (!WIFEXITED(status) && (WIFSIGNALED(status) || WIFSTOPPED(status))) {
+	/*
+	 * ERR_143 - process killed/stopped by signal
+	 *
+	 * DESCRIPTION
+	 *      The exit status returned by close_child() revealed that
+	 *      the child process was killed by a signal.  This could be
+	 *      due to the machine being brought down, if the signal is
+	 *      SIGTERM.  Other signals may represent genuine problems.
+	 *
+	 * ACTIONS
+	 *      If the child was killed with SIGTERM, defer the input
+	 *      addresses, otherwise fail the addresses and notify the
+	 *      postmaster.  This is probably not a problem that address
+	 *      owners should have to deal with.  If ignore_status is
+	 *      set, then the problem is ignored (except for SIGTERM).
+	 *
+	 * RESOLUTION
+	 *      Time to use your skill in tracking down unusual
+	 *      problems, exept of course in the case of SIGTERM....
+	 */
+	if (WIFSIGNALED(status) && WTERMSIG(status) == SIGTERM) {
+	    insert_addr_list(addr, defer, note_error(ERR_143, errstr));
+	    return;
+	}
+	if (tp->flags & PIPE_IGNORE_STATUS) {
+	    /*
+	     * log errors in the system log file, but don't do anything
+	     * about them, if ignore_status is set
+	     */
+	    struct addr *cur;
+	    
+	    for (cur = addr; cur; cur = cur->succ) {
+		write_log(WRITE_LOG_SYS, "note: %s ... %s", cur->in_addr, errstr);
+	    }
+	} else {
+	    insert_addr_list(addr,
+			     (tp->flags & PIPE_DEFER_ERRORS) ? defer : fail,
+			     note_error(ERR_NPOSTMAST | ERR_143, errstr));
+	    return;
+	}
     }
 
     if (write_error) {
@@ -604,7 +660,7 @@ get_pipe_env(tp, addr)
     struct transport *tp;		/* transport being used */
     struct addr *addr;			/* addrs being delivered to */
 {
-    static char *pipe_env[50];		/* lots of space for variables */
+    static char *pipe_env[50];		/* lots of space for lots of variables */
     static int inited = FALSE;		/* true if pipe_env is set up */
     char **next_env;			/* for stepping through pipe_env */
     static char **per_address_env;	/* start of per-address environment */
@@ -667,21 +723,28 @@ get_pipe_env(tp, addr)
 	}
 	*next_env++ = xprintf("HOME=%s", cur->home? cur->home: "/");
     } else {
-	cur = addr->parent;
-	if (cur && cur->remainder) {
-		*next_env++ = xprintf("DESTMBOX=%s", cur->remainder);
+	if (addr->parent && addr->parent->remainder) {
+	    *next_env++ = xprintf("DESTMBOX=%s", addr->parent->remainder);
 	}
 	if (addr->next_addr) {
 	    *next_env++ = xprintf("ADDR=%s", addr->next_addr);
 	}
+	if (addr->rem_prefix) {
+	    *next_env++ = xprintf("MBOX_PREFIX=%s", addr->rem_prefix);
+	}
+	if (addr->rem_suffix) {
+	    *next_env++ = xprintf("MBOX_SUFFIX=%s", addr->rem_suffix);
+	}
 	if (addr->next_host) {
 	    *next_env++ = xprintf("HOST=%s", addr->next_host);
 	}
-	*next_env++ = xprintf("HOME=%s", addr->home? addr->home: "/");
+	*next_env++ = xprintf("HOME=%s", addr->home ? addr->home : "/");
     }
 
     /* is there a suitable username to go into the environment */
-    for (cur = addr; cur && ! (cur->flags & ADDR_ISUSER); cur = cur->parent) ;
+    for (cur = addr; cur && ! (cur->flags & ADDR_ISUSER); cur = cur->parent) {
+	;
+    }
     if (cur) {
 	/* yes */
 	register char *user = cur->remainder;
