@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_114:smtplib.c,v 1.48 2001/07/29 23:11:32 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_115:smtplib.c,v 1.70 2003/06/05 20:24:36 woods Exp"
  */
 
 /*
@@ -26,47 +26,65 @@
  * which properly parses these messages in such a way that they will be treated
  * as the single entry that they are.
  */
+
+#define NEED_SOCKETS
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
+#include <setjmp.h>
 #include <signal.h>
 #include <ctype.h>
-#include "defs.h"
 
-#ifdef HAVE_STDLIB_H
+#ifdef STDC_HEADERS
 # include <stdlib.h>
-#endif
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
 
 #include "../smail.h"
+#include "../smailsock.h"
 #include "../main.h"
 #include "../jump.h"
 #include "../parse.h"
 #include "../addr.h"
-#include "../transport.h"
 #include "../spool.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "smtplib.h"
 #include "../extern.h"
 #include "../error.h"
 #include "../debug.h"
 #include "../log.h"
-
-#ifdef ANSI_C
-# define P_(x)  x
-# define VOLATILE volatile
-#else
-# define P_(x)  ()
-# define VOLATILE /**/
-#endif
+#include "../smailport.h"
 
 /* reply code groups, encoded in hex */
 #define POSITIVE_DEBUG		0x000	/* debugging messages */
@@ -117,12 +135,12 @@ static JUMP_ENVBUF timeout_buf;		/* timeouts jump here */
 static char *skip_to_eol __P((char *));
 static void log_malformed_ehlo_reply __P((char *, char *));
 static void do_smtp_shutdown __P((struct smtp *, int));
-static int write_command_maybe_wait __P((struct smtp *, unsigned, char *, int, char **));
-static int wait_write_command __P((struct smtp *, unsigned, char *, int, char **));
-static int write_command_nowait __P((struct smtp *, char *, int));
+static int write_command_maybe_wait __P((struct smtp *, unsigned int, char *, unsigned int, char **));
+static int wait_write_command __P((struct smtp *, unsigned int, char *, unsigned int, char **));
+static int write_command_nowait __P((struct smtp *, char *, unsigned int));
 static int flush_command_stream __P((struct smtp *, char **));
-static int wait_read_response __P((struct smtp *, unsigned, char **));
-static int read_response_internal __P((struct smtp *, unsigned, char **));
+static int wait_read_response __P((struct smtp *, unsigned int, char **));
+static int read_response_internal __P((struct smtp *, unsigned int, char **));
 static void catch_timeout __P((int));
 static struct error * no_remote __P((struct transport *, char *));
 static struct error * try_again __P((struct transport *, char *));
@@ -156,8 +174,8 @@ static struct error * read_failed __P((struct transport *));
  *		to an EHLO command.  This can only occur when
  *		try_ehlo is set (and HAVE_EHLO is defined).
  *
- * For SMTP_FAIL and SMTP_AGAIN (but not SMTP_EHLO_FAIL), return a
- * filled-in error structure.
+ * For SMTP_FAIL, SMTP_NOCONNECT, and SMTP_AGAIN (but not SMTP_EHLO_FAIL),
+ * return a filled-in error structure.
  */
 int
 smtp_startup(smtpb, error_p, try_ehlo)
@@ -209,6 +227,8 @@ smtp_startup(smtpb, error_p, try_ehlo)
     smtpb->esmtp_flags.ESMTP_8bitmime = 0;
     smtpb->esmtp_flags.ESMTP_atrn = 0;
     smtpb->esmtp_flags.ESMTP_auth = 0;
+    smtpb->esmtp_flags.ESMTP_binarymime = 0;
+    smtpb->esmtp_flags.ESMTP_checkpoint = 0;
     smtpb->esmtp_flags.ESMTP_chunking = 0;
     smtpb->esmtp_flags.ESMTP_dsn = 0;
     smtpb->esmtp_flags.ESMTP_enhancedstatuscodes = 0;
@@ -224,6 +244,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
     smtpb->esmtp_flags.ESMTP_starttls = 0;
     smtpb->esmtp_flags.ESMTP_turn = 0;
     smtpb->esmtp_flags.ESMTP_verb = 0;
+    smtpb->esmtp_flags.ESMTP_vrfy = 0;
     smtpb->esmtp_flags.ESMTP_x = 0;
 
 #ifdef HAVE_EHLO
@@ -236,16 +257,25 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	 *	5xx - fatal error   (try HELO, do not bounce!)
 	 */
 	STR_CLEAR(&smtp_out);
-	(void) str_printf(&smtp_out, "EHLO %s", sending_name ? sending_name : primary_name);
+# ifdef HAVE_BSD_NETWORKING
+	if (sending_name && get_inet_addr(sending_name) != INADDR_NONE) {
+	    (void) str_printf(&smtp_out, "EHLO [%s]", sending_name);
+	} else
+# endif
+	{
+	    (void) str_printf(&smtp_out, "EHLO %s", sending_name ? sending_name : primary_name);
+	}
 
 	reply = wait_write_command(smtpb, smtpb->short_timeout,
-				   STR(&smtp_out), (int) STR_LEN(&smtp_out),
+				   STR(&smtp_out), (unsigned int) STR_LEN(&smtp_out),
 				   &reply_text);
 
 	if (reply == REPLY_TIMEOUT || reply == REPLY_EOF) {
-	    /* Some gateways just terminate the conection when they receive an
-	     * EHLO.  We handle this case by returning a special value here
-	     * that can be used to force re-connecting to the buggy host.
+	    /*
+	     * Some broken servers just terminate the conection when they
+	     * receive an EHLO.  As per RFC 1869 4.7 we handle this case by
+	     * returning a special value here that can optionally be used to
+	     * force re-connecting to the buggy host and only issuing a HELO.
 	     */
 	    return SMTP_EHLO_FAIL;
 	}
@@ -255,7 +285,51 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	    return SMTP_NOCONNECT;	/* skip this host... */
 	}
 	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN) {
-	    /* remote SMTP closed, try again later */
+	    /* remote SMTP closing....  try again later */
+	    /*
+	     * Some mis-configured Postfix servers have been known to return
+	     * "402 Error: command not implemented" in response to unknown
+	     * commands.  If such a server lives behind a mis-configured Cisco
+	     * PIX firewall this will happen when we send EHLO and we will
+	     * never be able to establish a successful SMTP session.
+	     *
+	     * The proper fix is to issue a "no fixup protocol smtp 25" command
+	     * to the Cisco (and save the config to make it permanent, of
+	     * course) _AND_ then to also turn off 'soft_bounce' in the Postfix
+	     * server.  If the remote server's administrator(s) only do the
+	     * latter and do not also fix their firewall the then a slightly
+	     * more correct "502 Error:  command not implemented" will be
+	     * returned by Postfix forcing us to forego ESMTP and fall back to
+	     * sending a HELO.
+	     * 
+	     * This is all because the PIX fails to implement ESMTP, and in
+	     * doing so it "helpfully" translates the EHLO command into an
+	     * "XXXX" command.  In combination with the Postfix 'soft_bounce'
+	     * testing feature this causes an irreconcilable deadlock.  The
+	     * ESMTP specification is very nearly a decade old.  The latest
+	     * SMTP RFC (2821) includes EHLO as a mandatory feature by default.
+	     *
+	     * Unfortunately this is not the only SMTP violation committed by
+	     * the PIX.  You can recognize it because it translates all
+	     * characters except '2', '0', and ' ' in the 220 greeting to
+	     * asterisks, thus obliterating the mandatory 220 host name
+	     * parameter as well as any explicit indication of ESMTP support.
+	     *
+	     * The Cisco Firewall Team appear to be stuck back in 1992 or
+	     * earlier....
+	     *
+	     * Postfix postmasters who are forced to live behind a broken PIX
+	     * by lame-brained idiot security officers are never going to be
+	     * able to successfully use 'soft_bounce' for testing (not that
+	     * it's a good idea to use this so-called feature, even just for
+	     * testing -- test the real behaviour, not some broken imitation).
+	     *
+	     * In any case don't ask me to break Smail!  Smail's not the only
+	     * mailer which will refuse to deal with this situation.  At least
+	     * this way you won't have to un-freeze any messages destined to
+	     * the broken site and they'll be delivered almost as soon as the
+	     * remote configurations are fixed.
+	     */
 	    *error_p = try_again(smtpb->tp, reply_text);
 	    return SMTP_AGAIN;		/* he says we should come back later... */
 	}
@@ -275,7 +349,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		 * every line must begin with the three digits "250", followed
 		 * by a '-' for continued lines, and a ' ' for the final line
 		 */
-		if (strncmp(cp, "250", 3) != 0) {
+		if (strncmp(cp, "250", (size_t) 3) != 0) {
 		    log_malformed_ehlo_reply("Not a 250 reply!", reply_text);
 		    return SMTP_SUCCEED;
 		}
@@ -296,7 +370,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		/*
 		 * search for whitespace (or newline) after the keyword
 		 */
-		for (pp = cp; *(pp) && !isspace(*pp); ++pp) {
+		for (pp = cp; *(pp) && !isspace((int) *pp); ++pp) {
 		     ;
 		}
 		/*
@@ -304,7 +378,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		 * by whitespace, not an equal sign!  We'll generously skip
 		 * any whitespace before a newline too...
 		 */
-		while (isspace(*pp)) {
+		while (isspace((int) *pp)) {
 		    ++pp;
 		}
 		/*
@@ -314,9 +388,13 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		    smtpb->esmtp_flags.ESMTP_8bitmime = 1;
 		} else if (strncmpic(cp, "ATRN", sizeof("ATRN")-1) == 0) {
 		    smtpb->esmtp_flags.ESMTP_atrn = 1;
-		} else if (strncmpic(cp, "AUTH", sizeof("AUTH")-1) == 0) {
+		} else if (strncmpic(cp, "AUTH", sizeof("AUTH")-1) == 0 && cp[sizeof("AUTH")-1] == ' ') {
 		    /* XXX followed by a space separated list of the names of supported SASL mechanisms */
 		    smtpb->esmtp_flags.ESMTP_auth = 1;
+		} else if (strncmpic(cp, "BINARYMIME", sizeof("BINARYMIME")-1) == 0) {
+		    smtpb->esmtp_flags.ESMTP_binarymime = 1;
+		} else if (strncmpic(cp, "CHECKPOINT", sizeof("CHECKPOINT")-1) == 0) {
+		    smtpb->esmtp_flags.ESMTP_binarymime = 1;
 		} else if (strncmpic(cp, "CHUNKING", sizeof("CHUNKING")-1) == 0) {
 		    smtpb->esmtp_flags.ESMTP_chunking = 1;
 		} else if (strncmpic(cp, "DSN", sizeof("DSN")-1) == 0) {
@@ -326,9 +404,9 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		} else if (strncmpic(cp, "ETRN", sizeof("ETRN")-1) == 0) {
 		    smtpb->esmtp_flags.ESMTP_etrn = 1;
 		} else if (strncmpic(cp, "EXPN", sizeof("EXPN")-1) == 0) {
-		    ; /* NOP XXX Smail never requests EXPN so we can ignore it in here */
+		    smtpb->esmtp_flags.ESMTP_expn = 1;
 		} else if (strncmpic(cp, "HELP", sizeof("HELP")-1) == 0) {
-		    ; /* NOP XXX smail never requests HELP so we can ignore it in here */
+		    smtpb->esmtp_flags.ESMTP_help = 1;
 		} else if (strncmpic(cp, "ONEX", sizeof("ONEX")-1) == 0) {
 		    smtpb->esmtp_flags.ESMTP_onex = 1;
 		} else if (strncmpic(cp, "PIPELINING", sizeof("PIPELINING")-1) == 0) {
@@ -366,6 +444,8 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		    smtpb->esmtp_flags.ESMTP_turn = 1;
 		} else if (strncmpic(cp, "VERB", sizeof("VERB")-1) == 0) {
 		    smtpb->esmtp_flags.ESMTP_verb = 1;
+		} else if (strncmpic(cp, "VRFY", sizeof("VRFY")-1) == 0) {
+		    smtpb->esmtp_flags.ESMTP_vrfy = 1;
 		} else if (*cp == 'X' || *cp == 'x') {
 		    smtpb->esmtp_flags.ESMTP_x = 1;
 		} else {
@@ -385,28 +465,32 @@ smtp_startup(smtpb, error_p, try_ehlo)
 		}
 		cp = skip_to_eol(cp);
 	    }
-	    DEBUG9(DBG_DRIVER_MID, "destination supports ESMTP%s%s%s%s%s%s%s%s%s",
+	    DEBUG9(DBG_DRIVER_MID, "destination supports %s%s%s%s%s%s%s%s%s",
+		   smtpb->esmtp_flags.ESMTP_basic ? "ESMTP" : "[INTERNAL ERROR!]",
 		   smtpb->esmtp_flags.ESMTP_8bitmime ? " 8BITMIME" : "",
 		   smtpb->esmtp_flags.ESMTP_atrn ? " ATRN" : "",
 		   smtpb->esmtp_flags.ESMTP_auth ? " AUTH" : "",
+		   smtpb->esmtp_flags.ESMTP_binarymime ? " BINARYMIME" : "",
+		   smtpb->esmtp_flags.ESMTP_checkpoint ? " CHECKPOINT" : "",
 		   smtpb->esmtp_flags.ESMTP_chunking ? " CHUNKING" : "",
 		   smtpb->esmtp_flags.ESMTP_dsn ? " DSN" : "",
-		   smtpb->esmtp_flags.ESMTP_enhancedstatuscodes ? " ENHANCEDSTATUSCODES" : "",
+		   smtpb->esmtp_flags.ESMTP_enhancedstatuscodes ? " ENHANCEDSTATUSCODES" : "");
+	    DEBUG9(DBG_DRIVER_MID, "%s%s%s%s%s%s%s%s%s",
 		   smtpb->esmtp_flags.ESMTP_etrn ? " ETRN" : "",
 		   smtpb->esmtp_flags.ESMTP_expn ? " EXPN" : "",
-		   smtpb->esmtp_flags.ESMTP_help ? " HELP" : "");
-	    DEBUG9(DBG_DRIVER_MID, "%s%s%s%s%s%s%s%s%s",
+		   smtpb->esmtp_flags.ESMTP_help ? " HELP" : "",
 		   smtpb->esmtp_flags.ESMTP_onex ? " ONEX" : "",
 		   smtpb->esmtp_flags.ESMTP_pipelining ? " PIPELINING" : "",
 		   smtpb->esmtp_flags.ESMTP_saml ? " SAML" : "",
 		   smtpb->esmtp_flags.ESMTP_send ? " SEND" : "",
 		   smtpb->esmtp_flags.ESMTP_size ? " SIZE" : "",
-		   smtpb->esmtp_flags.ESMTP_soml ? " SOML" : "",
+		   smtpb->esmtp_flags.ESMTP_soml ? " SOML" : "");
+	    DEBUG5(DBG_DRIVER_MID, "%s%s%s%s%s.\n",
 		   smtpb->esmtp_flags.ESMTP_starttls ? " STARTTLS" : "",
 		   smtpb->esmtp_flags.ESMTP_turn ? " TURN" : "",
-		   smtpb->esmtp_flags.ESMTP_verb ? " VERB" : "");
-	    DEBUG1(DBG_DRIVER_MID, "%s.\n",
-		   smtpb->esmtp_flags.ESMTP_x ? " X*" : "");
+		   smtpb->esmtp_flags.ESMTP_verb ? " VERB" : "",
+		   smtpb->esmtp_flags.ESMTP_vrfy ? " VRFY" : "",
+		   smtpb->esmtp_flags.ESMTP_x ? " X*(private)" : "");
 	    /*
 	     * If debugging is turned on high enough, and if the remote server
 	     * advertised "VERB" then ask the remote server for more verbose
@@ -415,7 +499,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	     */
 	    if (smtpb->esmtp_flags.ESMTP_verb && debug >= DBG_DRIVER_MID) {
 		reply = wait_write_command(smtpb, smtpb->short_timeout,
-					   "VERB", sizeof("VERB") - 1,
+					   "VERB", (unsigned int) sizeof("VERB") - 1,
 					   &reply_text);
 		if (REPLY_GROUP(reply) == PSEUDO_TRY_AGAIN) {
 		    /* some other error ... */
@@ -435,7 +519,9 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	    }
 	    return SMTP_SUCCEED;
 	}
-	/* Fall through and try an old-fashioned HELO */ 
+	/*
+	 * Fall through and try an old-fashioned HELO
+	 */
     }
 #endif /* HAVE_EHLO */
     /*
@@ -446,10 +532,17 @@ smtp_startup(smtpb, error_p, try_ehlo)
      *  5xx - fatal error   (return message to sender)
      */
     STR_CLEAR(&smtp_out);
-    (void) str_printf(&smtp_out, "HELO %s", sending_name ? sending_name : primary_name);
+# ifdef HAVE_BSD_NETWORKING
+    if (sending_name && get_inet_addr(sending_name) != INADDR_NONE) {
+	(void) str_printf(&smtp_out, "HELO [%s]", sending_name);
+    } else
+# endif
+    {
+	(void) str_printf(&smtp_out, "HELO %s", sending_name ? sending_name : primary_name);
+    }
 
     reply = wait_write_command(smtpb, smtpb->short_timeout,
-			       STR(&smtp_out), (int) STR_LEN(&smtp_out),
+			       STR(&smtp_out), (unsigned int) STR_LEN(&smtp_out),
 			       &reply_text);
     if (REPLY_GROUP(reply) == PSEUDO_TRY_AGAIN) {
 	/* some other error ... */
@@ -468,6 +561,12 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	return SMTP_FAIL;	/* bouncey... bouncey... */
     }
 #else /* HAVE_EHLO */
+    /*
+     * RFC 1869 4.7 warns that some really broken implementations may not
+     * accept HELO after they have issued a failure response to EHLO, and we
+     * may may try to compensate by sending an RSET before sending the HELO,
+     * and ignore any error response from the RSET.
+     */
     if (reply == REPLY_SEQUENCE_ERROR) {
 	/* log 503 Bad sequence of commands errors */
 	write_log(WRITE_LOG_SYS, "%x after bad EHLO followed by HELO:\n%s", reply, reply_text);
@@ -478,7 +577,7 @@ smtp_startup(smtpb, error_p, try_ehlo)
     }
     if (reply != REPLY_OK) {	/* try RSET and then HELO one more time */
 	reply = wait_write_command(smtpb, smtpb->short_timeout,
-				   "RSET", sizeof("RSET") - 1,
+				   "RSET", (unsigned int) sizeof("RSET") - 1,
 				   &reply_text);
 	if (REPLY_GROUP(reply) == PSEUDO_TRY_AGAIN) {
 	    /* some other error ... */
@@ -490,9 +589,11 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	    *error_p = try_again(smtpb->tp, reply_text);
 	    return SMTP_AGAIN;
 	} else if (reply != REPLY_OK) {
-	    /* Some very buggy gateways don't accept any commands before they
-	     * get a HELO, not even RSET.  Fortunately it is usually safe to
-	     * ignore the error messages.
+	    /*
+	     * Some extremely buggy servers don't accept any commands before
+	     * they get a HELO, not even RSET.  Fortunately they are usually so
+	     * buggy that it is safe to just ignore their error messages and
+	     * simply carry on....
 	     */
 	    write_log(WRITE_LOG_SYS, "unexpected response to RSET:\n%s", reply_text);
 	}
@@ -500,10 +601,18 @@ smtp_startup(smtpb, error_p, try_ehlo)
 	 * the RSET command has been sent; try the old HELO again...
 	 */
 	STR_CLEAR(&smtp_out);
-	(void) str_printf(&smtp_out, "HELO %s", sending_name ? sending_name : primary_name);
+# ifdef HAVE_BSD_NETWORKING
+	/* if sending_name is a literal IP address, send it in []'s.... */
+	if (sending_name && get_inet_addr(sending_name) != INADDR_NONE) {
+	    (void) str_printf(&smtp_out, "HELO [%s]", sending_name);
+	} else
+# endif
+	{
+	    (void) str_printf(&smtp_out, "HELO %s", sending_name ? sending_name : primary_name);
+	}
 
 	reply = wait_write_command(smtpb, smtpb->short_timeout,
-				   STR(&smtp_out), (int) STR_LEN(&smtp_out),
+				   STR(&smtp_out), (unsigned int) STR_LEN(&smtp_out),
 				   &reply_text);
 	if (REPLY_GROUP(reply) == PSEUDO_TRY_AGAIN) {
 	    /* some other error */
@@ -521,10 +630,10 @@ smtp_startup(smtpb, error_p, try_ehlo)
     }
 #endif
     if (reply != REPLY_OK) {
-	/* fatal error, return message to sender */
+	/* some other fatal error, return message to sender */
 	*error_p = fatal_error(smtpb->tp, reply_text);
-	smtp_shutdown(smtpb);		/* be nice and say goodbye! */
-	return SMTP_FAIL;
+	smtp_shutdown(smtpb);	/* be nice and say goodbye! */
+	return SMTP_FAIL;	/* bouncey... bouncey.... */
     }
 
     /* connection established */
@@ -638,7 +747,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
      * to quote it as necessary.  We'll do this always whether we use the value
      * or not just to keep the logic here simple...
      */
-    local_part = quote_local_part(((addr->owner) ? addr->owner : get_sender_addr(tp)));
+    local_part = rfc821_quote_local_part(((addr->owner) ? addr->owner : get_sender_addr(tp)));
 
     str_printf(&smtp_out, "MAIL FROM:<%s%s%s>",
                error_sender ? ((islocal && tp->flags & LOCAL_XFORM) ? "+" : "") : local_part,
@@ -665,6 +774,12 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
      * idea either, but is a common practice.  Sigh.  We'll just avoid saying
      * what we're sending....
      */
+    /*
+     * XXX in theory we should only set the body type to 8BITMIME if that's how
+     * we received it.  We'd need a command-line flag to store in the queue
+     * file to do this though.  Similary we should only claim 7BIT if we know
+     * there are no high-bit chars in the body.
+     */
     if (smtpb->esmtp_flags.ESMTP_8bitmime) {
 	str_printf(&smtp_out, " BODY=8BITMIME");
     } else {
@@ -674,7 +789,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
 
     DEBUG(DBG_DRIVER_MID, "\n");	/* we're about to get more debug */
     reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-				     STR(&smtp_out), (int) STR_LEN(&smtp_out),
+				     STR(&smtp_out), (unsigned int) STR_LEN(&smtp_out),
 				     &reply_text);
     /*
      * Possible responses:
@@ -710,10 +825,10 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
 	 *  5xx - failure	       (return message to sender)
 	 */
 	STR_CLEAR(&smtp_out);
-	str_printf(&smtp_out, "RCPT TO:<%s>", quote_local_part(cur->next_addr));
+	str_printf(&smtp_out, "RCPT TO:<%s>", rfc821_quote_local_part(cur->next_addr));
 
 	reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-					 STR(&smtp_out), (int) STR_LEN(&smtp_out),
+					 STR(&smtp_out), (unsigned int) STR_LEN(&smtp_out),
 					 &reply_text);
 
 	if (REPLY_GROUP(reply) == NEGATIVE_TRY_AGAIN || REPLY_GROUP(reply) == PSEUDO_TRY_AGAIN) {
@@ -806,6 +921,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
 	}
     }
     if (!smtpb->esmtp_flags.ESMTP_pipelining && !okay) {
+	*error_p = fatal_error(tp, reply_text);
 	do_smtp_shutdown(smtpb, REPLY_TRANS_FAILURE);
 	return FAIL;
     }
@@ -819,7 +935,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
      *  5xx - fatal error        (return message to sender)
      */
     reply = write_command_maybe_wait(smtpb, smtpb->long_timeout,
-				     "DATA", sizeof("DATA") - 1,
+				     "DATA", (unsigned int) sizeof("DATA") - 1,
 				     &reply_text);
 
     /*
@@ -1029,7 +1145,7 @@ smtp_send(smtpb, addr, succeed, defer, fail, defer_failure, error_p)
 	return FAIL;
     }
 
-    insert_addr_list(okay, succeed, (struct error *)NULL);
+    insert_addr_list(okay, succeed, (struct error *) NULL);
     return SUCCEED;
 }
 
@@ -1044,7 +1160,7 @@ smtp_shutdown(smtpb)
     char *reply_text;			/* where to store reply text */
 
     (void) wait_write_command(smtpb, smtpb->short_timeout,
-			      "QUIT", sizeof("QUIT") - 1, &reply_text);
+			      "QUIT", (unsigned int) sizeof("QUIT") - 1, &reply_text);
 }
 
 /*
@@ -1071,9 +1187,9 @@ do_smtp_shutdown(smtpb, reply)
 static int
 write_command_maybe_wait (smtpb, timeout, text, len, reply_text)
     struct smtp *smtpb;			/* SMTP description block */
-    unsigned timeout;			/* read timeout */
+    unsigned int timeout;		/* read timeout */
     char *text;				/* text of command */
-    register int len;			/* length of command */
+    register unsigned int len;		/* length of command */
     char **reply_text;			/* text of response from remote */
 {
     if (smtpb->esmtp_flags.ESMTP_pipelining) {
@@ -1094,7 +1210,7 @@ wait_write_command(smtpb, timeout, text, len, reply_text)
     struct smtp *smtpb;			/* SMTP description block */
     unsigned timeout;			/* read timeout */
     char *text;				/* text of command */
-    register int len;			/* length of command */
+    register unsigned int len;		/* length of command */
     char **reply_text;			/* text of response from remote */
 {
     int reply;
@@ -1120,7 +1236,7 @@ static int
 write_command_nowait(smtpb, text, len)
     struct smtp *smtpb;			/* SMTP description block */
     char *text;				/* text of command */
-    register int len;			/* length of command */
+    register unsigned int len;		/* length of command */
 {
     register FILE *f = smtpb->out;
     register char *cp;
@@ -1139,7 +1255,7 @@ write_command_nowait(smtpb, text, len)
 #endif	/* NODEBUG */
 
     /* send out the command */
-    for (cp = text; len; cp++, --len) {
+    for (cp = text; len > 0; cp++, --len) {
 	putc(*cp, f);
     }
 
@@ -1194,19 +1310,26 @@ wait_read_response(smtpb, timeout, reply_text)
     return result;
 }
 
+#if ((__STDC__ - 0) <= 0)
+/* most traditional compilers won't optimize global variables */
+JUMPSIG save_sig;			/* previous alarm signal catcher */
+unsigned save_alarm;			/* previous alarm value */
+#else
+static volatile JUMPSIG save_sig;	/* previous alarm signal catcher */
+static volatile unsigned save_alarm;	/* previous alarm value */
+#endif
+
 /*
  * WARNING: do not put newlines in any faked reply_text messgaes!!!!
  */
 static int
 read_response_internal(smtpb, timeout, reply_text)
     struct smtp *smtpb;			/* SMTP description block */
-    unsigned timeout;			/* read timeout */
+    unsigned int timeout;			/* read timeout */
     char **reply_text;			/* return text of response here */
 {
     register int lstart;		/* offset to start of an input line */
     int reply_code;			/* SMTP reply code value */
-    VOLATILE unsigned save_alarm;	/* previous alarm value */
-    VOLATILE JUMPSIG save_sig;		/* previous alarm signal catcher */
     register int c;
     register char *rp;
 
@@ -1243,9 +1366,9 @@ read_response_internal(smtpb, timeout, reply_text)
 	STR_NEXT(&smtp_in, '\n');
 	rp = STR(&smtp_in);		/* get current pointer */
     } while (c != EOF &&
-	     isdigit(rp[lstart]) &&
-	     isdigit(rp[lstart + 1]) &&
-	     isdigit(rp[lstart + 2]) &&
+	     isdigit((int) rp[lstart]) &&
+	     isdigit((int) rp[lstart + 1]) &&
+	     isdigit((int) rp[lstart + 2]) &&
 	     rp[lstart + 3] == '-');
 
     /* replace last newline with a NUL byte */
@@ -1265,17 +1388,17 @@ read_response_internal(smtpb, timeout, reply_text)
      * finally it's safe to see if there was a read error....
      */
     if (c == EOF) {
-	*reply_text = xprintf("%x read error from remote SMTP process", REPLY_EOF);
+	*reply_text = xprintf("%x read error from remote SMTP process (connection closed unexpectedly)", REPLY_EOF);
 	return REPLY_EOF;
     }
     /* .... or other protocol error */
     rp = STR(&smtp_in);
-    if (! isdigit(rp[lstart]) ||
-	! isdigit(rp[lstart + 1]) ||
-	! isdigit(rp[lstart + 2]) ||
-	! isspace(rp[lstart + 3]))
+    if (! isdigit((int) rp[lstart]) ||
+	! isdigit((int) rp[lstart + 1]) ||
+	! isdigit((int) rp[lstart + 2]) ||
+	! isspace((int) rp[lstart + 3]))
     {
-	*reply_text = xprintf("%x protocol error in reply from remote SMTP process", REPLY_PROTO_ERROR);
+	*reply_text = xprintf("%x protocol error (no response code) in reply from remote SMTP process", REPLY_PROTO_ERROR);
 	return REPLY_PROTO_ERROR;
     }
 
@@ -1289,7 +1412,7 @@ read_response_internal(smtpb, timeout, reply_text)
 
 
 /*
- *	is_dot_string() - is a string compliant with RFC 821 dot-string?
+ *	rfc821_is_dot_string() - is a string compliant with RFC 821 dot-string?
  *
  * in RFC 821 a dot-string must not contain any <special> or <SP>, other than
  * "." of course, unless it's escaped with a "\", and with no double "."s and
@@ -1320,23 +1443,29 @@ read_response_internal(smtpb, timeout, reply_text)
  * Note the new RFC 2821 (which replaces 821) seems to concur with these rules.
  */
 int
-is_dot_string(s)
+rfc821_is_dot_string(s)
     char *s;
 {
     int c;
 
     while ((c = *s++)) {
 	if (c == '\\') {
-	    s++;				/* anything goes! */
+	    if (!isascii((int) *s)) {
+		return FALSE;
+	    }
+	    s++;				/* anything else goes! */
 	    continue;
 	}
 	if (c == '<' || c == '>' || c == '(' || c == ')' || c == '[' || c == ']' ||
 	    c == '\\' || c == ',' || c == ';' || c == ':' || c == '@' || c == '"' ||
-	    c == ' ' || (c == '.' && *s == '.') || iscntrl(c)) {
+	    c == ' ' || (c == '.' && *s == '.') || iscntrl((int) c)) {
+	    return FALSE;
+	}
+	if (!isascii(c)) {
 	    return FALSE;
 	}
     }
-    if (*(s-2) == '.') {
+    if (*(s-2) == '.') {			/* trailing-dot? */
 	return FALSE;
     }
 
@@ -1344,7 +1473,7 @@ is_dot_string(s)
 }
 
 /*
- *	is_quoted_string() - is a string compliant with RFC 821 quoted-string?
+ *	rfc821_is_quoted_string() - is a string compliant with RFC 821 quoted-string?
  *
  * in RFC-821 a quoted local part may be any one of the 128 ASCII
  * characters except <CR>, <LF>, quote ("), or backslash (\);
@@ -1364,7 +1493,7 @@ is_dot_string(s)
  *          <x> ::= any one of the 128 ASCII characters (no exceptions)
  */
 int
-is_quoted_string(s)
+rfc821_is_quoted_string(s)
     char *s;
 {
     int c;
@@ -1374,12 +1503,18 @@ is_quoted_string(s)
     }
     while ((c = *s++)) {
 	if (c == '\\') {
-	    s++;				/* anything goes! */
+	    if (!isascii((int) *s)) {
+		return FALSE;
+	    }
+	    s++;				/* anything else goes! */
 	    continue;
 	}
 	if (c == '\r' || c == '\n' || c == '\\') {
 	    return FALSE;
 	}
+	if (!isascii(c)) {
+	    return FALSE;
+	}
     }
     if (*(s-2) != '"') {
 	return FALSE;
@@ -1389,7 +1524,7 @@ is_quoted_string(s)
 }
 
 /*
- *	make_quoted_string() - make quoted-string from local-part
+ *	rfc821_make_quoted_string() - make quoted-string from local-part
  *
  * Put quotes around `s', and insert a backslash before characters that need
  * quoting when they part of a quoted-string.
@@ -1397,7 +1532,7 @@ is_quoted_string(s)
  * `at' might point to a domain-part, and if so we tack that onto the end.
  */
 char *
-make_quoted_string(s, at)
+rfc821_make_quoted_string(s, at)
     char *s;
     char *at;
 {
@@ -1415,14 +1550,17 @@ make_quoted_string(s, at)
     }
     STR_NEXT(&dst, '"');
     for (cp = s; (ch = *cp); cp++) {
-        if (ch > 127 || ch == '\r' || ch == '\n' || ch == '"' || ch == '\\') {
-            STR_NEXT(&dst, '\\');
+	if (ch > 127 || ch == '\r' || ch == '\n' || ch == '"' || ch == '\\') {
+	    STR_NEXT(&dst, '\\');
 	}
-        STR_NEXT(&dst, ch);
+	STR_NEXT(&dst, ch);
     }
     STR_NEXT(&dst, '"');
     if (at) {
-	STR_CAT(&dst, at);
+	STR_NEXT(&dst, '@');
+	STR_CAT(&dst, at);			/* append domain, and NUL */
+    } else {
+	STR_NEXT(&dst, '\0');			/* just NUL-terminate */
     }
     STR_DONE(&dst);
 
@@ -1431,28 +1569,29 @@ make_quoted_string(s, at)
 
 
 /*
- *	quote_local_part() - quote local part of address according to rfc 821
+ *	rfc821_quote_local_part() - quote local part of address according to rfc 821
  *
- * According to RFC 821, a local-part is a dot-string or a quoted-string.
- * We first see if the local-part is a dot-string.  If it is not, we turn
- * it into a quoted-string. Anything else would be too painful.
+ * According to RFC 821, a local-part is a dot-string or a quoted-string.  We
+ * first see if the local-part is a dot-string or a quoted-string.  If it is
+ * not, we just turn it into a quoted-string and be done with it.
  *
  * `lp' might be just a mailbox, or might be a fully qualified address....
  */
 char *
-quote_local_part(lp)
-    char *lp;
+rfc821_quote_local_part(lp)
+    char *lp;				/* local part string (writable) */
 {
-    char *at = NULL;
+    char *at = NULL;			/* pointer to any '@' char... */
 
-    if ((at = strchr(lp, '@'))) {
-	*at = '\0';
+    /* XXX NOTE: we don't allow route-addrs! */
+    if ((at = strrchr(lp, '@'))) {
+	*at = '\0';			/* XXX: should COPY_STRING() */
     }
-    if (!is_dot_string(lp)) {
-	lp = make_quoted_string(lp, at ? at++ : (char *) NULL);
+    if (!rfc821_is_dot_string(lp) && !rfc821_is_quoted_string(lp)) {
+	lp = rfc821_make_quoted_string(lp, at ? at++ : (char *) NULL);
     }
     if (at) {
-	*at = '@';
+	*at = '@';			/* put it back in the orig string */
     }
 
     return lp;
@@ -1543,7 +1682,7 @@ fatal_error(tp, reply_text)
      *	    or the message originator.
      *
      * RESOLUTION
-     *      The resolution depends upon the error message.  See RFC822
+     *      The resolution depends upon the error message.  See RFC821
      *	    for details.
      */
     error_text = xprintf("fatal error from %s transport:\n%s", tp->name, reply_text);
