@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:smtplib.h,v 1.13 2001/07/28 17:37:01 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:smtplib.h,v 1.15 2003/02/26 19:53:56 woods Exp"
  */
 
 /*
@@ -27,27 +27,34 @@
  * requires that we be able to count in binary and understand binary logic
  * idioms.  Bit-fields are the best way to represent flags (except for when you
  * want to mass-assign them all to one value! :-).
+ *
+ * The current official list of SMTP Service Extension keywords is here:
+ *
+ * 	http://www.iana.org/assignments/mail-parameters
  */
 typedef struct esmtpflags {
     unsigned int ESMTP_basic		: 1; /* got EHLO 250 reply */
     unsigned int ESMTP_8bitmime		: 1; /* rfc1652 */
     unsigned int ESMTP_atrn		: 1; /* rfc2645 */
     unsigned int ESMTP_auth		: 1; /* rfc2645 */
-    unsigned int ESMTP_chunking		: 1; /* rfc1830 */
+    unsigned int ESMTP_binarymime	: 1; /* rfc3030 */
+    unsigned int ESMTP_checkpoint	: 1; /* rfc1845 */
+    unsigned int ESMTP_chunking		: 1; /* rfc3030 */
     unsigned int ESMTP_dsn		: 1; /* rfc1891 */
     unsigned int ESMTP_enhancedstatuscodes : 1;	/* rfc2034 */
     unsigned int ESMTP_etrn		: 1; /* rfc1985 */
-    unsigned int ESMTP_expn		: 1; /* rfc821+rfc1869 */
-    unsigned int ESMTP_help		: 1; /* rfc821+rfc1869 */
+    unsigned int ESMTP_expn		: 1; /* optional cmd, as per rfc[2]821+rfc1869 */
+    unsigned int ESMTP_help		: 1; /* optional cmd, as per rfc[2]821+rfc1869 */
     unsigned int ESMTP_onex		: 1; /* One message transaction only [Eric Allman] */
-    unsigned int ESMTP_pipelining	: 1; /* rfc2197 */
-    unsigned int ESMTP_saml		: 1; /* rfc821+rfc1869 */
-    unsigned int ESMTP_send		: 1; /* rfc821+rfc1869 */
+    unsigned int ESMTP_pipelining	: 1; /* rfc2920 */
+    unsigned int ESMTP_saml		: 1; /* optional cmd, as per rfc[2]821+rfc1869 */
+    unsigned int ESMTP_send		: 1; /* optional cmd , as per rfc[2]821+rfc1869 */
     unsigned int ESMTP_size		: 1; /* rfc1870 */
-    unsigned int ESMTP_soml		: 1; /* rfc821+rfc1869 */
-    unsigned int ESMTP_starttls		: 1; /* rfc2487 */
-    unsigned int ESMTP_turn		: 1; /* rfc821+rfc1869 */
+    unsigned int ESMTP_soml		: 1; /* optional cmd, as per rfc[2]821+rfc1869 */
+    unsigned int ESMTP_starttls		: 1; /* rfc3207 */
+    unsigned int ESMTP_turn		: 1; /* optional cmd, as per rfc[2]821+rfc1869 */
     unsigned int ESMTP_verb		: 1; /* Verbose [Eric Allman] */
+    unsigned int ESMTP_vrfy		: 1; /* not optional, but some think it is */
     unsigned int ESMTP_x		: 1; /* unspported, non-standard, unregistered */
 } esmtpf_t;
 
@@ -73,7 +80,7 @@ struct smtp {
 extern int smtp_startup __P((struct smtp *, struct error **, int));
 extern int smtp_send __P((struct smtp *, struct addr *, struct addr **, struct addr **, struct addr **, int, struct error **));
 extern void smtp_shutdown __P((struct smtp *));
-extern int is_dot_string __P((char *));
-extern int is_quoted_string __P((char *));
-extern char *make_quoted_string __P((char *, char *));
-extern char *quote_local_part __P((char *));
+extern int rfc821_is_dot_string __P((char *));
+extern int rfc821_is_quoted_string __P((char *));
+extern char *rfc821_make_quoted_string __P((char *, char *));
+extern char *rfc821_quote_local_part __P((char *));
