@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/transports:RELEASE-3_2_0_114:tcpsmtp.c,v 1.57 2000/12/02 19:38:49 woods Exp"
+#ident	"@(#)smail/src/transports:RELEASE-3_2_0_115:tcpsmtp.c,v 1.71 2003/06/18 07:02:57 woods Exp"
  */
 
 /*
@@ -12,38 +12,79 @@
 
 /*
  * tcpsmtp.c:
- *      Send mail using the SMTP protocol.
+ *      Send mail using the SMTP protocol over TCP/IP.
  *
  * See note in smtplib.c regarding logging of SMTP reply text.
  */
 
-#define NEED_SOCKETS
+#include "defs.h"
+
+/*
+ * Compilation of this entire file depends on "HAVE_BSD_NETWORKING".
+ */
+#ifdef HAVE_BIND
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
-#if defined(POSIX_OS)
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
+
+#ifdef STDC_HEADERS
+# include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
 # endif
-#endif	/* not POSIX_OS */
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 
 #include "../smail.h"
+#include "../smailsock.h"
 #include "../main.h"
 #include "../parse.h"
 #include "../addr.h"
 #include "../bindsmtpth.h"
-#include "../transport.h"
 #include "../route.h"
 #include "../spool.h"
 #include "../exitcodes.h"
 #include "../lookup.h"
 #include "../alloc.h"
+#include "../list.h"
+#include "../smailstring.h"
 #include "../dys.h"
+#include "../transport.h"
 #include "../smailconf.h"
 #include "tcpsmtp.h"
 #include "smtplib.h"
@@ -51,6 +92,7 @@
 #include "../error.h"
 #include "../debug.h"
 #include "../log.h"
+#include "../smailport.h"
 
 /* functions local to this file */
 #ifdef HAVE_BIND
@@ -62,15 +104,15 @@ static char *set_short_timeout __P((char *, struct attribute *));
 static char *set_long_timeout __P((char *, struct attribute *));
 static char *set_timeout __P((unsigned int *, struct attribute *));
 static int tcpsmtp_connect __P((char *, struct in_addr *, int, char *, char **));
-static struct error *connect_failure __P((struct transport *, char *));
-static struct transport_hints *fake_addr_hints __P((char *, char **));
+static struct error *connect_failure __P((struct transport *, const char *));
+static struct transport_hints *fake_addr_hints __P((char *, const char **));
 
 static struct attr_table tcpsmtp_attributes[] = {
-    { "short_timeout", t_proc, NULL, (tup *)set_short_timeout, 0 },
-    { "long_timeout", t_proc, NULL, (tup *)set_long_timeout, 0 },
-    { "service", t_string, NULL, NULL, OFFSET(tcpsmtp_private, service) },
+    { "short_timeout", t_proc, 0, NULL, (tup *)set_short_timeout, 0 },
+    { "long_timeout", t_proc, 0, NULL, (tup *)set_long_timeout, 0 },
+    { "service", t_string, 0, NULL, NULL, OFFSET(tcpsmtp_private, service) },
 #ifdef HAVE_BIND
-    { "use_bind", t_boolean, NULL, NULL, TCPSMTP_USE_BIND },
+    { "use_bind", t_boolean, 0, NULL, NULL, TCPSMTP_USE_BIND },
     BIND_ATTRIBUTES(tcpsmtp_private, bindlib_attr),
 #endif
 };
@@ -95,14 +137,13 @@ tpd_tcpsmtp(addr, succeed, defer, fail)
     struct transport_hints *mx_hints;
     struct ipaddr_hint *ip_addr;
     char *service;
-    char *error_text = NULL;
+    const char *error_text = NULL;
     struct error *lckerr = NULL;	/* lock error structure */
     struct error *connerr = NULL;	/* temporary for any connection error */
     struct error *primary_err = NULL;	/* error from primary MX, if any */
     struct tcpsmtp_private *priv;
     time_t started = 0;			/* connect start time for retry_host_unlock() */
     int defer_failure = TRUE;		/* always reset by retry_host_lock()... */
-    int never_tried = TRUE;		/* set to false if we call tcpsmtp_internal() */
 
     priv = (struct tcpsmtp_private *) addr->transport->private;
     
@@ -177,7 +218,7 @@ tpd_tcpsmtp(addr, succeed, defer, fail)
 	int tmp_result = SMTP_AGAIN;
 
 	if (!EQ(mx_hints->hint_name, "mx")) {
-	    continue;		/* ignore all but MX RRs */
+	    continue;		/* ignore all but MX RRs (real or fake) */
 	}
 
 	/*
@@ -227,43 +268,45 @@ tpd_tcpsmtp(addr, succeed, defer, fail)
 		if (!primary_err) {
 		    primary_err = lckerr;
 		}
-		/* remember if we've been unable to call tcpsmtp_internal() */
 		retry_host_unlock((time_t) 0, lckerr);
 
-		continue;		/* keep on truckin' with the next one */
+		continue;		/* keep on truckin', try the next one */
 	    }
 	    /*
 	     * if the target's retry file was successfully locked, then let's
 	     * try to send the messsage...
 	     */
+	    connerr = NULL;
 	    (void) time(&started);
 	    tmp_result = tcpsmtp_internal(addr, succeed, defer, fail,
 					  ip_addr->hostname, &ip_addr->addr,
 					  AF_INET, service, defer_failure,
 					  &connerr);
-	    /*
-	     * remember if we've called tcpsmtp_internal() so that we don't try
-	     * to re-insert the address list into defer or fail!
-	     */
-	    never_tried = FALSE;
 	    if ((!primary_err || primary_err == lckerr) && connerr) {
-		primary_err = connerr;	/* remember the erorr for primary MX */
+		/*
+		 * Remember the error for the first connection attempt as this
+		 * is the one we'll log, overriding any lckerr saved above.
+		 * Norally this will be for the primary MX host, unless it just
+		 * failed or has recently failed and thus was passed over by
+		 * the retry lock.
+		 */
+		primary_err = connerr;
 	    }
 	    switch (tmp_result) {
 	    case SMTP_SUCCEED:
-		/* pass a NULL error so the retry file is removed */
+		/* explicitly pass a NULL error so the retry file is removed */
 		retry_host_unlock(started, (struct error *) NULL);
 		return;
 		/* NOTREACHED */
 
 	    case SMTP_FAIL:		/* we've been told to go away */
 		retry_host_unlock(started, connerr);
-		return;
+		return;			/* we will probably bounce it... */
 		/* NOTREACHED */
 
 	    case SMTP_AGAIN:		/* we've been told to come back later */
 		retry_host_unlock(started, connerr);
-		return;
+		return;			/* so don't bother with any lesser MX */
 		/* NOTREACHED */
 
 	    case SMTP_NOCONNECT:
@@ -274,21 +317,19 @@ tpd_tcpsmtp(addr, succeed, defer, fail)
 # undef mx_hint
     }
     /*
-     * If we've never called tcpsmtp_internal(), or if it returneed
-     * SMTP_NOCONNECT, then we still have to do something to manage the error
+     * If we've never called tcpsmtp_internal(), or if it returned
+     * SMTP_NOCONNECT, then we still have to do something to record the error
      * for these recipients....
+     *
+     * If we haven't yet set primary_err then all of the mx hints were either
+     * not proper MX RRs, or none had (valid) IP addresses.
+     *
+     * This shouldn't be possible, but you never know....
      */
     if (!primary_err) {
-	if (connerr) {
-	    primary_err = connerr;
-	    if (!defer_failure) {
-		primary_err->info |= ERR_NSOWNER;
-	    }
-	} else {
-	    error_text = xprintf("failed to find any MX hosts or IP addresses for %s", addr->next_host);
-	    primary_err = connect_failure(addr->transport, error_text);
-	    defer_failure = FALSE;	/* force the bounce... */
-	}
+	error_text = xprintf("failed to find any MX hosts with valid IP addresses for %s", addr->next_host);
+	primary_err = connect_failure(addr->transport, error_text);
+	defer_failure = FALSE;	/* force the bounce right away... */
     }
     insert_addr_list(addr, defer_failure ? defer : fail, primary_err);
 
@@ -412,7 +453,9 @@ tcpsmtp_bind_addr(addr, defer, fail)
  *	SMTP_NOCONNECT if another MX host should be tried (or the connection
  *	should be retried later)
  *
- *		- "addr" is left alone.
+ *		- "addr" is left alone (i.e. defer_failure is ignored).
+ *
+ *	ep is always set if return != SMTP_SUCCEED
  */
 static int
 tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service, defer_failure, ep)
@@ -499,9 +542,7 @@ tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service,
 	    break;
 
 	case SMTP_AGAIN:		/* server says try again */
-	    DEBUG1(DBG_DRIVER_LO,
-		   "\n  SMTP startup failed with a temporary SMTP rejection:\n    %s\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+	    DEBUG1(DBG_DRIVER_LO, "\n  SMTP startup failed with a temporary SMTP rejection:\n    %s\n", (*ep)->message);
 	    (void) fclose(smtpbuf.in);
 	    (void) fclose(smtpbuf.out);
 	    insert_addr_list(addr, defer_failure ? defer : fail, *ep);
@@ -510,9 +551,7 @@ tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service,
 	    /* NOTREACHED */
 
 	case SMTP_NOCONNECT:		/* read timeout or disconnect */
-	    DEBUG1(DBG_DRIVER_LO,
-		   "\n  link broken by:\n    %s\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+	    DEBUG1(DBG_DRIVER_LO, "\n  link broken by:\n    %s\n", (*ep)->message);
 	    (void) fclose(smtpbuf.in);
 	    (void) fclose(smtpbuf.out);
 
@@ -520,23 +559,18 @@ tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service,
 	    /* NOTREACHED */
 
 	case SMTP_EHLO_FAIL:		/* server hung up on us! */
-	    DEBUG1(DBG_DRIVER_LO,
-		   "\n  link broken by EHLO:\n    %s\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+	    DEBUG1(DBG_DRIVER_LO, "\n  link to '%s' broken by EHLO\n", hostname);
 	    (void) fclose(smtpbuf.in);
 	    (void) fclose(smtpbuf.out);
 	    /* write a message to the log file because it is interesting
 	     * to know which mailers exhibit this bug....
 	     */
-	    write_log(WRITE_LOG_SYS, "link broken to '%s' by EHLO!", hostname);
+	    write_log(WRITE_LOG_SYS, "link to '%s' broken by EHLO!", hostname);
 
 	    break;			/* back around to try again without EHLO */
-	    /* NOTREACHED */
 
 	case SMTP_FAIL:			/* remote server says "no!"... */
-	    DEBUG1(DBG_DRIVER_LO,
-		   "\n  SMTP startup failed with permanent SMTP rejection:\n    %s\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+	    DEBUG1(DBG_DRIVER_LO, "\n  SMTP startup failed with permanent SMTP rejection:\n    %s\n", (*ep)->message);
 	    (void) fclose(smtpbuf.in);
 	    (void) fclose(smtpbuf.out);
 	    /* bouncy... bouncy... */
@@ -548,12 +582,12 @@ tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service,
 	default:			/* defensive programming! */
 	    DEBUG1(DBG_DRIVER_LO,
 		   "\n  WARNING!  INTERNAL ERROR DETECTED:\n    %s.\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+		   (*ep && (*ep)->message) ? (*ep)->message : "(no error message given)");
 	    (void) fclose(smtpbuf.in);
 	    (void) fclose(smtpbuf.out);
 	    write_log(WRITE_LOG_PANIC, "IMPOSSIBLE result (%d) from smtp_startup() for %s: %s",
 		      result, hostname,
-		      *ep ? (*ep)->message : "(no error message given)");
+		      (*ep && (*ep)->message) ? (*ep)->message : "(no error message given)");
 	    insert_addr_list(addr, defer_failure ? defer : fail, *ep);
 
 	    return SMTP_AGAIN;
@@ -572,8 +606,12 @@ tcpsmtp_internal(addr, succeed, defer, fail, hostname, ipaddr, family, service,
 	    smtp_shutdown(&smtpbuf);
 	    DEBUG(DBG_DRIVER_LO, " done.\n")
 	} else {
-	    DEBUG1(DBG_DRIVER_LO, "\n    delivery FAILED: %s\n",
-		   *ep ? (*ep)->message : "(no error message given)");
+	    /*
+	     * note we are not defensive here -- the code in smtp_send() is
+	     * quite simple to verify manually since it can only return SUCCEED
+	     * or FAIL; and in this case we only dereference when debugging.
+	     */
+	    DEBUG1(DBG_DRIVER_LO, "\n    delivery FAILED: %s\n", (*ep)->message);
 	}
     }
 
@@ -676,36 +714,25 @@ set_long_timeout(struct_p, attr)
 
 static char *
 set_timeout(timeout, attr)
-    unsigned *timeout;                  /* set this timeout variable */
-    struct attribute *attr;             /* parsed attribute */
+    unsigned int *timeout;		/* set this timeout variable */
+    struct attribute *attr;		/* parsed attribute */
 {
     long l;
 
-    if (attr->value == on) {
-        return xprintf("%s: boolean form for non-boolean attribute",
-                       attr->name);
-    }
     l = ivaltol(attr->value);
     if (l < 0) {
-        return xprintf("%s: %s: malformed interval",
-                       attr->name, attr->value);
+	return xprintf("%s: %s: malformed interval",
+		       attr->name, attr->value);
     }
     *timeout = (unsigned)l;
-    if (*timeout != l) {
+    if (*timeout != (unsigned int) l) {
         return xprintf("%s: %s: interval too large", attr->name, attr->value);
     }
+
     return NULL;
 }
 
 
-/*
- * import h_errno; many systems don't define it in <netdb.h>
- */
-
-#if defined(HAVE_BIND) && !defined(OBSOLETE_RESOLVER)
-extern int h_errno;
-#endif
-
 /*
  * tcpsmtp_connect - return a socket connected to the remote host
  */
@@ -718,21 +745,22 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
     char *service;
     char **error;
 {
-    static int port = 0;                /* port to connect to */
+    static unsigned short port = 0;	/* port to connect to (XXX in_port_t) */
     struct servent *smtp_service;       /* service entry */
     struct sockaddr_in saddrin;		/* inet socket address */
     static char *save_error = NULL;     /* keep handle to free error msgs */
     int s;                              /* socket */
     char *error_text;
 
-    if (isdigit(*service)) {
+    if (isdigit((int) *service)) {
         error_text = NULL;
-        port = c_atol(service, &error_text);
+        port = htons((unsigned short) c_atol(service, &error_text));
+	/* XXX we should do more range checking than just this! */
         if (error_text) {
             *error = xprintf("invalid port: %s", service, error_text);
             return -1;
         }
-    } else if (! port) {
+    } else if (port == 0) {		/* port is static -- only do this once! */
         smtp_service = getservbyname(service, "tcp");
         if (! smtp_service) {
             *error = xprintf("service name %s not found", service);
@@ -757,19 +785,15 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
 	    if (save_error) {
 		xfree(save_error);
 	    }
-#if defined(HAVE_BIND) && !defined(OBSOLETE_RESOLVER)
 	    *error = save_error = xprintf("gethostbyname(%s): %s", sending_name, hstrerror(h_errno));
-#else
-	    *error = save_error = xprintf("gethostbyname(%s): failed.", sending_name);
-#endif
 	    return -1;
 	}
 	memset((char *) &lsaddrin, '\0', sizeof(lsaddrin));
 	memcpy((char *) &lsaddrin.sin_addr, hostentp->h_addr_list[0], sizeof(struct in_addr));
 	lsaddrin.sin_family = family;
 	lsaddrin.sin_port = 0;		/* any old port will do.... */
-	DEBUG2(DBG_DRIVER_MID, "\nbinding socket to local source addr (%s)[%s]... ", sending_name, inet_ntoa(lsaddrin.sin_addr));
-	if (bind(s, (struct sockaddr *) &lsaddrin, sizeof(lsaddrin)) < 0) {
+	DEBUG2(DBG_DRIVER_MID, "\n   binding socket to local source addr (%s)[%s]... ", sending_name, inet_ntoa(lsaddrin.sin_addr));
+	if (bind(s, (struct sockaddr *) &lsaddrin, /* XXX (socklen_t) */ sizeof(lsaddrin)) < 0) {
 	    if (save_error) {
 		xfree(save_error);
 	    }
@@ -783,7 +807,9 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
     saddrin.sin_family = family;
     saddrin.sin_port = port;
 
-    if (connect(s, (struct sockaddr *) &saddrin, sizeof(saddrin)) < 0) {
+    DEBUG2(DBG_DRIVER_HI, "\n   connecting socket to [%s]/%d... ", inet_ntoa(saddrin.sin_addr), ntohs(saddrin.sin_port));
+
+    if (connect(s, (struct sockaddr *) &saddrin, /* XXX (socklen_t) */ sizeof(saddrin)) < 0) {
         if (save_error) {
             xfree(save_error);
         }
@@ -798,7 +824,7 @@ tcpsmtp_connect(remote_host, ip_addr, family, service, error)
 static struct error *
 connect_failure(tp, connect_error_text)
     struct transport *tp;
-    char *connect_error_text;
+    const char *connect_error_text;
 {
     char *error_text;
 
@@ -828,8 +854,8 @@ connect_failure(tp, connect_error_text)
  */
 static struct transport_hints *
 fake_addr_hints(remote_host, errorp)
-    char *remote_host;			/* hostname to look up */
-    char **errorp;
+    char *remote_host;			/* hostname to look up (writable) */
+    const char **errorp;
 {
     struct in_addr s_inet;		/* internet address */
     struct hostent hostent;		/* addr for remote host */
@@ -871,8 +897,14 @@ fake_addr_hints(remote_host, errorp)
     } else {
         hp = gethostbyname(remote_host);
         if (!hp) {
-	    /* XXX cast of hstrerror() is to explicitly discard const */
-	    *errorp = (char *) hstrerror(h_errno);
+	    /*
+	     * NOTE: if you get a compiler warning about hstrerror() not being
+	     * declared, or one warning about "illegal combination of pointer
+	     * and integer", then you are not compiling with the headers from a
+	     * modern enough resolver library!  (regardless of what you think
+	     * you're doing ;-)
+	     */
+	    *errorp = hstrerror(h_errno);
             return NULL;
         }
     }
@@ -902,3 +934,5 @@ fake_addr_hints(remote_host, errorp)
 
     return hints;
 }
+
+#endif /* HAVE_BSD_NETWORKING */
