@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src:RELEASE-3_2_0_114:version.c,v 1.15 2000/09/03 17:00:19 woods Exp"
+#ident	"@(#)smail/src:RELEASE-3_2_0_115:version.c,v 1.19 2003/03/27 16:34:07 woods Exp"
  */
 
 /*
@@ -14,16 +14,27 @@
  * version:
  *	return the current smail version
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
-#include "defs.h"
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "alloc.h"
 #include "dys.h"
+#include "smailstring.h"
 #include "version.h"
 #include "extern.h"
+#include "smailport.h"
 
 /*
  * version - keep track of the smail version number
@@ -52,8 +63,7 @@ version()
      * form the version string for the first time if needed
      */
     if (!fmtver) {
-	fmtver = xmalloc(sizeof("Smail- x #dddddd") + strlen(version_number) + strlen(release_date));
-	(void) sprintf(fmtver, "Smail-%s %s #%d", version_number, release_date, compile_num);
+	fmtver = xprintf("Smail-%s %s #%d", version_number, release_date, compile_num);
     }
 
     /*
