@@ -1,6 +1,6 @@
 #! /bin/sh
 :
-#ident	"@(#)contrib:RELEASE-3_2_0_114:bouncemail.sh,v 1.5 2001/01/18 07:36:31 woods Exp"
+#ident	"@(#)contrib:RELEASE-3_2_0_115:bouncemail.sh,v 1.7 2003/01/16 19:33:56 woods Exp"
 #
 #	bouncemail.sh - bounce a mail message after one more try
 #
@@ -13,7 +13,7 @@ umask 022
 TMPDIR="X_TMP_DIR_X"
 if [ ! -d $TMPDIR ] ; then
 	if ! mkdir $TMPDIR ; then
-		echo "$0: $TMPDIR exists as a file!" 2>&1
+		echo "$0: $TMPDIR exists as a file!" 1>&2
 		exit 1
 	else
 		chmod 700 $TMPDIR
@@ -27,24 +27,28 @@ trap 'rc=$?; rm -f $TMPFILES; exit $rc' 0 1 2 3 13 15
 
 argv0=`basename $0`
 
-USAGE="Usage: $argv0 [-v] message_queue_id [...]"
+USAGE="Usage: $argv0 [-v[N]] message_queue_id [...]"
 
 case "$1" in
 -v*)
 	verbose="$1"
 	shift
 	;;
+-*)
+	echo $USAGE 1>&2
+	exit 2
+	;;
 esac
 
 if [ $# -lt 1 ] ; then
-	echo $USAGE 2>&1
+	echo $USAGE 1>&2
 	exit 2
 fi
 
 if expr "`id`" : '^uid=0(root)' >/dev/null ; then
 	:
 else
-	echo "$argv0:  ERROR:  you must be root to do this!" 2>&1
+	echo "$argv0:  ERROR:  you must be root to do this!" 1>&2
 	exit 1
 fi
 
@@ -102,7 +106,7 @@ fi
 
 $SMAIL_NAME -bp ${1+"$@"} > $TMPDIR/qu$$
 if [ ! -s $TMPDIR/qu$$ ] ; then
-	echo "$argv0: no message matching '${1+"$@"}'" 2>&1
+	echo "$argv0: no message matching '${1+"$@"}'" 1>&2
 	exit 1
 fi
 
@@ -116,7 +120,7 @@ case "$ans" in
 y*|Y*)
 	;;
 *)
-	echo "$argv0: Aborting..." 2>&1
+	echo "$argv0: Aborting..." 1>&2
 	exit 1
 	;;
 esac
