@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/util:RELEASE-3_2_0_114:dcasehost.c,v 1.8 2000/09/11 01:13:56 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_115:dcasehost.c,v 1.15 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -25,14 +25,42 @@
  * before downcase is done.
  */
 
+#include "defs.h"
+
+#include <sys/types.h>
 #include <stdio.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#if defined(__GLIBC__) && !defined(__OPTIMIZE_SIZE__)
+/*
+ * XXX __NO_STRING_INLINES may even be a better choice here, but this works for
+ * now for at least GLIBC 2.3 on Red Hat 8.0....
+ */
+# define __OPTIMIZE_SIZE__      1       /* avoid buggy inline macros */
+#endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
+#include "smailport.h"
 
 char *program;			/* argv[0] from main */
 
@@ -92,7 +120,7 @@ main(argc, argv)
 		f_end > buf && *f_end == '\n') {
 
 	        /* save field 1 with newline and NULL */
-	        strncpy(buf2, buf, (unsigned int)(f1_white-buf));
+	        strncpy(buf2, buf, (size_t)(f1_white-buf));
 		buf2[f1_white-buf] = '\n';
 		buf2[f1_white-buf+1] = '\0';
 
@@ -108,12 +136,12 @@ main(argc, argv)
 		 */
 		if (buf2[0] == ' ' || buf2[0] == '\t') {
 		    /* overwrite the old trailing newline */
-		    strncpy(&buf[f_end-f2_field], buf2, (unsigned int) (f1_white-buf+2));
+		    strncpy(&buf[f_end-f2_field], buf2, (size_t) (f1_white-buf+2));
 		    
 		} else {
 		    /* separate old line end and field 1 with a tab */
 		    buf[f_end-f2_field] = '\t';
-		    strncpy(&buf[f_end-f2_field+1], buf2, (unsigned int) (f1_white-buf+2));
+		    strncpy(&buf[f_end-f2_field+1], buf2, (size_t) (f1_white-buf+2));
 		}
 	    }
 	}
@@ -128,7 +156,7 @@ main(argc, argv)
 	    register char *p;	/* index */
 
 	    for (p=buf; p < f1_white; ++p) {
-		*p = lowercase((unsigned char) *p);
+		*p = tolower((int) *p);
 	    }
 	}
 
