@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/util:RELEASE-3_2_0_114:gleem.c,v 1.7 2000/09/03 17:01:55 woods Exp"
+#ident	"@(#)smail/src/util:RELEASE-3_2_0_115:gleem.c,v 1.11 2003/04/02 22:15:04 woods Exp"
  */
 
 /*
@@ -57,16 +57,37 @@
  *	 the single '#'.
  */
 
-#include <stdio.h>
 #include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#include <sys/types.h>
+#include <stdio.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-#ifdef HAVE_UNISTD_H
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
 #endif
+
 #include "smail.h"
 #include "main.h"
+#include "smailport.h"
 
 #define BUFFER_SIZE 4096		/* length of longest line */
 
@@ -318,7 +339,7 @@ strip_com( buf )
     /*
      * remove a comment
      */
-    p = index(buf, '#');
+    p = strchr(buf, '#');
     if (p != NULL) {
 
 	/* strip */
@@ -529,7 +550,7 @@ safemap( filename, linenum, buf )
     /* 
      * begins with an unsafe letter, is it really an unsafe word? 
      */
-    if (strncmp(buf, "delete", 6) == 0) {
+    if (strncmp(buf, "delete", (size_t) 6) == 0) {
 
 	/* leading chars are ok, but is if followed by space/tab? */
 	for (p = buf+6; *p; ++p) {
@@ -551,7 +572,7 @@ safemap( filename, linenum, buf )
 	    }
 	}
 
-    } else if (strncmp(buf, "adjust", 6) == 0) {
+    } else if (strncmp(buf, "adjust", (size_t) 6) == 0) {
 
 	/* leading chars are ok, but is if followed by space/tab? */
 	for (p = buf+6; *p; ++p) {
@@ -591,7 +612,7 @@ file_2_space( buf )
     /*
      * look for a file directive
      */
-    if (strncmp(buf, "file", 4) == 0) {
+    if (strncmp(buf, "file", (size_t) 4) == 0) {
 
 	/* leading chars are ok, but is if followed by space/tab? */
 	for (p = buf+4; *p; ++p) {
