@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/util:RELEASE-3_2_0_114:mkdbm.c,v 1.21 2001/01/18 01:08:18 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_115:mkdbm.c,v 1.32 2003/04/02 22:15:03 woods Exp"
  */
 
 /*
@@ -18,41 +18,56 @@
  *
  *	Usage:  mkdbm [-o database] [-fvndy] [input_file ...]
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+#include <limits.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-#if defined(POSIX_OS)
-# include <limits.h>
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
 # include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_LIMITS_H)
-#  include <limits.h>
-# endif
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
-#if (defined(POSIX_OS) || !defined(UNIX_BSD)) && !defined(ISC_SOCKET_TIME_BUG)
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
 # include <time.h>
 #else
-# include <sys/time.h>
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
-#include "smail.h"
-#include "main.h"
-#include "parse.h"
-#include "addr.h"
-#include "alloc.h"
-#include "dys.h"
-#include "route.h"
-#include "bindlib.h"
-#include "spool.h"
-#include "exitcodes.h"
+
 #if defined(UNIX_SYS5) || defined(POSIX_OS) || defined(USE_FCNTL)
 # include <fcntl.h>
 #else
@@ -60,22 +75,37 @@
 #  include <sys/file.h>
 # endif
 #endif
-#include "dbm_compat.h"
-#include "extern.h"
-
 #if !defined(HAVE_GETHOSTNAME) && defined(HAVE_UNAME)
 #include <sys/utsname.h>
 #endif
 
+#include "smail.h"
+#include "main.h"
+#include "parse.h"
+#include "addr.h"
+#include "alloc.h"
+#include "smailstring.h"
+#include "dys.h"
+#include "route.h"
+#include "bindlib.h"
+#include "spool.h"
+#include "exitcodes.h"
+#include "dbm_compat.h"
+#include "extern.h"
+#include "smailport.h"
+
 /* variables provided by this file */
 char *program;				/* argv[0] from main */
 int debug = 0;
 FILE *errfile;
+int only_testing = TRUE;
 
 /* variables local to this file */
 static char *out_name = NULL;
 static char *temp_dir = NULL;
+#ifndef DBM_SUFFIX
 static char *temp_pag = NULL;
+#endif
 static int downcase = FALSE;
 static int ypstamps = FALSE;
 static int verbose = FALSE;
@@ -98,7 +128,7 @@ static void add_yp_stamps __P((void));
 /*ARGSUSED*/
 int
 main(argc, argv)
-    int argc;
+    int argc;					/* UNUSED */
     char **argv;
 {
     char **files;
@@ -161,7 +191,7 @@ main(argc, argv)
 		      reccnt, maxlen);
     }
 
-    dbmclose();
+    (void) dbmclose();
     exit(EX_OK);
 
     /* NOTREACHED */
@@ -254,7 +284,9 @@ done(ex)
 {
     (void) dbmclose();
     (void) unlink(temp_dir);
+#ifndef DBM_SUFFIX
     (void) unlink(temp_pag);
+#endif
     exit(ex);
 }
 
@@ -329,12 +361,12 @@ build_temp_name(name)
     char *name;
 {
     char *fn;
-    unsigned int dirlen;
+    size_t dirlen;
     char *slash = strrchr(name, '/');
 
     if (slash) {
 	dirlen = slash - name;
-	fn = xmalloc((unsigned) (dirlen + sizeof("/dbmXXXXXX")));
+	fn = xmalloc(dirlen + sizeof("/dbmXXXXXX"));
 	(void) strncpy(fn, name, dirlen);
 	(void) strcpy(fn + dirlen, "/dbmXXXXXX");
     } else {
@@ -351,40 +383,13 @@ static void
 create_database(dbm_name)
     char *dbm_name;
 {
-#ifndef DBM_SUFFIX	/* not db emulation of ndbm */
-    int fd;
-#endif
-    int temp_len = strlen(dbm_name);
-
-#ifdef DBM_SUFFIX	/* db emulation of ndbm */
-
-    temp_dir = xmalloc(temp_len + sizeof(DBM_SUFFIX));
+#ifdef DBM_SUFFIX	/* indicates db(3) emulation of ndbm */
+    temp_dir = xmalloc(strlen(dbm_name) + sizeof(DBM_SUFFIX));
     (void) sprintf(temp_dir, "%s%s", dbm_name, DBM_SUFFIX);
+#endif
 
-#else
-
-    temp_dir = xmalloc(temp_len + sizeof(".dir"));
-    temp_pag = xmalloc(temp_len + sizeof(".pag"));
-    (void) sprintf(temp_dir, "%s.dir", dbm_name);
-    (void) sprintf(temp_pag, "%s.pag", dbm_name);
-
-    fd = creat(temp_dir, 0644);
-    if (fd < 0) {
-	(void) fprintf(stderr, "%s: cannot creat ", program);
-	(void) perror(temp_dir);
-	exit(EX_TEMPFAIL);
-    }
-    (void) close(fd);
-
-    fd = creat(temp_pag, 0644);
-    if (fd < 0) {
-	(void) fprintf(stderr, "%s: cannot creat ", program);
-	(void) perror(temp_pag);
-	(void) unlink(temp_dir);
-	exit(EX_TEMPFAIL);
-    }
-    (void) close(fd);
-
+#if defined(HAVE_DBM) && !defined(HAVE_NDBM)
+    mkdbfiles(dbm_name); /* ancient dbm won't create its own files */
 #endif
 
     /* open the new DBM database */
@@ -395,6 +400,37 @@ create_database(dbm_name)
     }
 }
 
+#if defined(HAVE_DBM) && !defined(HAVE_NDBM)
+static void
+mkdbfiles(dbm_name)
+    char *dbm_name;
+{
+    int temp_len = strlen(dbm_name);
+
+    temp_dir = xmalloc(temp_len + sizeof(".dir"));
+    temp_pag = xmalloc(temp_len + sizeof(".pag"));
+    (void) sprintf(temp_dir, "%s.dir", dbm_name);
+    (void) sprintf(temp_pag, "%s.pag", dbm_name);
+
+    fd = creat(temp_dir, 0644);
+    if (fd < 0) {
+	(void) fprintf(stderr, "%s: cannot create ", program);
+	(void) perror(temp_dir);
+	exit(EX_TEMPFAIL);
+    }
+    (void) close(fd);
+
+    fd = creat(temp_pag, 0644);
+    if (fd < 0) {
+	(void) fprintf(stderr, "%s: cannot create ", program);
+	(void) perror(temp_pag);
+	(void) unlink(temp_dir);
+	exit(EX_TEMPFAIL);
+    }
+    (void) close(fd);
+}
+#endif
+
 /*
  * add all entries in the file to the DBM database
  */
@@ -408,8 +444,9 @@ add_to_database(f)
 	register char *data;
 	datum key, content;
 
-	for (data = s; *data && *data != ':' && !isspace(*data); data++) ;
-
+	for (data = s; *data && *data != ':' && !isspace((int) *data); data++) {
+	    ;
+	}
 	if (*data) {
 	    *data++ = '\0';
 	    key.dptr = s;
@@ -424,7 +461,7 @@ add_to_database(f)
 		register char *sp;
 
 		for (sp = s; *sp; sp++) {
-		    *sp = lowercase((unsigned char) *sp);
+		    *sp = tolower((int) *sp);
 		}
 	    }
 	    if (store(key, content) < 0) {
@@ -499,26 +536,30 @@ rename_database(from, to)
     char *from;
     char *to;
 {
-    unsigned int fromlen = (unsigned)strlen(from);
-    unsigned int tolen = (unsigned)strlen(to);
-    char *from_dir = xmalloc(fromlen + sizeof(".dir"));
-    char *to_dir = xmalloc(tolen + sizeof(".dir"));
-#ifndef DBM_SUFFIX
-    char *to_pag = xmalloc(tolen + sizeof(".pag"));
-    char *from_pag = xmalloc(fromlen + sizeof(".pag"));
-#endif
-
+    unsigned int fromlen = (unsigned int) strlen(from);
+    unsigned int tolen = (unsigned int) strlen(to);
 #ifdef DBM_SUFFIX
+    char *from_dir = xmalloc(fromlen + sizeof(DBM_SUFFIX));
+    char *to_dir = xmalloc(tolen + sizeof(DBM_SUFFIX));
+
     (void) sprintf(from_dir, "%s%s", from, DBM_SUFFIX);
     (void) sprintf(to_dir, "%s%s", to, DBM_SUFFIX);
-#else
+
+#else /* !DBM_SUFFIX */
+
+    char *from_dir = xmalloc(fromlen + sizeof(".dir"));
+    char *to_dir = xmalloc(tolen + sizeof(".dir"));
+    char *to_pag = xmalloc(tolen + sizeof(".pag"));
+    char *from_pag = xmalloc(fromlen + sizeof(".pag"));
+
     (void) sprintf(from_dir, "%s.dir", from);
     (void) sprintf(from_pag, "%s.pag", from);
     (void) sprintf(to_dir, "%s.dir", to);
     (void) sprintf(to_pag, "%s.pag", to);
 
     (void) unlink(to_pag);
-#endif
+
+#endif /* DBM_SUFFIX */
 
     (void) unlink(to_dir);
     (void) sleep(2);			/* sleep at least 1 second */
@@ -544,7 +585,7 @@ rename_database(from, to)
 /*ARGSUSED*/
 char *
 qualify_domain(s)
-    char *s;
+    char *s;					/* UNUSED */
 {
     return NULL;
 }
@@ -553,7 +594,7 @@ qualify_domain(s)
 char *
 preparse_address(s, error)
     char *s;
-    char **error;
+    char **error;				/* UNUSED */
 {
     return s;
 }
