@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/util:RELEASE-3_2_0_114:mkline.c,v 1.11 2001/01/18 01:08:19 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_115:mkline.c,v 1.18 2003/04/02 22:15:03 woods Exp"
  */
 
 /*
@@ -15,32 +15,60 @@
  *	Take an alias, pathalias or similarly formed file and collapse
  *	into single-line records with comments stripped.
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "field.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "route.h"
 #include "bindlib.h"
 #include "spool.h"
 #include "exitcodes.h"
 #include "extern.h"
+#include "smailport.h"
 
 /* variables provided by this file */
 char *program;				/* argv[0] from main */
 int debug = 0;
 FILE *errfile;
+int only_testing = TRUE;
 int use_tabs_p = FALSE;
 int ignore_keys_p = FALSE;
 
@@ -51,7 +79,7 @@ static int mklistline __P((FILE *));
 /*ARGSUSED*/
 int
 main(argc, argv)
-    int argc;
+    int argc;				/* UNUSED */
     char *argv[];
 {
     int list_files_p = FALSE;		/* TRUE for -l option */
@@ -134,7 +162,7 @@ mkaliasline(f)
 	register struct token *toks;
 
 	if (! ignore_keys_p) {
-	    while (!isspace(*sp) && *sp != ':') {
+	    while (!isspace((int) *sp) && *sp != ':') {
 		sp++;
 	    }
 
@@ -148,7 +176,7 @@ mkaliasline(f)
 	if (error) {
 	    if (ignore_keys_p) {	/* wasn't split above... */
 		sp = s;
-		while (!isspace(*sp) && *sp != ':') {
+		while (!isspace((int) *sp) && *sp != ':') {
 		    sp++;
 		}
 
@@ -215,7 +243,7 @@ mklistline(f)
 /*ARGSUSED*/
 char *
 qualify_domain(s)
-    char *s;
+    char *s;				/* UNUSED */
 {
     return NULL;
 }
@@ -225,7 +253,7 @@ alloc_addr()
 {
     register struct addr *ret = (struct addr *)xmalloc(sizeof(struct addr));
 
-    bzero((char *)ret, sizeof(struct addr));
+    (void) memset((char *) ret, '\0', sizeof(struct addr)); /* XXX hope NULLs are all 0 bits */
     return ret;
 }
 
@@ -233,7 +261,7 @@ alloc_addr()
 char *
 preparse_address(s, error)
     char *s;
-    char **error;
+    char **error;			/* UNUSED */
 {
     return s;
 }
