@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/util:RELEASE-3_2_0_114:mksort.c,v 1.10 2001/07/29 20:13:24 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_115:mksort.c,v 1.18 2003/04/02 22:16:18 woods Exp"
  */
 
 /*
@@ -15,23 +15,50 @@
  *	Take a list of lines and sort them.  If the flag -f is given,
  *	then ignore case in comparisons.
  */
+
+#include "defs.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "addr.h"
 #include "field.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "exitcodes.h"
 #include "extern.h"
+#include "smailport.h"
 
 /* variables provided by this file */
 char *program;				/* argv[0] from main */
@@ -41,20 +68,20 @@ FILE *errfile;
 /* functions local to this file */
 static int cmp __P((const void *, const void *));
 static int cmpic __P((const void *, const void *));
-static char **mkvectors __P((unsigned int, char *));
+static char **mkvectors __P((size_t, char *));
 static int read_lines __P((struct str *, FILE *));
 
 /*ARGSUSED*/
 int
 main(argc, argv)
-    int argc;
+    int argc;					/* UNUSED */
     char **argv;
 {
     struct str strings;
-    int (*compare_fun) __P((const void *, const void *)) = cmp;
-    unsigned int ct = 0;
+    int (*compare_fun) __P((const void *, const void *));
+    size_t ct = 0;
     char **v;
-    int i;
+    size_t i;
 
     program = *argv++;
     errfile = stderr;
@@ -63,8 +90,9 @@ main(argc, argv)
     if (*argv && EQ(*argv, "-f")) {
 	compare_fun = cmpic;
 	argv++;
+    } else {
+	compare_fun = cmp;
     }
-
     STR_INIT(&strings);
     if (*argv == NULL) {
 	ct += read_lines(&strings, stdin);
@@ -120,7 +148,7 @@ read_lines(ssp, f)
 
 static char **
 mkvectors(ct, strings)
-    unsigned int ct;			/* count of strings */
+    size_t ct;				/* count of strings */
     char *strings;			/* null-terminated strings */
 {
     char **v;
@@ -139,16 +167,16 @@ cmp(a, b)
     const void *a;
     const void *b;
 {
-    char *s1 = *(char **) a;
-    char *s2 = *(char **) b;
+    const char *s1 = *(const char * const *) a;
+    const char *s2 = *(const char * const *) b;
     register int c1, c2;
 
     for (;;) {
 	c1 = *s1++;
 	c2 = *s2++;
-	if (isspace(c1) || c1 == ':')
+	if (isspace((int) c1) || c1 == ':')
 	    c1 = '\0';
-	if (isspace(c2) || c2 == ':')
+	if (isspace((int) c2) || c2 == ':')
 	    c2 = '\0';
 	if (c1 != c2)
 	    return c1 - c2;
@@ -163,16 +191,16 @@ cmpic(a, b)
     const void *a;
     const void *b;
 {
-    char *s1 = *(char **) a;
-    char *s2 = *(char **) b;
+    const char *s1 = *(const char * const *) a;
+    const char *s2 = *(const char * const *) b;
     register int c1, c2;
 
     for (;;) {
-	c1 = lowercase((unsigned char) *s1++);
-	c2 = lowercase((unsigned char) *s2++);
-	if (isspace(c1) || c1 == ':')
+	c1 = tolower((int) *s1++);
+	c2 = tolower((int) *s2++);
+	if (isspace((int) c1) || c1 == ':')
 	    c1 = '\0';
-	if (isspace(c2) || c2 == ':')
+	if (isspace((int) c2) || c2 == ':')
 	    c2 = '\0';
 	if (c1 != c2)
 	    return c1 - c2;
@@ -187,7 +215,7 @@ cmpic(a, b)
  */
 char *
 xmalloc(len)
-    unsigned int len;
+    size_t len;
 {
     register char *ret = malloc(len);
 
@@ -200,7 +228,7 @@ xmalloc(len)
 char *
 xrealloc(s, len)
     char *s;
-    unsigned int len;
+    size_t len;
 {
     register char *ret = realloc(s, len);
 
