@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/src/util:RELEASE-3_2_0_114:pathmerge.c,v 1.6 2000/09/03 17:01:56 woods Exp"
+#ident	"@(#)smail/src/util:RELEASE-3_2_0_115:pathmerge.c,v 1.12 2003/04/02 22:15:03 woods Exp"
  */
 
 /*
@@ -21,15 +21,36 @@
  *	given in the argument list can be `-' for the standard
  *	input.
  */
+
+#include "defs.h"
+
+#include <sys/types.h>
 #include <stdio.h>
 #include <errno.h>
 #include <ctype.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
+
+#ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
+# include <string.h>
+#endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
+#include "smailport.h"
 
 char *program;			/* who we are */
 
@@ -38,8 +59,6 @@ char *program;			/* who we are */
 #define TRUE 1
 #define FALSE 0
 
-#define lower(c) ((char)(lowercase(toascii((int)(c)))))	/* safe tolower */
-
 /*
  * file state
  */
@@ -187,7 +206,7 @@ main(argc,argv)
 				/* write out that line */
 				fputs(line[next], stdout);
 				/* save it */
-				strncpy(previous, line[next], MAXLINE);
+				strncpy(previous, line[next], (size_t) MAXLINE);
 			}
 			status[next] = readme; /* ready to read again */
 			/* skip lines that are equal to the selected one */
@@ -276,7 +295,7 @@ cmp(s, t)
 	/*
 	 * look for an irregularity
 	 */
-	while (*s == *t && !isspace(*s) && *s != '\0' && *s != ':') {
+	while (*s == *t && !isspace((int) *s) && *s != '\0' && *s != ':') {
 		++s;
 		++t;
 	}
@@ -286,18 +305,18 @@ cmp(s, t)
 	 * determine the relationship between the lines
 	 */
 	/* if we matched all the way, then note a line match */
-	if ((isspace(*s) || *s == ':' || *s == '\0') &&
-	    (isspace(*t) || *t == ':' || *t == '\0'))
+	if ((isspace((int) *s) || *s == ':' || *s == '\0') &&
+	    (isspace((int) *t) || *t == ':' || *t == '\0'))
 	{
 	    return (0);
 	/* if the first line ended early, it is before the second */
-	} else if (isspace(*s) || *s == '\0' || *s == ':') {
+	} else if (isspace((int) *s) || *s == '\0' || *s == ':') {
 		return (-1);
 	/* if the second line ended early, it is before the first */
-	} else if (isspace(*t) || *t == '\0' || *t == ':') {
+	} else if (isspace((int) *t) || *t == '\0' || *t == ':') {
 		return (1);
 	/* we hit a difference spot, note which is earler */
 	} else {
-		return ((lower(*s) < lower(*t)) ? -1 : 1);
+		return ((tolower((int) *s) < tolower((int) *t)) ? -1 : 1);
 	}
 }
