@@ -1,5 +1,5 @@
 /*
-#ident	"@(#)smail/util:RELEASE-3_2_0_114:unsharmap.c,v 1.16 2001/01/18 01:08:19 woods Exp"
+#ident	"@(#)smail/util:RELEASE-3_2_0_115:unsharmap.c,v 1.24 2003/04/02 22:15:03 woods Exp"
  */
 /*
  *    Copyright (C) 1988 Ronald S. Karr and Landon Curt Noll
@@ -64,38 +64,55 @@
  * exits by 0 if there was no errors, non-0 otherwise.
  */
  
+#include "defs.h"
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <time.h>
 #include <fcntl.h>
-#include "defs.h"
-#ifdef HAVE_STDLIB_H
+#include <limits.h>
+
+#ifdef STDC_HEADERS
 # include <stdlib.h>
+# include <stddef.h>
+#else
+# ifdef HAVE_STDLIB_H
+#  include <stdlib.h>
+# endif
 #endif
-#if defined(POSIX_OS)
-# include <limits.h>
-# include <unistd.h>
-#else /* not POSIX_OS */
-# if defined (HAVE_LIMITS_H)
-#  include <limits.h>
-# endif
-# if defined (HAVE_UNISTD_H)
-#  include <unistd.h>
-# endif
-#endif	/* not POSIX_OS */
+
 #ifdef HAVE_STRING_H
+# if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
+#  include <memory.h>
+# endif
 # include <string.h>
 #endif
+#ifdef HAVE_STRINGS_H
+# include <strings.h>
+#endif
+
+#ifdef __STDC__
+# include <stdarg.h>
+#else
+# include <varargs.h>
+#endif
+
+#if defined(HAVE_UNISTD_H)
+# include <unistd.h>
+#endif
+
 #include "smail.h"
 #include "main.h"
 #include "parse.h"
 #include "addr.h"
 #include "alloc.h"
+#include "smailstring.h"
 #include "dys.h"
 #include "route.h"
 #include "bindlib.h"
 #include "spool.h"
 #include "extern.h"
+#include "smailport.h"
 
 #define HEADER_OK 1			/* the article header is correct */
 #define HEADER_ERROR 2			/* bad header, skip and log */
@@ -104,6 +121,7 @@
 char *program;				/* our name */
 int debug = 0;
 FILE *errfile;
+int only_testing = TRUE;
 static char err[BUFSIZ+1];		/* error and log message */
 char buf[BUFSIZ+1];			/* input buffer */
 char mapname[BUFSIZ+1];			/* name of the output map file */
@@ -297,7 +315,7 @@ main( argc, argv )
  *
  * returns the date in ctime(3) format
  */
-char *
+static char *
 get_date()
 {
     time_t now = time((time_t *) NULL);   /* seconds since 1-jan-1970 0:00:00 GMT */
@@ -309,7 +327,7 @@ get_date()
 /*
  * log - write a message to the log
  */
-void
+static void
 logit( msg )
     char *msg;				/* the message to write */
 {
@@ -337,7 +355,7 @@ logit( msg )
  *         HEADER_ERROR is the header was bad and needs to be skipped & logged,
  *	   HEADER_SKIP if the file is not a UUCP shar file and should be skipped
  */
-int
+static int
 check_header( article )
     FILE *article;			/* the article stream */
 {
@@ -378,7 +396,7 @@ check_header( article )
 	switch (buf[0]) {
 	case 'n':
 	case 'N':			/* could be Newsgroups: */
-	    if (strncmpic("Newsgroups: ", buf, 12) == 0) {
+	    if (strncmpic("Newsgroups: ", buf, (size_t) 12) == 0) {
 		for (p = strcolon(newsgroups); p; p = strcolon((char *)NULL)) {
 		    if (strncmpic(buf + 12, p, strlen(p)) == 0 &&
 			buf[12 + strlen(p)] == '\n')
@@ -397,8 +415,8 @@ check_header( article )
 	    break;
 	case 's':
 	case 'S':			/* Subject: buf */
-	    if (strncmpic("Subject: ", buf, 9) == 0) {
-		if (strncmpic("Subject: UUCP ", buf, 14) == 0) {
+	    if (strncmpic("Subject: ", buf, (size_t) 9) == 0) {
+		if (strncmpic("Subject: UUCP ", buf, (size_t) 14) == 0) {
 		    saw_subject = 1;
 		    logit("    ");
 		    logit(buf);
@@ -411,7 +429,7 @@ check_header( article )
 	    break;
 	case 'm':
 	case 'M':			/* Message-ID: buf */
-	    if (strncmpic("Message-ID: ", buf, 12) == 0) {
+	    if (strncmpic("Message-ID: ", buf, (size_t) 12) == 0) {
 		    saw_message = 1;
 		    logit("    ");
 		    logit(buf);
@@ -419,7 +437,7 @@ check_header( article )
 	    break;
 	case 'd':
 	case 'D':			/* Message-ID: buf */
-	    if (strncmpic("Date: ", buf, 6) == 0) {
+	    if (strncmpic("Date: ", buf, (size_t) 6) == 0) {
 		saw_date = 1;
 		logit("    ");
 		logit(buf);
@@ -427,7 +445,7 @@ check_header( article )
 	    break;
 	case 'a':
 	case 'A':			/* Message-ID: buf */
-	    if (strncmpic("Approved: ", buf, 10) == 0) {
+	    if (strncmpic("Approved: ", buf, (size_t) 10) == 0) {
 		saw_approved = 1;
 		logit("    ");
 		logit(buf);
@@ -435,7 +453,7 @@ check_header( article )
 	    break;
 	}
 
-    } while(strcmp("\n", buf) != 0);	/* while heading the header */
+    } while (strcmp("\n", buf) != 0);	/* while heading the header */
 
     /*
      * report if er got everything
@@ -466,7 +484,7 @@ check_header( article )
 /*
  * skip_article - skip article, log and close it
  */
-void
+static void
 skip_article( article, filename )
     FILE *article;			/* the article stream */
     char *filename;			/* the filename we are skipping */
@@ -494,7 +512,7 @@ skip_article( article, filename )
 /*
  * ingore_article - ignore an article, close and report it
  */
-void
+static void
 ignore_article( article, filename )
     FILE *article;			/* the article stream */
     char *filename;			/* the filename we are skipping */
@@ -532,7 +550,7 @@ ignore_article( article, filename )
  * Any other line is printed to both logs as an error.  The "THE-FILE-NAME"
  * string is returned if a valid preamble is found, NULL otherwise.
  */
-char *
+static char *
 check_preamble( article )
     FILE *article;			/* the article stream */
 {
@@ -576,7 +594,7 @@ check_preamble( article )
 	/*
 	 * look for the echo line
 	 */
-	if (strncmp("echo shar: extracting ", buf, 22) == 0) {
+	if (strncmp("echo shar: extracting ", buf, (size_t) 22) == 0) {
 	    /* grab the mapname */
 	    for (p=buf+22, q=mapname, bad_echo=0; *p; ++p) {
 		/* be sure it is well formed */
@@ -614,7 +632,7 @@ check_preamble( article )
 	/*
 	 * watch for the cat line
 	 */
-	} else if (strncmp("cat << 'SHAR_EOF' > ", buf, 20) == 0) {
+	} else if (strncmp("cat << 'SHAR_EOF' > ", buf, (size_t) 20) == 0) {
 
 	    /*
 	     * compare cat filename against echo filename
@@ -624,7 +642,7 @@ check_preamble( article )
 			lineno, buf);
 		logit(err);
 		return(NULL);		/* bad preamble */
-	    } else if (strncmp(buf+20, mapname, (unsigned int) (q-mapname)) != 0) {
+	    } else if (strncmp(buf+20, mapname, (size_t) (q-mapname)) != 0) {
 		sprintf(err,
 			"error: line %d, echo mapname: %s != cat mapname: %s",
 			lineno, mapname, buf+20);
@@ -662,7 +680,7 @@ check_preamble( article )
  *
  * returns 1 if all was ok, 0 otherwise.
  */
-int
+static int
 write_map( article )
     FILE *article;			/* the article stream */
 {
@@ -810,7 +828,7 @@ write_map( article )
  *
  * returns a pointer to the newline, or NULL if none found
  */
-char *
+static char *
 check_newline( str )
     char *str;				/* check for newline in str */
 {
@@ -827,7 +845,7 @@ check_newline( str )
 /*ARGSUSED*/
 char *
 qualify_domain(s)
-    char *s;
+    char *s;				/* UNUSED */
 {
     return NULL;
 }
@@ -836,7 +854,7 @@ qualify_domain(s)
 char *
 preparse_address(s, error)
     char *s;
-    char **error;
+    char **error;			/* UNUSED */
 {
     return s;
 }
